import { __assign, __read, __spreadArray } from "tslib";
import * as React from 'react';
import { observer } from 'mobx-react-lite';
import { checkVisible, checkDisplay, getDom, generateEmit, isSlot, CALC_RPX_ROOT_VAL_NAME, generateWidgetAPIContext, checkObserverInstanceRef, } from '../utils';
import { get, set, isEqual } from 'lodash-es';
import { generateSlotMetaMap, getComponentChildren } from './RenderWrapper';
import { InjectContext } from '../router';
import { ErrorBoundary } from '../utils/error';
import { processCSSPropertiesUnit } from '@cloudbase/cals';
import { autorun, observable } from 'mobx';
import { REPEATER } from '../utils/constants';
export var ForContext = React.createContext({});
export var ScopeContext = React.createContext({});
export var DataContext = React.createContext({});
var getSafeComponentProps = function (_a) {
    var style = _a.style, classNameList = _a.classNameList;
    var componentProps = {};
    if (classNameList.length) {
        componentProps.className = classNameList.join(' ');
    }
    if (style && Object.keys(style).length) {
        componentProps.style = style;
    }
    return componentProps;
};
// TODO: 需要不断移除 dataBinds(style/classList)
function getBindData(_a) {
    var _b;
    var widgetData = _a.widgetData, forItems = _a.forItems, scopeContext = _a.scopeContext, codeContext = _a.codeContext, _c = _a.commonStyle, commonStyle = _c === void 0 ? {} : _c, _d = _a.classNameList, classNameList = _d === void 0 ? [] : _d, _e = _a.injectContext, injectContext = _e === void 0 ? {} : _e;
    // bindData
    if (Array.isArray(widgetData)) {
        widgetData =
            forItems.forIndexes === void 0 || widgetData.length === 0
                ? {}
                : get(widgetData, getForIndexes(forItems, widgetData));
    }
    widgetData = widgetData || {};
    var fieldData = __assign({}, widgetData);
    var isRootNode = widgetData && !widgetData.parent;
    var isInComposite = !!codeContext.$WEAPPS_COMP;
    // 再次计算 scope value
    for (var key in fieldData) {
        if (Object.prototype.hasOwnProperty.call(fieldData, key)) {
            var value = fieldData[key];
            if (value && value.__type === 'scopedValue') {
                try {
                    fieldData[key] = value.getValue(scopeContext || {});
                }
                catch (e) {
                    console.warn("Error computing data bind '".concat(key, "' error:"), e);
                    fieldData[key] = '';
                }
            }
        }
    }
    // bindStyle
    var bindStyle = fieldData.style || {};
    /**
     * 若指令 hidden
     * 则相当于强制绑定了 display: 'none',
     */
    if (!(injectContext.checkDisplay || checkDisplay)(fieldData)) {
        bindStyle = __assign(__assign({}, bindStyle), { display: 'none' });
    }
    // 复合组件第一层需要将最外层样式 style 挂到节点上
    var cssStyle = commonStyle || {};
    var propStyle = isInComposite && isRootNode ? ((_b = codeContext.$WEAPPS_COMP.props) === null || _b === void 0 ? void 0 : _b.style) || {} : {};
    var finalStyle = getFinalStyle(injectContext.processCssUnit, cssStyle, bindStyle, propStyle);
    // bindClassList
    var bindClassList = fieldData.classList || [];
    var finalClassNameList = getFinalClassNameList(classNameList, bindClassList);
    // 当前在复合组件中，且当前节点为根节点
    if (isInComposite && isRootNode) {
        if (widgetData.ownerWidget) {
            Object.keys(widgetData.ownerWidget).forEach(function (propName) {
                if (propName === 'role' || ['aria-', 'data-'].some(function (prefix) { return propName.startsWith(prefix); })) {
                    widgetData[propName] = widgetData.ownerWidget[propName];
                }
            });
        }
    }
    // 防止渲染时 data 的 style 与实际的 style 冲突
    delete fieldData.style;
    return {
        fieldData: fieldData,
        finalStyle: finalStyle,
        finalClassNameList: Array.from(new Set(finalClassNameList)),
    };
}
var _FOR_ERROR_CACHE_MAP = {};
function getForList(compId, dataBinds, parentForItems, dataContext, $w, $scope) {
    /**
     * For循环渲染
     * @important
     * @default undfined // 代表没有进行for循环绑定
     */
    var forList = [];
    try {
        // 绑定了 for 变量，计算值最差应该是空数组
        // 没有绑定，那就是 undefined
        clearTimeout(_FOR_ERROR_CACHE_MAP[compId]);
        var list = (dataBinds === null || dataBinds === void 0 ? void 0 : dataBinds._waFor)
            ? dataBinds._waFor(parentForItems, undefined, dataContext, $w, $scope) || []
            : undefined;
        // 绑定了for 并且计算了值，但是不是数组类型，应该进行兜底
        if (list && !Array.isArray(list)) {
            list = [];
        }
        forList = list;
    }
    catch (e) {
        if ((window === null || window === void 0 ? void 0 : window.__WEDA_LOG_LEVEL__) === 'debug') {
            // 计算值出错则使用空数组兜底
            _FOR_ERROR_CACHE_MAP[compId] = setTimeout(function () {
                console.debug("".concat(compId, "\u5FAA\u73AF\u7ED1\u5B9A\u9519\u8BEF"), e);
            }, 1000);
        }
        forList = [];
    }
    if (forList && !Array.isArray(forList)) {
        _FOR_ERROR_CACHE_MAP[compId] = setTimeout(function () {
            console.warn("".concat(compId, "\u5FAA\u73AF\u7ED1\u5B9A\u975E\u6570\u7EC4\u503C\uFF1A"), forList);
        }, 1000);
        forList = [];
    }
    return forList;
}
// FieldWrapper 依赖 currentWidgets scope 因此需要添加observer
var FieldWrapper = observer(_FieldWrapper);
function useObserverableRef(sourceKey) {
    if (sourceKey === void 0) { sourceKey = ''; }
    var pureRef = React.useRef();
    var ref = React.useRef();
    var observerRef = React.useMemo(function () {
        return checkObserverInstanceRef(sourceKey) ? observable(ref) : pureRef;
    }, []);
    return observerRef;
}
function useWidgetInstanceRef(widget) {
    var ref = useObserverableRef(widget === null || widget === void 0 ? void 0 : widget.widgetType);
    return widget
        ? React.useMemo(function () {
            widget._instanceRef.current = null;
            return widget._instanceRef;
        }, [widget._instanceRef])
        : ref;
}
var emptyObject = {};
function _FieldWrapper(_a) {
    var _b, _c, _d, _e, _f;
    var Field = _a.Field, componentSchema = _a.componentSchema, id = _a.id, data = _a.data, _g = _a.style, style = _g === void 0 ? emptyObject : _g, _h = _a.className, className = _h === void 0 ? '' : _h, events = _a.events, compositeParent = _a.compositeParent, context = _a.context, updateContext = _a.updateContext, _j = _a.renderSlot, renderSlot = _j === void 0 ? false : _j;
    var forContext = context.forContext, forIndexes = context.forIndexes, widgetsData = context.widgetsData, codeContext = context.codeContext, 
    //  dataContext,
    virtualFields = context.virtualFields;
    // trace(true);
    var injectContext = React.useContext(InjectContext);
    var scopeContext = React.useContext(ScopeContext);
    var _dataContext = React.useContext(DataContext);
    var indexRef = React.useRef();
    var typeRef = React.useRef();
    var _k = __read(React.useState(0), 2), _ = _k[0], forceUpate = _k[1];
    var isInComposite = !!codeContext.$WEAPPS_COMP;
    var isDataContainer = (_c = (_b = componentSchema === null || componentSchema === void 0 ? void 0 : componentSchema.extra) === null || _b === void 0 ? void 0 : _b.compConfig) === null || _c === void 0 ? void 0 : _c.isDataContainer;
    var currentWidget = Array.isArray(widgetsData) ? get(widgetsData, forIndexes) : widgetsData;
    var instanceRef = useWidgetInstanceRef(currentWidget);
    // context 变化强刷一次children，同时把 currentWidget._scope?.dataContext  添加到 observer 中
    var dispose = React.useMemo(function () {
        return autorun(function (r) {
            var _a;
            // r.trace();
            if (isDataContainer && ((_a = currentWidget === null || currentWidget === void 0 ? void 0 : currentWidget._scope) === null || _a === void 0 ? void 0 : _a.dataContext) && Object.values(currentWidget === null || currentWidget === void 0 ? void 0 : currentWidget._scope.dataContext)) {
                forceUpate(function (_) {
                    return _ + 1;
                });
            }
        });
    }, []);
    React.useEffect(function () {
        return function () {
            dispose();
        };
    }, []);
    var totalContext = React.useMemo(function () {
        var _a;
        var _b;
        var res = isDataContainer && ((_b = currentWidget === null || currentWidget === void 0 ? void 0 : currentWidget._scope) === null || _b === void 0 ? void 0 : _b.dataContext)
            ? __assign(__assign({}, _dataContext), (_a = {}, _a[id] = currentWidget._scope.dataContext, _a)) : _dataContext;
        return res;
    }, [_dataContext, _]);
    var watchedList = isInComposite
        ? [componentSchema]
        : [componentSchema, scopeContext, codeContext, injectContext, virtualFields];
    var staticResourceAttribute = ((_d = componentSchema.extra) === null || _d === void 0 ? void 0 : _d.staticResourceAttribute) || [];
    // 组件最终用于执行的事件函数
    var emit = React.useCallback(generateEmit((_f = (_e = (!isInComposite ? codeContext.$page : codeContext.$WEAPPS_COMP)) === null || _e === void 0 ? void 0 : _e.__internal__) === null || _f === void 0 ? void 0 : _f.$w, currentWidget, injectContext, isDataContainer
        ? [
            {
                key: "wa".concat(Date.now().toString().slice(-8)),
                trigger: 'onDataChange',
                isCapturePhase: false,
                data: {},
                dataBinds: {},
                instanceFunction: function (_a) {
                    var _b, _c, _d, _e, _f, _g;
                    var event = _a.event;
                    if (currentWidget === null || currentWidget === void 0 ? void 0 : currentWidget._scope) {
                        if (currentWidget._scope.dataContext) {
                            if (!isEqual(currentWidget._scope.dataContext.data, (_b = event === null || event === void 0 ? void 0 : event.detail) === null || _b === void 0 ? void 0 : _b.data)) {
                                currentWidget._scope.dataContext.data = (_c = event === null || event === void 0 ? void 0 : event.detail) === null || _c === void 0 ? void 0 : _c.data;
                            }
                            if (!isEqual(currentWidget._scope.dataContext.state, (_d = event === null || event === void 0 ? void 0 : event.detail) === null || _d === void 0 ? void 0 : _d.state)) {
                                currentWidget._scope.dataContext.state = (_e = event === null || event === void 0 ? void 0 : event.detail) === null || _e === void 0 ? void 0 : _e.state;
                            }
                        }
                        else {
                            currentWidget._scope.dataContext = { data: (_f = event === null || event === void 0 ? void 0 : event.detail) === null || _f === void 0 ? void 0 : _f.data, state: (_g = event === null || event === void 0 ? void 0 : event.detail) === null || _g === void 0 ? void 0 : _g.state };
                        }
                    }
                    return event.detial;
                },
            },
        ]
        : [], {}), [currentWidget]);
    // 组件最终传给组件props的，要保持引用防rerender
    var emitFn = React.useCallback(function (trigger, eventData, originalEvent) {
        var _a;
        return emit(trigger, componentSchema.listenerInstances, eventData, forContext, originalEvent, scopeContext, totalContext, !isInComposite ? (_a = codeContext.$page) === null || _a === void 0 ? void 0 : _a.id : '');
    }, [componentSchema.listenerInstances, forContext, totalContext]);
    React.useLayoutEffect(function () {
        return function () {
            instanceRef.current = null;
            // if (currentWidget) {
            //   currentWidget._getInstanceRef = function getInstanceRef() {
            //     return null;
            //   };
            // }
        };
    }, []);
    React.useLayoutEffect(function () {
        // if (currentWidget) {
        //   currentWidget._getInstanceRef = () => instanceRef;
        // }
        return function () {
            instanceRef.current = null;
            // if (currentWidget) {
            //   currentWidget._getInstanceRef = function getInstanceRef() {
            //     return null;
            //   };
            // }
        };
    }, [currentWidget]);
    if (!Array.isArray(staticResourceAttribute)) {
        staticResourceAttribute = [];
    }
    // 判断为容器组件，则增加一个onDataChange事件
    if (isDataContainer && !(events === null || events === void 0 ? void 0 : events.includes('onDataChange'))) {
        events.push('onDataChange');
    }
    // 多个组件的 slot 属性
    var slotMetaMap = React.useMemo(function () {
        return generateSlotMetaMap(componentSchema, {
            scopeContext: scopeContext,
            codeContext: codeContext,
            virtualFields: virtualFields,
            updateContext: updateContext,
            injectContext: injectContext,
        }, {
            renderSlot: renderSlot,
        });
    }, watchedList);
    var slotMap = React.useMemo(function () {
        return generateSlotMap(slotMetaMap, {
            forContext: forContext,
            dataContext: totalContext,
        });
    }, [slotMetaMap, totalContext, forContext]);
    var _selectableBlockEvents = React.useMemo(function () {
        var _a;
        var r = {
            // 选区的预览的click事件
            onCustomEvent: function (_a) {
                var index = _a.order, blockKey = _a.blockKey;
                if (index !== undefined) {
                    indexRef.current = index;
                    typeRef.current = blockKey;
                }
            },
            events: ((_a = componentSchema === null || componentSchema === void 0 ? void 0 : componentSchema.selectableBlock) === null || _a === void 0 ? void 0 : _a.events.map(function (item) { return item.name; })) || [],
            emit: function (trigger, eventData, originalEvent) {
                var _a, _b, _c;
                return emit(trigger, typeRef.current &&
                    indexRef.current !== undefined &&
                    ((_c = (_b = (_a = data === null || data === void 0 ? void 0 : data[typeRef.current]) === null || _a === void 0 ? void 0 : _a[indexRef.current]) === null || _b === void 0 ? void 0 : _b.selectableBlock) === null || _c === void 0 ? void 0 : _c.listenerInstances), eventData, forContext, originalEvent, scopeContext, totalContext);
            },
        };
        return r;
    }, [componentSchema, totalContext, scopeContext, forContext]);
    return (React.createElement(ErrorBoundary, { FallbackComponent: function (_a) {
            var error = _a.error;
            console.error("\u7EC4\u4EF6".concat(id, "\u53D1\u751F\u4E86\u9519\u8BEF\uFF0C\uFF1A").concat(error));
            return null;
        } },
        React.createElement(DataContext.Provider, { value: totalContext },
            React.createElement(Field, { ref: instanceRef, data: __assign(__assign(__assign({}, data), slotMap), { _selectableBlockEvents: _selectableBlockEvents }), id: id, emit: emitFn, events: events, compositeParent: compositeParent, forIndexes: forIndexes, "$node": currentWidget, "$widget": currentWidget === null || currentWidget === void 0 ? void 0 : currentWidget._userWidget, domRef: setGetDomApi(currentWidget, { codeContext: codeContext }), style: style, className: className, staticResourceAttribute: staticResourceAttribute }, getComponentChildren(componentSchema, {
                virtualFields: virtualFields,
                codeContext: codeContext,
                forContext: forContext,
                injectContext: injectContext,
                scopeContext: scopeContext,
                dataContext: totalContext,
                updateContext: updateContext,
            })))));
}
function SlotElementWrapper(_a) {
    var component = _a.component, forContext = _a.forContext, dataContext = _a.dataContext, _b = _a.codeContext, codeContext = _b === void 0 ? undefined : _b;
    return component({ forContext: forContext, dataContext: dataContext, codeContext: codeContext });
}
function generateSlotMap(slots, _a) {
    var forContext = _a.forContext, dataContext = _a.dataContext, _b = _a.codeContext, codeContext = _b === void 0 ? undefined : _b;
    var map = {};
    Object.keys(slots).forEach(function (slotProp) {
        var meta = slots[slotProp];
        if (meta.type === 'HOC') {
            set(map, slotProp, meta.node({ forContext: forContext, dataContext: dataContext }));
        }
        else {
            set(map, slotProp, React.createElement(SlotElementWrapper, { component: meta.node, forContext: forContext, dataContext: dataContext, codeContext: codeContext }));
        }
    });
    return map;
}
function SlotWrapper(props) {
    return props.children;
}
function Page(props) {
    return props.children;
}
export function getComponentRenderList(props) {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    var key = props.key, virtualFields = props.virtualFields, renderSlot = props.renderSlot, _j = props.codeContext, codeContext = _j === void 0 ? {} : _j, _k = props.scopeContext, scopeContext = _k === void 0 ? {} : _k, _l = props.genericComp, genericComp = _l === void 0 ? { propName: '' } : _l, _m = props.emitEvents, emitEvents = _m === void 0 ? [] : _m, _o = props.componentSchema, componentSchema = _o === void 0 ? {} : _o, _p = props.forContext, forContext = _p === void 0 ? {} : _p, _q = props.injectContext, injectContext = _q === void 0 ? {} : _q, _r = props.rootNode, rootNode = _r === void 0 ? false : _r, dataContext = props.dataContext, updateContext = props.updateContext;
    var compId = componentSchema.id;
    var isInComposite = !!codeContext.$WEAPPS_COMP;
    // 判断 widgets 是从 page 来的，还是组件来的
    var widgetsData = !isInComposite ? codeContext.$page.widgets[compId] : codeContext.$WEAPPS_COMP.widgets[compId];
    var classNameList = (((_a = componentSchema === null || componentSchema === void 0 ? void 0 : componentSchema.attributes) === null || _a === void 0 ? void 0 : _a.class) || '').split(/\s/).filter(function (item) { return !!item; });
    var dataBinds = ((_b = codeContext._dataBinds) === null || _b === void 0 ? void 0 : _b[compId]) || {};
    var FieldKey = genericComp.propName
        ? codeContext.node[genericComp.propName]
        : "".concat(componentSchema.module, ":").concat(componentSchema.component);
    var Field = componentSchema.type === 'PAGE' ? Page : virtualFields[FieldKey];
    var context = {
        scopeContext: scopeContext,
        forContext: forContext,
        codeContext: codeContext,
        dataContext: dataContext,
        virtualFields: virtualFields,
    };
    if (isSlot(componentSchema)) {
        var SlotRender = !compId ? SlotWrapper : injectContext.SlotRender || SlotWrapper;
        var slotId = "".concat((_d = (_c = componentSchema === null || componentSchema === void 0 ? void 0 : componentSchema.extra) === null || _c === void 0 ? void 0 : _c.__parent) === null || _d === void 0 ? void 0 : _d.id, ".").concat(compId);
        return (React.createElement(SlotRender, { component: slotId, componentSchema: componentSchema }, getComponentChildren(componentSchema, __assign(__assign({}, context), { injectContext: injectContext, updateContext: updateContext }))));
    }
    /**
     * 这里逻辑的增加是为了支持 Repeater 组件和 $w 作用域
     * 此逻辑作用于 2 层循环组件绑定了 1 层循环项的数组值
     * 1. 补全 forContext.forStack
     * 2. 为 getForList 的 dataBinds._waFor 方法提供正确的 widget 实例
     */
    var currentForParentWidget = Array.isArray(widgetsData) ? get(widgetsData, forContext.forIndexes || []) : widgetsData;
    /**
     * get forIndexes 后仍为数组则说明当前节点是绑定for循环的
     * 会再多一级，此刻仅为了验证
     */
    if (Array.isArray(currentForParentWidget)) {
        /**
         * 提前判断纯属是为了减少 mobx warn
         * 不然访问空数组的[0]总是打一个 warn 出来
         */
        if (currentForParentWidget.length) {
            currentForParentWidget = (_e = currentForParentWidget[0]) === null || _e === void 0 ? void 0 : _e.parent;
        }
        else {
            currentForParentWidget = undefined;
        }
    }
    // For循环渲染
    var forList = getForList(compId, dataBinds, forContext, dataContext, generateWidgetAPIContext((_g = (_f = (!isInComposite ? codeContext.$page : codeContext.$WEAPPS_COMP)) === null || _f === void 0 ? void 0 : _f.__internal__) === null || _g === void 0 ? void 0 : _g.$w, currentForParentWidget, forContext, scopeContext), scopeContext);
    if (forList) {
        return forList
            .map(function (item, index) {
            var _a, _b;
            var _c;
            var _d = forContext.forIndexes, forIndexes = _d === void 0 ? [] : _d, _e = forContext.forStack, forStack = _e === void 0 ? [] : _e;
            var forItemsIndexes = forIndexes.concat(index);
            var currentWidget = Array.isArray(widgetsData) ? get(widgetsData, forItemsIndexes) : widgetsData;
            var forContextListAlias;
            var listMeta = { currentItem: item, currentIndex: index };
            switch (currentWidget === null || currentWidget === void 0 ? void 0 : currentWidget.widgetType) {
                case "".concat(REPEATER.MODULE_NAME, ":").concat(REPEATER.REPEATER_ITEM_NAME):
                case "".concat(REPEATER.DEBUG_MODULE_NAME, ":").concat(REPEATER.REPEATER_ITEM_NAME): {
                    var repeaterWidget = currentWidget.parent;
                    forContextListAlias = (_a = {},
                        _a["".concat(repeaterWidget.forIndex) || 'currentIndex'] = listMeta.currentIndex,
                        _a["".concat(repeaterWidget.forItem) || 'currentItem'] = listMeta.currentItem,
                        _a);
                }
            }
            var forItems = __assign(__assign({}, forContext), (_b = {}, _b[compId] = item, _b.forIndexes = forItemsIndexes, _b.forStack = __spreadArray(__spreadArray([], __read(forStack), false), [
                __assign(__assign({}, listMeta), { alias: forContextListAlias }),
            ], false), _b));
            var _f = getBindData({
                widgetData: widgetsData,
                forItems: forItems,
                scopeContext: scopeContext,
                codeContext: codeContext,
                commonStyle: (_c = componentSchema.extra) === null || _c === void 0 ? void 0 : _c.commonStyle,
                classNameList: classNameList,
                injectContext: __assign(__assign({}, injectContext), { processCssUnit: isInComposite ? 'rpx' : injectContext.processCssUnit }),
            }), forItemData = _f.fieldData, forItemStyle = _f.finalStyle, forItemClassNameList = _f.finalClassNameList;
            var VisiblePlaceHolder = injectContext.VisiblePlaceHolder;
            if (!(injectContext.checkVisible || checkVisible)(forItemData)) {
                return VisiblePlaceHolder ? React.createElement(VisiblePlaceHolder, { id: compId }) : null;
            }
            var filedElement = (React.createElement(ForContext.Provider, { key: forItemData._key, value: forItems },
                React.createElement(FieldWrapper, __assign({ Field: Field, componentSchema: componentSchema, context: __assign(__assign({}, context), { widgetsData: widgetsData, forContext: forItems, forIndexes: forItemsIndexes }), id: compId, updateContext: updateContext, data: forItemData }, getSafeComponentProps({
                    style: forItemStyle,
                    classNameList: forItemClassNameList,
                }), { events: emitEvents, compositeParent: codeContext, renderSlot: renderSlot }))));
            return !(injectContext.checkDisplay || checkDisplay)(forItemData) && VisiblePlaceHolder ? (React.createElement(VisiblePlaceHolder, { id: compId }, filedElement)) : (filedElement);
        })
            .filter(function (item) { return !!item; });
    }
    // 修正 forIndexes
    var forIndexes = getForIndexes(forContext, widgetsData);
    // 单节点渲染
    var _s = getBindData({
        widgetData: widgetsData,
        forItems: forContext,
        scopeContext: scopeContext,
        codeContext: codeContext,
        commonStyle: (_h = componentSchema.extra) === null || _h === void 0 ? void 0 : _h.commonStyle,
        classNameList: classNameList,
        injectContext: __assign(__assign({}, injectContext), { processCssUnit: isInComposite ? 'rpx' : injectContext.processCssUnit }),
    }), fieldData = _s.fieldData, finalClassNameList = _s.finalClassNameList, finalStyle = _s.finalStyle;
    // false 或空字符串时
    if (!(injectContext.checkVisible || checkVisible)(fieldData)) {
        var VisiblePlaceHolder = injectContext.VisiblePlaceHolder;
        return VisiblePlaceHolder ? [React.createElement(VisiblePlaceHolder, { key: key, id: compId })] : [];
    }
    return [
        React.createElement(FieldWrapper, __assign({ key: key, Field: Field, componentSchema: componentSchema, context: __assign(__assign({}, context), { widgetsData: widgetsData, forIndexes: forIndexes }), id: compId, updateContext: updateContext, data: fieldData }, getSafeComponentProps({
            style: finalStyle,
            classNameList: finalClassNameList,
        }), { events: emitEvents, compositeParent: codeContext, renderSlot: renderSlot })),
    ];
}
export var CompRenderer = observer(function (props) {
    return getComponentRenderList(__assign(__assign({}, props), { scopeContext: React.createContext(ScopeContext), forContext: React.useContext(ForContext), injectContext: React.useContext(InjectContext) }));
});
function getFinalStyle(type, commonStyle, bindStyle, propStyle) {
    if (commonStyle === void 0) { commonStyle = {}; }
    if (bindStyle === void 0) { bindStyle = {}; }
    if (propStyle === void 0) { propStyle = {}; }
    var processOptions = {
        looseError: true,
        rpxCalcRoot: "var(".concat(CALC_RPX_ROOT_VAL_NAME, ")"),
    };
    var processCssUnit = type || 'rem';
    var remBindStyle = typeof bindStyle === 'object' ? processCSSPropertiesUnit(bindStyle, processCssUnit, processOptions) : {};
    return __assign(__assign(__assign({}, (processCSSPropertiesUnit(commonStyle, processCssUnit, processOptions) || {})), remBindStyle), propStyle);
}
function getFinalClassNameList(classNameList, bindClassList, widgetClassList) {
    if (classNameList === void 0) { classNameList = []; }
    if (bindClassList === void 0) { bindClassList = []; }
    if (widgetClassList === void 0) { widgetClassList = []; }
    return [].concat(classNameList, bindClassList, widgetClassList);
}
// HACK: 从后向前保证循环的深度与 forIndexes 一致
// 后续需要将 For 循环迁移到外层
export function getForIndexes(parentForItems, widgetsData) {
    return Array.isArray(widgetsData) && widgetsData.length > 0
        ? (parentForItems.forIndexes || []).slice(0 - getDeepArrLen(widgetsData))
        : undefined;
}
function getDeepArrLen(arr, len) {
    if (len === void 0) { len = 0; }
    if (Array.isArray(arr)) {
        return getDeepArrLen(arr.length > 0 ? arr[0] : null, len + 1);
    }
    else {
        return len;
    }
}
function setGetDomApi(currentWidget, _a) {
    var codeContext = _a.codeContext;
    if (!currentWidget)
        return React.createRef();
    var isComposite = !currentWidget.widgetType.startsWith('gsd-h5-react');
    var isInComposite = !!(codeContext === null || codeContext === void 0 ? void 0 : codeContext.$WEAPPS_COMP);
    // 如果当前是复合组件，不做 getDom 的挂载
    if (!isComposite) {
        if (!currentWidget.domRef) {
            currentWidget.domRef = React.createRef();
        }
        if (!currentWidget.getDom) {
            currentWidget.getDom = function (options) { return getDom(currentWidget.domRef.current, options); };
        }
        if (isInComposite && // 当前在复合组件内
            !currentWidget.parent && // 当前节点为复合组件的根节点
            codeContext.node && // 复合组件的 node 已经创建
            !codeContext.node.getDom // 复合组件的 node 未挂载 getDom 方法
        ) {
            codeContext.node.domRef = currentWidget.domRef;
            codeContext.node.getDom = currentWidget.getDom;
        }
    }
    return currentWidget.domRef;
}
