import { __assign, __awaiter, __generator, __read, __rest, __spreadArray } from "tslib";
// Import Libs and Handlers
import * as React from 'react';
import { autorun, observable, reaction } from 'mobx';
import { get, isEqual } from 'lodash-es';
import { app, createPageApi, $w as baseAPI } from '../app/global-api'; // 取对应子包app
import { AppRender } from '../handlers/RenderWrapper';
import { _WEDA_CLOUD_SDK as WEDA_CLOUD_SDK, AppContext } from '@cloudbase/weda-client';
import { CALC_RPX_ROOT_VAL_NAME, ENABLE_PAGE_ROOT_ID, generateEmit, reportTime } from '../utils';
import { bindFuncObj, checkAuth, createComputed, createWidgets, retryDataBinds, toCssText, defaultGenerateLoginUrl, } from '../utils/index';
import { generatePageLifecycleHandlers, usePageLifeCycle } from '../utils/lifecycle';
import { useScrollTop, OwnerComponentContext } from '../utils/hooks';
import { processCommonStyle2CSSProperties } from '@cloudbase/cals';
import { runtime } from '../utils/adapter';
import { loadMicroApp } from 'qiankun';
import { InjectContext } from '../router';
import { useLocation } from 'react-router-dom';
var createDataset = WEDA_CLOUD_SDK.createDataset, createStateDataSourceVar = WEDA_CLOUD_SDK.createStateDataSourceVar, generateParamsParser = WEDA_CLOUD_SDK.generateParamsParser, EXTRA_API = WEDA_CLOUD_SDK.EXTRA_API;
function useMemoOrDeep(value, memo) {
    var ref = React.useRef(value);
    return React.useMemo(function () {
        if (memo || !isEqual(ref.current, value)) {
            ref.current = value;
        }
        return ref.current;
    }, [value]);
}
function useObserverControledDataContext(inited, injectContext, initDataContext) {
    var dataContextRef = React.useRef(observable((injectContext === null || injectContext === void 0 ? void 0 : injectContext.dataContext) || initDataContext));
    return React.useMemo(function () {
        if (inited && (injectContext === null || injectContext === void 0 ? void 0 : injectContext.dataContext)) {
            if (dataContextRef.current) {
                // eslint-disable-next-line no-restricted-syntax
                for (var key in dataContextRef.current) {
                    if (!injectContext.dataContext[key]) {
                        dataContextRef.current[key] = null;
                    }
                }
            }
            // eslint-disable-next-line no-restricted-syntax
            for (var key in injectContext.dataContext) {
                dataContextRef.current[key] = injectContext.dataContext[key];
            }
            // setFourceUpdate(Date.now());
        }
        return dataContextRef.current;
    }, [inited, injectContext === null || injectContext === void 0 ? void 0 : injectContext.dataContext]);
}
// Init
// eslint-disable-next-line react/display-name
export default function App(_a) {
    var _page = _a.page, generateComponentsMap = _a.generateComponentsMap, generateActionsMap = _a.generateActionsMap, generateLowcodeModule = _a.generateLowcodeModule, generateWidgetsContext = _a.generateWidgetsContext, generateDataBinds = _a.generateDataBinds, generateComponentSchema = _a.generateComponentSchema, generateContext = _a.generateContext, onRender = _a.onRender, onMounted = _a.onMounted, _b = _a.initDataContext, initDataContext = _b === void 0 ? {} : _b, renderLoading = _a.renderLoading, options = _a.options;
    var page = useMemoOrDeep(_page, !options.reuseWidget);
    var injectContext = React.useContext(InjectContext);
    var _c = __read(React.useState(false), 2), pageContextInited = _c[0], setPageContextInited = _c[1];
    var _d = __read(React.useState(Date.now()), 2), _ = _d[0], setFourceUpdate = _d[1];
    // 数据容器的上下文数据
    var widgetsContextRef = React.useRef({});
    var _e = (options || {}).reuseWidget, reuseWidget = _e === void 0 ? false : _e;
    // 兼容 this.state / $page.state 两种模式
    var pageCodeContextRef = React.useRef((injectContext === null || injectContext === void 0 ? void 0 : injectContext.$page) || createPageApi());
    var $page = pageCodeContextRef.current;
    var lowcode = generateLowcodeModule();
    var pathname = useLocation().pathname;
    React.useEffect(function () {
        var _a;
        if (!(injectContext === null || injectContext === void 0 ? void 0 : injectContext.$page)) {
            var dataset = createDataset(page.id);
            var state = observable(lowcode.state || {});
            state.dataset = dataset;
            Object.assign($page, {
                id: page.id,
                label: (_a = page === null || page === void 0 ? void 0 : page.attributes) === null || _a === void 0 ? void 0 : _a.title,
                path: pathname,
                state: state,
                computed: createComputed(lowcode.computed || {}, pageCodeContextRef.current),
                handler: bindFuncObj(lowcode.handler || {}, pageCodeContextRef.current),
                randerer: lowcode.renderer,
                dataset: dataset,
            });
        }
        Object.assign($page, {
            __internal__: __assign(__assign({}, $page.__internal__), { $w: new Proxy(baseAPI, {
                    get: function (obj, prop) {
                        var _a;
                        /**
                         * 使用当前的实例进行覆盖
                         */
                        if (prop === '$page' || prop === 'page') {
                            return $page;
                        }
                        // 尝试代理页面级别组件实例
                        var pageWidget = (_a = $page.widgets) === null || _a === void 0 ? void 0 : _a[prop];
                        if (pageWidget) {
                            return pageWidget._userWidget;
                        }
                        return obj[prop];
                    },
                }) }),
        });
        runtime.$page = pageCodeContextRef.current;
        setPageContextInited(true);
    }, []);
    var dataContext = useObserverControledDataContext(pageContextInited, injectContext, initDataContext);
    var _f = React.useMemo(function () {
        if (pageContextInited) {
            var globalCtx = {
                $page: $page,
                app: app,
                $app: app,
                actionsMap: (generateActionsMap === null || generateActionsMap === void 0 ? void 0 : generateActionsMap()) || {},
            };
            var widgetsContext_1;
            var databinds = void 0;
            var componentSchema_1;
            if (generateContext) {
                var context = generateContext(globalCtx, page);
                widgetsContext_1 = context.widgetContext;
                databinds = context.dataBinds;
                componentSchema_1 = context.componentSchema;
            }
            else {
                widgetsContext_1 = generateWidgetsContext(page);
                databinds = generateDataBinds(globalCtx, page);
                componentSchema_1 = generateComponentSchema(globalCtx, page);
            }
            // widgets 内的 dataBinds 可能需要关联 widgets，需要重新执行 dataBinds
            $page._dataBinds = databinds;
            return { componentSchema: componentSchema_1, widgetsContext: widgetsContext_1 };
        }
        return {};
    }, [pageContextInited, page]), componentSchema = _f.componentSchema, widgetsContext = _f.widgetsContext;
    widgetsContextRef.current = React.useMemo(function () {
        if (pageContextInited) {
            var _a = createWidgets(__assign({ apiContext: $page }, (reuseWidget ? widgetsContextRef.current : {})), widgetsContext, $page._dataBinds || {}, dataContext), widgets_1 = _a.widgets, nodeMap = _a.nodeMap;
            Object.entries(widgets_1).forEach(function (_a) {
                var _b = __read(_a, 2), key = _b[0], value = _b[1];
                $page.widgets[key] = value;
            });
            retryDataBinds();
            if (reuseWidget) {
                /**
                 * 复用时处理挂载新的
                 * 还要处理老的
                 */
                Object.keys($page.widgets).forEach(function (key) {
                    if (!(key in widgets_1)) {
                        $page.widgets[key] = undefined;
                    }
                });
                var lastWdigets = (widgetsContextRef.current || {}).widgets;
                if (lastWdigets === null || lastWdigets === void 0 ? void 0 : lastWdigets._disposers) {
                    lastWdigets._disposers.forEach(function (dispose) { return dispose(); });
                    lastWdigets._disposers = [];
                }
            }
            return { widgetProps: widgetsContext, nodeMap: nodeMap, widgets: widgets_1 };
        }
        else {
            return widgetsContextRef.current;
        }
    }, [widgetsContext, dataContext]);
    React.useEffect(function () {
        return function () {
            var _a, _b, _c, _d;
            if ((_b = (_a = pageCodeContextRef.current) === null || _a === void 0 ? void 0 : _a.widgets) === null || _b === void 0 ? void 0 : _b._disposers) {
                (_d = (_c = pageCodeContextRef.current) === null || _c === void 0 ? void 0 : _c.widgets) === null || _d === void 0 ? void 0 : _d._disposers.forEach(function (dispose) { return dispose(); });
                pageCodeContextRef.current.widgets._disposers = [];
            }
        };
    }, []);
    return (pageContextInited && (React.createElement(AppContent, { page: page, lowcode: lowcode, componentSchema: componentSchema, pageCodeContext: pageCodeContextRef.current, virtualFields: generateComponentsMap(), onMounted: onMounted, onRender: onRender, 
        // dataContext={dataContext}
        /**
         * 更新数据容器的上下文的方法
         * 会传递到事件emit的地方，将从组件获取到的数据data赋值给上下文
         * 当组件卸载时，传过来的data为undefined即可
         * {
         *   id1: [{...}],
         *   id2: {...},
         *   id3: undefined,
         *   id4: null,
         *   ...
         * }
         * @param id
         * @param data
         */
        // updateContext={function (id, data) {
        //   if (id && !injectContext?.dataContext) {
        //     dataContext[id] = { data };
        //   }
        //   injectContext?.onDataContextChange?.(id, data);
        // }}
        options: options, renderLoading: renderLoading })));
}
function AppContent(_a) {
    var _this = this;
    var _b, _c, _d, _e, _f;
    var page = _a.page, lowcode = _a.lowcode, componentSchema = _a.componentSchema, pageCodeContext = _a.pageCodeContext, virtualFields = _a.virtualFields, _g = _a.onMounted, onMounted = _g === void 0 ? function (props) {
        var _a;
        // eslint-disable-next-line react/prop-types
        (_a = props === null || props === void 0 ? void 0 : props.default) === null || _a === void 0 ? void 0 : _a.call(props);
    } : _g, _h = _a.onRender, onRender = _h === void 0 ? function () { } : _h, 
    // dataContext,
    // updateContext,
    _j = _a.renderLoading, 
    // dataContext,
    // updateContext,
    renderLoading = _j === void 0 ? function () { return null; } : _j, options = _a.options;
    var _k = options.processCssUnit, processCssUnit = _k === void 0 ? 'rem' : _k;
    // 检查权限
    var _l = __read(React.useState(false), 2), weDaHasLogin = _l[0], setWeDaHasLogin = _l[1];
    var containerRef = React.useRef(null);
    var externalClassRef = React.useRef('');
    var microApp = React.useRef(null);
    var pureSrc = (_b = page.attributes) === null || _b === void 0 ? void 0 : _b.src;
    var isPure = !!pureSrc;
    var _m = React.useContext(InjectContext), _o = _m.checkPageAuth, checkPageAuth = _o === void 0 ? checkAuth : _o, _p = _m.generateLoginUrl, generateLoginUrl = _p === void 0 ? defaultGenerateLoginUrl : _p;
    var pageReadyResolveRef = React.useRef(function (data) { });
    var pageReadyCalledRef = React.useRef(false);
    var onReady = React.useCallback(function () {
        pageReadyCalledRef.current = true;
        pageReadyResolveRef.current(null);
    }, []);
    var events = React.useMemo(function () { return getPageEvents(pageCodeContext, componentSchema); }, [pageCodeContext, componentSchema]);
    var lifecycle = React.useMemo(function () {
        return generatePageLifecycleHandlers(__assign(__assign({}, lowcode === null || lowcode === void 0 ? void 0 : lowcode.lifecycle), { beforePageCustomLaunch: function (query) { return __awaiter(_this, void 0, void 0, function () {
                var checkAuthResult;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            onMounted({
                                page: page,
                                default: function () {
                                    var _a;
                                    var document = (runtime === null || runtime === void 0 ? void 0 : runtime.document) || window.document;
                                    document.title = ((_a = page.attributes) === null || _a === void 0 ? void 0 : _a.title) || page.id;
                                },
                            });
                            EXTRA_API.setParams(page.id, query || {});
                            initWatchMethods(pageCodeContext);
                            return [4 /*yield*/, checkPageAuth(app, app.id, pageCodeContext, generateLoginUrl)];
                        case 1:
                            checkAuthResult = _a.sent();
                            setWeDaHasLogin(checkAuthResult);
                            createStateDataSourceVar(page.id, generateParamsParser({ app: app, $page: pageCodeContext }));
                            onRender === null || onRender === void 0 ? void 0 : onRender();
                            reportTime('FIRST_PAGE_CONTENT_RENDERED', undefined, true);
                            return [2 /*return*/];
                    }
                });
            }); } }), events);
    }, []);
    // Web 环境页面级别生命周期
    usePageLifeCycle(__assign(__assign({}, lifecycle), { 
        // eslint-disable-next-line @typescript-eslint/no-misused-promises
        onPageReady: function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return __awaiter(_this, void 0, void 0, function () {
                var res;
                var _a;
                return __generator(this, function (_b) {
                    switch (_b.label) {
                        case 0: return [4 /*yield*/, new Promise(function (resolve) {
                                pageReadyResolveRef.current = resolve;
                                if (pageReadyCalledRef.current) {
                                    resolve(null);
                                }
                            })];
                        case 1:
                            _b.sent();
                            res = lifecycle === null || lifecycle === void 0 ? void 0 : (_a = lifecycle.onPageReady).call.apply(_a, __spreadArray([this], __read(args), false));
                            events === null || events === void 0 ? void 0 : events.ready();
                            return [2 /*return*/, res];
                    }
                });
            });
        } }), { app: app, page: pageCodeContext, options: { looseError: options === null || options === void 0 ? void 0 : options.looseError } }, []);
    // 切换页面滚动到顶部
    useScrollTop();
    React.useEffect(function () {
        var _a, _b;
        if (isPure) {
            if (weDaHasLogin) {
                if (!microApp.current) {
                    microApp.current = loadMicroApp({
                        name: page.id,
                        entry: pureSrc,
                        container: containerRef.current,
                    }, {
                        fetch: runtime.fetch,
                    });
                }
                else {
                    (_b = (_a = microApp.current).mount) === null || _b === void 0 ? void 0 : _b.call(_a);
                }
            }
            else if (microApp.current) {
                microApp.current.unmount();
            }
        }
    }, [weDaHasLogin]);
    React.useEffect(function () {
        app.__internal__.activePage = pageCodeContext;
        return function () {
            if (microApp.current) {
                microApp.current.unmount();
            }
        };
    }, []);
    React.useEffect(function () {
        function resize() {
            var _a, _b;
            var className = "wd-".concat(((_b = (_a = app === null || app === void 0 ? void 0 : app.__internal__) === null || _a === void 0 ? void 0 : _a.isMobile) === null || _b === void 0 ? void 0 : _b.call(_a)) ? 'h5' : 'pc', "-root");
            if (externalClassRef.current === className) {
                return;
            }
            if (containerRef.current.classList.contains(externalClassRef.current)) {
                containerRef.current.classList.replace(externalClassRef.current, className);
            }
            else {
                containerRef.current.classList.add(className);
            }
            externalClassRef.current = className;
        }
        resize();
        runtime === null || runtime === void 0 ? void 0 : runtime.addEventListener('resize', resize);
        return function () {
            runtime === null || runtime === void 0 ? void 0 : runtime.removeEventListener('resize', resize);
        };
    }, []);
    return (React.createElement("div", { id: ENABLE_PAGE_ROOT_ID ? 'wd-page-root' : undefined, ref: containerRef, className: "weapps-page weapps-page-".concat(page.id, " ").concat(externalClassRef.current) },
        React.createElement("style", { type: "text/css", dangerouslySetInnerHTML: {
                __html: "".concat(toCssText(((_c = page.attributes) === null || _c === void 0 ? void 0 : _c.style) ||
                    processCommonStyle2CSSProperties(((_d = page === null || page === void 0 ? void 0 : page.extra) === null || _d === void 0 ? void 0 : _d.commonStyle) || {}, {
                        defaultUnit: processCssUnit,
                        rpxCalcRoot: "var(".concat(CALC_RPX_ROOT_VAL_NAME, ")"),
                    }), 'body')).concat((lowcode.style || []).map(function (item) { return item.code; }).join('\n')),
            } }),
        weDaHasLogin
            ? !isPure &&
                (((_e = lowcode.lifecycle) === null || _e === void 0 ? void 0 : _e.onPageRender) ? ((_f = lowcode.lifecycle) === null || _f === void 0 ? void 0 : _f.onPageRender()) : (React.createElement(IAppRender, { virtualFields: virtualFields, componentSchema: componentSchema, codeContext: pageCodeContext, onReady: onReady })))
            : renderLoading({ message: '登录鉴权中...' })));
}
function IAppRender(_a) {
    var onReady = _a.onReady, props = __rest(_a, ["onReady"]);
    var _b = __read(React.useState(false), 2), mount = _b[0], setMounted = _b[1];
    React.useEffect(function () {
        setMounted(true);
    }, []);
    /**
     * @important
     * 需要子节点先触发，page 的最后触发，
     * 因此不在 setMount 时直接调用
     */
    React.useEffect(function () {
        if (mount) {
            onReady();
        }
    }, [mount]);
    var _c = React.useMemo(function () {
        return { appContext: { ready: mount }, ownerComponentContext: { attached: mount } };
    }, [mount]), appContext = _c.appContext, ownerComponentContext = _c.ownerComponentContext;
    return (React.createElement(AppContext.Provider, { value: appContext },
        React.createElement(OwnerComponentContext.Provider, { value: ownerComponentContext },
            React.createElement(AppRender, __assign({}, props)))));
}
function initWatchMethods(pageContext) {
    // const { watch = {}, watchState = {}, watchWidget = {}, watchEffects = {}} = pageAPI
    var _a = {}, _b = _a.watch, watch = _b === void 0 ? {} : _b, _c = _a.watchState, watchState = _c === void 0 ? {} : _c, _d = _a.watchWidget, watchWidget = _d === void 0 ? {} : _d, _e = _a.watchEffects, watchEffects = _e === void 0 ? {} : _e;
    // # watch effect
    var watchEffectDisposers = Object.keys(watchEffects).map(function (fnName) {
        return autorun(watchEffects[fnName].bind(pageContext));
    });
    var disposers = watchEffectDisposers;
    // # watch state
    Object.keys(watchState).map(function (key) { return runWatcher(parseWatcher(watchState[key]), pageContext.state, key, 'watchState'); });
    // # watch widgets
    Object.keys(watchWidget).map(function (key) {
        return runWatcher(parseWatcher(watchWidget[key]), pageContext.widgets, key, 'watchWidgets');
    });
    function runWatcher(_a, target, key, label) {
        var _b = _a === void 0 ? {} : _a, handler = _b.handler, immediate = _b.immediate;
        if (!handler) {
            console.error("Invalid ".concat(label, "(").concat(key, ") of ").concat(pageContext.node.widgetType, ", watch must a function or {handler: function}"));
            return;
        }
        var disposer = reaction(function () { return get(target, key); }, handler.bind(pageContext), { fireImmediately: immediate });
        disposers.push(disposer);
    }
}
function parseWatcher(watcher) {
    if (!watcher)
        return;
    if (watcher instanceof Function) {
        return { handler: watcher, immediate: false };
    }
    var handler = watcher.handler, _a = watcher.immediate, immediate = _a === void 0 ? false : _a;
    if (!(handler instanceof Function))
        return;
    return { handler: handler, immediate: immediate };
}
function getPageEvents(pageCodeContext, componentSchema) {
    var _a;
    var emit = generateEmit((_a = pageCodeContext.__internal__) === null || _a === void 0 ? void 0 : _a.$w);
    return new Proxy({}, {
        get: function (target, eventName) {
            return function (eventData) {
                return emit(eventName, componentSchema.listenerInstances, eventData, {}, {}, {}, {}, pageCodeContext.id);
            };
        },
    });
}
