import { __assign, __awaiter, __generator, __read, __spreadArray } from "tslib";
/* eslint-disable @typescript-eslint/no-empty-function */
/**
 * 生命周期处理函数
 */
import * as React from 'react';
import { throttle } from 'lodash-es';
import * as querystring from 'query-string';
import { runtime } from './adapter';
import { ENABLE_WECOM, getNetworkType } from './common';
import { signInWidthProviderCode } from './sso';
import { parseProviderTokenError } from './sso/error-handler';
import { auth } from '@cloudbase/weda-client';
var noops = function () { };
var APPLIFECYCLE = {
    onAppShow: noops,
    onAppHide: noops,
    onAppError: noops,
    onAppUnhandledRejection: noops,
    onAppLaunch: noops,
    beforeCustomLaunch: noops,
};
// 小程序 端使用lifeCycle
var alreadyInitAppLifeCycle = false;
var _HANDLER_MAP = {};
export function initAppLifeCycle(app, appLifeCycle, force) {
    if (force === void 0) { force = false; }
    appLifeCycle = __assign(__assign({}, APPLIFECYCLE), appLifeCycle);
    function scheduleAppLifecycle() {
        var _a, _b, _c, _d, _e, _f;
        return __awaiter(this, void 0, void 0, function () {
            var options, _g, query, state, code, e_1, url;
            return __generator(this, function (_h) {
                switch (_h.label) {
                    case 0:
                        options = app.getLaunchOptionsSync();
                        _g = options.query, query = _g === void 0 ? {} : _g;
                        state = query.state, code = query.code;
                        if (!(ENABLE_WECOM && query.code && query.state)) return [3 /*break*/, 6];
                        _h.label = 1;
                    case 1:
                        _h.trys.push([1, 4, 5, 6]);
                        return [4 /*yield*/, signInWidthProviderCode(state, code)];
                    case 2:
                        _h.sent();
                        return [4 /*yield*/, auth.getUserInfo(true)];
                    case 3:
                        _h.sent();
                        return [3 /*break*/, 6];
                    case 4:
                        e_1 = _h.sent();
                        console.error("\u4F01\u5FAE\u767B\u5F55\u5931\u8D25: ".concat(JSON.stringify({ state: state, code: code })), e_1);
                        app['showModal']({
                            content: "".concat(parseProviderTokenError(e_1), " [").concat(JSON.stringify({
                                reqId: (e_1 === null || e_1 === void 0 ? void 0 : e_1.request_id) || '',
                                state: state,
                                code: code,
                            }), "]"),
                            success: function (res) {
                                if (res.confirm) {
                                    runtime.location.reload();
                                }
                            },
                        });
                        throw e_1;
                    case 5:
                        delete query.state;
                        delete query.code;
                        delete query.appid;
                        url = new URL(runtime.location.href);
                        (_b = (_a = url === null || url === void 0 ? void 0 : url.searchParams) === null || _a === void 0 ? void 0 : _a.delete) === null || _b === void 0 ? void 0 : _b.call(_a, 'code');
                        (_d = (_c = url === null || url === void 0 ? void 0 : url.searchParams) === null || _c === void 0 ? void 0 : _c.delete) === null || _d === void 0 ? void 0 : _d.call(_c, 'state');
                        (_f = (_e = url === null || url === void 0 ? void 0 : url.searchParams) === null || _e === void 0 ? void 0 : _e.delete) === null || _f === void 0 ? void 0 : _f.call(_e, 'appid');
                        runtime.history.replaceState({ _isLoginRoot: true }, '', url.toString());
                        return [7 /*endfinally*/];
                    case 6: return [4 /*yield*/, (appLifeCycle === null || appLifeCycle === void 0 ? void 0 : appLifeCycle.beforeCustomLaunch(query))];
                    case 7:
                        _h.sent();
                        appLifeCycle === null || appLifeCycle === void 0 ? void 0 : appLifeCycle.onAppLaunch(options);
                        appLifeCycle === null || appLifeCycle === void 0 ? void 0 : appLifeCycle.onAppShow(options);
                        return [2 /*return*/];
                }
            });
        });
    }
    if (force) {
        removeAppLifeCycle();
    }
    // 应用级别事件监听
    if (!alreadyInitAppLifeCycle) {
        scheduleAppLifecycle();
        // 预留等客户端来触发
        _HANDLER_MAP = {
            appShow: function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return appLifeCycle.onAppShow.apply(appLifeCycle, __spreadArray([], __read(args), false));
            },
            appHide: function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return appLifeCycle.onAppHide.apply(appLifeCycle, __spreadArray([], __read(args), false));
            },
            error: function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return appLifeCycle.onAppError.apply(appLifeCycle, __spreadArray([], __read(args), false));
            },
            unhandledrejection: function (event) { return appLifeCycle.onAppUnhandledRejection(event === null || event === void 0 ? void 0 : event.reason); },
        };
        for (var event_1 in _HANDLER_MAP) {
            runtime.addEventListener(event_1, _HANDLER_MAP[event_1]);
        }
        alreadyInitAppLifeCycle = true;
    }
}
export function removeAppLifeCycle() {
    for (var event_2 in _HANDLER_MAP) {
        runtime.removeEventListener(event_2, _HANDLER_MAP[event_2]);
    }
    _HANDLER_MAP = {};
    alreadyInitAppLifeCycle = false;
}
var isReachBottom = false;
export function generatePageLifecycleHandlers(_a, events) {
    var _b = _a.beforePageCustomLaunch, beforePageCustomLaunch = _b === void 0 ? function (query) {
        return __awaiter(this, void 0, void 0, function () { return __generator(this, function (_a) {
            return [2 /*return*/];
        }); });
    } : _b, _c = _a.onPageLoad, onPageLoad = _c === void 0 ? function (query) { } : _c, _d = _a.onPageShow, onPageShow = _d === void 0 ? function () { } : _d, _e = _a.onPageReady, onPageReady = _e === void 0 ? function () { } : _e, _f = _a.onPageHide, onPageHide = _f === void 0 ? function () { } : _f, _g = _a.onPageUnload, onPageUnload = _g === void 0 ? function () { } : _g, _h = _a.onReachBottom, onReachBottom = _h === void 0 ? function () { } : _h, _j = _a.onShareAppMessage, onShareAppMessage = _j === void 0 ? function () { } : _j, _k = _a.onPageScroll, onPageScroll = _k === void 0 ? function () { } : _k, _l = _a.onResize, onResize = _l === void 0 ? function () { } : _l;
    function ensureDelegater(fn, eventsKey) {
        if (eventsKey === void 0) { eventsKey = ''; }
        return function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            typeof fn === 'function' && fn.call.apply(fn, __spreadArray([this], __read(args), false));
            if (eventsKey) {
                events[eventsKey]();
            }
        };
    }
    return {
        beforePageCustomLaunch: beforePageCustomLaunch,
        onPageLoad: function (query) {
            var callHandlerResult = typeof onPageLoad === 'function' ? onPageLoad.call(this, query) : function () { };
            events.load({ query: query });
            return callHandlerResult;
        },
        onPageShow: ensureDelegater(onPageShow, 'show'),
        onPageReady: ensureDelegater(onPageReady, 'ready'),
        onPageHide: ensureDelegater(onPageHide, 'hide'),
        onPageUnload: ensureDelegater(onPageUnload, 'unload'),
        onReachBottom: ensureDelegater(onReachBottom),
        onShareAppMessage: ensureDelegater(onShareAppMessage),
        onPageScroll: ensureDelegater(onPageScroll),
        onResize: ensureDelegater(onResize),
    };
}
// web 端使用lifeCycle
export function usePageLifeCycle(_a, context, deps) {
    var _b = _a.beforePageCustomLaunch, beforePageCustomLaunch = _b === void 0 ? function (query) {
        return __awaiter(this, void 0, void 0, function () { return __generator(this, function (_a) {
            return [2 /*return*/];
        }); });
    } : _b, _c = _a.onPageLoad, onPageLoad = _c === void 0 ? function (query) { } : _c, _d = _a.onPageShow, onPageShow = _d === void 0 ? function () { } : _d, _e = _a.onPageReady, onPageReady = _e === void 0 ? function () { } : _e, _f = _a.onPageHide, onPageHide = _f === void 0 ? function () { } : _f, _g = _a.onPageUnload, onPageUnload = _g === void 0 ? function () { } : _g, _h = _a.onReachBottom, onReachBottom = _h === void 0 ? function () { } : _h, _j = _a.onShareAppMessage, onShareAppMessage = _j === void 0 ? function () { } : _j, _k = _a.onPageScroll, onPageScroll = _k === void 0 ? function () { } : _k, _l = _a.onResize, onResize = _l === void 0 ? function () { } : _l;
    if (deps === void 0) { deps = []; }
    var reachBottomDistance = 50;
    try {
        var config = context.app.__internal__.getConfig().pages.find(function (page) { return page.id === context.page.id; });
        reachBottomDistance = config.reachBottomDistance;
        if (typeof reachBottomDistance !== 'number') {
            reachBottomDistance = 50;
        }
    }
    catch (e) { }
    React.useEffect(function () {
        function schedulePageLifecycle() {
            var _a, _b, _c, _d;
            return __awaiter(this, void 0, void 0, function () {
                var query, e_2;
                return __generator(this, function (_e) {
                    switch (_e.label) {
                        case 0:
                            _e.trys.push([0, 2, , 3]);
                            query = getCurrentPageQuery();
                            return [4 /*yield*/, (beforePageCustomLaunch === null || beforePageCustomLaunch === void 0 ? void 0 : beforePageCustomLaunch(query))];
                        case 1:
                            _e.sent();
                            (_a = onPageLoad === null || onPageLoad === void 0 ? void 0 : onPageLoad.call) === null || _a === void 0 ? void 0 : _a.call(onPageLoad, context.page, query);
                            (_b = onPageShow === null || onPageShow === void 0 ? void 0 : onPageShow.call) === null || _b === void 0 ? void 0 : _b.call(onPageShow, context.page);
                            (_c = onPageReady === null || onPageReady === void 0 ? void 0 : onPageReady.call) === null || _c === void 0 ? void 0 : _c.call(onPageReady, context.page);
                            return [3 /*break*/, 3];
                        case 2:
                            e_2 = _e.sent();
                            if (!((_d = context.options) === null || _d === void 0 ? void 0 : _d.looseError)) {
                                throw e_2;
                            }
                            else {
                                console.error('Catched error:', e_2);
                            }
                            return [3 /*break*/, 3];
                        case 3: return [2 /*return*/];
                    }
                });
            });
        }
        schedulePageLifecycle();
        return function () {
            var _a, _b, _c;
            try {
                (_a = onPageHide === null || onPageHide === void 0 ? void 0 : onPageHide.call) === null || _a === void 0 ? void 0 : _a.call(onPageHide, context.page, {});
                (_b = onPageUnload === null || onPageUnload === void 0 ? void 0 : onPageUnload.call) === null || _b === void 0 ? void 0 : _b.call(onPageUnload, context.page, {});
            }
            catch (e) {
                if (!((_c = context.options) === null || _c === void 0 ? void 0 : _c.looseError)) {
                    throw e;
                }
                else {
                    console.error('Catched error:', e);
                }
            }
        };
    }, deps);
    React.useEffect(function () {
        var _a, _b;
        runtime.onscroll = throttle(function () {
            // 变量scrollTop是滚动条滚动时，滚动条上端距离顶部的距离
            var scrollTop = document.documentElement.scrollTop || document.body.scrollTop;
            // 变量windowHeight是可视区的高度
            var windowHeight = document.documentElement.clientHeight || document.body.clientHeight;
            // 变量scrollHeight是滚动条的总高度（当前可滚动的页面的总高度）
            var scrollHeight = document.documentElement.scrollHeight || document.body.scrollHeight;
            onPageScroll.call(context.page, {
                scrollTop: runtime.pageYOffset,
            });
            // console.log(scrollTop, windowHeight, scrollTop + windowHeight, scrollHeight, isReachBottom)
            // 滚动条到底部
            if (scrollTop + windowHeight >= scrollHeight - reachBottomDistance) {
                // 要进行的操作
                if (!isReachBottom) {
                    isReachBottom = true;
                    onReachBottom.call(context.page);
                }
            }
            else {
                isReachBottom = false;
            }
        }, 300);
        // todo
        if (typeof onShareAppMessage === 'function') {
            runtime['onShareAppMessage'] = onShareAppMessage.bind(context.page);
        }
        runtime.onresize = throttle(function () {
            var _a;
            onResize.call(context.page, {
                size: {
                    windowWidth: runtime.innerWidth,
                    windowHeight: runtime.innerHeight,
                },
            });
            var device = (_a = context.page.__internal__.$w) === null || _a === void 0 ? void 0 : _a.device;
            if (device) {
                device.viewport = {
                    width: runtime.innerWidth,
                    height: runtime.innerHeight,
                };
            }
        }, 100);
        var navigator = (runtime === null || runtime === void 0 ? void 0 : runtime.navigator) || {};
        var connection = navigator.connection || navigator.mozConnection || navigator.webkitConnection;
        var isMobile = (_b = (_a = context.app) === null || _a === void 0 ? void 0 : _a.__internal__) === null || _b === void 0 ? void 0 : _b.isMobile;
        var connectionListener = function () {
            var _a;
            var device = (_a = context.page.__internal__.$w) === null || _a === void 0 ? void 0 : _a.device;
            if (isMobile === null || isMobile === void 0 ? void 0 : isMobile()) {
                device.networkType = getNetworkType(runtime);
            }
            else {
                device.networkType = '';
            }
        };
        connection === null || connection === void 0 ? void 0 : connection.addEventListener('change', connectionListener);
        return function () {
            runtime.onscroll = null;
            runtime.onresize = null;
            connection === null || connection === void 0 ? void 0 : connection.removeEventListener('change', connectionListener);
        };
    }, []);
}
function getCurrentPageQuery() {
    var queryObj = querystring.parse((location.search || '').split('?')[1] || '') || {};
    var hashQueryObj = querystring.parse((location.hash || '').split('?')[1] || '') || {};
    var query = __assign(__assign({}, hashQueryObj), queryObj);
    // query string parse decode succeess
    // Object.keys(query).forEach((key) => {
    //   query[key] = decodeURIComponent(query[key] as string);
    // });
    return query;
}
export function initWebConfig(app, appConfig) {
    // miniprogram.config 配置截取
    app.setConfig(appConfig);
}
