import { __assign, __awaiter, __generator } from "tslib";
import { resolveDataBinds, reportEvent } from '../common';
import { set as lodashSet } from 'lodash-es';
import { app } from '../../app/global-api';
import { runtime } from '../adapter';
import { untracked } from 'mobx';
import { generateWidgetAPIContext } from '../widgets';
var DEFAULT_MAX_TIMEOUT = 10 * 1000;
var Event = /** @class */ (function () {
    function Event(_a) {
        var _b = _a.type, type = _b === void 0 ? '' : _b, _c = _a.detail, detail = _c === void 0 ? undefined : _c, _d = _a.currentTarget, currentTarget = _d === void 0 ? undefined : _d, _e = _a.target, target = _e === void 0 ? undefined : _e, _f = _a._isCapturePhase, _isCapturePhase = _f === void 0 ? false : _f, origin = _a.origin;
        this.type = type;
        this.detail = detail;
        this.currentTarget = currentTarget;
        this.target = target;
        this._isCapturePhase = _isCapturePhase;
        this.origin = origin;
        return new Proxy(this, {
            get: function (target, prop) {
                switch (prop) {
                    case 'name': {
                        console.warn('[deprecated] event.name 将在未来版本放弃支持，请使用 event.type 替代');
                        return target.type;
                    }
                    case 'origin':
                    case 'target':
                    case 'currentTarget': {
                        return target[prop]
                            ? new Proxy(target[prop], {
                                get: function (target, p) {
                                    reportEvent("event.".concat(prop));
                                    return target[p];
                                },
                            })
                            : target[prop];
                        break;
                    }
                }
                return target[prop];
            },
        });
    }
    return Event;
}());
function emitEvent(trigger, listeners, args, scopeContext, dataContext, $w, pageId) {
    if (listeners === void 0) { listeners = []; }
    if (scopeContext === void 0) { scopeContext = {}; }
    if (pageId === void 0) { pageId = ''; }
    var targetListeners = listeners.filter(function (l) { return l.trigger === trigger; });
    var currentEvent = args === null || args === void 0 ? void 0 : args.event;
    function processListener(listener) {
        var _a, _b, _c, _d, _e;
        return __awaiter(this, void 0, void 0, function () {
            var nextEvent, res, e_1, res, e_2;
            return __generator(this, function (_f) {
                switch (_f.label) {
                    case 0:
                        // 当前非捕获Event，再判断冒泡行为
                        if (!(currentEvent === null || currentEvent === void 0 ? void 0 : currentEvent._isCapturePhase) && listener.noPropagation) {
                            ((_a = args === null || args === void 0 ? void 0 : args.originalEvent) === null || _a === void 0 ? void 0 : _a.stopPropagation) && ((_b = args === null || args === void 0 ? void 0 : args.originalEvent) === null || _b === void 0 ? void 0 : _b.stopPropagation());
                        }
                        nextEvent = new Event(__assign(__assign({}, currentEvent), { type: '', detail: '', origin: currentEvent }));
                        if (!(((listener === null || listener === void 0 ? void 0 : listener.isCapturePhase) || false) === ((currentEvent === null || currentEvent === void 0 ? void 0 : currentEvent._isCapturePhase) || false))) return [3 /*break*/, 8];
                        if (pageId && ((_c = $w === null || $w === void 0 ? void 0 : $w.page) === null || _c === void 0 ? void 0 : _c.id) && pageId !== ((_d = $w === null || $w === void 0 ? void 0 : $w.page) === null || _d === void 0 ? void 0 : _d.id)) {
                            console.error("Action error: [".concat(listener.trigger, "] \u9875\u9762\u751F\u547D\u5468\u671F\u7ED3\u675F\uFF0C\u94FE\u5F0F\u8C03\u7528\u4E2D\u65AD"));
                            return [2 /*return*/];
                        }
                        if (!((listener === null || listener === void 0 ? void 0 : listener.sourceKey) === 'platform:utils.If')) return [3 /*break*/, 5];
                        _f.label = 1;
                    case 1:
                        _f.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, invokeListener(listener, args, scopeContext, dataContext, $w)];
                    case 2:
                        res = _f.sent();
                        nextEvent.type = "".concat(listener.key, ".").concat(res ? 'success' : 'fail');
                        return [3 /*break*/, 4];
                    case 3:
                        e_1 = _f.sent();
                        nextEvent.type = "".concat(listener.key, ".fail");
                        return [3 /*break*/, 4];
                    case 4:
                        nextEvent.detail = currentEvent.detail;
                        return [3 /*break*/, 8];
                    case 5:
                        _f.trys.push([5, 7, , 8]);
                        return [4 /*yield*/, invokeListener(listener, args, scopeContext, dataContext, $w)];
                    case 6:
                        res = _f.sent();
                        nextEvent.type = "".concat(listener.key, ".success");
                        nextEvent.detail = res;
                        return [3 /*break*/, 8];
                    case 7:
                        e_2 = _f.sent();
                        if (e_2.message) {
                            e_2.message = "".concat(((_e = currentEvent === null || currentEvent === void 0 ? void 0 : currentEvent.currentTarget) === null || _e === void 0 ? void 0 : _e.id) || '', "[").concat(listener.trigger, "] ").concat(e_2.message);
                        }
                        nextEvent.type = "".concat(listener.key, ".fail");
                        nextEvent.detail = e_2;
                        // 之前 invoke 内部catch 了错误，不会抛错
                        // throw e
                        console.error("Action error: [".concat(listener.trigger, "]"), e_2);
                        return [3 /*break*/, 8];
                    case 8:
                        emitEvent(nextEvent.type, listeners, __assign(__assign({}, args), { event: nextEvent }), scopeContext, dataContext, $w, pageId);
                        return [2 /*return*/];
                }
            });
        });
    }
    if (!targetListeners.length && typeof trigger === 'string' && /\.fail$/.test(trigger)) {
        app.showToast({
            title: "\u4E8B\u4EF6\u54CD\u5E94\u5931\u8D25\uFF1A".concat(currentEvent === null || currentEvent === void 0 ? void 0 : currentEvent.detail),
            icon: 'error',
        });
    }
    else {
        targetListeners.forEach(function (listener) {
            processListener(listener);
        });
    }
}
export function generateEmit($w, currentWidget, // page 节点暂未生成
injectContext, extraListeners, extraForContext) {
    if (currentWidget === void 0) { currentWidget = undefined; }
    if (injectContext === void 0) { injectContext = undefined; }
    if (extraListeners === void 0) { extraListeners = []; }
    if (extraForContext === void 0) { extraForContext = {}; }
    return function (trigger, listeners, eventData, forItems, originalEvent, scopeContext, dataContext, pageId) {
        if (listeners === void 0) { listeners = []; }
        if (pageId === void 0) { pageId = ''; }
        listeners = listeners.concat(extraListeners);
        if (injectContext === null || injectContext === void 0 ? void 0 : injectContext.emit) {
            return injectContext === null || injectContext === void 0 ? void 0 : injectContext.emit(trigger, listeners, eventData, forItems, originalEvent, scopeContext, dataContext);
        }
        else {
            var event_1 = new Event({
                detail: eventData,
                type: trigger,
                target: currentWidget,
                currentTarget: currentWidget,
            });
            var forContext_1 = __assign(__assign({}, forItems), extraForContext);
            emitEvent(event_1.type, listeners, {
                event: event_1,
                forItems: forContext_1,
            }, scopeContext, dataContext, untracked(function () {
                return generateWidgetAPIContext($w, currentWidget, forContext_1, scopeContext);
            }), pageId);
        }
    };
}
function invokeListener(listener, args, scopeContext, dataContext, $w) {
    var _a, _b, _c;
    var key = listener.key, instanceFunction = listener.instanceFunction, _d = listener.data, data = _d === void 0 ? {} : _d, _e = listener.dataBinds, dataBinds = _e === void 0 ? {} : _e;
    if (!instanceFunction) {
        try {
            // @ts-ignore
            (_b = (_a = window._aegis) === null || _a === void 0 ? void 0 : _a.error) === null || _b === void 0 ? void 0 : _b.call(_a, 'SYSTEM_ERROR: listener not found', listener);
            console.error('$page current hander', (_c = runtime === null || runtime === void 0 ? void 0 : runtime['$page']) === null || _c === void 0 ? void 0 : _c.handler);
        }
        catch (e) { }
        throw new Error("".concat(key, " handler \u672A\u627E\u5230'"));
    }
    // ToDo resolve databinds
    var action = instanceFunction;
    var resolvedData = __assign({}, data);
    var resolvedDataBinds = resolveDataBinds(dataBinds, args.forItems, { event: args.event }, scopeContext, true, dataContext, $w);
    // eslint-disable-next-line no-restricted-syntax
    for (var key_1 in resolvedDataBinds) {
        if (resolvedDataBinds[key_1] && resolvedDataBinds[key_1].__type === 'scopedValue') {
            try {
                lodashSet(resolvedData, key_1, resolvedDataBinds[key_1].getValue(scopeContext));
            }
            catch (e) {
                lodashSet(resolvedData, key_1, '');
            }
        }
        else {
            lodashSet(resolvedData, key_1, resolvedDataBinds[key_1]);
        }
    }
    var params = __assign({ data: resolvedData }, args);
    return action(params);
}
