function prefixMsg(msg, prefix) {
    if (prefix === void 0) { prefix = ' - '; }
    return "".concat(msg ? prefix + msg : '');
}
export function errorPage(router) {
    return function (e, err) {
        var _a;
        var title;
        var desc;
        switch (err.error) {
            case 'unreachable':
                title = '网络错误';
                desc = "\u8BF7\u68C0\u67E5\u60A8\u7684\u7F51\u7EDC\u8FDE\u63A5\uFF0C\u7A0D\u540E\u91CD\u8BD5".concat(prefixMsg(e.error_description));
                break;
            case 'permission_denied':
                title = '权限不足';
                desc = "\u9519\u8BEF\u8BE6\u60C5 - ".concat(err.error_description);
                break;
            case '-':
                desc = err.error_description;
                break;
            case 'unauthorized_organization':
                // @todo @nanxi企微场景
                desc = '该应用可能未在您的企业安装，请重新安装，然后5分钟后重试';
                break;
            case '-':
                desc = err.error_description;
                break;
            case 'success':
                (_a = e.querySelector('.icon')) === null || _a === void 0 ? void 0 : _a.classList.add('hidden');
                title = '操作成功';
                desc = err.error_description;
                break;
            default:
                desc = "\u9519\u8BEF\u8BE6\u60C5 - ".concat(err.error || err.message).concat(prefixMsg(err.error_description));
                break;
        }
        if (title) {
            e.querySelector('.title').innerText = title;
        }
        if (desc) {
            e.querySelector('.desc').innerText = desc;
        }
        // 异常上报
        if (router.getUrlParams('redirect_uri')) {
            var redirectBtn = e.querySelector('.redirect-btn');
            redirectBtn.style.display = 'block';
            redirectBtn.onclick = function (e) {
                e.preventDefault();
                router.redirectURIorBackTo();
            };
        }
        router.loaded();
    };
}
// 登录错误处理
export var parseLoginError = function (e) {
    switch (e.error) {
        case 'not_found':
            return '用户不存在';
        case 'password_not_set':
            return '当前用户未设置密码，请使用验证码登录或第三方登录方式';
        case 'invalid_password':
            if (e.details && e.details.length > 0) {
                return "\u5BC6\u7801\u9A8C\u8BC1\u5931\u8D25, \u60A8\u8FD8\u6709 ".concat(e.details[0].remaining, " \u6B21\u91CD\u8BD5\u673A\u4F1A");
            }
            return '密码不正确';
        case 'user_pending':
            return '该用户未激活';
        case 'user_blocked':
            return '该用户被停用';
        case 'invalid_status':
            if (e.details && e.details.length > 0) {
                var retryInSeconds = e.details[0].retry_in;
                var retryInTime = retryInSeconds < 3600 ? "".concat(retryInSeconds / 60, "\u5206\u949F") : "".concat(retryInSeconds / 3600, "\u5C0F\u65F6");
                return "\u60A8\u5DF2\u7ECF\u8D85\u8FC7\u4E86\u5BC6\u7801\u6700\u5927\u91CD\u8BD5\u6B21\u6570\uFF0C\u8BF7\u4E8E ".concat(retryInTime, " \u540E\u91CD\u8BD5");
            }
            return '您已经超过了密码最大重试次数，请稍后重试';
        default:
            return '您输入的帐号或密码不正确，请重新输入';
    }
};
// 注册错误处理
export var parseSignUpError = function (e) {
    switch (e.error) {
        case 'failed_precondition':
            return '您输入的手机号或邮箱已被注册，请使用其他号码';
        default:
            return parseVerifyError(e);
    }
};
// 验证码验证token
export var parseVerifyError = function (e) {
    switch (e.error) {
        case 'invalid_argument':
            return '您输入的验证码不正确或已过期';
        case 'aborted':
            return '您尝试的次数过多，稍后重试';
        default:
            return parseCommonError(e);
    }
};
// 三方登录表单错误
export var parseProviderSignError = function (e) {
    switch (e.error) {
        case 'permission_denied':
            return '您当前的会话已过期，请返回重试';
        case 'invalid_argument':
            return '您输入的验证码不正确或已过期';
        case 'aborted':
            return '您尝试的次数过多，稍后重试';
        default:
            return parseCommonError(e);
    }
};
// 重置密码错误
export var parseResetPasswordError = function (e) {
    switch (e.error) {
        case 'invalid_argument':
            return '您输入的密码不符合要求, 请重新输入';
        default:
            return parseCommonError(e);
    }
};
// 验证码验证token
export var parseProviderTokenError = function (e) {
    switch (e.error) {
        case 'failed_precondition':
            return "\u4ECE\u7B2C\u4E09\u65B9\u83B7\u53D6\u7528\u6237\u4FE1\u606F\u5931\u8D25".concat(prefixMsg(e.error_description));
        case 'not_found':
            return "\u6CA1\u6709\u627E\u5230\u5339\u914D\u7684\u7528\u6237\n\u8BF7\u8054\u7CFB\u7BA1\u7406\u5458\u786E\u8BA4\u60A8\u7684\u914D\u7F6E";
        default:
            return parseCommonError(e);
    }
};
export function parseValidateError(e, verificationTarget) {
    switch (e.error) {
        case 'failed_precondition':
            switch (verificationTarget) {
                case 'USER':
                    return '你输入的手机号或邮箱已被注册，请使用其他号码';
                case 'NOT_USER':
                    return '该用户已被注册，请使用其他号码';
                default:
                    return '短信功能未开启，请联系管理员开启';
            }
        case 'payment_required':
            return '短信包剩余条数不足，请联系管理员充值';
        case 'aborted':
            return '你尝试的次数过多，稍后重试';
        case 'unavailable':
            return "\u53D1\u4EF6\u670D\u52A1\u4E0D\u53EF\u7528\uFF0C\u8BF7\u7A0D\u540E\u91CD\u8BD5".concat(prefixMsg(e.error_description));
        default:
            return parseCommonError(e);
    }
}
export var parseCommonError = function (e) {
    var msg;
    switch (e.error) {
        case 'unreachable':
            msg = "\u7F51\u7EDC\u9519\u8BEF\uFF0C\u8BF7\u68C0\u67E5\u60A8\u7684\u7F51\u7EDC\u8FDE\u63A5\uFF0C\u7A0D\u540E\u91CD\u8BD5".concat(prefixMsg(e.error_description));
            break;
        case 'resource_exhausted':
            return '您尝试过于频繁，请稍后重试';
        default:
            msg = "\u670D\u52A1\u5F02\u5E38, \u8BF7\u7A0D\u540E\u91CD\u8BD5".concat(prefixMsg(e.error || e.message)).concat(prefixMsg(e.error_description));
            break;
    }
    return msg;
};
