import { __read, __values } from "tslib";
var Router = /** @class */ (function () {
    function Router(loading, location) {
        var _this = this;
        this._routes = new Map();
        this._paramsResolvers = new Map();
        this._loading = loading;
        this._location = location;
        this._urlParams = this.parseURLParams();
        document.body.addEventListener('click', function (event) {
            var _a;
            var target = event.target;
            if ((target === null || target === void 0 ? void 0 : target.tagName) === 'A' && ((_a = target === null || target === void 0 ? void 0 : target.dataset) === null || _a === void 0 ? void 0 : _a.link) !== undefined) {
                var href = target.getAttribute('href');
                if (href.indexOf('://') > 0) {
                    return true;
                }
                event.preventDefault();
                _this.show(href);
            }
        });
    }
    Router.prototype.add = function (id, fn) {
        var e_1, _a;
        if (typeof id === 'string') {
            this._routes.set(id, fn);
            return;
        }
        try {
            for (var id_1 = __values(id), id_1_1 = id_1.next(); !id_1_1.done; id_1_1 = id_1.next()) {
                var v = id_1_1.value;
                this._routes.set(v, fn);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (id_1_1 && !id_1_1.done && (_a = id_1.return)) _a.call(id_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    Router.prototype.addParamsResolver = function (param, fn, weight) {
        if (!weight) {
            weight = 0;
        }
        this._paramsResolvers.set(param, {
            fn: fn,
            weight: weight,
        });
    };
    Router.prototype.show = function (id, data) {
        var ele = document.querySelector(".l-".concat(id));
        var currentEle = document.querySelector('.l-block:not(.hidden)');
        if (currentEle) {
            currentEle.classList.add('hidden');
            if (currentEle instanceof HTMLFormElement) {
                currentEle.reset();
            }
        }
        ele.classList.remove('hidden');
        var action = this._routes.get(id);
        if (action && ele instanceof HTMLFormElement) {
            action(ele, data);
        }
    };
    Router.prototype.start = function () {
        var _this = this;
        if (this._urlParams.size > 0) {
            var callback_1;
            this._urlParams.forEach(function (value, key) {
                var tmp = _this._paramsResolvers.get(key);
                if (tmp) {
                    if (!callback_1) {
                        callback_1 = tmp;
                    }
                    else {
                        if (tmp.weight > callback_1.weight) {
                            callback_1 = tmp;
                        }
                    }
                }
            });
            if (callback_1) {
                callback_1.fn(this._urlParams);
                return;
            }
        }
        this.show('main');
    };
    Router.prototype.loading = function () {
        this._loading.style.display = 'block';
    };
    Router.prototype.loaded = function () {
        this._loading.style.display = 'none';
    };
    Router.prototype.redirectURIorBackTo = function (backTo) {
        var redirectUri = decodeURIComponent(this._urlParams.get('redirect_uri') || '');
        if (redirectUri) {
            this._location.replace(backTo || redirectUri);
        }
    };
    Router.prototype.getUrlParams = function (key) {
        return this._urlParams.get(key);
    };
    Router.prototype.setUrlParams = function (key, value) {
        this._urlParams.set(key, value);
    };
    Router.prototype.checkIfHasParamsResolver = function () {
        var e_2, _a;
        try {
            for (var _b = __values(this._paramsResolvers), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = __read(_c.value, 1), key = _d[0];
                if (this._urlParams.get(key)) {
                    return true;
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return false;
    };
    Router.prototype.parseURLParams = function () {
        var search;
        var location = this._location;
        if (location.search.length > 2) {
            search = location.search.substring(1);
        }
        else if (location.hash.indexOf('=') > 0) {
            search = location.hash.substring(1);
        }
        var params = new Map();
        if (!search) {
            return params;
        }
        search.split('&').forEach(function (item) {
            var _a = __read(item.split('='), 2), key = _a[0], value = _a[1];
            params.set(key, value);
        });
        return params;
    };
    return Router;
}());
export { Router };
