"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadMockDependencies = exports.loadRuntimeDependencies = exports.loadExternalResources = exports.formatDependiencesMap = exports.loadDependiences = exports.loadDependencies = exports.injectExternals = exports.upsertFavicon = void 0;
var tslib_1 = require("tslib");
var React = tslib_1.__importStar(require("react"));
var ReactDom = tslib_1.__importStar(require("react-dom"));
var server_1 = tslib_1.__importDefault(require("react-dom/server"));
var MobxReactLite = tslib_1.__importStar(require("mobx-react-lite"));
var Mobx = tslib_1.__importStar(require("mobx"));
var lodash_es_1 = require("lodash-es");
var History = tslib_1.__importStar(require("history"));
var ReactErrorBoundary = tslib_1.__importStar(require("react-error-boundary"));
var cals_1 = require("@cloudbase/cals");
var hooks_1 = require("../hooks");
var index_1 = require("./index");
var constants_1 = require("../constants");
var weda_client_1 = require("@cloudbase/weda-client");
function loadUmdModule(jsUrl, umdModuleName, targetDoc, cache, module) {
    if (targetDoc === void 0) { targetDoc = document; }
    if (cache === void 0) { cache = false; }
    if (module === void 0) { module = true; }
    return new Promise(function (resolve, reject) {
        var win = targetDoc.defaultView;
        if (cache) {
            if (module && Object.prototype.hasOwnProperty.call(win, umdModuleName)) {
                return resolve(win[umdModuleName]);
            }
            if (!module) {
                var scripts = Array.from(targetDoc.body.querySelectorAll('script'));
                if (scripts.find(function (item) { return (item === null || item === void 0 ? void 0 : item.className) === umdModuleName; })) {
                    return resolve(null);
                }
            }
        }
        var script = targetDoc.createElement('script');
        script.setAttribute('src', jsUrl);
        script.setAttribute('class', umdModuleName);
        script.addEventListener('load', function () {
            if (module) {
                if (Object.prototype.hasOwnProperty.call(win, umdModuleName)) {
                    return resolve(win[umdModuleName]);
                }
                else {
                    var error = new Error("Fail to load UMD module ".concat(umdModuleName, " from [").concat(jsUrl, "]."));
                    return reject(error);
                }
            }
            else {
                return resolve(null);
            }
        });
        script.addEventListener('error', function (e) {
            var error = new Error("main bundle [".concat(umdModuleName, "] load failed from [").concat(jsUrl, "]: ").concat((e === null || e === void 0 ? void 0 : e.message) || ''));
            reject(error);
        });
        targetDoc.body.appendChild(script);
    });
}
function loadCssModule(url, bundleName, entryDoc, cache, module) {
    if (entryDoc === void 0) { entryDoc = document; }
    if (cache === void 0) { cache = false; }
    if (module === void 0) { module = true; }
    if (!url) {
        return Promise.resolve();
    }
    return new Promise(function (resolve) {
        var className = module ? "@weda-materials-style-".concat(bundleName) : bundleName;
        if (cache) {
            var links = Array.from(entryDoc.head.querySelectorAll('link'));
            if (links.find(function (item) { return (item === null || item === void 0 ? void 0 : item.className) === className; })) {
                return resolve();
            }
        }
        var link = entryDoc.createElement('link');
        link.type = 'text/css';
        link.rel = 'stylesheet';
        link.className = className;
        link.href = url;
        link.addEventListener('load', function () { return resolve(); });
        link.addEventListener('error', function (e) {
            console.warn("Load [".concat(bundleName, "] meta style failed, skip..."), e);
            resolve();
        });
        entryDoc.head.appendChild(link);
    });
}
function upsertFavicon(favicon, app, entryDoc) {
    if (entryDoc === void 0) { entryDoc = document; }
    if (!favicon) {
        return;
    }
    var _a = (app || {}).__internal__, __internal__ = _a === void 0 ? {} : _a;
    var resolveStaticResourceUrl = (__internal__ || {}).resolveStaticResourceUrl;
    var href = resolveStaticResourceUrl(favicon);
    var links = Array.from(entryDoc.head.querySelectorAll('link[rel~="icon"]'));
    if (links.length) {
        links.forEach(function (link) {
            link.href = href;
        });
    }
    else {
        var link = entryDoc.createElement('link');
        link.type = 'image/x-icon';
        link.rel = 'shortcut icon';
        link.href = href;
        entryDoc.head.appendChild(link);
    }
}
exports.upsertFavicon = upsertFavicon;
function injectExternals(runtime) {
    if (runtime === void 0) { runtime = index_1.runtime; }
    var externals = {
        // 兼容配置
        react: runtime.React || React,
        React: runtime.React || React,
        ReactDOM: runtime.ReactDOM || ReactDom,
        ReactDom: runtime.ReactDOM || ReactDom,
        ReactDomServer: server_1.default,
        Mobx: runtime.mobx || Mobx,
        'mobx-react-lite': MobxReactLite,
        MobxReactLite: MobxReactLite,
        LodashGet: lodash_es_1.get,
        LodashRemove: lodash_es_1.remove,
        LodashSet: lodash_es_1.set,
        ReactErrorBoundary: ReactErrorBoundary,
        History: History,
        WedaAppContext: runtime.WedaAppContext || weda_client_1.AppContext,
    };
    for (var key in externals) {
        runtime[key] = externals[key];
    }
}
exports.injectExternals = injectExternals;
function loadDependencies(data, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var controlPromises, componentPromises, cssPromises, dependencies, runtime, cache, _a, controls, components, res;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    controlPromises = [];
                    componentPromises = [];
                    cssPromises = [];
                    dependencies = Array.isArray(data) ? data : data.dependencies;
                    runtime = !Array.isArray(data) && data.runtime ? data.runtime : index_1.runtime;
                    cache = typeof options === 'boolean' ? options : (options === null || options === void 0 ? void 0 : options.cache) || false;
                    dependencies.forEach(function (item) {
                        if (!item.isComposite) {
                            controlPromises.push(loadUmdModule(item.controlJsPath || item.mainJsPath.replace(/\.main\.js/, '.control.js'), "@weapps-materials-control-".concat(item.name), runtime.document, cache)
                                .then(function (res) {
                                res.name = item.name;
                                res.version = item.version;
                                return res;
                            })
                                .catch(function (e) {
                                console.warn(e);
                                return null;
                            }));
                        }
                        componentPromises.push(loadUmdModule(item.mainJsPath, "@weapps-materials-main-".concat(item.name), runtime.document, cache).then(function (res) {
                            res.name = item.name;
                            res.isComposite = item.isComposite;
                            return res;
                        }, function (e) {
                            console.warn(e);
                            return null;
                        }));
                        cssPromises.push(loadCssModule(item.mainCssPath, item.name, runtime.document, cache));
                    });
                    return [4 /*yield*/, Promise.all([controlPromises, componentPromises, cssPromises].map(function (arr) { return Promise.all(arr); }))];
                case 1:
                    _a = tslib_1.__read.apply(void 0, [_b.sent(), 2]), controls = _a[0], components = _a[1];
                    res = formatDependiencesMap(components, controls);
                    return [2 /*return*/, res];
            }
        });
    });
}
exports.loadDependencies = loadDependencies;
exports.loadDependiences = loadDependencies;
function formatDependiencesMap(components, controls) {
    var componentsMap = {};
    var componentActionsMap = {};
    components
        .filter(function (item) { return !!item; })
        .forEach(function (lib) {
        var _a, _b;
        var _c = lib.schemaVersion, schemaVersion = _c === void 0 ? '' : _c;
        var isPlainProps = true;
        if (lib.isComposite ||
            isNaN(Number((_a = schemaVersion.split('.')) === null || _a === void 0 ? void 0 : _a[0])) ||
            Number((_b = schemaVersion.split('.')) === null || _b === void 0 ? void 0 : _b[0]) < 3) {
            isPlainProps = false;
        }
        var _loop_1 = function (componentName) {
            var Field = lib.components[componentName];
            var name_1 = Field.displayName || Field.name || 'Field';
            var processWrapper = function (props, ref) {
                var processedProps = (0, hooks_1.useComponentProps)(props, isPlainProps);
                return React.createElement(Field, tslib_1.__assign({}, processedProps, { ref: ref }));
            };
            processWrapper.displayName = "ForwordRef(".concat(name_1, ")");
            componentsMap["".concat(lib.name, ":").concat(componentName)] = React.forwardRef(processWrapper);
        };
        for (var componentName in lib.components || {}) {
            _loop_1(componentName);
        }
        for (var actionName in lib.actions) {
            componentActionsMap["".concat(lib.name, ":").concat(actionName)] = lib.actions[actionName];
        }
    });
    var controlMap = controls
        .filter(function (item) { return !!item; })
        .reduce(function (map, lib) {
        map["".concat(lib.name, "@").concat(lib.version)] = lib;
        return map;
    }, {});
    // console.log('controlMap', controlMap);
    // console.log('componentsMap', componentsMap);
    return {
        controlMap: controlMap,
        componentsMap: componentsMap,
        componentActionsMap: componentActionsMap,
    };
}
exports.formatDependiencesMap = formatDependiencesMap;
function loadExternalResources(_a, options) {
    var _b = _a.externalResources, externalResources = _b === void 0 ? [] : _b, _c = _a.runtime, runtime = _c === void 0 ? index_1.runtime : _c;
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var scriptPromises, cssPromises, cache, externalResources_1, externalResources_1_1, externalResource;
        var e_1, _d;
        return tslib_1.__generator(this, function (_e) {
            switch (_e.label) {
                case 0:
                    scriptPromises = [];
                    cssPromises = [];
                    cache = options === null || options === void 0 ? void 0 : options.cache;
                    try {
                        for (externalResources_1 = tslib_1.__values(externalResources), externalResources_1_1 = externalResources_1.next(); !externalResources_1_1.done; externalResources_1_1 = externalResources_1.next()) {
                            externalResource = externalResources_1_1.value;
                            switch (externalResource.type) {
                                case cals_1.EExternalResourceType.JSUrl: {
                                    scriptPromises.push(loadUmdModule(externalResource.url, externalResource.url, runtime.document, cache, false).catch(function (e) {
                                        console.warn(e);
                                        return null;
                                    }));
                                    break;
                                }
                                case cals_1.EExternalResourceType.CSSUrl: {
                                    cssPromises.push(loadCssModule(externalResource.url, externalResource.url, runtime.document, cache, false));
                                    break;
                                }
                            }
                        }
                    }
                    catch (e_1_1) { e_1 = { error: e_1_1 }; }
                    finally {
                        try {
                            if (externalResources_1_1 && !externalResources_1_1.done && (_d = externalResources_1.return)) _d.call(externalResources_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                    }
                    return [4 /*yield*/, Promise.all(tslib_1.__spreadArray(tslib_1.__spreadArray([], tslib_1.__read(scriptPromises), false), tslib_1.__read(cssPromises), false))];
                case 1:
                    _e.sent();
                    return [2 /*return*/];
            }
        });
    });
}
exports.loadExternalResources = loadExternalResources;
function loadRuntimeDependencies() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a;
        return tslib_1.__generator(this, function (_b) {
            return [2 /*return*/, {
                    controlMap: {},
                    componentsMap: (_a = {},
                        _a["".concat(constants_1.REPEATER.MODULE_NAME, ":").concat(constants_1.REPEATER.REPEATER_NAME)] = Repeater,
                        _a["".concat(constants_1.REPEATER.MODULE_NAME, ":").concat(constants_1.REPEATER.REPEATER_ITEM_NAME)] = Block,
                        _a),
                    componentActionsMap: {},
                }];
        });
    });
}
exports.loadRuntimeDependencies = loadRuntimeDependencies;
function loadMockDependencies() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            return [2 /*return*/, {
                    controlMap: {},
                    componentsMap: {
                    // eslint-disable-next-line react/display-name
                    // [`${OFFICIAL_LIB_KEY}:WdText`]: React.forwardRef((props: React.PropsWithChildren<{}>, ref) => {
                    //   // eslint-disable-next-line react/prop-types
                    //   const processedProps = useComponentProps(props as any, true);
                    //   return <DebugText {...processedProps} ref={ref} />;
                    // }),
                    },
                    componentActionsMap: {},
                }];
        });
    });
}
exports.loadMockDependencies = loadMockDependencies;
var Repeater = React.forwardRef(function (props, ref) {
    var _a = (0, hooks_1.useComponentProps)(props, true), data = _a.data, children = _a.children, $node = _a.$node;
    React.useImperativeHandle(ref, function () {
        return {
            data: data,
            items: $node.items,
        };
    }, [data, $node.children]);
    return typeof children === 'function' ? children() : children;
});
Repeater.displayName = constants_1.REPEATER.REPEATER_NAME;
var Block = React.forwardRef(function (props, ref) {
    // eslint-disable-next-line react/prop-types
    var children = props.children;
    return typeof children === 'function' ? children() : children;
});
Block.displayName = 'Block';
// eslint-disable-next-line react/display-name
var DebugText = React.forwardRef(function (props, ref) {
    // eslint-disable-next-line react/prop-types
    var id = props.id, text = props.text, events = props.events;
    var _a = tslib_1.__read(React.useState('23232'), 2), value = _a[0], setValue = _a[1];
    React.useImperativeHandle(ref, function () {
        return {
            test: value,
            text: '123',
            console: function () {
                var arg = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    arg[_i] = arguments[_i];
                }
                console.log.apply(console, tslib_1.__spreadArray(['id', id], tslib_1.__read(arg), false));
            },
        };
    }, []);
    React.useEffect(function () {
        setInterval(function () {
            setValue(String(Math.random()));
            console.log('>>>>>>>>>>>>>>.');
            ref.current.text = String(Math.random());
        }, 4000);
    }, []);
    return (React.createElement("p", { id: id, onClick: function (e) { return events.tap({}, { originEvent: e }); }, "data-testid": "text" }, String(text)));
});
//# sourceMappingURL=dependencies.js.map