"use strict";
/* eslint-disable no-new-func */
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateDataBindForDataset = exports.generateDataBinds = exports.getComponentSchema = exports.getComponentsInfo = void 0;
var tslib_1 = require("tslib");
var common_1 = require("../common");
var cals_1 = require("@cloudbase/cals");
var lodash_es_1 = require("lodash-es");
var index_1 = require("./index");
var constants_1 = require("../constants");
var index_2 = require("../index");
function getComponentsInfo(dependencies) {
    var outputObj = {};
    dependencies.forEach(function (_a) {
        var materialName = _a.name, components = _a.components, isComposite = _a.isComposite, version = _a.version;
        if (isComposite) {
            components.forEach(function (component) {
                var lowCodes = component.lowCodes, componentInstances = component.componentInstances, compItem = tslib_1.__rest(component, ["lowCodes", "componentInstances"]);
                var sourceKey = "".concat(materialName, ":").concat(compItem.name);
                outputObj[sourceKey] = tslib_1.__assign(tslib_1.__assign({ isComposite: isComposite }, compItem), { 
                    /**
                     * @deprecated
                     * 勿新增引用
                     */
                    _version: version });
            });
        }
        else {
            components.forEach(function (_a) {
                var name = _a.name, meta = _a.meta, emitEvents = _a.emitEvents, _b = _a.events, events = _b === void 0 ? undefined : _b, isContainer = _a.isContainer, compConfig = _a.compConfig, dataForm = _a.dataForm, properties = _a.properties;
                outputObj["".concat(materialName, ":").concat(name)] = tslib_1.__assign(tslib_1.__assign({ isComposite: isComposite }, meta), { events: events, emitEvents: emitEvents, isContainer: isContainer, compConfig: compConfig, dataForm: dataForm, properties: properties, 
                    /**
                     * @deprecated
                     * 勿新增引用
                     */
                    _version: version });
            });
        }
    });
    return outputObj;
}
exports.getComponentsInfo = getComponentsInfo;
function getInputProps(componentsMetaMap) {
    var outputObj = {};
    var _loop_1 = function (key) {
        var component = componentsMetaMap[key];
        var sourceKey = key;
        var inputProps = (component === null || component === void 0 ? void 0 : component.inputProps) || (component === null || component === void 0 ? void 0 : component.syncProps);
        if (inputProps) {
            outputObj[sourceKey] = inputProps;
        }
        var compItem = component;
        Object.keys(compItem.dataForm || {}).forEach(function (key) {
            var _a;
            var _b, _c;
            var inputProps = ((_b = compItem.dataForm[key]) === null || _b === void 0 ? void 0 : _b.inputProp) || ((_c = compItem.dataForm[key]) === null || _c === void 0 ? void 0 : _c.syncProp);
            if (inputProps) {
                outputObj[sourceKey] = tslib_1.__assign((_a = {}, _a[key] = inputProps, _a), (outputObj[sourceKey] || {}));
            }
        });
    };
    for (var key in componentsMetaMap) {
        _loop_1(key);
    }
    return outputObj;
}
/**
 * utils end
 */
function splitDynamicMap(data) {
    var staticData = {};
    var dataBinds = [];
    for (var key in data) {
        var meta = data[key];
        if (!meta.type || meta.type === cals_1.PropBindType.static) {
            staticData[key] = meta.value;
        }
        else if (meta.type === cals_1.PropBindType.slot) {
            continue;
        }
        else {
            dataBinds.push({
                type: meta.type,
                bindDataPath: meta.value,
                propertyPath: key,
            });
        }
    }
    return { staticData: staticData, dataBinds: dataBinds };
}
function walkThroughCals(cals, process, options) {
    var e_1, _a, e_2, _b;
    var _c, _d;
    if (options === void 0) { options = {}; }
    var _e = cals.items, items = _e === void 0 ? [] : _e;
    cals.extra = cals.extra || {};
    if (((_c = cals === null || cals === void 0 ? void 0 : cals.directives) === null || _c === void 0 ? void 0 : _c[':if']) === false) {
        cals.items = [];
    }
    var filteredItems = [];
    try {
        for (var items_1 = tslib_1.__values(items), items_1_1 = items_1.next(); !items_1_1.done; items_1_1 = items_1.next()) {
            var item = items_1_1.value;
            if (((_d = item === null || item === void 0 ? void 0 : item.directives) === null || _d === void 0 ? void 0 : _d[':if']) === false) {
                continue;
            }
            filteredItems.push(item);
            item.extra = item.extra || {};
            item.extra.__parent = cals;
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (items_1_1 && !items_1_1.done && (_a = items_1.return)) _a.call(items_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    cals.items = filteredItems;
    // 先根遍历 因为父上的状态可能需要带下去
    process(cals, options);
    try {
        for (var items_2 = tslib_1.__values(items), items_2_1 = items_2.next(); !items_2_1.done; items_2_1 = items_2.next()) {
            var item = items_2_1.value;
            walkThroughCals(item, process, options);
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (items_2_1 && !items_2_1.done && (_b = items_2.return)) _b.call(items_2);
        }
        finally { if (e_2) throw e_2.error; }
    }
    return cals;
}
function parseDirectiveIfAndDispaly(directiveValue) {
    var value;
    switch (directiveValue) {
        case true:
        case false: {
            value = directiveValue;
            break;
        }
        case undefined: {
            value = true;
            break;
        }
    }
    return value;
}
/**
 * widgets 不包括page 的 widges
 */
function getComponentSchema(ctx, page, isComposite, componentsInfoMap, wrapperClass, options) {
    if (isComposite === void 0) { isComposite = false; }
    if (componentsInfoMap === void 0) { componentsInfoMap = {}; }
    var componentInputProps = getInputProps(componentsInfoMap) || {};
    var copyJson = JSON.parse(JSON.stringify(page, function (key, value) {
        if (value === null || value === void 0 ? void 0 : value.component) {
            var component_1 = value;
            if ((value === null || value === void 0 ? void 0 : value.component) &&
                ("".concat(component_1.module, ":").concat(component_1.component) === "".concat(constants_1.REPEATER.MODULE_NAME, ":").concat(constants_1.REPEATER.REPEATER_NAME) ||
                    "".concat(component_1.module, ":").concat(component_1.component) === "".concat(constants_1.REPEATER.DEBUG_MODULE_NAME, ":").concat(constants_1.REPEATER.REPEATER_NAME))) {
                // 给 Repeater 组件加一层虚拟项组件，在虚拟项组件上挂 for 循环
                var _a = component_1.items, items = _a === void 0 ? [] : _a;
                var firstChildComponent = items[0];
                if ("".concat(firstChildComponent === null || firstChildComponent === void 0 ? void 0 : firstChildComponent.module, ":").concat(firstChildComponent === null || firstChildComponent === void 0 ? void 0 : firstChildComponent.component) ===
                    "".concat(constants_1.REPEATER.MODULE_NAME, ":").concat(constants_1.REPEATER.REPEATER_ITEM_NAME) ||
                    "".concat(firstChildComponent === null || firstChildComponent === void 0 ? void 0 : firstChildComponent.module, ":").concat(firstChildComponent === null || firstChildComponent === void 0 ? void 0 : firstChildComponent.component) ===
                        "".concat(constants_1.REPEATER.DEBUG_MODULE_NAME, ":").concat(constants_1.REPEATER.REPEATER_ITEM_NAME)) {
                    return value;
                }
                component_1.items = [
                    {
                        component: constants_1.REPEATER.REPEATER_ITEM_NAME,
                        module: component_1.module,
                        directives: {
                            ':if': true,
                            ':for': component_1.attributes[':data'],
                            key: component_1.attributes.key || '',
                            ':display': component_1.directives[':display'],
                        },
                        items: items.map(function (item) {
                            if (component_1.directives[':display'] !== undefined && component_1.directives[':display'] !== 'true') {
                                if (!item.directives) {
                                    item.directives = {};
                                }
                                var current = item.directives[':display'] || 'true';
                                item.directives[':display'] = "(\n".concat(component_1.directives[':display'], "\n) && (\n").concat(current, "\n)");
                            }
                            return item;
                        }),
                        id: "$weda_internal_".concat(component_1.id), // uniqueId() 解决在编辑器中切换组件导致 children 不断增加的问题
                    },
                ];
            }
        }
        return value;
    }));
    var compWidgets = {};
    var compDataBinds = {};
    var _a = options.processCssUnit, processCssUnit = _a === void 0 ? 'rem' : _a, _b = options.looseError, looseError = _b === void 0 ? false : _b;
    var componentSchemaJson = walkThroughCals(copyJson, function (component, options) {
        var e_3, _a;
        var _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v;
        if (options === void 0) { options = {}; }
        var parentComponent = component.extra.__parent;
        if ((0, common_1.isSlot)(component)) {
            var slotId = "".concat(parentComponent.id, ".").concat(component.id);
            var parentComponentSourceKey = "".concat(parentComponent.module, ":").concat(parentComponent.component);
            var sourceKey_1 = "".concat(parentComponentSourceKey, ".").concat(component.id);
            var _w = (0, common_1.getScopeSlotMeta)(parentComponent, component.id, {
                componentsInfoMap: componentsInfoMap,
            }), isScopeSlot = _w.isScopeSlot, scopeSlotConfig = _w.config, _x = _w.useScopeWidgetData, useScopeWidgetData = _x === void 0 ? false : _x;
            if (isScopeSlot) {
                compWidgets[slotId] = {
                    _visible: true,
                    _display: true,
                    style: {},
                    classList: [],
                    widgetType: sourceKey_1,
                    _parentId: parentComponent.id,
                    getConfig: function () { return ({}); },
                    _waForKey: undefined,
                };
                if (!useScopeWidgetData) {
                    options.useScopeWidgetData = false;
                }
                if ((_b = scopeSlotConfig === null || scopeSlotConfig === void 0 ? void 0 : scopeSlotConfig['x-slot-directives']) === null || _b === void 0 ? void 0 : _b.for) {
                    compDataBinds[slotId] = generateDataBinds(ctx, [
                        {
                            type: cals_1.PropBindType.expression,
                            bindDataPath: "$w.".concat(parentComponent.id, ".").concat((_c = scopeSlotConfig === null || scopeSlotConfig === void 0 ? void 0 : scopeSlotConfig['x-slot-directives']) === null || _c === void 0 ? void 0 : _c.for),
                            // bindDataPath: '$page.dataset.state.list',
                            propertyPath: '_waFor',
                        },
                    ], {
                        isComposite: isComposite,
                        looseError: looseError,
                        key: "\u7EC4\u4EF6 ".concat(slotId),
                        useScopeWidgetData: true,
                    });
                }
            }
            return;
        }
        var componentName = component.component, module = component.module, _y = component.attributes, attributes = _y === void 0 ? {} : _y;
        var _z = (component.extra || {}).commonStyle, commonStyle = _z === void 0 ? {} : _z;
        var processedCommonStyle = attributes.style
            ? attributes.style
            : (0, cals_1.processCommonStyle2CSSProperties)(commonStyle, {
                defaultUnit: processCssUnit,
                rpxCalcRoot: "var(".concat(index_2.CALC_RPX_ROOT_VAL_NAME, ")"),
            });
        if (processedCommonStyle) {
            component.extra.commonStyle = processedCommonStyle;
        }
        var sourceKey = "".concat(module, ":").concat(componentName);
        var _0 = attributes.class, className = _0 === void 0 ? '' : _0;
        var deserializeCtx = { page: page };
        var dynamicStyle = component[':style']
            ? (0, cals_1.deserializeValue)(deserializeCtx, ':style', component[':style']).value
            : undefined;
        var dynamicClassList = component[':class']
            ? (0, cals_1.deserializeValue)(deserializeCtx, ':class', component[':class']).value
            : undefined;
        var dynamicIf = ((_d = component.directives) === null || _d === void 0 ? void 0 : _d[':if'])
            ? (0, cals_1.deserializeValue)(deserializeCtx, ':if', (_e = component.directives) === null || _e === void 0 ? void 0 : _e[':if']).value
            : undefined;
        var dynamicDisplay = ((_f = component.directives) === null || _f === void 0 ? void 0 : _f[':display'])
            ? (0, cals_1.deserializeValue)(deserializeCtx, ':display', (_g = component.directives) === null || _g === void 0 ? void 0 : _g[':display']).value
            : undefined;
        var dynamicFor = ((_h = component.directives) === null || _h === void 0 ? void 0 : _h[':for'])
            ? (0, cals_1.deserializeValue)(deserializeCtx, ':for', (_j = component.directives) === null || _j === void 0 ? void 0 : _j[':for']).value
            : undefined;
        if (dynamicIf != undefined &&
            (dynamicIf.value == undefined || dynamicIf.value === true || dynamicIf.value === false)) {
            dynamicIf = undefined;
        }
        if (dynamicDisplay != undefined &&
            (dynamicDisplay.value == undefined || dynamicDisplay.value === true || dynamicDisplay.value === false)) {
            dynamicDisplay = undefined;
        }
        if (dynamicFor) {
            var type = dynamicFor.type, value = dynamicFor.value;
            if (type === cals_1.PropBindType.expression) {
                dynamicFor.value = value.replace(/^(.*?in\s)/, '');
            }
        }
        var data = (0, cals_1.deserializeAttributes)(deserializeCtx, attributes);
        var _1 = splitDynamicMap(data), dataBinds = _1.dataBinds, staticData = _1.staticData;
        var componentInfo = componentsInfoMap[sourceKey];
        if (componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.compConfig) {
            component.extra.compConfig = componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.compConfig;
        }
        if (componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.events) {
            component.extra.emitEvents = componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.events.map(function (item) { return item.name; });
        }
        else if (componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.emitEvents) {
            component.extra.emitEvents = componentInfo.emitEvents.map(function (item) { return item.eventName; });
        }
        // 生成 widgets/dataBinds
        if (!(0, common_1.isSlot)(component) && component.id) {
            var _visible = parseDirectiveIfAndDispaly((_k = component.directives) === null || _k === void 0 ? void 0 : _k[':if']);
            var _display = parseDirectiveIfAndDispaly((_l = component === null || component === void 0 ? void 0 : component.directives) === null || _l === void 0 ? void 0 : _l[':display']);
            if (component.type !== 'PAGE') {
                var _parentId = null;
                if (parentComponent) {
                    _parentId = parentComponent.id;
                    if (parentComponent.type === 'PAGE') {
                        _parentId = (_o = (_m = parentComponent.extra) === null || _m === void 0 ? void 0 : _m.__parent) === null || _o === void 0 ? void 0 : _o.id;
                    }
                    else if ((0, common_1.isSlot)(parentComponent)) {
                        _parentId = (_q = (_p = parentComponent.extra) === null || _p === void 0 ? void 0 : _p.__parent) === null || _q === void 0 ? void 0 : _q.id;
                        var isScopeSlot = (0, common_1.getScopeSlotMeta)(parentComponent.extra.__parent, parentComponent.id, {
                            componentsInfoMap: componentsInfoMap,
                        }).isScopeSlot;
                        if (isScopeSlot) {
                            _parentId = "".concat((_r = parentComponent.extra.__parent) === null || _r === void 0 ? void 0 : _r.id, ".").concat(parentComponent.id);
                        }
                    }
                }
                compWidgets[component.id] = tslib_1.__assign(tslib_1.__assign({}, Object.entries(staticData).reduce(function (data, _a) {
                    var _b = tslib_1.__read(_a, 2), propsKey = _b[0], propsValue = _b[1];
                    var value = propsValue;
                    if (value && Array.isArray(value)) {
                        value = value.map(function (_, index) {
                            var _a, _b;
                            var block = (_a = attributes[propsKey]) === null || _a === void 0 ? void 0 : _a[index]; // 因为 value 里的是序列化过的，尤其是选区的是时候
                            var blockInstances = (_b = block === null || block === void 0 ? void 0 : block.selectableBlock) === null || _b === void 0 ? void 0 : _b.listeners;
                            if (blockInstances) {
                                block.selectableBlock.listenerInstances = generateListnerInstances(ctx, blockInstances || [], {
                                    isComposite: isComposite,
                                    looseError: looseError,
                                    useScopeWidgetData: options.useScopeWidgetData === undefined ? true : options.useScopeWidgetData,
                                });
                                return block;
                            }
                            else {
                                // 普通的非选区的，返回staticData 里的值
                                return _;
                            }
                        });
                    }
                    data[propsKey] = value;
                    return data;
                }, {})), { 
                    /**
                     * 此处新增key，需要同步定义 widgets.ts/widgetKeys 常量
                     */
                    _visible: _visible, _display: _display, style: tslib_1.__assign({}, processedCommonStyle), classList: className.split(/\s/).filter(function (item) { return !!item; }), widgetType: sourceKey, _parentId: _parentId, getConfig: function () {
                        var _a;
                        return (_a = componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.compConfig) !== null && _a !== void 0 ? _a : {};
                    }, _waForKey: (_s = component.directives) === null || _s === void 0 ? void 0 : _s.key });
            }
            var processedDataBinds = tslib_1.__spreadArray([], tslib_1.__read(dataBinds), false);
            var dynamicMetaList = [
                { property: '_visible', meta: dynamicIf },
                { property: '_display', meta: dynamicDisplay },
                { property: '_waFor', meta: dynamicFor },
                { property: 'style', meta: dynamicStyle },
                { property: 'classList', meta: dynamicClassList },
            ];
            try {
                for (var dynamicMetaList_1 = tslib_1.__values(dynamicMetaList), dynamicMetaList_1_1 = dynamicMetaList_1.next(); !dynamicMetaList_1_1.done; dynamicMetaList_1_1 = dynamicMetaList_1.next()) {
                    var item = dynamicMetaList_1_1.value;
                    var property = item.property, meta = item.meta;
                    if (meta === null || meta === void 0 ? void 0 : meta.value) {
                        processedDataBinds.push({
                            type: meta.type,
                            bindDataPath: meta.value,
                            propertyPath: property,
                        });
                    }
                }
            }
            catch (e_3_1) { e_3 = { error: e_3_1 }; }
            finally {
                try {
                    if (dynamicMetaList_1_1 && !dynamicMetaList_1_1.done && (_a = dynamicMetaList_1.return)) _a.call(dynamicMetaList_1);
                }
                finally { if (e_3) throw e_3.error; }
            }
            if (processedDataBinds.length > 0) {
                compDataBinds[component.id] = generateDataBinds(ctx, processedDataBinds, {
                    isComposite: isComposite,
                    looseError: looseError,
                    key: "\u7EC4\u4EF6 ".concat(component.id),
                    useScopeWidgetData: options.useScopeWidgetData === undefined ? true : options.useScopeWidgetData,
                });
            }
        }
        component.listenerInstances = generateListnerInstances(ctx, component.listeners || [], {
            isComposite: isComposite,
            looseError: looseError,
            key: "\u7EC4\u4EF6\u4E8B\u4EF6 ".concat(component.id),
            useScopeWidgetData: options.useScopeWidgetData === undefined ? true : options.useScopeWidgetData,
        });
        delete component.listeners;
        if (!(0, common_1.isEmptyObj)(attributes)) {
            // 如果是复合组件的根节点，则补充 wrapperClass
            if (isComposite) {
                if (!((_v = (_u = (_t = component === null || component === void 0 ? void 0 : component.extra) === null || _t === void 0 ? void 0 : _t.__parent) === null || _u === void 0 ? void 0 : _u.extra) === null || _v === void 0 ? void 0 : _v.__parent)) {
                    component.attributes = {};
                    component.attributes.class = "".concat(component.attributes.class, " ").concat(wrapperClass);
                }
            }
            // 组件双向绑定
            var syncProps_1 = componentInputProps[sourceKey];
            if (syncProps_1) {
                Object.keys(syncProps_1).forEach(function (syncPropKey) {
                    var syncProp = syncProps_1[syncPropKey];
                    var syncPropArr = syncProp !== null && syncProp !== void 0 ? syncProp : [];
                    // 统一转成数组处理
                    if (!Array.isArray(syncProp)) {
                        syncPropArr = [syncProp];
                    }
                    else {
                        syncPropArr = syncPropArr;
                    }
                    syncPropArr.forEach(function (_a) {
                        // 双向绑定需要优先第一个执行，否则会出现输入框的拼音被打断的问题
                        var changeEvent = _a.changeEvent, valueFromEvent = _a.valueFromEvent;
                        function withContext(context) {
                            var instance = new index_1.runtime.Function('context', "return function({ event, forItems }) {\n                  const $for = forItems;\n                  const $index = $for.forIndexes ? $for.forIndexes[$for.forIndexes.length - 1] : undefined;\n                  const wid = ".concat(isComposite ? 'this.widgets' : 'context.$page.widgets', ".").concat(component.id, ";\n                  const widgetData = $for.forIndexes && $for.forIndexes.length > 0 ? context.get(wid, forItems.forIndexes) : wid;\n                  widgetData.").concat(syncPropKey, " = ").concat(valueFromEvent, "\n                }"))(context);
                            return ctx.this ? instance.bind(this) : instance;
                        }
                        component.listenerInstances.unshift({
                            trigger: changeEvent,
                            instanceFunction: withContext(tslib_1.__assign(tslib_1.__assign({}, ctx), { get: lodash_es_1.get })),
                        });
                    });
                });
            }
        }
    });
    return {
        widgets: compWidgets,
        dataBinds: compDataBinds,
        componentSchema: componentSchemaJson,
    };
}
exports.getComponentSchema = getComponentSchema;
// convert data binds to functions for performance & simplicity
function generateDataBinds(ctx, dataBinds, options) {
    var dataBindFuncs = {};
    var _a = (options || {}).useScopeWidgetData, useScopeWidgetData = _a === void 0 ? true : _a;
    dataBinds.forEach(function (bind) {
        var _a;
        var _b, _c;
        if (!bind.bindDataPath) {
            return console.warn('bindDataPath error', bind.propertyPath);
        }
        var databindFunction = function () { };
        var funcCode = "function (){}";
        var alias = "\nconst $for = forItems;\nconst $index = $for.forIndexes ? $for.forIndexes[$for.forIndexes.length - 1] : undefined;\n";
        try {
            if (bind.type === cals_1.PropBindType.forItem) {
                funcCode = "function(forItems){ return forItems.".concat(bind.bindDataPath, " } ");
            }
            else if (bind.type === 'scope') {
                funcCode = "function(_, __, ___, ____, $scope){ ".concat(useScopeWidgetData
                    ? 'return $scope.${bind.bindDataPath};'
                    : "return ({__type: \"scopedValue\", getValue: function($scope){ return $scope.".concat(bind.bindDataPath, "}});"), " }");
            }
            else if (bind.type === cals_1.PropBindType.expression) {
                if (options === null || options === void 0 ? void 0 : options.isComposite) {
                    funcCode = "function(forItems, $context, $w) {".concat(alias, " return (\n").concat(bind.bindDataPath.replace(/\$comp/g, 'this.$WEAPPS_COMP').replace(/;$/, '') || 'undefined', "\n)}");
                }
                else {
                    var code = bind.bindDataPath.replace(/;$/, '');
                    var content = "".concat(alias, " return (\n").concat(code || 'undefined', "\n)");
                    if ((!useScopeWidgetData && /\$scope\./.test(code)) || code === '$scope') {
                        funcCode = "(forItems, event, $context, $w, $scope) => ({__type: \"scopedValue\", getValue: ($scope) => { ".concat(content, " }})");
                    }
                    else {
                        funcCode = "(forItems, event, $context, $w, $scope) => {".concat(content, "}");
                    }
                }
            }
            else if (bind.type === cals_1.PropBindType.prop) {
                var bindDataPath = bind.bindDataPath;
                var isNegated = bindDataPath.startsWith('!');
                if (isNegated)
                    bindDataPath = bindDataPath.replace(/^!/, '');
                if (options === null || options === void 0 ? void 0 : options.isComposite) {
                    // 复合组件在预览时其实就是page，所以沿用page的变量即可
                    funcCode = "function() { return ".concat(isNegated ? '!' : '').concat('this.$WEAPPS_COMP', ".props.data.").concat(bindDataPath, " }");
                }
                else {
                    // 复合组件在预览时其实就是page，所以沿用page的变量即可
                    funcCode = "function() { return ".concat(isNegated ? '!' : '', "$page.props.data.").concat(bindDataPath, " }");
                }
            }
            else {
                var isGlobal = bind.bindDataPath.startsWith('global.');
                var returnValue = '';
                var PREFIX_MAP_1 = (_a = {},
                    _a[cals_1.PropBindType.state] = 'state',
                    _a[cals_1.PropBindType.computed] = 'computed',
                    // [PropBindType.dataVar]: 'dataVar',
                    _a[cals_1.PropBindType.stateData] = 'dataset.state',
                    _a[cals_1.PropBindType.paramData] = 'dataset.params',
                    _a);
                switch (bind.type) {
                    case cals_1.PropBindType.state:
                    case cals_1.PropBindType.computed:
                    // case PropBindType.dataVar:
                    case cals_1.PropBindType.stateData:
                    case cals_1.PropBindType.paramData: {
                        if (isGlobal) {
                            returnValue = bind.bindDataPath.replace(/^global([.[])/, function (_, $1) { return "app.".concat(PREFIX_MAP_1[bind.type]).concat($1); });
                        }
                        else {
                            if (options === null || options === void 0 ? void 0 : options.isComposite) {
                                returnValue = bind.bindDataPath
                                    .replace(/^comp-\w+([.[])/, function (_, $1) { return "this.$WEAPPS_COMP.".concat(PREFIX_MAP_1[bind.type]).concat($1); })
                                    .replace(/^\$\w+_\d+([.[])/, function (_, $1) { return "this.$WEAPPS_COMP.".concat(PREFIX_MAP_1[bind.type]).concat($1); });
                            }
                            else {
                                returnValue = bind.bindDataPath.replace(/^\w+([.[])/, function (_, $1) { return "$page.".concat(PREFIX_MAP_1[bind.type]).concat($1); });
                            }
                        }
                        funcCode = "function(){ return ".concat(returnValue, " }");
                        break;
                    }
                }
            }
            var imports = Object.keys(((_b = ctx === null || ctx === void 0 ? void 0 : ctx.app) === null || _b === void 0 ? void 0 : _b.utils) || {}).map(function (key) {
                return "const ".concat(key, " = app.utils[\"").concat(key, "\"]");
            });
            if (imports.length) {
                imports.push('');
            }
            try {
                databindFunction = new index_1.runtime.Function('context', "const { $page, app, $app } = context || {};\n          ".concat(imports.join(';'), "\n          return ").concat(funcCode))(ctx);
            }
            catch (e) {
                var errorType = e instanceof SyntaxError ? '语法错误' : '错误';
                e.message = "".concat(options.key ? "".concat(options.key, ".") : '').concat(bind.propertyPath, " \u7ED1\u5B9A").concat(errorType, ": ").concat((_c = e.message) === null || _c === void 0 ? void 0 : _c.replace(/^Unexpected token/, '非法字符'));
                throw e;
            }
            if (ctx.this) {
                databindFunction = databindFunction.bind(ctx.this);
            }
        }
        catch (e) {
            if (options === null || options === void 0 ? void 0 : options.looseError) {
                console.error('generate dynamic data error:', e);
                databindFunction = function () {
                    throw e;
                };
            }
            else {
                throw e;
            }
        }
        dataBindFuncs[bind.propertyPath] = databindFunction;
    });
    return dataBindFuncs;
}
exports.generateDataBinds = generateDataBinds;
/**
 * dataset 特殊的绑定格式
 * 要求：入参分别为 app ， page
 * 绑定只有变量，参数，表达式的绑定
 */
function generateDataBindForDataset(bind) {
    var _a;
    var expr = '';
    var type = bind.type, bindDataPath = bind.bindDataPath;
    if (type === cals_1.PropBindType.static || !type) {
        expr = typeof bindDataPath === 'string' ? "'".concat(bindDataPath, "'") : JSON.stringify(bindDataPath);
    }
    else if (type === cals_1.PropBindType.expression) {
        expr = bindDataPath;
    }
    else {
        var isGlobal = bindDataPath.startsWith('global.');
        var isCopositive = bindDataPath.startsWith('comp-') || bindDataPath.startsWith('$comp_');
        var PREFIX_MAP = (_a = {},
            _a[cals_1.PropBindType.state] = 'state',
            _a[cals_1.PropBindType.computed] = 'computed',
            _a[cals_1.PropBindType.dataVar] = 'dataVar',
            _a[cals_1.PropBindType.stateData] = 'dataset.state',
            _a[cals_1.PropBindType.paramData] = 'dataset.params',
            _a);
        switch (type) {
            case cals_1.PropBindType.state:
            case cals_1.PropBindType.computed:
            case cals_1.PropBindType.dataVar:
            case cals_1.PropBindType.stateData:
            case cals_1.PropBindType.paramData: {
                if (isCopositive) {
                    expr = bindDataPath
                        .replace(/^comp-\w+/, "$comp.".concat(PREFIX_MAP[type]))
                        .replace(/^\$comp_\w+/, "$comp.".concat(PREFIX_MAP[type]));
                }
                else {
                    expr = bindDataPath
                        .replace(/^\w+./, "".concat(isGlobal ? 'app' : '$page', ".").concat(PREFIX_MAP[type], "."))
                        .replace(/^comp-\w+/, "$comp.".concat(PREFIX_MAP[type]))
                        .replace(/^\$comp_\w+/, "$comp.".concat(PREFIX_MAP[type]));
                }
            }
        }
    }
    return new index_1.runtime.Function('app, $page', "const $app = app;\n    const imports = Object.keys(app && app.utils || {}).map(function (key) {\n      return `const ${key} = app.utils[\"${key}\"]`;\n    });\n    if(imports.length) {\n      imports.push('')\n    }\n    const body =  imports.join(';')+`return (\n".concat(expr || 'undefined', "\n)`\n   return new Function(\"\", body)()"));
}
exports.generateDataBindForDataset = generateDataBindForDataset;
function generateListnerInstances(ctx, listenerInstances, options) {
    var _this = this;
    if (options === void 0) { options = {
        isComposite: false,
        key: '',
        useScopeWidgetData: true,
    }; }
    var _a = options.isComposite, isComposite = _a === void 0 ? false : _a;
    return listenerInstances.map(function (listener) {
        var _a, _b, _c, _d, _e, _f, _g;
        var generatedListener = {
            key: listener.id,
            trigger: listener.eventName,
            isCapturePhase: !!listener.isCapturePhase,
            noPropagation: listener.noPropagation,
            instanceFunction: function () { },
            data: {},
            dataBinds: {},
        };
        if (listener.type === cals_1.ActionType.Material) {
            var _h = listener.handler || {}, name_1 = _h.name, module_1 = _h.module;
            generatedListener.instanceFunction = new index_1.runtime.Function('context', "const { $page, app, $app, actionsMap = {} } = context || {};\n        return function(...args){ return actionsMap[\"".concat(module_1, ":").concat(name_1, "\"]?.apply?.(null, args)} "))(ctx);
        }
        else if (listener.type === cals_1.ActionType.PropEvent) {
            if (isComposite) {
                var instance = new index_1.runtime.Function('{ data, event, forItems }', "\n          event.detail.data = data;\n          this.props.emit('".concat(
                // TODO: CHECK listener.handler.name 具体是什么
                listener.handler.name || listener.jsCode, "', event.detail);"));
                generatedListener.instanceFunction = ctx.this ? instance.bind(_this) : instance;
            }
            else {
                generatedListener.instanceFunction = new index_1.runtime.Function('context', "return (...args) { return context.$page.props.events.".concat(((_a = listener.handler) === null || _a === void 0 ? void 0 : _a.name) || listener.jsCode, ".apply(null, args);}"))(ctx);
            }
        }
        else if (listener.type === cals_1.ActionType.Inline) {
            // 内联 handler
            var instance = new index_1.runtime.Function('{event, data, forItems}', "return ".concat(listener.jsCode));
            if (isComposite) {
                generatedListener.instanceFunction = ctx.this ? instance.bind(_this) : instance;
            }
            else {
                generatedListener.instanceFunction = instance.bind(ctx.$page);
            }
        }
        else if (listener.type === cals_1.ActionType.Platform) {
            var name_2 = ((_b = listener.handler) === null || _b === void 0 ? void 0 : _b.name) || listener.jsCode;
            generatedListener.sourceKey = "".concat(((_c = listener.handler) === null || _c === void 0 ? void 0 : _c.module) || '', ":").concat(((_d = listener.handler) === null || _d === void 0 ? void 0 : _d.name) || '');
            generatedListener.instanceFunction = new index_1.runtime.Function('context', "return function({data}){ return context.app.".concat(name_2, "(data) } "))(ctx);
        }
        else if (listener.type === cals_1.ActionType.DataSource) {
            generatedListener.instanceFunction = new index_1.runtime.Function('context', "return function({data}) {return context.app.cloud.callDataSource(data)}")(ctx);
        }
        else {
            // Lowcode action(handler)
            if (isComposite) {
                generatedListener.instanceFunction = new index_1.runtime.Function('', "return this.handler.".concat(((_e = listener.handler) === null || _e === void 0 ? void 0 : _e.name) || listener.jsCode, ".bind(this)")).bind(ctx.this)();
            }
            else {
                var name_3 = ((_f = listener.handler) === null || _f === void 0 ? void 0 : _f.name) || listener.jsCode;
                generatedListener.instanceFunction = new index_1.runtime.Function('context', "return context.$page.handler.".concat(name_3))(ctx);
            }
        }
        var processedParams = (0, cals_1.deserializeDynamicMap)({
            page: { id: ctx.$page.id, type: cals_1.EComponentType.PAGE },
        }, (_g = listener.handler) === null || _g === void 0 ? void 0 : _g.params);
        var _j = splitDynamicMap(processedParams), staticData = _j.staticData, dataBinds = _j.dataBinds;
        generatedListener.data = staticData;
        generatedListener.dataBinds = generateDataBinds(ctx, dataBinds, tslib_1.__assign(tslib_1.__assign({}, options), { key: "".concat(options.key ? "".concat(options.key, ".") : '', "listener.").concat(listener.eventName) }));
        return generatedListener;
    });
}
//# sourceMappingURL=generate.js.map