"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resumeUrlParams = exports.saveUrlParams = exports.getCookie = exports.setCookie = void 0;
function setCookie(name, val, expiresInSecond) {
    var date = new Date();
    var value = val;
    date.setTime(date.getTime() + expiresInSecond * 1000);
    document.cookie = "".concat(name, "=").concat(value || '', "; expires=").concat(date.toUTCString(), "; path=/");
}
exports.setCookie = setCookie;
function getCookie(name) {
    var ca = document.cookie.split(';');
    var caLen = ca.length;
    var cookieName = "".concat(name, "=");
    var c;
    for (var i = 0; i < caLen; i += 1) {
        c = ca[i].replace(/^\s+/g, '');
        if (c.indexOf(cookieName) == 0) {
            return c.substring(cookieName.length);
        }
    }
    return undefined;
}
exports.getCookie = getCookie;
// 需存储参数key对照表
var SAVE_KEY_MAP = {
    envId: 'env_id',
    appId: 'app_id',
    clientId: 'client_id',
    // 登录配置版本号
    configVersion: 'config_version',
    // 回跳url
    redirectUrl: 'redirect_uri',
    // 登录来源
    source: 'source',
    // 登录态存储方式
    storageMode: 'storage_mode',
};
// 存储页面参数到cookie，缓存10分钟
function saveUrlParams(key, router) {
    var data = {};
    Object.keys(SAVE_KEY_MAP).forEach(function (key) {
        data[key] = router.getUrlParams(SAVE_KEY_MAP[key]);
    });
    setCookie(key, JSON.stringify(data), 600);
}
exports.saveUrlParams = saveUrlParams;
// 从cookie中恢复页面参数
function resumeUrlParams(key, router) {
    try {
        var data_1 = JSON.parse(getCookie(key));
        Object.keys(SAVE_KEY_MAP).forEach(function (key) {
            router.setUrlParams(SAVE_KEY_MAP[key], data_1[key]);
        });
        setCookie(key, '', -1);
    }
    catch (e) {
        console.log('无法获取原始数据', e);
    }
}
exports.resumeUrlParams = resumeUrlParams;
//# sourceMappingURL=cookies.js.map