"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkObserverInstanceRef = exports.generateWidgetAPIContext = exports.UserWidget = exports.createWidgets = exports.retryDataBinds = exports.findWidgets = exports.getDom = exports.checkDisplay = exports.checkVisible = exports.WidgetsContext = exports.WIDGET_API_MAP = exports.widgetKeys = void 0;
var tslib_1 = require("tslib");
var React = tslib_1.__importStar(require("react"));
var mobx_1 = require("mobx");
var lodash_es_1 = require("lodash-es");
var eventProxy_1 = require("./eventProxy");
var adapter_1 = require("./adapter");
var common_1 = require("./common");
var constants_1 = require("./constants");
var WIDGET_API_LIST = [
    'getConfig',
    'findWidgets',
    'getWidgetsByType',
    'extends',
    'addWidgetsListener',
    'removeWidgetsListener',
    'getOwnerWidget',
    'getDom',
    'domRef',
    '_listeners',
    '_instanceRef',
    '_getInstanceRef',
    '_scope',
    '_descendants',
    '_userWidget',
];
exports.widgetKeys = tslib_1.__spreadArray(tslib_1.__spreadArray([], tslib_1.__read(WIDGET_API_LIST), false), [
    '_key',
    'parent',
    '_visible',
    '_display',
    '_parentId',
    '_waForKey',
    '_disposers',
], false);
exports.WIDGET_API_MAP = WIDGET_API_LIST.reduce(function (map, key) {
    map[key] = true;
    return map;
}, {});
exports.WidgetsContext = React.createContext({ parent: null });
function checkVisible(_a) {
    var id = _a.id, _b = _a._visible, _visible = _b === void 0 ? undefined : _b;
    return _visible !== false && _visible !== '';
}
exports.checkVisible = checkVisible;
function checkDisplay(_a) {
    var id = _a.id, _b = _a._display, _display = _b === void 0 ? true : _b;
    return _display !== false && _display !== '';
}
exports.checkDisplay = checkDisplay;
function getDom(element, options) {
    if (!element) {
        console.warn('getDom 接口未传入有效的 element');
        return Promise.resolve({});
    }
    var result = {};
    if (options.id)
        result.id = element.id;
    if (options.dataset)
        result.dataset = element.dataset;
    if (options.rect) {
        var rect = element.getBoundingClientRect();
        Object.assign(result, {
            left: rect.left,
            right: rect.right,
            top: rect.top,
            bottom: rect.bottom,
        });
    }
    if (options.size) {
        var rect = element.getBoundingClientRect();
        Object.assign(result, {
            width: rect.width,
            height: rect.height,
        });
    }
    if (options.scrollOffset) {
        Object.assign(result, {
            scrollLeft: element.scrollLeft,
            scrollTop: element.scrollTop,
        });
    }
    if (options.properties && Array.isArray(options.properties)) {
        options.properties.forEach(function (propName) {
            result[propName] = element.getAttribute(propName);
        });
    }
    if (options.computedStyle && Array.isArray(options.computedStyle)) {
        var computedStyle_1 = adapter_1.runtime.getComputedStyle(element);
        options.computedStyle.forEach(function (propName) {
            result[propName] = computedStyle_1[propName];
        });
    }
    return Promise.resolve(result);
}
exports.getDom = getDom;
function findWidgets(widget, parentType, filterFn, includeInvisibleDescendants) {
    if (!widget)
        return [];
    var _a = widget.children, children = _a === void 0 ? [] : _a;
    var matched = children.filter(filterFn);
    // 过滤掉不可见后代
    if (!includeInvisibleDescendants) {
        children = children.filter(function (item) { return checkVisible(item); });
    }
    children.forEach(function (child) {
        // 如果递归过程中发现了自己，则停止递归
        matched = matched.concat(findWidgets(child, parentType, filterFn, includeInvisibleDescendants));
    });
    // 过滤掉不可见的匹配项
    if (!includeInvisibleDescendants) {
        matched = matched.filter(function (item) { return checkVisible(item); });
    }
    return matched;
}
exports.findWidgets = findWidgets;
var retryQueue = [];
// 递归执行
function retryDataBinds(tryTime) {
    if (tryTime === void 0) { tryTime = 10; }
    var fn = retryQueue.shift();
    if (!fn || tryTime <= 0) {
        retryQueue = [];
        return;
    }
    try {
        fn({ silent: tryTime > 1 });
    }
    catch (e) {
        console.error('retryDataBinds', e);
    }
    retryDataBinds(tryTime - 1);
}
exports.retryDataBinds = retryDataBinds;
function createWidgets(ctx, widgetProps, dataBinds, controledDataContext, widgets) {
    var _a = createWidgetTree(ctx.nodeMap, ctx.widgetProps, widgetProps, dataBinds), nodeTree = _a.nodeTree, nodeMap = _a.nodeMap;
    /**
     * 函数存在副作用
     * 因为构建过程中立即就对当前值进行了访问
     */
    var widgetsResult = createSubTree({ apiContext: ctx.apiContext, preWidgets: ctx.widgets, parentLevelWidgets: null }, nodeTree, {}, controledDataContext, widgets);
    return { widgets: widgetsResult, nodeMap: nodeMap };
    /**
     * @param {*} pre widgets
     * @param {*} curForNode
     * @param {*} forItems
     * @param {*} parentLevelWidgets
     * @returns for dispose
     */
    function runFor(apiContext, preWidgets, curForNode, forItems, parentLevelWidgets, controledDataContext) {
        var parentWidget = getNodeParentWidget(curForNode, parentLevelWidgets);
        /**
         * 组件节点
         */
        var nodeId = curForNode.id;
        var _waForKey = curForNode.value._waForKey;
        var timer = {};
        var dispose = (0, mobx_1.autorun)(function (r) {
            var forList = [];
            try {
                clearTimeout(timer[nodeId]);
                /**
                 * untrace 收集context 阶段不添加observer
                 * 消费数据阶段再添加
                 */
                var dataContext = (0, mobx_1.untracked)(function () { return (0, common_1.generateDataContext)(parentWidget); });
                // TODO: forContext
                var $scope_1 = (0, mobx_1.untracked)(function () { return generateScopeContext(parentWidget); });
                var $w = (0, mobx_1.untracked)(function () { var _a; return generateWidgetAPIContext((_a = apiContext === null || apiContext === void 0 ? void 0 : apiContext.__internal__) === null || _a === void 0 ? void 0 : _a.$w, parentWidget, forItems, $scope_1); });
                var list = dataBinds[nodeId]._waFor(forItems, undefined, dataContext, $w, $scope_1);
                if (!Array.isArray(list)) {
                    timer[nodeId] = setTimeout(function () {
                        console.warn(nodeId, 'For 循环绑定的数据并不是数组，请检查');
                    }, 1000);
                }
                else {
                    forList = list;
                }
            }
            catch (e) {
                if ((window === null || window === void 0 ? void 0 : window.__WEDA_LOG_LEVEL__) === 'debug') {
                    timer[nodeId] = setTimeout(function () {
                        console.debug("".concat(nodeId, "\u5FAA\u73AF\u7ED1\u5B9A\u9519\u8BEF"), e);
                    }, 1000);
                }
            }
            // r.trace();
            // 让 forList 进行监听
            forList.forEach(function () { });
            (0, mobx_1.untracked)(function () {
                // dispose widgets before reused instead
                // disposeWidgets(parentLevelWidgets[curForNode.id]);
                var exsitMap = forList.reduce(function (map, item, index) {
                    var cache = item === null || item === void 0 ? void 0 : item[_waForKey];
                    if (map[cache] === undefined) {
                        map[cache] = index;
                    }
                    return map;
                }, {});
                // exsitMap = {};
                var forWidgets = (parentLevelWidgets === null || parentLevelWidgets === void 0 ? void 0 : parentLevelWidgets[nodeId]) || [];
                var existingWidgetMap = {};
                var existingWidgetIndexMap = {};
                var extraWidgetsIndexMap = {};
                forWidgets.forEach(function (widget, index) {
                    var _a;
                    if (exsitMap[widget._key] !== undefined) {
                        existingWidgetMap[widget._key] = { index: exsitMap[widget._key], widgets: (_a = {}, _a[widget.id] = widget, _a) };
                        /**
                         * 此处依赖了 existingWidgetMap[widget._key].widgets 的引用，
                         * 为了直接可以通过 index 访问到 existingWidgetMap 里的值进行编辑
                         * 但是依赖引用关系，存在维护风险
                         */
                        existingWidgetIndexMap[index] = existingWidgetMap[widget._key].widgets;
                        // need to use uqique key
                        exsitMap[widget._key] = undefined;
                    }
                    else {
                        extraWidgetsIndexMap[index] = widget;
                    }
                });
                // clean nodes of previouse for run
                dfsTree(curForNode, function (node) {
                    var arr = parentLevelWidgets[node.id] || [];
                    /**
                     * clone 上次 for 已有的 widgets
                     */
                    if (node.id !== curForNode.id) {
                        arr.forEach(function (item, index) {
                            if (existingWidgetIndexMap[index]) {
                                existingWidgetIndexMap[index][node.id] = item;
                            }
                        });
                    }
                    /**
                     * 重头开始生成
                     * 清空原有 arr 并保持引用不变
                     */
                    arr.splice(0, arr.length);
                }, undefined);
                /**
                 * 明确已经不会复用的节点，清除 mobx observer
                 * 并递归清理子节点
                 */
                for (var index in extraWidgetsIndexMap) {
                    disposeWidgets([extraWidgetsIndexMap[index]]);
                }
                if (parentWidget) {
                    (0, lodash_es_1.remove)(parentWidget.children, function (_a) {
                        var id = _a.id;
                        return id === nodeId;
                    });
                }
                var isInRepeaterChild = (parentWidget === null || parentWidget === void 0 ? void 0 : parentWidget.widgetType) === "".concat(constants_1.REPEATER.MODULE_NAME, ":").concat(constants_1.REPEATER.REPEATER_NAME) ||
                    (parentWidget === null || parentWidget === void 0 ? void 0 : parentWidget.widgetType) === "".concat(constants_1.REPEATER.DEBUG_MODULE_NAME, ":").concat(constants_1.REPEATER.REPEATER_NAME);
                forList.forEach(function (item, index) {
                    var _a, _b;
                    var forContextListAlias;
                    var _c = forItems.forStack, forStack = _c === void 0 ? [] : _c;
                    var listMeta = { currentItem: item, currentIndex: index };
                    if (isInRepeaterChild) {
                        forContextListAlias = (_a = {},
                            _a["".concat(parentWidget.forIndex) || 'currentIndex'] = listMeta.currentIndex,
                            _a["".concat(parentWidget.forItem) || 'currentItem'] = listMeta.currentItem,
                            _a);
                    }
                    var subForItems = tslib_1.__assign(tslib_1.__assign({}, forItems), (_b = {}, _b[nodeId] = item, _b.forIndexes = tslib_1.__spreadArray([], tslib_1.__read(((forItems === null || forItems === void 0 ? void 0 : forItems.forIndexes) || [])), false).concat(listMeta.currentIndex), _b.forStack = tslib_1.__spreadArray(tslib_1.__spreadArray([], tslib_1.__read(forStack), false), [
                        tslib_1.__assign(tslib_1.__assign({}, listMeta), { alias: forContextListAlias }),
                    ], false), _b));
                    createSubTree({ apiContext: apiContext, preWidgets: preWidgets, parentLevelWidgets: parentLevelWidgets, existingWidgetMap: existingWidgetMap }, curForNode, subForItems, controledDataContext);
                });
                // 非初始化时遇到需要重新构建 dataBinds
                retryDataBinds();
            });
        });
        return dispose;
    }
    function createSubTree(ctx, curForNode, subForItems, controledDataContext, widgets) {
        if (widgets === void 0) { widgets = { _disposers: [] }; }
        var apiContext = ctx.apiContext, preWidgets = ctx.preWidgets, parentLevelWidgets = ctx.parentLevelWidgets, _a = ctx.existingWidgetMap, existingWidgetMap = _a === void 0 ? {} : _a;
        var parentWidget = getNodeParentWidget(curForNode, parentLevelWidgets);
        /**
         * 因为 构建 widgets 过程中对 $page.widgets 进行了访问，需要使用副作用的方式，保持 $page.widgets 引用
         */
        // const widgets = { _disposers: [] };
        if (!widgets._disposers) {
            widgets._disposers = [];
        }
        var _b = subForItems.forIndexes, forIndexes = _b === void 0 ? [] : _b;
        var currentIndex = forIndexes[(forIndexes === null || forIndexes === void 0 ? void 0 : forIndexes.length) - 1];
        // traverse down the tree to set all widgets
        dfsTree(curForNode, function (node, parentNode, cache) {
            var e_1, _a;
            var _b, _c, _d;
            var parentForWidgetArr = (parentLevelWidgets === null || parentLevelWidgets === void 0 ? void 0 : parentLevelWidgets[node.id]) || [];
            var _waForKey = node.value._waForKey;
            var key = (_b = subForItems[node.id]) === null || _b === void 0 ? void 0 : _b[_waForKey];
            var forExistingWidgetMap = {};
            var forExsitWidget;
            /**
             * for 起始节点，根据 existingWidgetMap 判断复用
             */
            if (node.id === curForNode.id) {
                if (existingWidgetMap[key] && existingWidgetMap[key].index === currentIndex) {
                    forExistingWidgetMap = existingWidgetMap[key].widgets || {};
                    forExsitWidget = forExistingWidgetMap[node.id];
                    if (forExsitWidget) {
                        cache[node.id] = {
                            widgets: forExistingWidgetMap,
                        };
                    }
                }
            }
            else if (cache[parentNode === null || parentNode === void 0 ? void 0 : parentNode.id]) {
                forExsitWidget = cache[parentNode === null || parentNode === void 0 ? void 0 : parentNode.id].widgets[node.id] || null;
                if (forExsitWidget) {
                    cache[node.id] = cache[parentNode === null || parentNode === void 0 ? void 0 : parentNode.id];
                }
            }
            var preExistWidget = node.exist
                ? Array.isArray(preWidgets === null || preWidgets === void 0 ? void 0 : preWidgets[node.id])
                    ? (0, lodash_es_1.get)(preWidgets === null || preWidgets === void 0 ? void 0 : preWidgets[node.id], subForItems.forIndexes)
                    : preWidgets === null || preWidgets === void 0 ? void 0 : preWidgets[node.id]
                : null;
            var existedWidget = forExsitWidget || preExistWidget || null;
            if (node.forCount === curForNode.forCount) {
                /**
                 * 同一层循环作用域内，当前节点与 curForNode（循环根节点）在同一级循环作用域中
                 * 即没有再开辟新级别的 for 循环
                 * Leaf node
                 */
                var w_1 = existedWidget;
                if (!existedWidget) {
                    w_1 = (0, mobx_1.observable)(tslib_1.__assign(tslib_1.__assign({}, node.value), { id: node.id, 
                        /**
                         * TODO: parent children 的维护，
                         * widget 挂载/卸载 要修改对应的 _descendants
                         */
                        parent: null, children: [], _key: key, _scope: {
                            id: node.id,
                            dataContext: controledDataContext[node.id] || undefined,
                            forContext: subForItems[node.id],
                        }, _disposers: [], _listeners: new eventProxy_1.EventProxy(), _descendants: {} }));
                    // 提供一个给 Node 挂载 API 的方式
                    (0, mobx_1.untracked)(function () {
                        if (!w_1.getConfig) {
                            // 兼容非复合组件调用 getConfig 能力
                            w_1.getConfig = function () { return ({}); };
                        }
                        w_1.findWidgets = function (type, includeInvisibleDescendants) {
                            return findWidgets(w_1, w_1.widgetType, type, includeInvisibleDescendants);
                        };
                        w_1.getWidgetsByType = function (type, includeInvisibleDescendants) {
                            return w_1.findWidgets(function (currentWidget) { return currentWidget.widgetType === type; }, includeInvisibleDescendants);
                        };
                        w_1.extends = function (name, fnOrData) {
                            return Object.defineProperty(w_1, name, {
                                value: fnOrData,
                                writable: true,
                            });
                        };
                        w_1.addWidgetsListener = function (name, listener) { return w_1._listeners.on(name, listener); };
                        w_1.removeWidgetsListener = function (name, listener) { return w_1._listeners.cancel(name, listener); };
                        w_1.getOwnerWidget = function () { return null; }; // 默认带上一个 widget
                        // eslint-disable-next-line react/display-name
                        var instanceRef = React.createRef();
                        if (checkObserverInstanceRef(w_1.widgetType)) {
                            w_1._instanceRef = instanceRef;
                        }
                        else {
                            Object.defineProperty(w_1, '_instanceRef', {
                                get: function () {
                                    return instanceRef;
                                },
                            });
                        }
                        w_1._getInstanceRef = function () { return w_1._instanceRef; }; // 默认初始值
                        Object.defineProperty(w_1, '_methods', {
                            get: function () {
                                var _a;
                                var instance = this._getInstanceRef();
                                return (_a = instance === null || instance === void 0 ? void 0 : instance.current) === null || _a === void 0 ? void 0 : _a.methods;
                            },
                        });
                        /**
                         * UserWidget
                         */
                        w_1._userWidget = new UserWidget(w_1);
                        /**
                         * 该 widget 为 Repeater 组件
                         */
                        switch (w_1.widgetType) {
                            case "".concat(constants_1.REPEATER.MODULE_NAME, ":").concat(constants_1.REPEATER.REPEATER_NAME):
                            case "".concat(constants_1.REPEATER.DEBUG_MODULE_NAME, ":").concat(constants_1.REPEATER.REPEATER_NAME): {
                                // Repeater.items 代理
                                if (!w_1.items) {
                                    Object.defineProperty(w_1, 'items', {
                                        get: function () {
                                            return (w_1.children || []).map(function (item) {
                                                var descendants = {};
                                                Object.keys((item === null || item === void 0 ? void 0 : item._descendants) || {}).forEach(function (key) {
                                                    descendants[key] = item._descendants[key]._userWidget;
                                                });
                                                return descendants;
                                            });
                                        },
                                    });
                                }
                            }
                        }
                    });
                }
                else {
                    if (!forExsitWidget && preExistWidget) {
                        if (node.changed) {
                            try {
                                for (var _e = tslib_1.__values(node.changed), _f = _e.next(); !_f.done; _f = _e.next()) {
                                    var key_1 = _f.value;
                                    w_1[key_1] = node.value[key_1];
                                }
                            }
                            catch (e_1_1) { e_1 = { error: e_1_1 }; }
                            finally {
                                try {
                                    if (_f && !_f.done && (_a = _e.return)) _a.call(_e);
                                }
                                finally { if (e_1) throw e_1.error; }
                            }
                        }
                        /**
                         * 复用
                         * 组件 tree 发生变化时，叶子节点 children 进行一次清理
                         */
                        if (!((_c = node.children) === null || _c === void 0 ? void 0 : _c.length)) {
                            w_1.children = [];
                        }
                        // 仅仅为了受控模式下更新 dataContext
                        if (controledDataContext === null || controledDataContext === void 0 ? void 0 : controledDataContext[w_1.id]) {
                            w_1._scope.dataContext = controledDataContext[w_1.id];
                        }
                    }
                    disposeWidgets([existedWidget], true);
                }
                /**
                 * 1. 正常新建widgets，父亲记录一下
                 * 2. for 循环清理了 curForNode.parent.children，此处若children 复用，则继承一下
                 */
                if (!existedWidget || (node.id === curForNode.id && !forExsitWidget && preExistWidget)) {
                    var parent_1 = parentNode ? widgets[parentNode.id] : parentWidget;
                    if (parent_1) {
                        w_1.parent = parent_1;
                        // 只有可显示 visible 的才存入 children 里
                        if (checkVisible(w_1)) {
                            parent_1.children.push(w_1);
                        }
                    }
                }
                (_d = parentForWidgetArr.push) === null || _d === void 0 ? void 0 : _d.call(parentForWidgetArr, w_1);
                widgets[node.id] = w_1;
                /**
                 * TODO: _descendants map 的维护，widget 挂载/卸载 要修改对应的 _descendants
                 */
                if (widgets === null || widgets === void 0 ? void 0 : widgets[node._ancestorId]) {
                    // 在虚拟项 RepeaterItem 下挂载所有的子孙 _descendants
                    // 这里不要用 lodashSet，否则会出现刷新几次，出现 Error: [mobx.array] Index out of bounds 的错误，原因暂不明
                    widgets[node._ancestorId]._descendants = widgets[node._ancestorId]._descendants || {};
                    widgets[node._ancestorId]._descendants[node.id] = widgets[node.id];
                }
                // Setup data binds
                Object.keys(dataBinds[node.id] || {})
                    .sort(function (a, b) {
                    return a.length - b.length > 0 ? 1 : -1;
                })
                    .map(function (prop) {
                    if (prop === '_waFor') {
                        return;
                    }
                    var timer = null;
                    function getBindData(curForNodeId, options) {
                        if (options === void 0) { options = { silent: false }; }
                        var disposeError = false;
                        var dispose = (0, mobx_1.autorun)(function (reaction) {
                            // reaction.trace();
                            try {
                                clearTimeout(timer);
                                /**
                                 * untrace 收集context 阶段不添加observer
                                 * 消费数据阶段再添加
                                 */
                                var dataContext = (0, mobx_1.untracked)(function () { return (0, common_1.generateDataContext)(w_1); });
                                var $scope_2 = (0, mobx_1.untracked)(function () { return generateScopeContext(w_1); });
                                var $w = (0, mobx_1.untracked)(function () { var _a; return generateWidgetAPIContext((_a = apiContext === null || apiContext === void 0 ? void 0 : apiContext.__internal__) === null || _a === void 0 ? void 0 : _a.$w, w_1, subForItems, $scope_2); });
                                // Computed data bind in the next tick since data bind may read widgets data
                                var value_1 = dataBinds[node.id][prop](subForItems, undefined, dataContext, $w, $scope_2);
                                if ((prop === '_visible' || prop === '_display') && value_1 === undefined) {
                                    value_1 = false;
                                }
                                var paths_1 = prop.split('.').filter(function (key) { return !!key; });
                                if (paths_1.length > 1) {
                                    // 一定要 untracked 不然爆栈了
                                    (0, mobx_1.untracked)(function () {
                                        var pre = (0, lodash_es_1.get)(w_1, prop);
                                        (0, lodash_es_1.set)(w_1, prop, value_1);
                                        /**
                                         * 深层的对象内部值变化不会触发 obverver，因此浅拷贝对象
                                         */
                                        if (Array.isArray(w_1[paths_1[0]])) {
                                            if (!(0, lodash_es_1.isEqual)(pre, value_1)) {
                                                w_1[paths_1[0]] = tslib_1.__spreadArray([], tslib_1.__read(w_1[paths_1[0]]), false);
                                            }
                                        }
                                        else if ((0, common_1.isPlainObject)(w_1[paths_1[0]])) {
                                            if (!(0, lodash_es_1.isEqual)(pre, value_1)) {
                                                w_1[paths_1[0]] = tslib_1.__assign({}, w_1[paths_1[0]]);
                                            }
                                        }
                                    });
                                }
                                else {
                                    // 普通 key 直接赋值
                                    w_1[prop] = value_1;
                                }
                                disposeError = false;
                            }
                            catch (e) {
                                if (prop === '_visible' || prop === '_display') {
                                    w_1[prop] = false;
                                }
                                else {
                                    retryQueue.push(function (opts) {
                                        reaction.dispose();
                                        return getBindData(curForNodeId, opts);
                                    });
                                    disposeError = true;
                                }
                                if (!(options === null || options === void 0 ? void 0 : options.silent)) {
                                    if ((window === null || window === void 0 ? void 0 : window.__WEDA_LOG_LEVEL__) === 'debug') {
                                        timer = setTimeout(function () {
                                            console.debug("\u6570\u636E\u7ED1\u5B9A\u8BA1\u7B97\u9519\u8BEF ".concat(w_1.id, ".").concat(prop), e);
                                            timer = null;
                                        }, 1000);
                                    }
                                }
                            }
                        });
                        if (!disposeError) {
                            if (curForNodeId) {
                                widgets[curForNode.id]._disposers.push(dispose);
                            }
                            else {
                                widgets._disposers.push(dispose);
                            }
                        }
                    }
                    getBindData(curForNode.id);
                });
            }
            else {
                if (existedWidget && forExsitWidget) {
                    widgets[node.id] = existedWidget;
                }
                else {
                    widgets[node.id] = (0, mobx_1.observable)([]);
                }
                if (parentLevelWidgets) {
                    parentLevelWidgets[node.id].push(widgets[node.id]);
                }
            }
        }, undefined);
        /**
         * run for of next level
         * 此处本质上再递归调用多级 For 生成 tree，
         * dfs 前(enter) 保证本级 widgets 建立完成，下一级 For 绑定都为空数组
         * dfs 中 递归下级 For 来向空数组中 push 值
         * dfs 后(exit) 本级所有的 widgets 都已经建立完成可以使用
         * 但当 root -> A -> B 三层结构时发现顺序为
         * root enter -> A enter -> A exit -> B enter -> B exit -> root exit
         * 不符合递归处理期望
         * 查到进展为：dfs B 时 runfor autorun 直接返回了值，然后 callback 才会再触发，导致先走到了 A 的退出逻辑
         * autorun 收 mobx Reaction 调度， 根据 autorun 的逻辑，最终 reaction.getDisposer()，dispose 里会 startBath 可能和此处有关
         */
        dfsTree(curForNode, function (node, parentNode) {
            if (node.forCount === curForNode.forCount + 1 && dataBinds[node.id] && dataBinds[node.id]._waFor) {
                /**
                 * node 上有 for 绑定，widgets[node.id] 为数组，不应对数组赋值
                 * 此外 _disposers 都为数组类型，非对象类型
                 * orgin: widgets[node.id]._disposers = { dataBinds: [] };
                 */
                var dispose = runFor(apiContext, preWidgets, node, subForItems, widgets, controledDataContext);
                curForNode.id && widgets[curForNode.id]._disposers.push(dispose);
            }
        }, undefined);
        return widgets;
    }
}
exports.createWidgets = createWidgets;
function getNodeParentWidget(node, widgets) {
    var _a;
    return ((_a = node === null || node === void 0 ? void 0 : node.parent) === null || _a === void 0 ? void 0 : _a.id) && (widgets === null || widgets === void 0 ? void 0 : widgets[node.parent.id]);
}
function createWidgetTree(preNodes, preWidgets, widgets, dataBinds) {
    var virtualRoot = { children: [], forCount: 0 };
    var nodes = Object.keys(widgets).reduce(function (result, id) {
        var currentValue = widgets[id];
        result[id] = {
            id: id,
            value: currentValue,
            children: [],
            parent: null,
            forCount: 0,
            exist: false,
            changed: [],
        };
        return result;
    }, {});
    // Create widgets tree API
    Object.keys(nodes).map(function (id) {
        var curNode = nodes[id];
        var parent = nodes[widgets[id]._parentId];
        if (!parent) {
            virtualRoot.children.push(curNode);
            return;
        }
        curNode.parent = parent;
        parent.children.push(curNode);
    });
    virtualRoot.children.map(addForCount);
    // dfs, add forCount
    function addForCount(node) {
        var _a;
        if (node.parent) {
            node.forCount = node.parent.forCount;
            var widgetType = node.parent.value.widgetType;
            if (widgetType === "".concat(constants_1.REPEATER.MODULE_NAME, ":").concat(constants_1.REPEATER.REPEATER_ITEM_NAME) ||
                widgetType === "".concat(constants_1.REPEATER.DEBUG_MODULE_NAME, ":").concat(constants_1.REPEATER.REPEATER_ITEM_NAME)) {
                // 从 Repeater 的循环虚拟项下的结点开始，结点的循环祖先是循环虚拟项
                node._ancestorId = node.parent.id;
            }
            if (node.parent._ancestorId) {
                // Repeater 作用域内的所有子孙（排除里面的 Repeater 组件，它本身又产生深一层的作用域）继承父级的循环信息
                if (!node._ancestorId) {
                    node._ancestorId = node.parent._ancestorId;
                }
            }
        }
        if ((_a = dataBinds[node.id]) === null || _a === void 0 ? void 0 : _a._waFor) {
            node.forCount += 1;
        }
        node.children.map(addForCount);
    }
    dfsTree(virtualRoot, function (node, parent) {
        var currentValue = node.value;
        var preValue = preWidgets === null || preWidgets === void 0 ? void 0 : preWidgets[node.id];
        var preNode = preNodes === null || preNodes === void 0 ? void 0 : preNodes[node.id];
        var exist = false;
        var changed = [];
        // 多级for绑定的情况会发生父for不变化，子for更新的情况，此时子for生成会有异常，因此复用只复用一层保证渲染正常
        // https://lcap.cloud.tencent.com/preview/?u=https%3A%2F%2Fci-source-1303824488.cos.ap-shanghai.myqcloud.com%2Flcp%2Fapp-h057yqaw%2Fappdata-1666253755745.json&appId=app-h057yqaw#/nav
        if ((!parent || (parent === null || parent === void 0 ? void 0 : parent.exist)) && preValue && preNode) {
            if (preValue.widgetType === (currentValue === null || currentValue === void 0 ? void 0 : currentValue.widgetType) &&
                preValue._parentId === currentValue._parentId &&
                preNode.forCount === node.forCount) {
                exist = true;
                for (var key in currentValue) {
                    if (!exports.WIDGET_API_MAP[key] && !(0, lodash_es_1.isEqual)(preValue[key], currentValue[key])) {
                        changed.push(key);
                    }
                }
            }
            node.exist = exist;
            node.changed = changed;
        }
    }, undefined);
    return { nodeTree: virtualRoot, nodeMap: nodes };
}
function dfsTree(node, fn, parent, cache, scope) {
    if (cache === void 0) { cache = {}; }
    if (scope === void 0) { scope = { dataContext: {} }; }
    if (node.value) {
        fn(node, parent, cache, scope);
    }
    node.children.map(function (e) { return dfsTree(e, fn, node.value ? node : null, cache, scope); });
}
/**
 * dispose autorun
 * TODO: 是否要同步清理 parent children 引用，防止内存泄漏
 */
function disposeWidgets(widgets, noRecursive) {
    if (widgets === void 0) { widgets = []; }
    if (noRecursive === void 0) { noRecursive = false; }
    widgets.forEach(function (widget) {
        /**
         * 解除 parent 依赖
         */
        // widget.parent = null;
        var disposers = widget._disposers;
        if (disposers) {
            disposers.map(function (dispose) { return dispose(); });
            disposers.splice(0, disposers.length);
        }
        if (!noRecursive) {
            disposeWidgets(widget.children);
            /**
             * 解除children 的引用
             */
            // widget.children = [];
        }
    });
}
/**
 * 对外（用户）的 Widget
 */
var UserWidget = /** @class */ (function () {
    function UserWidget(widget) {
        this._widget = null;
        this._widget = widget;
        return new Proxy(this, {
            get: function (target, prop) {
                if (prop in target) {
                    return target[prop];
                }
                var custom = target.custom;
                /**
                 * @important
                 * 优先 custom
                 * 会出现一些 target 时序还不存在的情况
                 * 因此 custom[prop] 先添加进依赖
                 */
                if ((custom === null || custom === void 0 ? void 0 : custom[prop]) && prop in custom) {
                    return custom[prop];
                }
                // 再尝试内置的方法和属性
                if (prop in target.sys) {
                    return target.sys[prop];
                }
                return undefined;
            },
            /**
             * userwidget 不可写
             */
            set: function () {
                return false;
            },
        });
    }
    Object.defineProperty(UserWidget.prototype, "description", {
        get: function () {
            var id = this._widget.id;
            return "\n    \u4F7F\u7528\u8BF4\u660E\uFF1A\n    1. sys \u547D\u540D\u7A7A\u95F4\u4E0B\u4E3A\u5185\u7F6E\u65B9\u6CD5\uFF0C\u53EF\u901A\u8FC7 $w.".concat(id, ".sys.<\u6210\u5458> \u6216 $w.").concat(id, ".<\u6210\u5458> \u8BBF\u95EE\u3002\n       \u8BBF\u95EE id \u793A\u4F8B: $w.").concat(id, ".id \u6216 $w.").concat(id, ".sys.id\n    2. custom \u547D\u540D\u7A7A\u95F4\u4E0B\u4E3A\u7528\u6237\u81EA\u5B9A\u4E49\u5BF9\u5916\u66B4\u9732\u7684\u6210\u5458\uFF0C\u53EF\u901A\u8FC7 $w.").concat(id, ".custom.<\u6210\u5458> \u6216 $w.").concat(id, ".<\u6210\u5458> \u8BBF\u95EE\u3002\n       \u8BBF\u95EE\u81EA\u5B9A\u4E49\u65B9\u6CD5 hello \u793A\u4F8B: $w.").concat(id, ".hello() \u6216 $w.").concat(id, ".custom.hello()\n    3. \u901A\u8FC7 $w.").concat(id, ".<\u6210\u5458> \u8BBF\u95EE\u65F6\uFF0C\u5982\u679C\u81EA\u5B9A\u4E49\u6210\u5458\u4E0E\u5185\u7F6E\u6210\u5458\u91CD\u540D\uFF0C\u5219\u81EA\u5B9A\u4E49\u6210\u5458\u8986\u76D6\u5185\u7F6E\u6210\u5458\u3002\n    4. [\u6CE8\u610F]: \u8BF7\u4E0D\u8981\u76F4\u63A5\u8BBF\u95EE _widget\uFF0C\u5B83\u91CC\u9762\u7684\u6210\u5458\u4E3A\u5185\u90E8\u5B9E\u73B0\uFF0C\u968F\u65F6\u53EF\u80FD\u8FDB\u884C\u8C03\u6574\n    ");
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UserWidget.prototype, "sys", {
        // 内置属性和方法命名空间
        get: function () {
            var _a, _b;
            var widget = this._widget;
            var _c = tslib_1.__read(((_b = (_a = widget.widgetType) === null || _a === void 0 ? void 0 : _a.split) === null || _b === void 0 ? void 0 : _b.call(_a, ':')) || [], 2), module = _c[0], component = _c[1];
            return {
                /**
                 * 内置属性
                 */
                get id() {
                    return widget.id;
                },
                get module() {
                    return module;
                },
                get component() {
                    return component;
                },
                get name() {
                    return widget.widgetType;
                },
                get parent() {
                    var _a;
                    return (_a = widget.parent) === null || _a === void 0 ? void 0 : _a._userWidget;
                },
                get children() {
                    var _a;
                    return ((_a = widget.children) === null || _a === void 0 ? void 0 : _a.map(function (item) { return item._userWidget; })) || [];
                },
                /**
                 * 内置方法
                 */
                closest: function (filter) {
                    var parent = this.parent;
                    if (!filter)
                        return parent;
                    while (parent) {
                        var matched = filter(parent);
                        if (matched)
                            return parent;
                        parent = parent.parent;
                    }
                    return null;
                },
            };
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UserWidget.prototype, "custom", {
        get: function () {
            var _this = this;
            var userCustomMember = (0, mobx_1.untracked)(function () {
                var _a, _b;
                var instanceRef = (_b = (_a = _this._widget)._getInstanceRef) === null || _b === void 0 ? void 0 : _b.call(_a);
                var _c = (instanceRef === null || instanceRef === void 0 ? void 0 : instanceRef.current) || {}, _d = _c.methods, methods = _d === void 0 ? {} : _d, restInstance = tslib_1.__rest(_c, ["methods"]);
                var widget = _this._widget;
                var proxyTarget = tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, widget._methods), restInstance), methods);
                return new Proxy(proxyTarget, {
                    get: function (_, prop) {
                        var _a, _b;
                        if (prop in methods) {
                            return methods[prop];
                        }
                        /**
                         * 此处要保持从 ref 开始访问
                         * 因为proxy 内部如果直接从 ref.current 开始访问，刚开始是可能遇到的是空值，并且空对象兜底
                         * 那么此处实际上监听的即为兜底的空对象，非 observer 对象
                         */
                        if (prop !== 'methods' && prop in (instanceRef.current || {})) {
                            return (_a = instanceRef.current) === null || _a === void 0 ? void 0 : _a[prop];
                        }
                        return (_b = widget === null || widget === void 0 ? void 0 : widget._methods) === null || _b === void 0 ? void 0 : _b[prop];
                    },
                });
            });
            return userCustomMember;
        },
        enumerable: false,
        configurable: true
    });
    return UserWidget;
}());
exports.UserWidget = UserWidget;
function generateWidgetAPIContext($w, widget, forContext, scopeContext) {
    if ($w === void 0) { $w = {}; }
    if (scopeContext === void 0) { scopeContext = {}; }
    return new Proxy($w, {
        get: function (target, prop) {
            // 尝试代理 currentItem 和 currentIndex
            // 用 slice 确保 forStack 的长度不会超过 forIndexes
            var _a = forContext.forStack, forStack = _a === void 0 ? [] : _a;
            /**
             * for context 变量优先
             * 从后往前找，即最深层循环优先
             */
            for (var i = forStack.length - 1; i >= 0; i--) {
                var meta = forStack[i];
                var map = meta.alias || {};
                if (prop in map) {
                    return map[prop];
                }
            }
            var res = (function findWidgets() {
                var _a;
                // 尝试代理同级 widget
                if (widget) {
                    var parent_2 = widget.parent;
                    while (parent_2) {
                        if ((_a = parent_2._descendants) === null || _a === void 0 ? void 0 : _a[prop]) {
                            return parent_2._descendants[prop]._userWidget;
                        }
                        parent_2 = parent_2.parent;
                    }
                }
                // 尝试代理到全局的 $w
                // 这里如果不加 $page.widgets[prop] 会因为 window 还没有 $page 而发生错误
                return target[prop];
                // return ($page.widgets[prop] && $page.widgets[prop]._userWidget) || window.$w[prop];
            })();
            /**
             * 如果是作用域插槽，则合并插槽动态访问 与 widget 静态访问
             * 否则只包含 widget
             */
            var id = prop;
            var scope = (0, mobx_1.untracked)(function () { return scopeContext[id]; });
            return !scope
                ? res
                : new Proxy(res, {
                    get: function (target, prop) {
                        if (prop in scope) {
                            /**
                             * 一定要访问原始值，不然就 observer 不到
                             */
                            return scopeContext[id][prop];
                        }
                        return target[prop];
                    },
                });
        },
    });
}
exports.generateWidgetAPIContext = generateWidgetAPIContext;
function generateScopeContext(widget) {
    var _a;
    var scopeContext = {};
    var _loop_1 = function () {
        var current = widget;
        var id = current._scope.id;
        if (id.includes('.')) {
            var _b = tslib_1.__read(id.split('.'), 2), componentId = _b[0], slotName_1 = _b[1];
            Object.defineProperty(scopeContext, componentId, {
                get: function () {
                    var _a;
                    return _a = {}, _a[slotName_1] = current._scope.scopeContext, _a;
                },
            });
            widget = (_a = widget.parent) === null || _a === void 0 ? void 0 : _a.parent;
        }
        else {
            // 此处采用代理的方式，是为了可以获取到最新的 _scope.dataContext 防止 dataContext 引用被重新赋值
            Object.defineProperty(scopeContext, id, {
                get: function () {
                    return current._scope.scopeContext;
                },
            });
            widget = widget.parent;
        }
    };
    while (widget === null || widget === void 0 ? void 0 : widget._scope) {
        _loop_1();
    }
    return scopeContext;
}
function checkObserverInstanceRef(sourceKey) {
    if (sourceKey === void 0) { sourceKey = ''; }
    return !(sourceKey === null || sourceKey === void 0 ? void 0 : sourceKey.startsWith('CLOUDBASE_BUSSINESS')) && common_1.ENABLE_OBSERVER_REF;
}
exports.checkObserverInstanceRef = checkObserverInstanceRef;
//# sourceMappingURL=widgets.js.map