import sdk from '@tcwd/weapps-sdk/lib/app-h5-sdk';

import {
  createWebApp,
  IWebAppAPI,
  _ACTIONS_KEY as ACTIONS_KEY,
  _ROUTER_KEY as ROUTER_KEY,
  _WEDA_CLOUD_SDK as WEDA_CLOUD_SDK,
} from '@cloudbase/weda-client';
import { runtime } from '../utils/adapter/index';
import { getEnumOptions } from '../utils/initGlobalVar';
import { get as lodashGet } from 'lodash-es';
import { OFFICIAL_LIB_KEY } from '@cloudbase/cals';
import { observable } from 'mobx';

interface IBaseState {
  state: {
    [key: string]: any;
  };
  computed: {
    [key: string]: any;
  };
  dataset?: {
    state?: {};
  };
}

enum EPLATFORM {
  WEB = 'WEB',
  /**
   * 拼写错误
   * 兼容性无法修改
   */
  MINIPROGRAM = 'MINIPROGRAME',
}

const DEFAULT_INIT_DATA = {
  processAction: (key, action) => {
    switch (key) {
      case ACTIONS_KEY.showModal: {
        return function (obj) {
          const showModal = runtime[`@weapps-materials-main-${OFFICIAL_LIB_KEY}`]?.actions?.showModal;
          const isMobile = (app?.__internal__ as any)?.isMobile;
          if (isMobile && !isMobile() && showModal) {
            return showModal({ data: obj, _origin: action });
          }
          return action(obj);
        };
      }
      case ACTIONS_KEY.showToast: {
        return function (obj) {
          const showMessage = runtime[`@weapps-materials-main-${OFFICIAL_LIB_KEY}`]?.actions?.showMessage;
          if (showMessage) {
            return showMessage({ data: obj, _origin: action });
          }
          return action(obj);
        };
      }
    }
    return action;
  },
};

export const subPackageName = '';
export let app = createGlboalApi(DEFAULT_INIT_DATA);
export const $w = new Proxy(app.__internal__.$w, {
  get(_, prop: string) {
    return app.__internal__.$w?.[prop];
  },
});

export function registerApp(_app = createGlboalApi(DEFAULT_INIT_DATA)) {
  _injectApp2Runtime(_app, true);
  app = _app;
}

mountAPIs(sdk);
export interface IAppAPI extends IWebAppAPI, IBaseState {
  domain: string;
  app?: IAppAPI;
  $app?: IAppAPI;
  mainApp?: IAppAPI;
  common: {};
  enumOptions?: {};
  /**
   * 内置方法
   */
  setState: (state) => void;
  invoke: (params) => void;
  utils: IWebAppAPI['utils'] & {
    formatEnum: (path, optionname, app) => string;
  };
}

function createGlboalApi(
  initData: {
    id?: string;
    domain?: string;
    appConfig?: {
      id?: string;
      envId?: string;
      envVersion?: 'preview' | 'production';
      staticResourceDomain?: string;
      pages: { id: string }[];
    };
    processAction?: (key, action) => any;
  } = {},
): IAppAPI {
  const webApp = createWebApp(initData);
  const globalAPI: IAppAPI = new Proxy(
    Object.assign(webApp, {
      domain: initData.domain || '',
      state: {},
      computed: {},
      dataset: {},
      common: {},
      enumOptions: {},
      // i18n,
      ...webApp,
      utils: {
        ...webApp.utils,
        formatEnum: (path, optionname, app) => {
          let isSingle = Array.isArray(path);
          // 这个app是后来添加进去的
          let options = app?.enumOptions;
          let parseOptions = options[optionname] ? options[optionname] : getEnumOptions(optionname);
          if (parseOptions === '') {
            return !isSingle ? path : path.join(',');
          }
          let multiTmp = [];
          let value = !isSingle
            ? JSON.parse(parseOptions)?.find((item) => item.key === path)?.value
            : JSON.parse(parseOptions)
                ?.filter((item) => path.some((pathValue) => item.key === pathValue))
                .map((item) => multiTmp.push(item.value));
          return !isSingle ? value : multiTmp.join(',');
        },
      },
      /**
       * setState
       */
      setState: (userSetState) => {
        Object.keys(userSetState).forEach((keyPath) => {
          globalAPI.utils.set(globalAPI.dataset.state, keyPath, userSetState[keyPath]);
        });
      },
      invoke(params) {
        const $page = globalAPI.utils.getCurrentPage();
        return $page.invokeComponentMethod(params);
      },
    }),
    {
      get: (obj, prop) => {
        if (prop === 'app' || prop === '$app') {
          return app;
        } else {
          return obj[prop];
        }
      },
      // set: (obj, prop, value) => {
      //   if (prop === 'app' || prop === '$app') {
      //     console.warn('$app 对象中 app/$app 为只读');
      //     return false;
      //   } else {
      //     // 修改 obj
      //     return true;
      //   }
      // },
    },
  );
  _injectApp2Runtime(globalAPI);

  return globalAPI;
}

function _injectApp2Runtime(globalAPI, hard = false) {
  // 给全局挂上 mainApp/subApp
  // The global api exposed to lowcode
  if (subPackageName) {
    globalAPI.mainApp = runtime.$app;
    (runtime as any)[`$$subapp_${subPackageName}`] = globalAPI;
  } else {
    // 预览区调试栏可调试
    // 暴露给复合组件，代码组件来引用
    runtime.app = hard ? globalAPI : runtime.app || globalAPI;
    runtime.$app = new Proxy(runtime.app, {
      get(obj, prop) {
        return obj ? obj[prop] : undefined;
      },
      set(obj, prop, value) {
        if (obj) {
          return (obj[prop] = value);
        } else {
          return undefined;
        }
      },
    });

    runtime.$w = new Proxy($w || globalAPI?.__internal__?.$w || {}, {
      get(_, prop: string) {
        const $page = runtime.app?.__internal__?.activePage || runtime.$page;
        if ($page?.__internal__?.$w?.[prop]) {
          return $page?.__internal__?.$w?.[prop];
        }

        /**
         * 此处应该统一到 $page 的生成上
         * 但是有可能有受控 $page 注入的情况
         * 理想应该在注入的入口完成，如果有遗漏的话，打开这里来兜底
         */
        // let pageWidget = lodashGet($page, `widgets.${prop}`);
        // if (pageWidget) {
        //   return pageWidget._userWidget;
        // }

        return null;
      },
    });
  }
}

export function removeGlboalApi() {
  if (subPackageName) {
    (runtime as any)[`$$subapp_${subPackageName}`] = undefined;
  } else {
    runtime.app = undefined;
    runtime.$app = undefined;
  }
}

export interface IPageAPI extends IBaseState {
  __internal__?: {
    $w?: any;
  };
  $page?: IPageAPI;
  id?: string;
  handler: {
    [key: string]: any;
  };
  props: {
    [key: string]: any;
  };
  widgets: {
    [key: string]: any;
  };
  setState: (state) => void;
  _dataBinds: {
    [key: string]: any;
  };
}

export function createPageApi(): IPageAPI {
  const $page: IPageAPI = new Proxy(
    {
      __internal__: {},
      state: {},
      computed: {},
      handler: {},
      props: {},
      widgets: observable({}),
      dataset: {},
      setState: (userSetState) => {
        Object.keys(userSetState).forEach((keyPath) => {
          app.utils.set($page.dataset?.state, keyPath, userSetState[keyPath]);
        });
      },
      _dataBinds: {},
      invokeComponentMethod({ component, method, params }) {
        const widgetData = lodashGet($page.widgets, component);
        if (Array.isArray(widgetData)) {
          throw new Error(`调用方法失败：id为${component}的组件拥有多个实例`);
          // widgetData.forEach(widget => {
          //   if(!widget._methods[method]) {
          //     throw new Error('NOT_FOUND')
          //   } else {
          //     actions.push(widget._methods[method])
          //   }
          // })
        } else {
          const action = widgetData?._userWidget?.custom?.[method];
          if (typeof action !== 'function') {
            throw new Error(`调用方法失败：未找到id为${component}下的方法${method}`);
          }
          return action(params);
        }
      },
    },
    {
      get(obj, prop) {
        return prop === '$page' ? obj : obj[prop];
      },
    },
  );
  return $page;
}

// 分app 和 wx 挂载app
function mountAPIs(sdks) {
  Object.keys(sdks).forEach((item) => {
    let action = sdks[item];
    if (!(item in ACTIONS_KEY) && !(item in ROUTER_KEY)) {
      app[item] = action;
    }
  });
  return app;
}
