import React from 'react';
import { useClassNames, Tag, Text } from 'tea-component';
import classNames from 'classnames';
import { Variable, ApprovalHistoryConstants } from '../../constants';
import HistoryNodeContent from './HistoryNodeContent';
const { STEP_STATUS_STYLE_MAP, ELEMENT_TYPE_MAP } = ApprovalHistoryConstants;
const HistoryNode = ({ dataSource }) => {
    const { elementType, approvalType, status, nodeName, approvalUserList, triggerTime, rejectTo, extra, } = dataSource;
    const { Margin } = useClassNames();
    const isApprovalNode = ![
        ELEMENT_TYPE_MAP.START_EVENT,
        ELEMENT_TYPE_MAP.END_EVENT,
        ELEMENT_TYPE_MAP.REVOKE_EVENT,
    ].includes(elementType);
    const isStartNode = elementType === ELEMENT_TYPE_MAP.START_EVENT;
    const isRevokeNode = elementType === ELEMENT_TYPE_MAP.REVOKE_EVENT;
    const renderSubTitle = () => {
        if (isRevokeNode) {
            return (React.createElement(React.Fragment, null,
                React.createElement("span", { className: classNames('title', Margin.Left['2n'], `${Variable}-step__item-content__header-title-revoke`) }, approvalType),
                React.createElement("span", { className: classNames('title', Margin.Left['2n']) }, extra)));
        }
        if (isStartNode) {
            return (React.createElement(Text, { theme: "weak", className: Margin.Left['2n'] }, approvalType));
        }
        if (isApprovalNode) {
            return (React.createElement(Tag, { theme: "primary", className: Margin.Left['2n'] }, approvalType));
        }
        return null;
    };
    const renderTitleExtra = () => {
        if (isStartNode) {
            return React.createElement("div", { className: `${Variable}-step__item-content__header-des` }, triggerTime);
        }
        return null;
    };
    return (React.createElement("div", { className: `${Variable}-step__item is-${STEP_STATUS_STYLE_MAP[status]}` },
        React.createElement("div", { className: `${Variable}-step__item-icon` },
            React.createElement("i", { className: `${Variable}-step__item-icon-order` })),
        React.createElement("div", { className: `${Variable}-step__item-content` },
            React.createElement("div", { className: `${Variable}-step__item-content__header` },
                React.createElement("div", { className: `${Variable}-step__item-content__header-title` },
                    React.createElement("span", { className: "title" }, nodeName),
                    renderSubTitle()),
                renderTitleExtra()),
            isApprovalNode ? (React.createElement(HistoryNodeContent, { dataSource: approvalUserList, rejectTo: rejectTo })) : null)));
};
export default HistoryNode;
