import React from 'react';
import classNames from 'classnames';
import { useClassNames, Text, Tooltip } from 'tea-component';
import { Variable, ApprovalHistoryConstants } from '../../constants';
const { STEP_STATUS_STYLE_MAP, APPROVER_NODE_STATUS_STYLE_MAP, APPROVAL_COMMENTS, APPROVAL_STATUS_MAP, REJECT_TO, FIELD_UPDATE_TEXT_MAP, } = ApprovalHistoryConstants;
const NodeStatusExtra = ({ rejectTo, relatedUser, status }) => {
    const { Margin } = useClassNames();
    if (status === APPROVAL_STATUS_MAP.REJECTED) {
        return (React.createElement(React.Fragment, null,
            React.createElement(Text, { theme: "text", className: classNames(`${Variable}-step__sub-item__header-status-text`, Margin.Left['1n']) }, REJECT_TO),
            React.createElement(Tooltip, { title: rejectTo },
                React.createElement(Text, { overflow: true, theme: "text", verticalAlign: "middle", style: { maxWidth: 100 }, className: classNames(`${Variable}-step__sub-item__header-status-text`, Margin.Left['4n']) }, rejectTo))));
    }
    if (relatedUser) {
        return (React.createElement(Text, { theme: "text", verticalAlign: "middle", className: classNames(`${Variable}-step__sub-item__header-status-text`, Margin.Left['4n']) }, relatedUser));
    }
    return null;
};
const NodeBody = ({ isShow, fieldUpdateList, content, time }) => {
    const { Margin } = useClassNames();
    const renderFieldUpdateHistoryList = (el) => {
        // 渲染更新操作且没有错误的Dom
        if (el.isUpdate && !el.isError) {
            return (React.createElement(React.Fragment, null,
                el.fieldName,
                "\uFF1A",
                FIELD_UPDATE_TEXT_MAP.FROM,
                React.createElement("em", null,
                    el.sourceValue,
                    " "),
                FIELD_UPDATE_TEXT_MAP.TO,
                React.createElement("em", null, el.targetValue)));
        }
        return (React.createElement(React.Fragment, null,
            el.fieldName,
            "\uFF1A", el === null || el === void 0 ? void 0 :
            el.itemId,
            (el === null || el === void 0 ? void 0 : el.isError) ? (React.createElement(Text, { theme: "text", className: classNames(`${Variable}-step__sub-item__body-field-failed`, Margin.Left['1n']) }, FIELD_UPDATE_TEXT_MAP.FAILED)) : null));
    };
    if (!isShow) {
        return (React.createElement("div", { className: `${Variable}-step__sub-item__body` },
            fieldUpdateList.map((el) => (React.createElement("div", { className: `${Variable}-step__sub-item__body-text`, key: el.fieldName }, renderFieldUpdateHistoryList(el)))),
            React.createElement("div", { className: classNames(`${Variable}-step__sub-item__body-des`, {
                    [`${Variable}-step__sub-item__body-des-hidden`]: !content,
                }) }, `${APPROVAL_COMMENTS}：${content}`),
            React.createElement("div", { className: `${Variable}-step__sub-item__body-time` }, time)));
    }
    return null;
};
const HistoryNodeContent = ({ dataSource, rejectTo }) => (React.createElement("div", { className: `${Variable}-step__item-content__body` },
    React.createElement("div", { className: `${Variable}-step__sub` }, dataSource.map((el) => {
        const { userName, status, statusName, commentContent, commentTime, fieldUpdateList, relatedUser, isApproving, } = el;
        return (React.createElement("div", { key: el.userName, className: `${Variable}-step__sub-item is-${STEP_STATUS_STYLE_MAP[status]}` },
            React.createElement("div", { className: `${Variable}-step__sub-item__header` },
                React.createElement("div", { className: `${Variable}-step__sub-item__header-title` }, userName),
                React.createElement("div", { className: `${Variable}-step__sub-item__header-status` },
                    React.createElement(Text, { theme: APPROVER_NODE_STATUS_STYLE_MAP[status], className: `${Variable}-step__sub-item__header-status-text` }, statusName),
                    React.createElement(NodeStatusExtra, { status: status, rejectTo: rejectTo, relatedUser: relatedUser }))),
            React.createElement(NodeBody, { isShow: isApproving, fieldUpdateList: fieldUpdateList, content: commentContent, time: commentTime })));
    }))));
export default HistoryNodeContent;
