import React, { useEffect } from 'react';
import { message } from 'tea-component';
import { useSetState } from 'ahooks';
import { ModelForm } from '../Form';
import { PageContent, PageCountDown, PageNullDb, PageErrorContent } from '../PageContent';
import { DataSource, navigateToModelPage, isInIde } from '../../utils';
import { useAuthFields } from '../../hooks';
import { CREATE_BUTTONS } from '../../constants';
import { alertErrorMessage } from '../../utils/feedback';
/**
 * 新增组件
 */
export const ModelCreate = (props) => {
    const { bindMetadata = {}, // data属性
    buttons = CREATE_BUTTONS, // 选区按钮集合
    _selectableBlockEvents: selectableBlockEvents, // 选区的事件集合
    id: compId, // 组件的id
    ...rest } = props;
    const { datasource, enabledFieldNames } = bindMetadata;
    const dbName = datasource === null || datasource === void 0 ? void 0 : datasource.name;
    const [{ showFields, requireNoRwFields, getFieldsLoading, errorMessageInfo }, setState] = useSetState({
        showFields: [],
        requireNoRwFields: [],
        getFieldsLoading: true,
        errorMessageInfo: {
            status: false,
        },
    });
    useEffect(() => {
        try {
            async function getUseAuthFields() {
                const { showFields, requireNoRwFields } = await useAuthFields(dbName, enabledFieldNames);
                setState({
                    showFields,
                    requireNoRwFields,
                    getFieldsLoading: false,
                });
            }
            getUseAuthFields();
        }
        catch (err) {
            setState({
                getFieldsLoading: false,
                errorMessageInfo: {
                    status: true,
                    title: '获取数据源字段失败',
                    message: err,
                },
            });
            console.error(`获取数据源字段失败,错误信息:${err}`);
        }
    }, [dbName]);
    if (errorMessageInfo === null || errorMessageInfo === void 0 ? void 0 : errorMessageInfo.status) {
        return React.createElement(PageErrorContent, { errorMessageInfo: errorMessageInfo, ...rest });
    }
    // 1、编辑器内未绑定数据源时返回
    if (!dbName) {
        return React.createElement(PageNullDb, { ...rest });
    }
    // 2、无编辑权限字段时返回
    if (showFields.length === 0 && !getFieldsLoading) {
        if (isInIde()) {
            return React.createElement(PageContent, { ...rest }, "\u672A\u627E\u5230\u5B57\u6BB5\u4FE1\u606F");
        }
        return React.createElement(PageCountDown, { dbName: dbName, ...rest });
    }
    // 3、正常返回
    return (React.createElement(PageContent, { loading: getFieldsLoading && !isInIde(), ...rest },
        React.createElement(ModelForm, { action: "create", fields: showFields, compId: compId, buttons: buttons, selectableBlockEvents: selectableBlockEvents, onSubmit: async (v) => {
                try {
                    if (requireNoRwFields.length > 0) {
                        message.warning({ content: '当前有不具备权限的字段是必填项，无法进行保存' });
                        return;
                    }
                    const subValue = {};
                    // 提交时只提交有读写权限的字段
                    Object.keys(v).forEach((item) => {
                        const value = showFields === null || showFields === void 0 ? void 0 : showFields.find((k) => k.name === item);
                        if (value && (value === null || value === void 0 ? void 0 : value.readWriteAuth) === 'rw') {
                            subValue[item] = v[item];
                        }
                    });
                    const db = new DataSource(datasource.name);
                    await db.createRecord(subValue);
                    message.success({ content: '创建成功' });
                    navigateToModelPage('list', {
                        dbName,
                    });
                }
                catch (error) {
                    alertErrorMessage({ message: `创建失败 ${error.message}` });
                }
            }, initialData: undefined })));
};
