import React, { useContext } from 'react';
import { useRequest } from 'ahooks';
import { Form, Status } from 'tea-component';
import { getFieldRender, getFieldValue } from '../Table/FieldRender';
import { DataSource, getRecordId, isEmpty, isInIde, getPrimaryField } from '../../utils';
import { useMediaQuery } from '../../hooks';
import { DetailLoadingContext } from './index';
import { alertErrorMessage } from '../../utils/feedback';
export const DataItemView = ({ fields, dbName, setRecordData, recordId }) => {
    // 数据 id
    const { id } = getRecordId(recordId);
    const isH5 = useMediaQuery('(max-width: 1024px)');
    const setDetailLoading = useContext(DetailLoadingContext);
    const { data, loading } = useRequest(async () => {
        if (!id && !isInIde()) {
            alertErrorMessage({ message: '记录 ID 不存在，加载数据失败' });
            return {};
        }
        const datasource = new DataSource(dbName);
        const data = await datasource.getRecord(id);
        setRecordData && setRecordData(data);
        if (isEmpty(data) && !isInIde()) {
            alertErrorMessage({ message: '记录 ID 不存在，加载数据失败' });
            return {};
        }
        // 主列字段获取
        const datas = getPrimaryField(data);
        return datas;
    }, {
        refreshDeps: [id, dbName],
        onError: () => setDetailLoading === null || setDetailLoading === void 0 ? void 0 : setDetailLoading(false),
        onSuccess: () => setDetailLoading === null || setDetailLoading === void 0 ? void 0 : setDetailLoading(false),
    });
    // 未找到数据，或在加载中
    if (!data || loading) {
        return (React.createElement("div", { style: { textAlign: 'center' } },
            React.createElement(Status, { size: "l", icon: "loading", title: loading ? '加载中' : '加载完成', description: loading ? '正在加载页面' : '暂无数据' })));
    }
    return (React.createElement("div", { className: "model-detail-view" },
        React.createElement(Form, { readonly: true, className: isH5 ? 'tea-form--vertical model-detail-h5' : '' + ' model-form' }, fields.map((field, key) => {
            const fieldRender = getFieldRender(field, {
                showTip: false,
                modelType: 'detail',
                relation: data[`@${field.name}`],
            });
            const value = getFieldValue(field.name, data);
            return (React.createElement(Form.Item, { label: field.title, key: key },
                React.createElement(Form.Text, null, fieldRender(value))));
        }))));
};
