import React from 'react';
const initialState = { error: null };
class ErrorBoundary extends React.Component {
    constructor() {
        super(...arguments);
        Object.defineProperty(this, "state", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: initialState
        });
    }
    // capture error
    static getDerivedStateFromError(error) {
        return { error };
    }
    componentDidCatch(error, info) {
        var _a, _b;
        console.error('组件边界错误', error);
        (_b = (_a = this.props).onError) === null || _b === void 0 ? void 0 : _b.call(_a, error, info);
    }
    render() {
        const { error } = this.state;
        const { fallbackRender, FallbackComponent, fallback } = this.props;
        if (error !== null) {
            const props = {
                error,
            };
            if (React.isValidElement(fallback)) {
                return fallback;
            }
            if (typeof fallbackRender === 'function') {
                return fallbackRender(props);
            }
            if (FallbackComponent) {
                return React.createElement(FallbackComponent, { ...props });
            }
            return `Error Render: ${error === null || error === void 0 ? void 0 : error.message}`;
        }
        return this.props.children;
    }
}
export default ErrorBoundary;
