/* eslint-disable no-param-reassign */
import React, { useState, useEffect, useMemo } from 'react';
import { Justify, Button, Icon } from 'tea-component';
import { Popup } from 'react-vant';
import { safeJsonParse } from '../../../../utils';
import { WedaUserTransfer } from '../WedaUserTransfer';
import './index.less';
export const FlowUserSelect = (props) => {
    const { visible, approveField, mode, close } = props;
    const [selected, setSelected] = useState([]);
    const [selectedUser, setSelectedUser] = useState([]);
    const [collapse, setCollapseStatus] = useState(true);
    useEffect(() => {
        document.title = '选择人员';
        const selectedAvatar = sessionStorage.getItem('selected');
        selectedAvatar && setSelected(safeJsonParse(selectedAvatar || '[]'));
    }, []);
    const selectedLen = useMemo(() => selected.length, [selected]);
    const removeId = (id) => {
        setSelected(selected.filter((item) => item !== id));
        setSelectedUser(selectedUser.filter((item) => (item === null || item === void 0 ? void 0 : item.UserId) !== id));
    };
    const handleHighLightSelected = (selected, flowData) => {
        let v = '';
        const { opinionUserMap, approveInfo } = flowData;
        selected.forEach((id) => {
            const user = selectedUser.find((user) => (user === null || user === void 0 ? void 0 : user.UserId) === id) || {};
            const name = `@${user === null || user === void 0 ? void 0 : user.Name}`;
            v = `${v + name} `;
            if (!opinionUserMap[id]) {
                opinionUserMap[id] = name;
            }
        });
        flowData.approveInfo = approveInfo + v;
        flowData.opinionUserMap = { ...opinionUserMap };
    };
    const saveSelect = () => {
        var _a;
        const flowData = (_a = safeJsonParse(sessionStorage.getItem('FlowActionsMobile'))) !== null && _a !== void 0 ? _a : {};
        if (approveField === 'assigneeIds') {
            flowData.assigneeIds = selected;
            sessionStorage.setItem('hasSignSubmit', 'hasSignSubmit');
        }
        else if (approveField === 'highLightApprovers') {
            handleHighLightSelected(selected, flowData);
        }
        else {
            flowData[approveField] = selected;
        }
        sessionStorage.setItem('FlowActionsMobile', JSON.stringify(flowData));
        sessionStorage.setItem(approveField, JSON.stringify(selectedUser));
        // history.go(-1);
        close === null || close === void 0 ? void 0 : close();
    };
    const onWedaUserSelectChange = ({ keys, rows }) => {
        setSelected(keys);
        setSelectedUser(rows);
    };
    useEffect(() => {
        let lastTouchEnd = 0;
        const touchstartFn = (event) => {
            if (event.touches.length > 1) {
                event.preventDefault();
            }
        };
        const touchendFn = (event) => {
            const now = new Date().getTime();
            if (now - lastTouchEnd <= 300) {
                event.preventDefault();
            }
            lastTouchEnd = now;
        };
        const gesturestartFn = (event) => {
            event.preventDefault();
        };
        document.addEventListener('touchstart', touchstartFn);
        document.addEventListener('touchend', touchendFn, false);
        // 阻止双指放大
        document.addEventListener('gesturestart', gesturestartFn);
        return () => {
            document.removeEventListener('touchstart', touchstartFn);
            document.removeEventListener('touchend', touchendFn);
            document.removeEventListener('gesturestart', gesturestartFn);
        };
    }, []);
    return (React.createElement(Popup, { visible: visible, position: "bottom", className: "flow-user-select-mobile" },
        React.createElement("section", { className: "page-info__wrapper-mobile" },
            React.createElement(WedaUserTransfer, { multiple: mode === 'multiple', isMobile: true, onSelectedChange: onWedaUserSelectChange, value: selected, className: "flow-user-select-mobile-table" }),
            React.createElement("footer", { className: "selected-info" },
                React.createElement(Justify, { left: React.createElement("div", { className: "selected-info__left" },
                        React.createElement("p", { className: "selected-count", onClick: () => setCollapseStatus(false) },
                            React.createElement("span", { style: { display: 'inline-block' } }, `已选：${selectedLen}人`),
                            React.createElement("span", { className: "collapse-spread" })),
                        React.createElement("p", { className: "selected-people" }, selectedUser.map((item) => item === null || item === void 0 ? void 0 : item.Name).join(','))), right: React.createElement(React.Fragment, null,
                        React.createElement(Button, { className: "submit-btn", type: "primary", onClick: saveSelect }, `确认(${selectedLen})`),
                        React.createElement(Button, { type: "link", onClick: () => {
                                close === null || close === void 0 ? void 0 : close();
                            } }, "\u5173\u95ED")) })),
            React.createElement(Popup, { className: "SelectPersonOrg-popup", visible: !collapse, position: "bottom", style: { height: '440px' } },
                React.createElement(React.Fragment, null,
                    React.createElement("p", { className: "selected-title" },
                        React.createElement(Button, { type: "text", onClick: () => setCollapseStatus(true) }, "\u53D6\u6D88"),
                        React.createElement("span", null, `已选：${selectedLen}人`),
                        React.createElement(Button, { type: "link", onClick: saveSelect }, "\u5B8C\u6210")),
                    React.createElement("div", { className: "selected-person" }, selectedUser.map((item) => (React.createElement(Justify, { key: item === null || item === void 0 ? void 0 : item.UserId, left: React.createElement("span", { className: "selected-person__name" }, item === null || item === void 0 ? void 0 : item.Name), right: React.createElement(Icon, { onClick: () => removeId(item === null || item === void 0 ? void 0 : item.UserId), type: "dismiss" }) })))))))));
};
