import React, { useState, useEffect, useRef } from 'react';
import HighlightWithinTextarea from 'react-highlight-within-textarea';
import { UserSelectModel } from '../UserSelectModel';
import './index.css';
export const HighlightTextarea = ({ size = 'pc', placeholder = '请输入', value, onChange, opinionUserMap, setOpinionUserMap, setHighlightPositionMap, }) => {
    const [selectPersonModalShow, setSelectPersonModalShow] = useState(false);
    const range = useRef();
    const [placeholderVisible, setPlaceholderVisible] = useState(!value);
    const [highlightPositions, setHighlightPositions] = useState([]);
    const [copyMsg, setCopyMsg] = useState('');
    useEffect(() => {
        if (!value) {
            setPlaceholderVisible(true);
            setOpinionUserMap({});
            return;
        }
        setPlaceholderVisible(false);
        const { res: highlightPositions, obj: highlightPositionObj } = checkHighLight(value);
        setHighlightPositions([...highlightPositions]);
        setHighlightPositionMap({ ...highlightPositionObj });
        const newOpinionUserMap = {};
        // value每次change后需要再次检查opinionUserMap
        for (const [id, name] of Object.entries(opinionUserMap)) {
            if (value.indexOf(name) !== -1) {
                newOpinionUserMap[id] = name;
            }
        }
        setOpinionUserMap(newOpinionUserMap);
    }, [value]);
    const checkHighLight = (text) => {
        const highlightList = Object.values(opinionUserMap);
        const len = highlightList.length;
        let v = text;
        let res = [];
        // 回显时使用，必须顺序排列
        let obj = {};
        for (let i = 0; i < len; i++) {
            const indexof = v.indexOf(highlightList[i]);
            const itemLen = highlightList[i].length;
            if (indexof !== -1) {
                const endIndex = indexof + itemLen;
                res.push([indexof, endIndex]);
                obj[indexof] = endIndex;
                const fillText = new Array(itemLen).fill(' ').join(''); // 空格填充
                v = v.substring(0, indexof) + fillText + v.substring(endIndex);
            }
        }
        if (res.length > 0 && v.includes('@')) {
            // 继续检查剩下文本，是否存在重复@
            const { res: arr, obj: objCopy } = checkHighLight(v);
            res = res.concat(arr);
            obj = { ...obj, ...objCopy };
        }
        return { res, obj };
    };
    const changeValue = (addValue) => {
        var _a;
        const r = range.current;
        (_a = r === null || r === void 0 ? void 0 : r.insertNode) === null || _a === void 0 ? void 0 : _a.call(r, document.createTextNode(addValue));
        onChange(`${value}${addValue}`);
    };
    const getRange = () => {
        // 弹起人员选择弹框
        setSelectPersonModalShow(true);
        // 获取光标选区位置
        const s = window.getSelection();
        const r = s.getRangeAt(0);
        range.current = r;
    };
    const countAtSignal = (value) => {
        let count1 = 0;
        let count2 = 0;
        for (const v of value) {
            if (v === '@') {
                count1 += 1;
            }
        }
        for (const v of copyMsg) {
            if (v === '@') {
                count2 += 1;
            }
        }
        return count1 === count2;
    };
    return (React.createElement("div", { className: "highlight-textarea-content" },
        React.createElement("div", { className: "highlight-textarea", onKeyUp: (e) => {
                if (size === 'pc' && e.shiftKey && (e.key === '@' || e.keyCode === 50)) {
                    getRange();
                    return;
                }
                if (size === 'pc' && e.keyCode === 50) {
                    // 判断当下value值是否存在2
                    if (!countAtSignal(value)) {
                        getRange();
                    }
                }
                setTimeout(() => setCopyMsg(value), 300);
            }, onFocus: () => {
                setPlaceholderVisible(false);
            }, onBlur: () => {
                if (!value) {
                    setPlaceholderVisible(true);
                }
            } },
            React.createElement(HighlightWithinTextarea, { value: value, onChange: (value) => {
                    onChange(value);
                }, highlight: highlightPositions }),
            placeholderVisible && React.createElement("div", { className: "highlight-textarea-placeholder" }, placeholder)),
        React.createElement(UserSelectModel, { visible: selectPersonModalShow, placeholder: "\u8BF7\u9009\u62E9\u901A\u77E5\u4EBA\u5458", onClose: () => {
                setSelectPersonModalShow(false);
            }, onChange: (val) => {
                let v = '';
                const newOpinionUserMap = { ...opinionUserMap };
                (val || []).forEach((item, index) => {
                    const name = `@${item.text}`;
                    v = `${v + item.text} `;
                    if (index < val.length - 1) {
                        v = `${v}@`;
                    }
                    if (!opinionUserMap[item.value]) {
                        newOpinionUserMap[item.value] = name;
                    }
                });
                setOpinionUserMap({ ...newOpinionUserMap });
                changeValue(v || ' ');
                setSelectPersonModalShow(false);
            }, value: [] })));
};
