import React from 'react';
import { useEffect, useRef, useState } from 'react';
import { Transfer, Table, SearchBox, Bubble, Text, Modal, Button, LoadingTip } from 'tea-component';
import to from 'await-to-js';
import { useGlobalContext } from '../../../../context';
import { getWedaUserList } from '../../services';
import './index.less';
const { selectable, removeable, scrollable, radioable } = Table.addons;
export function WedaUserTransfer({ visible, onClose, setOptions, multiple = true, isMobile = false, onSelectedChange, value, tableClassName, }) {
    var _a;
    const [targetKeys, setTargetKeys] = useState([]);
    const [selectedRows, setSelectedRows] = useState([]);
    const [inputValue, setInputValue] = useState('');
    const [userList, setUserList] = useState([]);
    const originUserList = useRef([]);
    const [loading, setLoading] = useState(false);
    const { callWedaApi, IS_WEDA_IDE } = useGlobalContext();
    useEffect(() => {
        getWedaUser({ PageNo: 1, PageSize: 200 });
    }, []);
    useEffect(() => {
        if (!visible) {
            onSearchUser('');
        }
    }, [visible]);
    useEffect(() => {
        var _a;
        if (isMobile) {
            setTargetKeys(value);
        }
        else {
            setTargetKeys(value === null || value === void 0 ? void 0 : value.map((i) => i === null || i === void 0 ? void 0 : i.value));
        }
        if ((_a = originUserList.current) === null || _a === void 0 ? void 0 : _a.length) {
            const initSelected = value === null || value === void 0 ? void 0 : value.map((i) => {
                var _a;
                return (_a = originUserList.current) === null || _a === void 0 ? void 0 : _a.find((user) => {
                    if (isMobile)
                        return (user === null || user === void 0 ? void 0 : user.UserId) === i;
                    return (user === null || user === void 0 ? void 0 : user.UserId) === (i === null || i === void 0 ? void 0 : i.value);
                });
            });
            setSelectedRows(initSelected || []);
        }
    }, [value, (_a = originUserList.current) === null || _a === void 0 ? void 0 : _a.length]);
    const getWedaUser = async (params) => {
        setLoading(true);
        const [err, result] = await to(getWedaUserList(params, { callWedaApi, IS_WEDA_IDE }));
        setLoading(false);
        if (!err) {
            originUserList.current = (result === null || result === void 0 ? void 0 : result.UserList) || [];
            setUserList((result === null || result === void 0 ? void 0 : result.UserList) || []);
        }
    };
    const onSourceTableChange = (keys, context) => {
        if (multiple) {
            setTargetKeys(keys);
            const selectedRows = transSelectedRows(context);
            setSelectedRows(selectedRows);
            onSelectedChange === null || onSelectedChange === void 0 ? void 0 : onSelectedChange({ keys, rows: selectedRows });
        }
        else {
            setTargetKeys([keys]);
            setSelectedRows([context === null || context === void 0 ? void 0 : context.record]);
            onSelectedChange === null || onSelectedChange === void 0 ? void 0 : onSelectedChange({ keys: [keys], rows: [context === null || context === void 0 ? void 0 : context.record] });
        }
    };
    const transSelectedRows = (context) => {
        const { checked, record, recordKey } = context;
        const index = selectedRows.findIndex((val) => (val === null || val === void 0 ? void 0 : val.UserId) === recordKey);
        const newSelectedRows = selectedRows.concat();
        if (checked) {
            newSelectedRows.push(record);
        }
        else {
            newSelectedRows.splice(index, 1);
        }
        return newSelectedRows;
    };
    const onSearchUser = (value) => {
        var _a;
        setInputValue(value);
        if (value) {
            const list = (_a = originUserList.current) === null || _a === void 0 ? void 0 : _a.filter((item) => { var _a; return (_a = item === null || item === void 0 ? void 0 : item.Name) === null || _a === void 0 ? void 0 : _a.includes(value); });
            setUserList(list);
        }
        else {
            setUserList(originUserList.current);
        }
    };
    const onSubmitUser = () => {
        setOptions(selectedRows.map((item) => ({
            text: item.Name,
            value: item.UserId,
        })));
        onCloseModal();
    };
    const onRemoveUser = (key) => {
        setTargetKeys(targetKeys === null || targetKeys === void 0 ? void 0 : targetKeys.filter((i) => i !== key));
        setSelectedRows(selectedRows === null || selectedRows === void 0 ? void 0 : selectedRows.filter((i) => i.UserId !== key));
    };
    const onCloseModal = () => {
        setTargetKeys([]);
        setSelectedRows([]);
        onClose === null || onClose === void 0 ? void 0 : onClose();
    };
    if (isMobile) {
        if (loading) {
            return React.createElement(LoadingTip, { loadingText: "\u52A0\u8F7D\u4E2D" });
        }
        return (React.createElement(React.Fragment, null,
            React.createElement(SearchBox, { className: "weda-workbench__flow-user-search", value: inputValue, onChange: (value) => onSearchUser(value) }),
            React.createElement(SourceTable, { dataSource: userList, targetKeys: targetKeys, onChange: onSourceTableChange, multiple: multiple, className: tableClassName })));
    }
    return (React.createElement(Modal, { visible: visible, caption: "\u4EBA\u5458\u9009\u62E9", onClose: onCloseModal, size: "l" },
        React.createElement(Modal.Body, null,
            React.createElement(Transfer, { leftCell: React.createElement(Transfer.Cell, { scrollable: false, title: "\u8BF7\u9009\u62E9\u7528\u6237", header: React.createElement(SearchBox, { value: inputValue, onChange: (value) => onSearchUser(value) }) },
                    React.createElement(SourceTable, { dataSource: userList, targetKeys: targetKeys, onChange: onSourceTableChange, multiple: multiple, className: tableClassName })), rightCell: React.createElement(Transfer.Cell, { title: `已选择 (${(targetKeys === null || targetKeys === void 0 ? void 0 : targetKeys.length) || 0})` },
                    React.createElement(TargetTable, { dataSource: selectedRows, onRemove: onRemoveUser })) })),
        React.createElement(Modal.Footer, null,
            React.createElement(Button, { type: "primary", onClick: onSubmitUser }, "\u786E\u5B9A"),
            React.createElement(Button, { type: "weak", onClick: onCloseModal }, "\u53D6\u6D88"))));
}
const columns = [
    {
        key: 'Name',
        header: '用户名',
        render: (record) => (record === null || record === void 0 ? void 0 : record.Name) ? (React.createElement(Bubble, { content: record === null || record === void 0 ? void 0 : record.Name },
            React.createElement(Text, { overflow: true, style: { cursor: 'pointer' } }, record === null || record === void 0 ? void 0 : record.Name))) : (React.createElement(Text, null, "-")),
    },
    {
        key: 'UserDepartment',
        header: '所属部门',
        render: (record) => {
            var _a, _b;
            const showStr = (_a = record === null || record === void 0 ? void 0 : record.Orgs) === null || _a === void 0 ? void 0 : _a.map((i) => (i === null || i === void 0 ? void 0 : i.OrgName) || '').join();
            return ((_b = record === null || record === void 0 ? void 0 : record.Orgs) === null || _b === void 0 ? void 0 : _b.length) ? (React.createElement(Bubble, { content: showStr },
                React.createElement(Text, { overflow: true, style: { cursor: 'pointer' } }, showStr))) : (React.createElement(Text, null, "-"));
        },
    },
];
function SourceTable({ dataSource, targetKeys, onChange, multiple, className }) {
    const addons = [
        scrollable({
            maxHeight: 310,
        }),
        multiple
            ? selectable({
                value: targetKeys,
                onChange,
                rowSelect: true,
            })
            : radioable({
                value: targetKeys === null || targetKeys === void 0 ? void 0 : targetKeys[0],
                onChange,
                rowSelect: true,
            }),
    ];
    return (React.createElement(Table, { records: dataSource, recordKey: "UserId", rowDisabled: (record) => record.status === 'stopped', rowClassName: (record) => record.status, columns: columns, addons: addons, className: className }));
}
function TargetTable({ dataSource, onRemove }) {
    return React.createElement(Table, { records: dataSource, recordKey: "UserId", columns: columns, addons: [removeable({ onRemove })] });
}
