import to from 'await-to-js';
import { fetchProcess, getApprovalPageDetail, fetchInstanceOperation } from '../services';
import { PAGE_TYPE } from '../constants';
import { IFlowDetailProps, IPageInfoProps } from './types';
import { CommonFlowDataUtils } from './utils';
export async function getCommonFlowData(config, flowTaskId) {
    const { currentPageType } = config;
    const pageTypeParams = currentPageType === PAGE_TYPE.CREATEFLOW ? PAGE_TYPE.TO_DO : currentPageType;
    const flowDetail = await getProcessInstance(config, pageTypeParams, flowTaskId);
    const pageInfo = await getPageInfo(config, flowDetail, pageTypeParams);
    const flowSteps = await fetchFlowSteps(config, flowDetail);
    return {
        flowDetail,
        ...pageInfo,
        ...flowSteps,
    };
}
export async function getProcessInstance(config, pageTypeParams, flowTaskId) {
    const result = {};
    const { isWedaIde, callWedaApi } = config;
    const params = {
        flowTaskId,
        pageType: pageTypeParams,
    };
    const options = { isWedaIde, callWedaApi };
    const [err, res] = await to(fetchProcess(params, options));
    if (!err) {
        return res;
    }
    return new IFlowDetailProps();
}
export async function getPageInfo(config, flowDetail, pageType) {
    const params = {
        instanceId: flowDetail === null || flowDetail === void 0 ? void 0 : flowDetail.instanceId,
        flowCode: flowDetail === null || flowDetail === void 0 ? void 0 : flowDetail.flowCode,
        nodeId: flowDetail === null || flowDetail === void 0 ? void 0 : flowDetail.currentNodeId,
        id: flowDetail === null || flowDetail === void 0 ? void 0 : flowDetail.ccId,
        appVersion: flowDetail === null || flowDetail === void 0 ? void 0 : flowDetail.flowVersion,
        isCompleted: flowDetail === null || flowDetail === void 0 ? void 0 : flowDetail.isCompleted,
        pageType,
        taskId: flowDetail === null || flowDetail === void 0 ? void 0 : flowDetail.taskId,
    };
    const [err, res] = await to(getApprovalPageDetail(params, config));
    if (!err) {
        const initialValues = CommonFlowDataUtils.translateObjectMap(res);
        const { isReadonly } = CommonFlowDataUtils.getFormState(config);
        const flowLoadData = {
            initialValues,
            formType: isReadonly ? 'read' : 'create',
        };
        return {
            pageInfo: res,
            flowLoadData,
        };
    }
    return {
        pageInfo: new IPageInfoProps(),
        flowLoadData: { initialValues: {}, formType: 'read' },
    };
}
export async function fetchFlowSteps(config, flowDetail) {
    var _a;
    const [err, res] = await to(fetchInstanceOperation({ instanceId: flowDetail.instanceId }, config));
    if (!err) {
        const processData = (res === null || res === void 0 ? void 0 : res.elementInfos) || [];
        const steps = CommonFlowDataUtils.transProcessData(processData, config.safeJsonParse);
        const ccUserMaps = ((_a = processData === null || processData === void 0 ? void 0 : processData[0]) === null || _a === void 0 ? void 0 : _a.ccUserMaps) || {};
        return {
            steps,
            ccUserMaps,
        };
    }
    return {
        steps: [],
        ccUserMaps: {},
    };
}
