import { useEffect, useRef } from 'react';
import EventEmitter from 'eventemitter3';
/** 流程通用接口请求hook */
export function useCommonFlowRequest({ moduleName, request, onListen }) {
    const flowRequestRef = useRef();
    useEffect(() => {
        flowRequestRef.current = CommonRequestInstance.getFlow();
        flowRequestRef.current.initModule(moduleName);
        flowRequestRef.current.listen(moduleName, onListen);
        flowRequestRef.current.request(requestData, moduleName);
        return () => {
            var _a;
            (_a = flowRequestRef.current) === null || _a === void 0 ? void 0 : _a.destroyModule(moduleName);
        };
    }, []);
    const requestData = async () => {
        var _a;
        const data = await request();
        (_a = flowRequestRef.current) === null || _a === void 0 ? void 0 : _a.requestSuccess(data);
    };
    return;
}
const EVENT_NAME = 'flow-send-data';
/**
 * 流程各个组件的请求控制中心
 * @description
 * - 管理五个流程组件中公用的后端流程数据，只进行一次请求，将请求结果发送到流程中五个组件内部
 */
class CommonRequestInstance {
    constructor() {
        /** 请求状态 */
        Object.defineProperty(this, "requestState", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 'init'
        });
        /** 数据接收状态 */
        Object.defineProperty(this, "receivedState", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        /** 模块卸载状态 */
        Object.defineProperty(this, "moduleMountState", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        /** 消息通知实体 */
        Object.defineProperty(this, "messageHub", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new EventEmitter()
        });
        /** 公用数据缓存 */
        Object.defineProperty(this, "cacheData", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
    }
    static getFlow() {
        if (!CommonRequestInstance.flow) {
            CommonRequestInstance.flow = new CommonRequestInstance();
        }
        return CommonRequestInstance.flow;
    }
    /** 初始化模块信息，为模块卸载状态注册初始化标记 */
    initModule(moduleName) {
        this.moduleMountState[moduleName] = true;
    }
    /**
     * 模块销毁
     * @description 模块对应的UI组件卸载之后，需要标记到`moduleMountState`中，当所有模块对应组件都被卸载，则将此实例属性重置
     */
    destroyModule(moduleName) {
        this.moduleMountState[moduleName] = false;
        this.receivedState[moduleName] = false;
        const allUnmount = Object.values(this.moduleMountState).every((state) => !state);
        if (allUnmount) {
            this.requestState = 'init';
            this.moduleMountState = {};
            this.receivedState = {};
        }
        this.messageHub.removeAllListeners();
    }
    /**
     * 流程请求控制
     * @description
     *  - 当请求状态是loading，执行return 避免重复请求
     *  - 当请求状态是loaded，发送已请求的缓存数据
     */
    request(callback, moduleName) {
        if (this.requestState === 'loaded') {
            if (!this.receivedState[moduleName]) {
                this.send(this.cacheData);
                return;
            }
            return;
        }
        if (this.requestState === 'loading') {
            return;
        }
        this.requestState = 'loading';
        callback === null || callback === void 0 ? void 0 : callback();
    }
    /**
     * 数据请求成功后的回调
     */
    requestSuccess(data) {
        this.cacheData = data;
        this.send(data);
        this.requestState = 'loaded';
    }
    /**
     * 向各个模块发送数据
     */
    send(data) {
        this.messageHub.emit(EVENT_NAME, data);
    }
    /**
     * 模块接收数据
     */
    listen(moduleName, callback) {
        this.messageHub.once(EVENT_NAME, (data) => {
            if (!this.receivedState[moduleName]) {
                callback === null || callback === void 0 ? void 0 : callback(data);
                this.receivedState[moduleName] = true;
            }
        });
    }
}
