import React, { useState, useRef } from 'react';
import { ConfigProvider } from 'tea-component';
import { globalContextConnect, useGlobalContext } from '../../../context';
import { useMediaQuery } from '../../../hooks';
import { safeJsonParse } from '../../../utils';
import { getCommonFlowData } from './getCommonFlowData';
import { ICommonFlowDataProps } from './types';
import { CommonFlowDataUtils } from './utils';
import { useCommonFlowRequest, useElementMediaQuery } from './hooks';
export * from './types';
/**
 * 各个模块 通用的高阶函数 返回一个Container组件，包含一些公共属性和逻辑
 */
export function createModuleFrame(Component, moduleName) {
    const BaseModuleContainer = (props) => {
        const { events } = props;
        const [loading, setLoading] = useState(false);
        const [commonFlowData, setCommonFlowData] = useState(new ICommonFlowDataProps());
        const containerRef = useRef();
        const isH5 = useMediaQuery('(max-width: 1024px)');
        const isNarrowElement = useElementMediaQuery(containerRef.current, 768);
        const { IS_WEDA_IDE, callWedaApi } = useGlobalContext();
        const { currentPageType, flowTaskId } = CommonFlowDataUtils.getCurrentFlowPageQuery(IS_WEDA_IDE);
        useCommonFlowRequest({
            moduleName,
            request: async () => {
                var _a;
                setLoading(true);
                const data = await getCommonFlowData(flowLocalProps, flowTaskId);
                setLoading(false);
                (_a = events === null || events === void 0 ? void 0 : events.flowLoad) === null || _a === void 0 ? void 0 : _a.call(events, data.flowLoadData);
                return data;
            },
            onListen: (data) => {
                setCommonFlowData(data);
            },
        });
        const flowLocalProps = {
            isH5,
            isNarrowElement,
            isWedaIde: IS_WEDA_IDE,
            currentPageType,
            callWedaApi,
            safeJsonParse,
        };
        return (React.createElement(ConfigProvider, { classPrefix: "wedatea2td" },
            React.createElement("div", { ...props, ref: containerRef },
                React.createElement(Component, { ide: props, flowLocalProps: flowLocalProps, commonFlowData: commonFlowData, loading: loading }))));
    };
    return globalContextConnect(BaseModuleContainer);
}
