import React from 'react';
import moment from 'moment';
import { Card, Button, Form } from 'tea-component';
import { FlowStatusText, FlowModuleText, getModuleTextProps, getModuleContentStyle } from '../../components';
import { pressTask } from './utils';
import './Basic.less';
const { Title, Content } = FlowModuleText;
export const Basic = (props) => {
    const { commonFlowData, flowLocalProps, ide } = props;
    const { flowDetail, pageInfo } = commonFlowData;
    const { isWedaIde, callWedaApi, currentPageType } = flowLocalProps;
    const textProps = getModuleTextProps(ide);
    // 是否为发起流程页面
    const isStartFlowPage = ['DRAFT', 'CREATEFLOW', 'TODO'].includes(currentPageType) && (pageInfo === null || pageInfo === void 0 ? void 0 : pageInfo.showSubmitApp);
    const onPressTask = () => {
        const params = { instanceId: flowDetail.instanceId };
        const options = { isWedaIde, callWedaApi };
        pressTask(params, options);
    };
    const formOptions = [
        { label: '发起人', value: flowDetail.startedBy },
        {
            label: '发起时间',
            value: flowDetail.startTime ? moment(flowDetail.startTime).format('YYYY-MM-DD HH:mm:ss') : '--',
        },
        { label: '结束时间', value: flowDetail.endTime ? moment(flowDetail.endTime).format('YYYY-MM-DD HH:mm:ss') : '--' },
        {
            label: '流程状态',
            value: React.createElement(FlowStatusText, { flow: flowDetail, style: getModuleContentStyle(ide) }),
            useOriginValue: true,
        },
    ];
    if (!isStartFlowPage) {
        return (React.createElement(Card.Body, { className: "page-process__progress-card", title: React.createElement(Title, { ...textProps }), operation: React.createElement("div", { className: "detail__operations" }, currentPageType === 'CREATE' && !flowDetail.isCompleted && (React.createElement(Button, { type: "link", style: { fontSize: '14px' }, onClick: onPressTask }, "\u50AC\u529E"))) },
            React.createElement(Form, { className: "mf-progress-info__form" }, formOptions.map(({ label, value, useOriginValue }) => (React.createElement(Form.Item, { label: React.createElement(Content, { ...textProps }, `${label}：`) }, useOriginValue ? value : React.createElement(Content, { ...textProps }, value)))))));
    }
    return null;
};
