import React, { useState, useEffect } from 'react';
import { Drawer, Button, Form, Icon } from 'tea-component';
import { HighlightTextarea, UserSelectModel } from '../../components';
import { useControlContext } from './provider';
import './ApprovalDrawer.less';
const ExtraDOM = (props) => {
    const { 
    // fileList,
    approveInfo, onChange, 
    // appCode,
    type, operationName, statusFlag, setStatusFlag, statusMessage, setStatusMessage, opinionUserMap, setOpinionUserMap, setHighlightPositionMap, } = props;
    const approveInfoLabel = type === 'handle' ? `${operationName}意见` : '审批意见';
    return (React.createElement(Form.Item, { label: approveInfoLabel, required: type === 'reject', status: statusFlag, message: statusMessage },
        React.createElement(HighlightTextarea, { value: approveInfo, placeholder: `请输入您的${approveInfoLabel},可@通知他人`, opinionUserMap: opinionUserMap, setOpinionUserMap: setOpinionUserMap, setHighlightPositionMap: setHighlightPositionMap, onChange: (value) => {
                onChange('approveInfo', value);
                setStatusFlag('');
                setStatusMessage(null);
                if (type === 'reject' && !value.trim()) {
                    setStatusFlag('error');
                    setStatusMessage(`请输入${approveInfoLabel}`);
                }
                if (value.length > 500) {
                    setStatusFlag('error');
                    setStatusMessage(`${approveInfoLabel}长度不可超过500，请检查`);
                }
            } })));
};
const ApprovePerson = (props) => {
    const { label, approvers, show, onChange } = props;
    const removeItem = (index) => {
        const approversAvatar = approvers.filter((item, idx) => idx !== index);
        onChange(approversAvatar);
    };
    return (React.createElement(Form.Item, { label: label, className: "btn-wrapper__black person-btn__wrapper" },
        React.createElement(Button, { onClick: show },
            React.createElement("span", { style: { fontSize: '16px', lineHeight: '16px' } }, "+"),
            " ",
            '人员'),
        approvers.length > 0
            ? approvers.map(({ text }, index) => (React.createElement("div", { className: "person-item", key: `person-${index}` },
                React.createElement("span", null, text),
                React.createElement(Icon, { type: "dismiss", onClick: () => removeItem(index) }))))
            : null));
};
export const ApprovalDrawer = (props) => {
    var _a, _b, _c, _d, _e, _f;
    const { pageInfo, flowDetail } = props;
    const { handleType, approveDrawerVisible, submitLoading, onTriggerFlowEvent, onApproveDrawerVisibleChange } = useControlContext();
    const { startAppCode: appCode, currentNodeName: nodeName, operationName, currentApproverName } = flowDetail || {};
    const { buttons, showCarbonCopy } = pageInfo || {};
    const [fileList, setFileList] = useState((_a = props.fileList) !== null && _a !== void 0 ? _a : []);
    const [approveInfo, setApproveInfo] = useState((_b = props.approveInfo) !== null && _b !== void 0 ? _b : '');
    const [opinionUserMap, setOpinionUserMap] = useState({});
    const [highlightPositionMap, setHighlightPositionMap] = useState({});
    const [approvers, setApprovers] = useState((_c = props.approvers) !== null && _c !== void 0 ? _c : []);
    const [modalVisible, setModalVisible] = useState(false);
    const [statusFlag, setStatusFlag] = useState('');
    const [statusMessage, setStatusMessage] = useState(null);
    const showModal = () => setModalVisible(true);
    const hideModal = () => setModalVisible(false);
    const stateMap = {
        fileList: {
            default: [],
            fn: setFileList,
        },
        // addTag: {
        //   default: false,
        //   fn: setTagStatus,
        // },
        approveInfo: {
            default: '',
            fn: setApproveInfo,
        },
        approvers: {
            default: [],
            fn: setApprovers,
        },
        statusFlag: {
            default: '',
            fn: setStatusFlag,
        },
        statusMessage: {
            default: null,
            fn: setStatusMessage,
        },
    };
    const approvalType = {
        agree: {
            title: `${(buttons === null || buttons === void 0 ? void 0 : buttons[1]) || '同意'}`,
            dom: true,
            personLabel: '抄送',
        },
        reject: {
            title: `${(buttons === null || buttons === void 0 ? void 0 : buttons[2]) || '拒绝'}`,
            dom: true,
            personLabel: '抄送',
        },
        transfer: {
            title: '转办',
            dom: false,
            personLabel: '转办',
        },
        handle: {
            title: '处理',
            dom: true,
            personLabel: '处理',
        },
    };
    useEffect(() => {
        if (!approveDrawerVisible) {
            Object.values(stateMap).forEach(({ default: defaultValue, fn }) => fn(defaultValue));
        }
        else {
            // postUserInfo().then((res) => {
            //   if (res?.data) {
            //     setApprover(res?.data?.name);
            //   }
            // });
        }
    }, [approveDrawerVisible]);
    const handleSubmit = () => {
        var _a;
        // const getIdReg = /^(\d+)&(?:user|group),(?:.+)$/;
        if (submitLoading)
            return;
        const obj = {
            operationComment: approveInfo,
            opinionUserMap,
            highlightPositionList: JSON.stringify(highlightPositionMap),
            attachmentInfo: JSON.stringify(fileList),
            ccUserIds: approvers.map((item) => item.value).join(','),
            turnToUserId: (_a = approvers === null || approvers === void 0 ? void 0 : approvers[0]) === null || _a === void 0 ? void 0 : _a.value,
        };
        switch (handleType) {
            case 'agree':
                Reflect.deleteProperty(obj, 'turnToUserId');
                break;
            case 'reject':
                Reflect.deleteProperty(obj, 'ccUserIds');
                Reflect.deleteProperty(obj, 'turnToUserId');
                break;
            case 'transfer':
                Reflect.deleteProperty(obj, 'ccUserIds');
                break;
        }
        onTriggerFlowEvent(obj);
    };
    const onChange = (key, value) => {
        stateMap[key].fn(value);
    };
    return (React.createElement(React.Fragment, null,
        React.createElement(Drawer, { showMask: true, size: 'l', className: "approval-drawer drawer", 
            // outerClickClosable={false}
            visible: approveDrawerVisible, title: handleType === 'handle' ? `${operationName}意见` : `审批意见-${(_d = approvalType === null || approvalType === void 0 ? void 0 : approvalType[handleType]) === null || _d === void 0 ? void 0 : _d.title}`, onClose: onApproveDrawerVisibleChange, footer: React.createElement("div", { className: "btn-wrapper" },
                React.createElement(Button, { className: "submit-btn", type: "primary", loading: submitLoading, onClick: () => {
                        if (handleType === 'reject' && !approveInfo.trim()) {
                            setStatusFlag('error');
                            setStatusMessage('请输入审批意见');
                            return;
                        }
                        if (approveInfo.length > 500) {
                            setStatusFlag('error');
                            setStatusMessage('审批意见长度不可超过500，请检查');
                            return;
                        }
                        handleSubmit();
                    } }, "\u63D0\u4EA4"),
                React.createElement(Button, { onClick: onApproveDrawerVisibleChange }, "\u53D6\u6D88")) },
            React.createElement(Form, { className: "approval-form__drawer" },
                React.createElement(Form.Item, { label: '当前节点' },
                    React.createElement("span", { className: "form-item__plain" }, nodeName)),
                React.createElement(Form.Item, { label: handleType === 'handle' ? '处理人' : '审批人' },
                    React.createElement("span", { className: "form-item__plain" }, currentApproverName || '--')),
                ((_e = approvalType === null || approvalType === void 0 ? void 0 : approvalType[handleType]) === null || _e === void 0 ? void 0 : _e.dom) ? (React.createElement(ExtraDOM, { type: handleType, appCode: appCode, onChange: onChange, fileList: fileList, approveInfo: approveInfo, operationName: operationName, statusFlag: statusFlag, setStatusFlag: setStatusFlag, statusMessage: statusMessage, setStatusMessage: setStatusMessage, opinionUserMap: opinionUserMap, setOpinionUserMap: setOpinionUserMap, setHighlightPositionMap: setHighlightPositionMap })) : null,
                handleType !== 'reject' && (showCarbonCopy || handleType === 'transfer') ? (React.createElement(ApprovePerson, { approvers: approvers, label: (_f = approvalType === null || approvalType === void 0 ? void 0 : approvalType[handleType]) === null || _f === void 0 ? void 0 : _f.personLabel, show: showModal, onChange: (val) => onChange('approvers', val) })) : null)),
        React.createElement(UserSelectModel, { visible: modalVisible, onChange: (val) => onChange('approvers', val), onClose: hideModal, value: approvers })));
};
