import React, { useState, useEffect, useRef } from 'react';
import { Icon, Button } from 'tea-component';
import { Popup, Field } from 'react-vant';
import { HighlightTextarea } from '../../components';
import { useControlContext } from './provider';
import { ACTION_TYPE } from './constants';
import './ApprovalPopup.less';
const ExtraDOM = (props) => {
    const { type, statusMessage, setStatusMessage, operationName, opinionUserMap, setOpinionUserMap, setHighlightPositionMap, prevFn, onUserSelectModelVisibleChange, onUserSelectModalInfoChange, safeJsonParse, userSelectModelVisible, } = props;
    const [approveInfo, setApproveInfo] = useState('');
    useEffect(() => {
        if (!userSelectModelVisible) {
            const popupData = sessionStorage.getItem('FlowActionsMobile');
            if (popupData) {
                const { approveInfo: localApproveInfo } = safeJsonParse(popupData);
                setApproveInfo(localApproveInfo);
            }
        }
    }, [userSelectModelVisible]);
    const toSelectPerson = () => {
        prevFn === null || prevFn === void 0 ? void 0 : prevFn(approveInfo);
        onUserSelectModelVisibleChange();
        onUserSelectModalInfoChange({ approveField: 'highLightApprovers', mode: 'multiple' });
    };
    const approveInfoLabel = type === 'handle' ? `${operationName}意见` : '审批意见';
    return (React.createElement(React.Fragment, null,
        React.createElement(Field, { className: "highlightTextarea-cell_wrapper", label: React.createElement("div", { className: "highlightTextarea-mobile__wrapper" },
                React.createElement("span", { className: "label" },
                    type === 'reject' && React.createElement("span", { style: { color: '#ee0a24' } }, "*"),
                    approveInfoLabel),
                React.createElement("span", { onClick: toSelectPerson, className: "extra-dom" }, "@\u67D0\u4EBA"),
                React.createElement(HighlightTextarea, { value: approveInfo || '', size: "mobile", placeholder: `请输入您的${approveInfoLabel},可@通知他人`, opinionUserMap: opinionUserMap, setOpinionUserMap: setOpinionUserMap, setHighlightPositionMap: setHighlightPositionMap, onChange: (value) => {
                        setApproveInfo(value);
                        setStatusMessage(null);
                        if (type === 'reject' && !value.trim()) {
                            setStatusMessage(`请输入${approveInfoLabel}`);
                        }
                        if (value.length > 500) {
                            setStatusMessage(`${approveInfoLabel}长度不可超过500，请检查`);
                        }
                    } }),
                statusMessage && React.createElement("div", { className: "pant-field__error-message" }, statusMessage)), placeholder: "" })));
};
const ApprovePerson = (props) => {
    const { label, onChange, approvers, prevFn, type, approveField = 'approvers', onUserSelectModelVisibleChange, onUserSelectModalInfoChange, safeJsonParse, } = props;
    const getUserMap = () => {
        var _a;
        return (_a = safeJsonParse(sessionStorage.getItem('approvers'))) === null || _a === void 0 ? void 0 : _a.reduce((p, c) => {
            // eslint-disable-next-line no-param-reassign
            p[c === null || c === void 0 ? void 0 : c.UserId] = c === null || c === void 0 ? void 0 : c.Name;
            return p;
        }, {});
    };
    const toSelectPerson = () => {
        prevFn === null || prevFn === void 0 ? void 0 : prevFn();
        onUserSelectModelVisibleChange();
        onUserSelectModalInfoChange({
            approveField,
            mode: [ACTION_TYPE.AGREE, ACTION_TYPE.TRANSFER].includes(type) ? 'multiple' : 'single',
        });
    };
    const removeItem = (index) => {
        const approversAvatar = approvers.filter((item, idx) => idx !== index);
        onChange(approversAvatar);
    };
    return (React.createElement(React.Fragment, null,
        React.createElement(Field, { label: React.createElement(React.Fragment, null,
                React.createElement("span", { className: "label" }, label)), className: "label-execute__btn", name: "approvers", placeholder: "" },
            React.createElement("span", { className: "btn-show-modal", onClick: toSelectPerson },
                React.createElement("i", { className: "apa-icon apa-icon-plus-blue" }),
                "\u4EBA\u5458")),
        (approvers === null || approvers === void 0 ? void 0 : approvers.length) > 0
            ? approvers === null || approvers === void 0 ? void 0 : approvers.map((item, index) => {
                var _a;
                return (React.createElement("div", { className: "person-item", key: `person-item_${index}` },
                    React.createElement("span", null, ((_a = getUserMap()) === null || _a === void 0 ? void 0 : _a[item]) || item),
                    React.createElement(Icon, { type: "dismiss", onClick: () => removeItem(index) })));
            })
            : null));
};
export const ApprovalPopup = (props) => {
    var _a, _b, _c, _d, _e;
    const { pageInfo, flowDetail, safeJsonParse } = props;
    const { handleType, approveDrawerVisible, submitLoading, approver, approvers, fileList, opinionUserMap, highlightPositionMap, userSelectModelVisible, setFileList, setApprovers, setApprover, setOpinionUserMap, setHighlightPositionMap, onApproveDrawerVisibleChange, onUserSelectModelVisibleChange, onUserSelectModalInfoChange, onTriggerFlowEvent, } = useControlContext();
    const { startAppCode: appCode, currentNodeName: nodeName, operationName } = flowDetail || {};
    const { buttons, showCarbonCopy } = pageInfo || {};
    const [statusMessage, setStatusMessage] = useState('');
    const approveInfoRef = useRef('');
    const stateMap = {
        fileList: {
            default: [],
            fn: setFileList,
        },
        approvers: {
            default: [],
            fn: setApprovers,
        },
        approver: {
            default: '',
            fn: setApprover,
        },
        statusMessage: {
            default: '',
            fn: setStatusMessage,
        },
    };
    const approvalType = {
        agree: {
            title: `${(buttons === null || buttons === void 0 ? void 0 : buttons[1]) || '同意'}`,
            dom: true,
            personLabel: '抄送',
        },
        reject: {
            title: `${(buttons === null || buttons === void 0 ? void 0 : buttons[2]) || '拒绝'}`,
            dom: true,
            personLabel: '抄送',
        },
        transfer: {
            title: '转办',
            dom: false,
            personLabel: '转办',
        },
        handle: {
            title: '处理',
            dom: true,
            personLabel: '抄送',
        },
    };
    useEffect(() => {
        if (!approveDrawerVisible) {
            // resetValue?.();
            Object.values(stateMap).forEach(({ default: defaultValue, fn }) => fn(defaultValue));
        }
        else if (!approver) {
            if (!props.approver) {
                // postUserInfo().then((res) => {
                //   if (res?.data) {
                //     setApprover(res?.data?.name);
                //   }
                // });
            }
            else {
                Object.entries(stateMap).forEach(([key, value]) => value.fn(props[key]));
            }
        }
    }, [approveDrawerVisible]);
    useEffect(() => {
        setApprovers(props.approvers);
    }, [props.approvers]);
    useEffect(() => {
        setFileList(props.fileList);
    }, [props.fileList]);
    useEffect(() => {
        var _a;
        setOpinionUserMap((_a = props.opinionUserMap) !== null && _a !== void 0 ? _a : {});
    }, [props.opinionUserMap]);
    useEffect(() => {
        var _a;
        setHighlightPositionMap((_a = props.highlightPositionMap) !== null && _a !== void 0 ? _a : {});
    }, [props.highlightPositionMap]);
    const onChange = (key, value) => {
        stateMap[key].fn(value);
    };
    const handleSubmit = () => {
        var _a;
        const obj = {
            operationComment: approveInfoRef.current || '',
            opinionUserMap,
            highlightPositionList: JSON.stringify(highlightPositionMap),
            attachmentInfo: JSON.stringify(fileList),
            ccUserIds: approvers === null || approvers === void 0 ? void 0 : approvers.map((item) => item.value).join(','),
            turnToUserId: (_a = approvers === null || approvers === void 0 ? void 0 : approvers[0]) === null || _a === void 0 ? void 0 : _a.value,
        };
        switch (handleType) {
            case 'agree':
                Reflect.deleteProperty(obj, 'turnToUserId');
                break;
            case 'reject':
                Reflect.deleteProperty(obj, 'ccUserIds');
                Reflect.deleteProperty(obj, 'turnToUserId');
                break;
            case 'transfer':
                Reflect.deleteProperty(obj, 'ccUserIds');
                break;
        }
        onTriggerFlowEvent(obj);
    };
    const dynamiticHeight = handleType === 'transfer' ? '500px' : '540px';
    const saveTemp = (approveInfo) => {
        approveInfoRef.current = approveInfo;
        const temp = {
            fileList,
            approveInfo,
            opinionUserMap,
            highlightPositionMap,
            approvers,
            handleType,
            approveDrawerVisible,
            appCode,
            // addApprovers,
            // tagType,
            // addTag,
            nodeName,
            approver,
        };
        sessionStorage.setItem('FlowActionsMobile', JSON.stringify(temp));
    };
    return (React.createElement(React.Fragment, null,
        React.createElement(Popup, { visible: approveDrawerVisible, position: "bottom", style: { height: dynamiticHeight }, onClose: onApproveDrawerVisibleChange, closeable: true, className: "cp-vant-popup", overlayStyle: { zIndex: 998 } },
            React.createElement(React.Fragment, null,
                React.createElement("div", { className: "popup-content-wrapper" },
                    React.createElement("h3", { className: "popup-title" }, handleType === 'handle' ? `${operationName}意见` : `审批意见 - ${(_a = approvalType === null || approvalType === void 0 ? void 0 : approvalType[handleType]) === null || _a === void 0 ? void 0 : _a.title}`),
                    React.createElement(Field, { value: nodeName, label: "\u5F53\u524D\u8282\u70B9", readOnly: true }),
                    React.createElement(Field, { value: ((_c = (_b = window === null || window === void 0 ? void 0 : window._WedaHostConfig) === null || _b === void 0 ? void 0 : _b.wedaUser) === null || _c === void 0 ? void 0 : _c.name) || '', label: handleType === 'handle' ? '处理人' : '审批人', readOnly: true }),
                    ((_d = approvalType === null || approvalType === void 0 ? void 0 : approvalType[handleType]) === null || _d === void 0 ? void 0 : _d.dom) && approveDrawerVisible ? (React.createElement(ExtraDOM, { type: handleType, appCode: appCode, onChange: onChange, fileList: fileList, operationName: operationName, statusMessage: statusMessage, setStatusMessage: setStatusMessage, opinionUserMap: opinionUserMap, setOpinionUserMap: setOpinionUserMap, setHighlightPositionMap: setHighlightPositionMap, prevFn: saveTemp, onUserSelectModelVisibleChange: onUserSelectModelVisibleChange, onUserSelectModalInfoChange: onUserSelectModalInfoChange, safeJsonParse: safeJsonParse, userSelectModelVisible: userSelectModelVisible })) : null,
                    handleType !== 'reject' && (showCarbonCopy || handleType === 'transfer') ? (React.createElement(ApprovePerson, { type: handleType, prevFn: saveTemp, approvers: approvers, label: (_e = approvalType === null || approvalType === void 0 ? void 0 : approvalType[handleType]) === null || _e === void 0 ? void 0 : _e.personLabel, onChange: (val) => onChange('approvers', val), onUserSelectModelVisibleChange: onUserSelectModelVisibleChange, onUserSelectModalInfoChange: onUserSelectModalInfoChange, safeJsonParse: safeJsonParse })) : null),
                React.createElement(Button, { className: "success-btn apa-height-xl", type: "primary", loading: submitLoading, onClick: () => {
                        var _a;
                        if (handleType === 'reject' && !approveInfoRef.current.trim()) {
                            setStatusMessage('请输入审批意见');
                            return;
                        }
                        if (((_a = approveInfoRef.current) === null || _a === void 0 ? void 0 : _a.length) > 500) {
                            setStatusMessage('审批意见长度不可超过500，请检查');
                            return;
                        }
                        handleSubmit();
                    } }, "\u63D0\u4EA4")))));
};
