import { useState, useRef, useEffect } from 'react';
import { message } from 'tea-component';
import { postApprovalFlow, postApprovalSign, postFlowTaskList } from '../../../services';
import { ACTION_TYPE, ACTION_TYPE_MAP } from '../constants';
import { ControlModelUtils } from '../utils';
export const useControlModel = (params) => {
    const { commonFlowData, flowLocalProps, ide } = params;
    const { flowDetail, pageInfo, ccUserMaps } = commonFlowData;
    const { isWedaIde, callWedaApi } = flowLocalProps;
    const { events } = ide || {};
    const ifSubmitAndCc = (pageInfo === null || pageInfo === void 0 ? void 0 : pageInfo.showSubmitApp) && (pageInfo === null || pageInfo === void 0 ? void 0 : pageInfo.showCarbonCopy);
    const [approveDrawerVisible, setApproveDrawerVisible] = useState(false);
    const [userModalVisible, setUserModalVisible] = useState(false);
    const [submitApplyLoading, setSubmitApplyLoading] = useState(false);
    const [submitLoading, setSubmitLoading] = useState(false);
    const [handleType, setHandleType] = useState('');
    const [flowTaskInfoModalVisible, setFlowTaskInfoModalVisible] = useState(false); // 下一节点展示人弹窗
    const [flowTaskInfoList, setFlowTaskInfoList] = useState([]); // 审批人列表
    const [allInfos, setAllInfos] = useState([]);
    const [flowTaskIds, setFlowTaskIds] = useState([]);
    const [ccUserModelVisible, setCcUserModelVisible] = useState(false);
    const [ccUsers, setCcUsers] = useState([]);
    useEffect(() => {
        var _a;
        const ccUsers = (_a = Object.entries(ccUserMaps || {})) === null || _a === void 0 ? void 0 : _a.map(([value, text]) => ({
            text,
            value,
        }));
        setCcUsers(ccUsers);
    }, [ccUserMaps]);
    const pageFormData = useRef({});
    const onAddCounterSign = async (assigneeIds) => {
        try {
            const res = await postApprovalSign({
                instanceId: flowDetail.instanceId,
                nodeId: flowDetail.currentNodeId,
                taskId: flowDetail.taskId,
                assigneeIds: assigneeIds.map((item) => item.value),
            }, { callWedaApi });
            (res === null || res === void 0 ? void 0 : res.status) && location.reload();
        }
        catch (ex) {
            console.log(ex);
        }
    };
    const onControlChick = (type, e) => {
        var _a;
        if (isWedaIde) {
            return;
        }
        (_a = events === null || events === void 0 ? void 0 : events.tap) === null || _a === void 0 ? void 0 : _a.call(events, {
            onGetFormData: (value) => {
                pageFormData.current = { ...pageFormData.current, ...value };
                if (type === ACTION_TYPE.SUBMIT) {
                    onTriggerFlowEvent({ buttonAction: ACTION_TYPE_MAP.submit });
                }
                else if (type === ACTION_TYPE.SAVEDRAFT) {
                    onSaveDraft();
                }
                else {
                    onApproveDrawerVisibleChange();
                    setHandleType(type);
                }
            },
        }, {
            originEvent: e,
        });
    };
    const onApproveDrawerVisibleChange = () => {
        const newApproveDrawerVisible = !approveDrawerVisible;
        setApproveDrawerVisible(newApproveDrawerVisible);
        !approveDrawerVisible && setHandleType('');
    };
    const onUserModalVisibleChange = () => {
        setUserModalVisible(!userModalVisible);
    };
    const onFlowTaskInfoModalVisibleChange = () => {
        setFlowTaskInfoModalVisible(!flowTaskInfoModalVisible);
    };
    const convertFormat = (data) => {
        if (data.length) {
            return data.map((v) => {
                var _a, _b, _c, _d;
                const item = { ...v };
                if (!((_a = item.value) === null || _a === void 0 ? void 0 : _a.recordId) && ((_b = item.value) === null || _b === void 0 ? void 0 : _b.fieldValueMap)) {
                    item.value.recordId = ((_d = (_c = item.value) === null || _c === void 0 ? void 0 : _c.fieldValueMap) === null || _d === void 0 ? void 0 : _d.recordId) || '';
                }
                return item;
            });
        }
        return [];
    };
    const onSaveDraft = () => {
        const params = {
            buttonAction: ACTION_TYPE_MAP[ACTION_TYPE.SAVEDRAFT],
            ccUserIds: (ccUsers === null || ccUsers === void 0 ? void 0 : ccUsers.length) ? ccUsers.map(({ value }) => value).join(',') : undefined,
            flowCode: flowDetail.flowCode,
            instanceId: flowDetail.instanceId,
            nodeId: flowDetail.currentNodeId,
            taskId: flowDetail.taskId,
            appVersion: flowDetail.flowVersion,
            variableInfos: ControlModelUtils.translateVariableInfos(pageFormData.current, pageInfo) || [],
        };
        params.variableInfos = convertFormat(params.variableInfos);
        postApprovalFlow({ ...params, AppCode: flowDetail.startAppCode }, { callWedaApi })
            .then((res) => {
            if (res) {
                message.success({
                    content: '保存草稿成功',
                });
                ControlModelUtils.navigateToFlowCenter();
            }
        })
            .catch((err) => {
            message.error({ content: '保存失败' });
        });
    };
    const onTriggerFlowEvent = (data) => {
        setSubmitApplyLoading(true);
        if (!pageFormData.current) {
            message.warning({
                content: '请检查申请信息是否填写完整',
            });
            setSubmitApplyLoading(false);
            return;
        }
        setSubmitLoading(true);
        const params = {
            buttonAction: ACTION_TYPE_MAP[handleType],
            ccUserIds: (ccUsers === null || ccUsers === void 0 ? void 0 : ccUsers.length) ? ccUsers.map(({ value }) => value).join(',') : undefined,
            flowCode: flowDetail.flowCode,
            instanceId: flowDetail.instanceId,
            nodeId: flowDetail.currentNodeId,
            taskId: flowDetail.taskId,
            // turnToUserId: data.turnToUserId,
            appVersion: flowDetail.flowVersion,
            variableInfos: ControlModelUtils.translateVariableInfos(pageFormData.current, pageInfo) || [],
            ...data,
        };
        params.variableInfos = convertFormat(params.variableInfos);
        postApprovalFlow({ ...params, AppCode: flowDetail === null || flowDetail === void 0 ? void 0 : flowDetail.startAppCode }, { callWedaApi })
            .then((res) => {
            var _a;
            if (res) {
                sessionStorage.removeItem('FlowActionsMobile');
                if (res.toBeSubmittedFlowTaskFlag) {
                    setApproveDrawerVisible(false);
                    setFlowTaskInfoModalVisible(res === null || res === void 0 ? void 0 : res.toBeSubmittedFlowTaskFlag);
                    setFlowTaskInfoList((res === null || res === void 0 ? void 0 : res.toBeSubmittedFlowTaskInfo) || []);
                    const tempArr = [];
                    (_a = res === null || res === void 0 ? void 0 : res.toBeSubmittedFlowTaskInfo) === null || _a === void 0 ? void 0 : _a.forEach((item) => {
                        tempArr.push(item.waitFlowTaskId);
                    });
                    setAllInfos(tempArr);
                    setTimeout(() => {
                        postFlowTaskList({
                            flowTaskIds: flowTaskIds.length ? flowTaskIds : tempArr,
                        }, { callWedaApi }).then((res) => {
                            if (res.status) {
                                setFlowTaskInfoModalVisible(false);
                                ControlModelUtils.navigateToFlowCenter();
                            }
                        });
                    }, 600000);
                }
                else {
                    ControlModelUtils.navigateToFlowCenter();
                }
            }
        })
            .finally(() => {
            setSubmitLoading(false);
            setSubmitApplyLoading(false);
        });
    };
    // 取消流程编辑 清除流程记录
    const onClearInstance = async () => {
        // history.push(`/flowCenter?tabId=${pageType}`);
        ControlModelUtils.navigateToFlowCenter();
    };
    const onCloseFlowTaskInfoModal = () => {
        postFlowTaskList({
            flowTaskIds: flowTaskIds.length ? flowTaskIds : allInfos,
        }, { callWedaApi }).then((res) => {
            if (res.status) {
                onFlowTaskInfoModalVisibleChange();
                ControlModelUtils.navigateToFlowCenter();
            }
        });
    };
    return {
        userModalVisible,
        approveDrawerVisible,
        submitApplyLoading,
        submitLoading,
        handleType,
        flowTaskInfoModalVisible,
        flowTaskInfoList,
        allInfos,
        flowTaskIds,
        ccUserModelVisible,
        ccUsers,
        ifSubmitAndCc,
        setUserModalVisible,
        setCcUsers,
        setCcUserModelVisible,
        onAddCounterSign,
        onClearInstance,
        onControlChick,
        onApproveDrawerVisibleChange,
        onUserModalVisibleChange,
        onTriggerFlowEvent,
        onFlowTaskInfoModalVisibleChange,
        setFlowTaskIds,
        onSaveDraft,
        onCloseFlowTaskInfoModal,
    };
};
