import { useState } from 'react';
import { revokeFlow, fetchBackNodeList, rollBackFlow } from '../../../services';
import { ControlModelUtils } from '../utils';
// 撤销/回退逻辑
export const useRevokeAndBackHook = (params) => {
    const { flowDetail } = params.commonFlowData;
    const { callWedaApi } = params.flowLocalProps;
    const [showRevokeModal, setShowRevokeModal] = useState(false);
    const [showBackModal, setShowBackModal] = useState(false);
    const [backNodeList, setBackNodeList] = useState([]);
    const [backNode, setBackNode] = useState('');
    const doneRevokeFn = (param) => {
        const { pageType } = param;
        // 调用撤销接口
        revokeFlow({
            flowCode: flowDetail.flowCode,
            appVersion: flowDetail.flowVersion,
            instanceId: flowDetail.instanceId,
            revokeType: pageType === 'CREATE' ? 2 : 1,
            taskId: pageType === 'DONE' ? flowDetail.taskId : undefined,
        }, { callWedaApi }).then((res) => {
            if (res) {
                setShowRevokeModal(false);
                // history.push('/flowCenter');
                ControlModelUtils.navigateToFlowCenter();
            }
        });
    };
    const getNodeList = () => 
    // 获取回退节点列表
    fetchBackNodeList({
        flowCode: flowDetail.flowCode,
        appVersion: flowDetail.flowVersion,
        instanceId: flowDetail.instanceId,
    }, { callWedaApi }).then((res) => {
        if (res === null || res === void 0 ? void 0 : res.data) {
            let preNodeId = '';
            const options = res.data.map((item) => {
                if (item.lastNode) {
                    preNodeId = item.elementId;
                }
                return {
                    ...item,
                    value: item.elementId,
                    text: item.elementName,
                };
            });
            setBackNodeList(options);
            setBackNode(preNodeId);
        }
    });
    const doneBackFn = () => {
        var _a;
        // 调用回退接口
        rollBackFlow({
            flowCode: flowDetail.flowCode,
            appVersion: flowDetail.flowVersion,
            instanceId: flowDetail.instanceId,
            taskId: flowDetail.taskId,
            nodeId: flowDetail.currentNodeId,
            sourceType: 1,
            targetNodeId: backNode,
            targetActTaskId: (_a = backNodeList.find((item) => item.value === backNode)) === null || _a === void 0 ? void 0 : _a.actTaskId,
        }, { callWedaApi }).then((res) => {
            if (res) {
                setShowBackModal(false);
                // history.push('/flowCenter');
                ControlModelUtils.navigateToFlowCenter();
            }
        });
    };
    return {
        showRevokeModal,
        setShowRevokeModal,
        doneRevokeFn,
        doneBackFn,
        showBackModal,
        setShowBackModal,
        getNodeList,
        backNodeList,
        backNode,
        setBackNode,
    };
};
