import React from 'react';
import moment from 'moment';
import classNames from 'classnames';
import { Text, Status, LoadingTip, Card } from 'tea-component';
import { HighLightComment, FlowModuleText, getModuleTextProps, getModuleContentStyle } from '../../components';
import { STATUS } from '../../constants';
import { getStatusStr, showTwoUserList } from './utils';
const ApproveProcess = (props) => {
    const { commonFlowData, flowLocalProps, ide, loading } = props;
    const flowModelContentStyle = getModuleContentStyle(ide);
    const { steps } = commonFlowData;
    const { safeJsonParse } = flowLocalProps;
    const renderContent = () => {
        if (loading) {
            return React.createElement(LoadingTip, null);
        }
        if ((steps === null || steps === void 0 ? void 0 : steps.length) === 0) {
            return React.createElement(Status, { icon: "blank", size: "l", title: "\u6682\u65E0\u6D41\u8F6C\u8BB0\u5F55", ...props });
        }
        return (React.createElement("div", { className: classNames('page-process__progress', 'page-process__progress__app-page') },
            React.createElement("div", { className: "approval-progress" }, (steps || []).map((item, index) => {
                var _a, _b, _c, _d, _e, _f;
                const details = item.contentInfo;
                return (React.createElement("div", { className: "approval-progress__item", key: index },
                    React.createElement("div", { className: `approval-progress__dot ${!item.status ? '' : item.status}` }),
                    React.createElement("div", { className: "approval-progress__content" },
                        React.createElement("div", { className: "approval-progress__row tea-mb-2n" },
                            item.status === STATUS.DROP ? (React.createElement("div", { className: "approval-progress__dept", style: { color: 'rgb(231, 87, 87)', ...flowModelContentStyle } }, item.result)) : (React.createElement("div", { className: "approval-progress__dept", style: flowModelContentStyle }, item.name)),
                            ![STATUS.CC, STATUS.DROP].includes(item.status) && (React.createElement("div", { className: "approval-progress__user", style: flowModelContentStyle },
                                item.status === STATUS.START ? '发起人' : '审批人',
                                "\uFF1A",
                                showTwoUserList(item.operaName, item.approvalStrategy))),
                            ![STATUS.START, STATUS.END, STATUS.CC].includes(item.status) && getStatusStr(item.status)),
                        React.createElement("div", { className: "approval-progress__row  approval-progress__row--start tea-mb-2n" },
                            details.createTime && (React.createElement(Text, { theme: "weak", className: "approval-progress__date" }, moment(details.createTime).format('YYYY-MM-DD HH:mm:ss'))),
                            item.status === STATUS.CC && (React.createElement(Text, { theme: "weak", style: { maxWidth: 500 }, overflow: true, tooltip: true },
                                "\u6284\u9001\uFF1A", (_b = (_a = details.ccUserInfos) === null || _a === void 0 ? void 0 : _a.map((i) => i.name)) === null || _b === void 0 ? void 0 :
                                _b.join('、'))),
                            details.ccUsers && (React.createElement(Text, { theme: "weak", style: { maxWidth: 500 }, overflow: true, tooltip: true },
                                "\u6284\u9001\uFF1A", (_c = Object.values(details.ccUsers)) === null || _c === void 0 ? void 0 :
                                _c.join('、'))),
                            item.status === STATUS.TRANS && (React.createElement(Text, { theme: "weak" },
                                "\u56DE\u9000\u5230\u6307\u5B9A\u8282\u70B9\uFF1A",
                                (details === null || details === void 0 ? void 0 : details.operation) ? (_d = safeJsonParse(details === null || details === void 0 ? void 0 : details.operation)[1]) === null || _d === void 0 ? void 0 : _d.name : '')),
                            !!details.operationComment && (React.createElement(Text, { theme: "weak" },
                                "\u5BA1\u6279\u610F\u89C1\uFF1A",
                                React.createElement(HighLightComment, { details: details })))),
                        React.createElement("div", { className: "approval-progress__row", style: {
                                flexDirection: 'column',
                                alignItems: 'flex-start',
                            } },
                            item.status === 'countersign' && (React.createElement(Text, { theme: "weak", className: "approval-progress__advice-item" },
                                React.createElement("i", { className: "apa-icon apa-icon-label tea-mr-2n" }),
                                "\u52A0\u7B7E\uFF1A",
                                (details === null || details === void 0 ? void 0 : details.operation)
                                    ? safeJsonParse(details.operation)
                                        .map((i) => i.name)
                                        .join('，')
                                    : '')),
                            details.isCC && (React.createElement(Text, { theme: "weak", className: "approval-progress__advice-item" },
                                React.createElement("i", { className: "apa-icon apa-icon-trans tea-mr-2n" }),
                                "\u6284\u9001\uFF1A", (_f = (_e = details.ccUserInfos) === null || _e === void 0 ? void 0 : _e.map((i) => i.name)) === null || _f === void 0 ? void 0 :
                                _f.join('、')))))));
            }))));
    };
    return (React.createElement(Card.Body, { className: "page-process__progress-card", title: React.createElement(FlowModuleText.Title, { ...getModuleTextProps(ide) }) }, renderContent()));
};
export default ApproveProcess;
