import { toTransKeyCase } from './utils';
import { chartMockData, pageDetailMockData, processMockData, instanceMockData } from './ideData';
// 获取批办过程接口
export const fetchInstanceOperation = async (params, { callWedaApi, isWedaIde }) => {
    if (isWedaIde) {
        return toTransKeyCase(processMockData, 'toLowerCase');
    }
    if (!(params === null || params === void 0 ? void 0 : params.instanceId)) {
        return;
    }
    const result = await callWedaApi({
        action: 'GetInstanceOperation',
        data: toTransKeyCase(params, 'toUpperCase'),
    });
    return toTransKeyCase(result, 'toLowerCase');
};
/**
 * 获取流程实例运行图（新版）
 * @param {instanceId: string} params
 */
export const fetchProcessInfoList = async (params, { isWedaIde, callWedaApi }) => {
    if (isWedaIde) {
        return toTransKeyCase(chartMockData, 'toLowerCase');
    }
    const result = await callWedaApi({
        action: 'GetProcessCanvasInfo',
        data: toTransKeyCase(params, 'toUpperCase'),
    });
    return toTransKeyCase(result, 'toLowerCase');
};
/**
 * 获取流程信息
 * @param {instanceId: string} params
 */
export const fetchProcess = async (params, { isWedaIde, callWedaApi }) => {
    if (isWedaIde) {
        return toTransKeyCase(instanceMockData, 'toLowerCase');
    }
    const result = await callWedaApi({
        action: 'DescribeProcessInstance',
        data: toTransKeyCase(params, 'toUpperCase'),
    });
    return toTransKeyCase(result, 'toLowerCase');
};
// 获取流程信息详情
export async function getApprovalPageDetail(params, { isWedaIde, callWedaApi }) {
    if (isWedaIde) {
        return toTransKeyCase(pageDetailMockData, 'toLowerCase');
    }
    const result = await callWedaApi({
        action: 'GetApprovalPageDetail',
        data: toTransKeyCase(params, 'toUpperCase'),
    });
    return toTransKeyCase(result, 'toLowerCase');
}
//  撤销流程接口
export const revokeFlow = async (params, { callWedaApi }) => {
    const result = await callWedaApi({
        action: 'Revoke',
        data: toTransKeyCase(params, 'toUpperCase'),
    });
    return toTransKeyCase(result, 'toLowerCase');
};
// 获取回退节点列表
export const fetchBackNodeList = async (params, { callWedaApi }) => {
    const result = await callWedaApi({
        action: 'GetPreElementList',
        data: toTransKeyCase(params, 'toUpperCase'),
    });
    return toTransKeyCase(result, 'toLowerCase');
};
// 回退流程接口
export const rollBackFlow = async (params, { callWedaApi }) => {
    const result = await callWedaApi({
        action: 'RollBack',
        data: toTransKeyCase(params, 'toUpperCase'),
    });
    return toTransKeyCase(result, 'toLowerCase');
};
// 审批流页面提交
export const postApprovalFlow = async (params, { callWedaApi }) => {
    const result = await callWedaApi({
        action: 'CompleteTask',
        data: toTransKeyCase(params, 'toUpperCase', 'variableInfos'),
    });
    return toTransKeyCase(result, 'toLowerCase');
};
// 审批流页面会签
export const postApprovalSign = async (params, { callWedaApi }) => {
    const result = await callWedaApi({
        action: 'AddCounterSignAssignee',
        data: toTransKeyCase(params, 'toUpperCase'),
    });
    return toTransKeyCase(result, 'toLowerCase');
};
// 审批人列表提交
export const postFlowTaskList = async (params, { callWedaApi }) => {
    const result = await callWedaApi({
        action: 'SubmitFlowTaskList',
        data: toTransKeyCase(params, 'toUpperCase'),
    });
    return toTransKeyCase(result, 'toLowerCase');
};
// 流程催办
export const postPressTask = async (params, { callWedaApi }) => {
    const result = await callWedaApi({
        action: 'PressTask',
        data: toTransKeyCase(params, 'toUpperCase'),
    });
    return toTransKeyCase(result, 'toLowerCase');
};
