import React, { useMemo } from 'react';
import { createForm, FormProvider, Field } from '../formily-react';
import { Modal, Form, Input } from 'tea-component';
import { ApprovalUserSelector } from './ApprovalUserSelector';
import { Submit, FormItem } from '../Form/FormItem';
import { SIGN_MESSAGE, SIGN_TYPE_MAP, TURN_SIGN_MESSAGE } from '../../constants';
import Space from '../Space';
import { useFlowContext } from './Context';
import './index.less';
const REQUEST_URL_MAP = {
    FORWARD_SIGN: 'AddSignTask',
    BACKWARD_SIGN: 'AddSignTask',
    TURN_SIGN: 'TurnSignTask',
};
export const ApprovalSignModal = (props) => {
    const { onClose, visible, doAfterSubmit, userNode, modalSignType, nodes } = props || {};
    const { wedaUserId, callWorkflow, showError } = useFlowContext();
    let title;
    const form = useMemo(() => createForm(), []);
    const { SignTaskInfo = {} } = userNode || {};
    const { SignTaskTypeList = [] } = SignTaskInfo;
    let SignTaskType = SignTaskTypeList[0];
    // 加签
    if (modalSignType === 'addSign') {
        title = SIGN_TYPE_MAP[SignTaskType];
    }
    else {
        title = '转签';
        SignTaskType = 'TURN_SIGN';
    }
    const done = () => {
        window.app.showToast({
            title: '操作成功',
            icon: 'success',
        });
        onClose();
        doAfterSubmit && doAfterSubmit();
    };
    const onSubmit = async (v) => {
        var _a, _b;
        const UserId = wedaUserId || '';
        const TaskId = (_b = (_a = nodes[nodes.length - 1]) === null || _a === void 0 ? void 0 : _a.ProcessNode) === null || _b === void 0 ? void 0 : _b.TaskId;
        if (!callWorkflow)
            return;
        const data = {
            TaskId,
            UserId,
            Message: v.Message,
            TargetUserId: v.TargetUserId,
        };
        // 如果是转签
        if (modalSignType === 'addSign') {
            data.AddSignTaskType = SignTaskType;
        }
        try {
            await callWorkflow({
                action: REQUEST_URL_MAP[SignTaskType],
                data,
            });
            done();
        }
        catch (error) {
            showError(error);
        }
    };
    return (React.createElement(Modal, { visible: visible, caption: title, onClose: onClose },
        React.createElement(Modal.Body, null,
            React.createElement(FormProvider
            // @ts-ignore
            , { 
                // @ts-ignore
                form: form },
                React.createElement(Form, { style: { width: 300, marginBottom: '15px' }, className: "model-form" },
                    React.createElement(Field, { name: "TargetUserId", 
                        // @ts-ignore
                        title: modalSignType === 'addSign' ? '加签人' : '转签人', required: true, decorator: [FormItem], component: [ApprovalUserSelector, { single: true }] }),
                    React.createElement(Field, { name: "Message", 
                        // @ts-ignore
                        title: modalSignType === 'addSign' ? SIGN_MESSAGE : TURN_SIGN_MESSAGE, decorator: [FormItem], component: [Input.TextArea] })),
                React.createElement(Space, { align: "center" },
                    React.createElement(Submit, { onCancel: onClose, onSubmit: onSubmit }, "\u786E\u8BA4"))))));
};
