import React, { useEffect, useState, useRef } from 'react';
import { SelectMultiple, StatusTip, Select } from 'tea-component';
import { getApproversList, getApproversListByName } from '../../services';
import { useFlowContext } from './Context';
const DEFAULT_USER_PARAM = { PageNo: 1, PageSize: 20 };
/**
 * 用户搜索框
 */
export const ApprovalUserSelector = (props) => {
    const { defaultValue = [], onChange, single, ...rest } = props || {};
    const [isInit, setIsInit] = useState(false); // 是否已经初始化完成，用于合并无效值
    const [voidValue, setVoidValue] = useState(defaultValue); // 无效值，若存在会合并进已选中
    const [valueOptions, setValueOptions] = useState([]); // 已选中的 options
    const [listOptions, setListOptions] = useState([]); // 分页查询时的 options
    const [queryOptions, setQueryOptions] = useState([]); // 搜索查询时的 options
    const [userTotal, setUserTotal] = useState(0); // 分页时的总数
    const [userParam, setUserParam] = useState(DEFAULT_USER_PARAM); // 分页时的参数
    const [inputValue, setInputValue] = useState(''); // 搜索值
    const [status, setStatus] = useState(null); // 加载状态
    const timeRef = useRef(null); // 搜索节流
    const [singleValue, setSingleValue] = useState(null);
    const { callWedaApi, showToast, showError } = useFlowContext();
    // 初始化传入值
    useEffect(() => {
        let PageNo = 1;
        const PageSize = 100;
        const setInitDataHandle = async () => {
            let resp = {};
            try {
                resp = await getApproversList({ UserIds: defaultValue, PageNo, PageSize }, callWedaApi, showError, showToast);
            }
            catch (e) { }
            const respOptions = ((resp === null || resp === void 0 ? void 0 : resp.UserList) || []).map((item) => ({
                text: item.Name,
                value: item.UserId,
                tooltip: item.Name,
            }));
            const respOptionValues = respOptions.map((item) => item.value);
            setVoidValue((vals) => vals.filter((d) => !respOptionValues.includes(d)));
            setValueOptions((opts) => [].concat(opts, respOptions));
            if ((resp === null || resp === void 0 ? void 0 : resp.Total) > PageNo * PageSize) {
                PageNo += 1;
                await setInitDataHandle();
            }
            else {
                setIsInit(true);
            }
        };
        defaultValue.length > 0 && setInitDataHandle();
        setListDataHandle();
    }, []);
    // 完成初始化后，添加无效用户及触发 onChange 事件
    useEffect(() => {
        if (isInit) {
            // eslint-disable-next-line no-underscore-dangle
            const _valueOptions = [].concat(voidValue.map((d) => ({
                text: `${d}（该用户不存在）`,
                value: d,
                tooltip: `用户ID${d}不存在，请勿选择`,
            })), valueOptions);
            setValueOptions(_valueOptions);
            onChange && onChange(defaultValue, _valueOptions);
        }
    }, [isInit]);
    // 设置分页查询
    const setListDataHandle = async (_param = {}) => {
        // eslint-disable-next-line no-param-reassign
        _param = { ...userParam, ..._param };
        let resp = {};
        try {
            resp = await getApproversList(_param, callWedaApi, showError, showToast);
        }
        catch (e) { }
        const respOptions = ((resp === null || resp === void 0 ? void 0 : resp.UserList) || []).map((item) => ({
            text: item.Name,
            value: item.UserId,
        }));
        setListOptions((opts) => [...opts, ...respOptions]);
        setUserTotal((resp === null || resp === void 0 ? void 0 : resp.Total) || 0);
        setUserParam(_param);
    };
    // 设置按名称搜索查询，已选中的选项将被置顶在搜索结果列表
    const setQueryDataHandle = async (val) => {
        setStatus('loading');
        if (val) {
            let resp = [];
            try {
                resp = await getApproversListByName(val, callWedaApi, showError, showToast);
            }
            catch (e) { }
            const respOptions = (resp || []).map((item) => ({
                text: item.Name,
                value: item.UserId,
            }));
            setQueryOptions(respOptions);
        }
        else {
            setQueryOptions([]);
        }
        setInputValue(val);
        setStatus(null);
    };
    let options = [];
    let placeholderText = '';
    const value = valueOptions.map((item) => item.value);
    let component = SelectMultiple;
    if (inputValue) {
        const userSet = new Set(valueOptions.map((item) => item.value));
        options = [].concat(valueOptions, queryOptions.filter((item) => !userSet.has(item.value)));
    }
    else {
        const userSet = new Set(listOptions.map((item) => item.value));
        options = [].concat(valueOptions.filter((item) => !userSet.has(item.value)), listOptions);
        placeholderText = `共${userTotal}个，已展示前${options.length - voidValue.length}个，${userTotal - options.length + voidValue.length ? '下拉可加载更多' : '已加载全部'}，输入可搜索。`;
    }
    if (single) {
        // @ts-ignore
        component = Select;
    }
    const componentProps = {
        appearance: 'button',
        boxSizeSync: true,
        clearable: true,
        filter: () => true,
        searchable: true,
        staging: false,
        size: 'full',
        options,
        searchPlaceholder: placeholderText,
        tips: status && React.createElement(StatusTip, { status: status }),
        ...rest,
        value: single ? singleValue : value,
        type: 'simulate',
        autoClearSearchValue: false,
        onChange: (vals) => {
            if (single) {
                setSingleValue(vals);
                onChange && onChange(vals);
            }
            else {
                // eslint-disable-next-line no-underscore-dangle
                const _valueOptions = options.filter((item) => vals.includes(item.value));
                setValueOptions(_valueOptions);
                onChange && onChange(vals, _valueOptions);
            }
        },
        onScrollBottom: () => {
            if (listOptions.length < userTotal) {
                setListDataHandle({
                    PageNo: userParam.PageNo + 1,
                    PageSize: undefined,
                });
            }
        },
        onSearch: (val) => {
            window.clearTimeout(timeRef.current);
            timeRef.current = setTimeout(() => {
                setQueryDataHandle(val);
            }, 300);
        },
    };
    return React.createElement(component, componentProps, null);
};
