import React, { useState, useMemo } from 'react';
import { Field, FormProvider, createForm } from '../formily-react';
import { Modal, Form, Input, Select } from 'tea-component';
import { getFormField } from '../Form/Field';
import { getInitialValues } from '../Form/utils';
import { Submit, FormItem } from '../Form/FormItem';
import { ApprovalUserModal } from './ApprovalUserModal';
import Space from '../Space';
import { getApprovedTask, getRejectedTask, getNextApprovalNodeInfo } from '../../services';
import { PASS, REJECT_TO, APPROVAL_MESSAGE, REJECT_TO_ANY_NODE, REJECT_TO_LAST_NODE, REJECT_TO_START_NODE, SAME_USER_AS_PREV, REJECT, } from '../../constants';
import { getVariables } from '../../utils';
import { useFlowContext } from './Context';
import './index.less';
export const OperateModal = (props) => {
    const { onClose, visible, fields = [], type, doAfterSubmit, nodes = [], rejectType, ProcessInstanceId, rejectNodes = [], } = props || {};
    const [approvalUserModalVisible, setApprovalUserModalVisible] = useState(false);
    const form = useMemo(() => createForm({ initialValues: getInitialValues('create', fields) }), []);
    const { callWorkflow, showToast, showError } = useFlowContext();
    // 获取驳回弹窗 驳回至下拉数据， 取节点前n-1个
    const getRejectToList = () => rejectNodes.map((option) => {
        const { NodeName, NodeKey } = option;
        return {
            value: NodeKey,
            text: NodeName,
        };
    });
    // 渲染驳回至下拉框
    const renderRejectTo = () => {
        if (type === PASS)
            return null;
        return rejectType === REJECT_TO_ANY_NODE ? (React.createElement(Field, { name: "NodeId", 
            // @ts-ignore
            title: REJECT_TO, required: true, decorator: [FormItem], component: [rejectToFormItem] })) : null;
    };
    // 获取驳回节点的id
    const getNodeId = (formData) => {
        var _a, _b, _c, _d;
        let nodeId = '';
        const { length } = nodes || [];
        let index = 0;
        switch (rejectType) {
            // 上一个节点
            case REJECT_TO_LAST_NODE:
                // 后台返回的数据 前2个数据 都是第一个节点
                if (length > 3) {
                    index = length - 2;
                }
                nodeId = (_b = (_a = nodes[index]) === null || _a === void 0 ? void 0 : _a.ProcessNode) === null || _b === void 0 ? void 0 : _b.NodeKey;
                break;
            //  任意节点
            case REJECT_TO_ANY_NODE:
                nodeId = formData.NodeId;
                break;
            // 发起人节点
            case REJECT_TO_START_NODE:
                nodeId = (_d = (_c = nodes[0]) === null || _c === void 0 ? void 0 : _c.ProcessNode) === null || _d === void 0 ? void 0 : _d.NodeKey;
                break;
            default:
                nodeId = '';
        }
        return nodeId;
    };
    // 获取提交参数
    const getSubmitParams = () => {
        var _a, _b;
        // @ts-ignore
        const formValues = form.values || {};
        const TaskId = nodes[nodes.length - 1] && ((_b = (_a = nodes[nodes.length - 1]) === null || _a === void 0 ? void 0 : _a.ProcessNode) === null || _b === void 0 ? void 0 : _b.TaskId);
        const requestParams = {
            TaskId,
            Variables: getVariables(formValues, fields),
            Message: formValues === null || formValues === void 0 ? void 0 : formValues.message,
        };
        if (type === REJECT) {
            const NodeId = getNodeId(formValues);
            requestParams.NodeId = NodeId;
        }
        return requestParams;
    };
    const done = (showSuccess = true) => {
        showSuccess &&
            (showToast === null || showToast === void 0 ? void 0 : showToast({
                title: '操作成功',
                icon: 'success',
            }));
        onClose();
        if (approvalUserModalVisible) {
            setApprovalUserModalVisible(false);
        }
        doAfterSubmit && doAfterSubmit();
    };
    // 审批通过确定
    const handleApprovalSubmit = async (ApproveUserList) => {
        var _a, _b;
        const params = getSubmitParams();
        const lastNode = (_a = nodes[nodes.length - 1]) === null || _a === void 0 ? void 0 : _a.ProcessNode;
        // 选择审批人弹窗不需要走此逻辑
        if (!(ApproveUserList === null || ApproveUserList === void 0 ? void 0 : ApproveUserList.length)) {
            const nextApprovalNodeInfo = await getNextApprovalNodeInfo(ProcessInstanceId, lastNode === null || lastNode === void 0 ? void 0 : lastNode.NodeKey, params === null || params === void 0 ? void 0 : params.Variables, lastNode === null || lastNode === void 0 ? void 0 : lastNode.AddSignTaskType, callWorkflow, showError, showToast);
            if (((_b = nextApprovalNodeInfo === null || nextApprovalNodeInfo === void 0 ? void 0 : nextApprovalNodeInfo.ManualUserNode) === null || _b === void 0 ? void 0 : _b.ApprovalUserType) === SAME_USER_AS_PREV) {
                setApprovalUserModalVisible(true);
                return;
            }
        }
        if (ApproveUserList === null || ApproveUserList === void 0 ? void 0 : ApproveUserList.length) {
            params.ApproveUserList = ApproveUserList;
        }
        try {
            await getApprovedTask(params, callWorkflow, showToast);
            done();
        }
        catch (error) {
            done(false);
            showError(error);
        }
    };
    // 驳回确定
    const handleRejectSubmit = async () => {
        const params = getSubmitParams();
        try {
            await getRejectedTask(params, callWorkflow, showToast);
            done();
        }
        catch (error) {
            done(false);
            showError(error);
        }
    };
    const onSubmit = async () => {
        switch (type) {
            case PASS:
                await handleApprovalSubmit();
                break;
            case REJECT:
                await handleRejectSubmit();
                break;
            default:
                () => console.error('暂不支持此操作');
        }
    };
    const rejectToFormItem = ({ onChange, value, style }) => (React.createElement("div", null,
        React.createElement(Select, { style: style, boxSizeSync: true, size: "full", type: "simulate", appearance: "button", clearable: true, options: getRejectToList(), value: value, filter: () => true, onChange: (value) => onChange(value) })));
    return (React.createElement(Modal, { visible: visible, caption: `审批${type === PASS ? '通过' : '驳回'}`, onClose: onClose },
        React.createElement(Modal.Body, null,
            React.createElement(FormProvider
            // @ts-ignore
            , { 
                // @ts-ignore
                form: form },
                React.createElement(Form, { style: { width: 300, marginBottom: '15px' }, className: "model-form" },
                    fields.map((field, i) => getFormField(field, i)),
                    React.createElement(Field, { name: "message", 
                        // @ts-ignore
                        title: APPROVAL_MESSAGE, decorator: [FormItem], component: [Input.TextArea] }),
                    renderRejectTo()),
                React.createElement(Space, { align: "center" },
                    React.createElement(Submit, { onCancel: onClose, onSubmit: onSubmit }, "\u786E\u8BA4"))),
            approvalUserModalVisible && (React.createElement(ApprovalUserModal, { visible: approvalUserModalVisible, onClose: () => setApprovalUserModalVisible(false), onSubmit: handleApprovalSubmit })))));
};
