// @ts-nocheck
import React, { useState, useEffect } from 'react';
import classnames from 'classnames';
import { useRequest } from 'ahooks';
import { Button, Icon, StatusTip, Dropdown, List, Text, useClassNames, Bubble, ConfigProvider } from 'tea-component';
import { OperateModal } from './OperateModal';
import { ApprovalHistoryModal } from './ApprovalHistoryModal';
import { FlowModal } from './FlowModal';
import { ApprovalSignModal } from './ApprovalSignModal';
import { FlowRevokeModal } from './FlowRevokeModal';
import { PageContent } from '../PageContent';
import { mergeFieldsInfo, setObjectData, isRenderOperateButton, isRenderRevokeButton, getPageQuery } from '../../utils';
import { getProcessInstanceInfoByItemId } from '../../services';
import { PASS, REJECT, Variable, APPROVAL_TYPE, APPROVAL_STATUS, APPROVAL_TEXT_THEME, APPROVAL_ICON_TYPE, NODE_WRAPPER_CLASS, } from '../../constants';
import Provider, { useFlowContext } from './Context';
import { EnumProvider } from '../../context';
import '../../styles/index.less';
const FlowCardComponent = (props) => {
    const { processInstanceId, processKey, bindMetadata = {}, onSubmitCallback } = props || {};
    const { IS_WEDA_IDE, wedaUserId, wedaRelatedRoles, getDataSourceByName, callWorkflow, showToast, showError } = useFlowContext();
    const [operateModalVisible, setOperateModalVisible] = useState(false);
    const [historyModalVisible, setHistoryModalVisible] = useState(false);
    const [flowModalVisible, setFlowModalVisible] = useState(false);
    const [signModalVisible, setSignModalVisible] = useState(false);
    const [userNode, setUserNode] = useState(null);
    const [flowInfo, setFlowInfo] = useState({
        type: '',
        nodes: [],
        fields: [],
        ProcessDefinitionName: '',
        ProcessInstanceId: '',
        isRenderButton: false,
        rejectType: false,
        properties: null,
        rejectNodes: [],
        HistoricApproveNodeCount: 0,
    });
    const [modalSignType, setModalSignType] = useState(null);
    const { Margin } = useClassNames();
    const { datasource = {} } = bindMetadata;
    const { name: datasourceName } = datasource;
    const { id: itemId } = getPageQuery();
    // 获取节点信息
    const { data, loading, run: getNodesInfo } = useRequest(getProcessInstanceInfoByItemId, { manual: true });
    const initData = async () => {
        var _a;
        // 根据数据源id获取数据源数据
        const dataSourceRes = getDataSourceByName(datasourceName);
        const { schema = {} } = dataSourceRes || {};
        // 所有的字段
        const { properties } = schema;
        const viewId = schema['x-viewId'];
        // 根据数据源id 获取节点信息
        const nodesData = await getNodesInfo({
            processInstanceId,
            itemId,
            datasourceName,
            processKey,
            viewId,
            callWorkflow,
            showToast,
            showError,
            IS_WEDA_IDE,
        });
        const { ProcessNodeDetailList, ProcessDefinitionName, ProcessInstanceId, Status, HistoricNodeList, HistoricApproveNodeCount, } = nodesData || {};
        const nodes = ProcessNodeDetailList || [];
        // 获取当前审批节点， 取排序好节点的最后一个
        const { length } = nodes;
        const { ManualUserNode = {}, NodeKey, TaskId } = ((_a = nodes[length - 1]) === null || _a === void 0 ? void 0 : _a.ProcessNode) || {};
        // 审批节点的配置字段信息都在userNode字段中
        ManualUserNode.nodeId = NodeKey;
        setUserNode(ManualUserNode);
        // 是否渲染操作按钮
        const isRenderButton = isRenderOperateButton(ManualUserNode === null || ManualUserNode === void 0 ? void 0 : ManualUserNode.ApprovalUserList, Status, TaskId, ManualUserNode === null || ManualUserNode === void 0 ? void 0 : ManualUserNode.ApprovalRoleIdList, ManualUserNode === null || ManualUserNode === void 0 ? void 0 : ManualUserNode.DatasourceUserFieldList, wedaUserId, wedaRelatedRoles, IS_WEDA_IDE);
        const isRenderRevoke = isRenderRevokeButton({ nodes, status: Status }, wedaUserId);
        // 设置流程节点相关信息
        const flowData = setObjectData(flowInfo, {
            nodes,
            ProcessDefinitionName,
            ProcessInstanceId,
            Status,
            isRenderButton,
            isRenderRevoke,
            properties,
            rejectNodes: HistoricNodeList,
            HistoricApproveNodeCount: HistoricApproveNodeCount || 0,
        });
        setFlowInfo(flowData);
    };
    useEffect(() => {
        // 初始化数据
        initData();
    }, [processKey]);
    // 获取审批节点的信息
    const getApprovalNodeInfo = (type) => {
        const { properties } = flowInfo;
        const { ApproveMethodExtension = {}, RejectMethodExtension = {} } = userNode || {};
        const { ApprovalDatasource = {} } = ApproveMethodExtension;
        const { ApprovalDatasource: rejectApprovalDatasource, RejectOptionType } = RejectMethodExtension;
        const { DatasourceFieldList: approvalEditFields } = ApprovalDatasource;
        const { DatasourceFieldList: rejectEditFields } = rejectApprovalDatasource || {};
        const editFields = type === PASS ? approvalEditFields : rejectEditFields;
        return {
            fields: [...mergeFieldsInfo(properties, editFields)],
            rejectType: RejectOptionType,
        };
    };
    // 驳回弹窗
    const reject = () => {
        const { fields, rejectType } = getApprovalNodeInfo(REJECT);
        const flowData = setObjectData(flowInfo, { type: REJECT, fields, rejectType });
        setFlowInfo(flowData);
        setOperateModalVisible(true);
    };
    // 通过弹窗
    const pass = async () => {
        const { fields } = getApprovalNodeInfo(PASS);
        const flowData = setObjectData(flowInfo, { type: PASS, fields });
        setFlowInfo(flowData);
        setOperateModalVisible(true);
    };
    const handleProcessNodeDetailList = () => {
        const { nodes, HistoricApproveNodeCount } = flowInfo;
        const processNodeDetailList = (nodes || []).concat();
        processNodeDetailList.shift();
        if (HistoricApproveNodeCount < 3) {
            return processNodeDetailList;
        }
        return [
            processNodeDetailList[0],
            { ellipsis: true },
            ...processNodeDetailList.slice(processNodeDetailList.length - 2),
        ];
    };
    const renderApprovalNodes = () => {
        const nodes = handleProcessNodeDetailList();
        return nodes.map((node, index) => {
            const { ManualUserNode, Status, TriggerNode } = (node === null || node === void 0 ? void 0 : node.ProcessNode) || {};
            const { ApprovalUserNameList = [], NodeName, ApprovalRoleNameList = [], DatasourceUserFieldList = [], } = ManualUserNode || {};
            const isEllipsis = node === null || node === void 0 ? void 0 : node.ellipsis;
            const approvers = [
                ...DatasourceUserFieldList.reduce((list, data) => {
                    list.push(...data === null || data === void 0 ? void 0 : data.UserFieldTitleList);
                    return list;
                }, []),
                ...ApprovalRoleNameList,
                ...ApprovalUserNameList,
            ];
            // 渲染发起人节点
            if (index === 0) {
                return (React.createElement("div", { className: `${Variable}-approval__info-item`, key: index },
                    React.createElement(Icon, { type: "success" }),
                    React.createElement("div", { className: `${Variable}-approval__info-item__name` }, TriggerNode === null || TriggerNode === void 0 ? void 0 : TriggerNode.StartUser),
                    React.createElement("div", { className: `${Variable}-approval__info-item__des` }, "\u53D1\u8D77")));
            }
            // 渲染省略号节点
            if (isEllipsis) {
                return (React.createElement("div", { className: `${Variable}-approval__info-item`, key: index },
                    React.createElement("div", { className: `${Variable}-approval__info-item__more` }, "\u2026")));
            }
            // 渲染其他节点
            return (React.createElement("div", { className: `${Variable}-approval__info-item`, key: index },
                React.createElement(Icon, { type: APPROVAL_ICON_TYPE[Status] }),
                (approvers === null || approvers === void 0 ? void 0 : approvers.length) ? (React.createElement(Bubble, { content: approvers.join() },
                    React.createElement("div", { className: classnames(`${Variable}-approval__info-item__name`, `${Variable}-approval__info-item__name-ellipsis`) }, approvers.join()))) : null,
                ManualUserNode ? (React.createElement("div", { className: `${Variable}-approval__info-item__text` }, APPROVAL_TYPE[ManualUserNode === null || ManualUserNode === void 0 ? void 0 : ManualUserNode.ApprovalType] || '')) : null,
                React.createElement("div", { className: `${Variable}-approval__info-item__des` }, NodeName),
                React.createElement("div", { className: `${Variable}-approval__info-item__status` },
                    React.createElement(Text, { theme: APPROVAL_TEXT_THEME[Status] }, APPROVAL_STATUS[Status]))));
        });
    };
    const viewHistoryApproval = () => {
        setHistoryModalVisible(true);
    };
    const viewProcess = () => {
        setFlowModalVisible(true);
    };
    // 未找到数据，或在加载中
    if (loading) {
        return (React.createElement("div", { style: { textAlign: 'center' }, ...props },
            React.createElement(StatusTip, { status: "loading" })));
    }
    // 如果没有数据 且是运行态 直接不显示组件
    if (!loading && !data && !IS_WEDA_IDE)
        return null;
    const menu = () => {
        const { SignTaskInfo = {} } = userNode || {};
        const { EnableSignTask, EnableTurnSign } = SignTaskInfo;
        const { isRenderButton } = flowInfo;
        return (React.createElement(List, { type: "option" },
            isRenderButton && EnableSignTask ? (React.createElement(List.Item, { onClick: () => {
                    setModalSignType('addSign');
                    setSignModalVisible(true);
                } }, "\u52A0\u7B7E")) : null,
            isRenderButton && EnableTurnSign ? (React.createElement(List.Item, { onClick: () => {
                    setModalSignType('turnSign');
                    setSignModalVisible(true);
                } }, "\u8F6C\u7B7E")) : null,
            React.createElement(List.Item, { onClick: () => {
                    viewHistoryApproval();
                } }, "\u67E5\u770B\u5386\u53F2\u5BA1\u6279"),
            React.createElement(List.Item, { onClick: viewProcess }, "\u67E5\u770B\u5BA1\u6279\u5168\u6D41\u7A0B")));
    };
    const doAfterSubmit = () => {
        initData();
        if (onSubmitCallback && typeof onSubmitCallback === 'function') {
            onSubmitCallback();
        }
    };
    const { ProcessDefinitionName, fields, type, Status, isRenderButton, isRenderRevoke, rejectType, nodes, ProcessInstanceId, rejectNodes, } = flowInfo;
    return (React.createElement(ConfigProvider, { classPrefix: "wedatea2td" },
        React.createElement(EnumProvider, { fields: fields },
            React.createElement(PageContent, { ...props },
                React.createElement("div", { className: classnames(`${Variable}-approval`, NODE_WRAPPER_CLASS[Status]), style: { marginBottom: '20px' } },
                    React.createElement("div", { className: `${Variable}-approval__header` },
                        React.createElement("div", { className: `${Variable}-approval__header-title` },
                            React.createElement("i", { className: `${Variable}-icon ${Variable}-icon--approval` }),
                            React.createElement("span", { className: `${Variable}-approval__header-title__text` }, ProcessDefinitionName)),
                        React.createElement("div", { className: `${Variable}-approval__header-more` },
                            isRenderButton ? (React.createElement(React.Fragment, null,
                                React.createElement(Button, { type: "primary", onClick: pass, className: Margin.Right['2n'] }, "\u901A\u8FC7"),
                                React.createElement(Button, { type: "weak", onClick: reject, className: Margin.Right['2n'] }, "\u9A73\u56DE"))) : null,
                            React.createElement(FlowRevokeModal, { isRenderRevoke: isRenderRevoke, ProcessInstanceId: ProcessInstanceId, doAfterSubmit: doAfterSubmit }),
                            React.createElement(Dropdown, { clickClose: false, button: React.createElement(Button, { type: "icon", icon: "more" }), appearance: "pure" }, menu))),
                    React.createElement("div", { className: `${Variable}-approval__body` },
                        React.createElement("div", { className: `${Variable}-approval__info` }, renderApprovalNodes()),
                        React.createElement("div", { className: `${Variable}-approval__status` },
                            React.createElement(Icon, { type: APPROVAL_ICON_TYPE[Status] }),
                            React.createElement(Text, { theme: APPROVAL_TEXT_THEME[Status] }, APPROVAL_STATUS[Status])))),
                operateModalVisible && (React.createElement(OperateModal, { visible: operateModalVisible, type: type, fields: fields || [], onClose: () => setOperateModalVisible(false), rejectType: rejectType, nodes: nodes, doAfterSubmit: doAfterSubmit, ProcessInstanceId: ProcessInstanceId, rejectNodes: rejectNodes })),
                historyModalVisible && (React.createElement(ApprovalHistoryModal, { visible: historyModalVisible, ProcessInstanceId: ProcessInstanceId, onClose: () => setHistoryModalVisible(false), title: ProcessDefinitionName })),
                flowModalVisible && (React.createElement(FlowModal, { visible: flowModalVisible, onClose: () => setFlowModalVisible(false), ProcessInstanceId: ProcessInstanceId, title: ProcessDefinitionName })),
                signModalVisible && (React.createElement(ApprovalSignModal, { visible: signModalVisible, onClose: () => setSignModalVisible(false), userNode: userNode, nodes: nodes, modalSignType: modalSignType, doAfterSubmit: doAfterSubmit }))))));
};
export const FlowCard = (props) => (React.createElement(Provider, null,
    React.createElement(FlowCardComponent, { ...(props || {}) })));
