import React, { memo, useContext } from 'react';
import { Handle, Position } from 'react-flow-renderer';
import { Button, Bubble, Text, Tag } from 'tea-component';
import { FlowContext } from '../index';
import { FLOW_NODE_STATUS_APPROVE, FLOW_NODE_STATUS_REJECT, FLOW_NODE_STATUS_UNEXECUTED, } from '../constants';
const sourceHandleStyle = {
    backgroundColor: '#fff',
    borderColor: '#999999',
    pointerEvents: 'none',
};
const sourceHandleExcutedStyle = {
    backgroundColor: '#fff',
    borderColor: '#006eff',
    pointerEvents: 'none',
};
// 隐藏线头的端点
const targetHandleStyle = {
    opacity: 0,
};
// 审批节点
const ApprovalNode = ({ ...restProps }) => {
    const { flowData } = useContext(FlowContext);
    const { id } = restProps;
    const data = (flowData === null || flowData === void 0 ? void 0 : flowData.find((item) => item.id === id)) || {};
    const { status } = data;
    const { NodeName: initNodeName, ApprovalUserNameList: initApprovalUserNameList = [], ApprovalRoleNameList = [], DatasourceUserFieldList = [], ApprovalType, } = (data === null || data === void 0 ? void 0 : data.ManualUserNode) || {};
    const DName = initNodeName;
    const EName = [
        ...DatasourceUserFieldList.reduce((list, data) => {
            list.push(...data === null || data === void 0 ? void 0 : data.UserFieldTitleList);
            return list;
        }, []),
        ...ApprovalRoleNameList,
        ...initApprovalUserNameList,
    ].join();
    return (React.createElement("div", { style: { opacity: !status || status === FLOW_NODE_STATUS_UNEXECUTED ? 0.5 : 1 } },
        React.createElement(Handle, { type: "target", position: Position.Top, id: "stop", style: targetHandleStyle }),
        React.createElement("div", { className: "flow-workflow flow-workflow--approval" },
            React.createElement("div", { className: `flow-workflow__inner ${status === 'EXECUTING' ? 'flow-workflow__inner_current' : ''}` },
                React.createElement("div", { className: "flow-workflow__icon" },
                    React.createElement("i", { className: "flow-icon flow-icon--approval" }),
                    React.createElement("div", { className: "flow-workflow__icon-media" },
                        React.createElement("div", { className: "flow-workflow__icon-media--title" },
                            DName ? (React.createElement(Bubble, { content: DName },
                                React.createElement(Text, { style: { cursor: 'pointer' }, overflow: true }, DName))) : (React.createElement(Text, { overflow: true }, "\u5BA1\u6279\u8282\u70B9")),
                            React.createElement(Tag, { theme: "primary" }, ApprovalType === 'APPROVE_MULTI' ? '多人会签' : '单人审批')))),
                React.createElement("div", { className: "flow-workflow__form" },
                    React.createElement("div", { className: "flow-workflow__form-item" },
                        React.createElement("div", { className: "flow-workflow__form-item--label" }, "\u5BA1\u6279\u4EBA"),
                        React.createElement("div", { className: "flow-workflow__form-item--main" },
                            React.createElement(Button, { type: "text" }, EName ? (React.createElement(Bubble, { content: EName },
                                React.createElement(Text, { style: { cursor: 'pointer' }, overflow: true }, EName))) : (React.createElement(Text, { overflow: true }, "\u672A\u8BBE\u7F6E")))))))),
        React.createElement(Handle, { type: "source", position: Position.Bottom, id: "start", 
            // @ts-ignore
            style: status === FLOW_NODE_STATUS_APPROVE || status === FLOW_NODE_STATUS_REJECT
                ? sourceHandleExcutedStyle
                : sourceHandleStyle })));
};
export default memo(ApprovalNode);
