import React, { useEffect, useContext } from 'react';
import { Text, Bubble } from 'tea-component';
import { useZoomPanHelper, useStoreState } from 'react-flow-renderer';
import html2canvas from 'html2canvas';
import { FlowContext } from '../index';
// 流程编辑器浮动控制按钮
export default function FlowControl({ flowRef, flowName }) {
    // 缩放操作，来自useZoomPanHelper
    const { zoomIn, zoomOut, fitView } = useZoomPanHelper();
    // 获取主数据
    const { elements } = useContext(FlowContext);
    // 平移缩放封装在store的transform里，以数组存放，缩放比率在第三个位置
    const transform = useStoreState((store) => store.transform);
    // 导出为图片
    const exportPNG = () => {
        html2canvas(flowRef.current).then((canvas) => {
            const imgData = canvas.toDataURL('image/png').replace('image/png', 'image/octet-stream');
            const aTemp = document.createElement('a');
            aTemp.href = imgData;
            aTemp.download = `审批全流程-${flowName}.png`;
            aTemp.click();
        });
    };
    // 图表渲染完毕触发
    useEffect(() => {
        setTimeout(() => {
            fitView();
        }, 300);
    }, [elements]);
    return (React.createElement(React.Fragment, null,
        React.createElement(Bubble, { content: "\u7F29\u5C0F" },
            React.createElement("div", { className: "react-flow__controls-button react-flow__controls-zoomOut", onClick: (_e) => zoomOut() })),
        React.createElement(Text, { overflow: true, className: "react-flow__controls-button react-flow__controls-scale" }, `${(transform[2] * 100).toFixed()}%`),
        React.createElement(Bubble, { content: "\u653E\u5927" },
            React.createElement("div", { className: "react-flow__controls-button react-flow__controls-zoomIn", onClick: (_e) => zoomIn() })),
        React.createElement(Bubble, { content: "\u9002\u5E94\u5C4F\u5E55" },
            React.createElement("div", { className: "react-flow__controls-button react-flow__controls-fitView", onClick: () => fitView() })),
        React.createElement(Bubble, { content: "\u5BFC\u51FA\u6D41\u7A0B\u56FE" },
            React.createElement("div", { className: "react-flow__controls-button react-flow__controls-export", onClick: exportPNG }))));
}
