import React, { useContext } from 'react';
import { FlowContext } from '../index';
import { getSmoothStepPath } from 'react-flow-renderer';
import { FLOW_BRANCHTYPE_MULTI_START, FLOW_BRANCHTYPE_MULTI_END, FLOW_SINGLE_NODE, FLOW_NODE_STATUS_UNEXECUTED, FLOW_GATEWAY_NODE, } from '../constants';
// 含有加号的连线
const PlusEdge = ({ source, target, sourceX, sourceY, targetX, targetY, sourcePosition, targetPosition, style = {}, data, }) => {
    const { branchType } = data;
    const borderRadius = 0;
    const { flowData } = useContext(FlowContext);
    let status = '';
    const { status: targetStatus, branchFinished: targetFinished, type: targetType, } = (flowData === null || flowData === void 0 ? void 0 : flowData.find((item) => item.id === target)) || {};
    const { status: sourceStatus, type, branchFinished: sourceFinished, } = (flowData === null || flowData === void 0 ? void 0 : flowData.find((item) => item.id === source)) || {};
    if (type === FLOW_SINGLE_NODE ||
        (targetStatus !== FLOW_NODE_STATUS_UNEXECUTED && sourceStatus !== FLOW_NODE_STATUS_UNEXECUTED)) {
        status = targetStatus;
    }
    // 汇聚分支的前段path
    const edgePathBefore = getSmoothStepPath({
        sourceX,
        sourceY,
        sourcePosition,
        targetX,
        // 调参获得
        targetY: sourceY,
        targetPosition,
        borderRadius,
    });
    // 汇聚分支的后段path
    const edgePathAfter = getSmoothStepPath({
        sourceX: targetX,
        // 调参获得
        sourceY,
        sourcePosition,
        targetX,
        targetY: targetY + 4,
        targetPosition,
        borderRadius,
    });
    // 单侧分支的前段path
    const singlePathBefore = getSmoothStepPath({
        sourceX,
        sourceY,
        sourcePosition,
        targetX: sourceX,
        // 调参获得
        targetY,
        targetPosition,
        borderRadius,
    });
    // 单侧分支的后段path
    const singlePathAfter = getSmoothStepPath({
        sourceX,
        // 调参获得
        sourceY: targetY,
        sourcePosition,
        targetX,
        targetY,
        targetPosition,
        borderRadius,
    });
    // 默认path
    const path = getSmoothStepPath({
        sourceX,
        // 调参获得
        sourceY,
        sourcePosition,
        targetX,
        targetY: targetY + (targetType === FLOW_GATEWAY_NODE ? 8 : 4),
        targetPosition,
        borderRadius,
    });
    let pathClass = '';
    if (!status || status === FLOW_NODE_STATUS_UNEXECUTED) {
        if (targetFinished && sourceFinished) {
            pathClass = 'react-flow__edge-path react-flow__edge-path_branchFinished';
        }
        else {
            pathClass = 'react-flow__edge-path react-flow__edge-path_animated';
        }
    }
    else {
        pathClass = 'react-flow__edge-path react-flow__edge-path_approved';
    }
    // svg path
    const svgPath = () => {
        switch (branchType) {
            case FLOW_BRANCHTYPE_MULTI_END:
                return (React.createElement(React.Fragment, null,
                    React.createElement("path", { style: style, className: pathClass, d: singlePathBefore }),
                    React.createElement("path", { style: style, className: pathClass, d: singlePathAfter })));
            case FLOW_BRANCHTYPE_MULTI_START:
                return (React.createElement(React.Fragment, null,
                    React.createElement("path", { style: style, className: pathClass, d: edgePathBefore }),
                    React.createElement("path", { style: style, className: pathClass, d: edgePathAfter })));
            default:
                return React.createElement("path", { style: style, className: pathClass, d: path });
        }
    };
    return svgPath();
};
export default PlusEdge;
