import { isEdge } from 'react-flow-renderer';
// 引入流程自定义常量
import { FLOW_BRANCHTYPE_SINGLE, FLOW_BRANCHTYPE_MULTI_START, FLOW_BRANCHTYPE_MULTI_END, FLOW_NODE_STATUS_UNEXECUTED, } from '../constants';
/**
 * 获取两个节点间所有节点id
 * 两个节点在同一层
 * @param args
 * @returns
 */
export const getIdsBetweenNodes = (args) => {
    const { from, to, data } = args;
    const nodes = [];
    nodes.push(from);
    const branchs = data.filter((node) => node.source === from);
    branchs.forEach((branch) => {
        const getLineNodes = (startEdge) => {
            data.forEach((node) => {
                if (startEdge.target === node.id && nodes.findIndex((n) => n === node.id) === -1) {
                    nodes.push(node.id);
                    const orders = data.filter((o) => o.source === node.id);
                    orders.forEach((order) => {
                        if (!order || node.id === to)
                            return;
                        getLineNodes(order);
                    });
                }
            });
        };
        getLineNodes(branch);
    });
    return nodes;
};
/**
 * 初始化流程数据
 * @param data
 * @returns
 */
export const initData = (data) => data.map((els) => {
    const flowElement = els;
    const { source, target, branchEnd, status, id } = flowElement;
    if (isEdge(flowElement)) {
        flowElement.type = 'plus';
        if (data.filter((dataItem) => dataItem.source === source).length > 1) {
            // 汇聚分支连线起点
            flowElement.data = { branchType: FLOW_BRANCHTYPE_MULTI_START };
        }
        else if (data.filter((dataItem) => dataItem.target === target).length > 1) {
            // 汇聚分支连线终点
            flowElement.data = { branchType: FLOW_BRANCHTYPE_MULTI_END };
        }
        else {
            // 单侧分支连线
            flowElement.data = { branchType: FLOW_BRANCHTYPE_SINGLE };
        }
    }
    else {
        // 节点初始位置
        flowElement.position = { x: 0, y: 0 };
        flowElement.data = {};
        const endNode = data.find((dataItem) => dataItem.id === branchEnd) || {};
        const isNotUnexecuted = endNode.status !== FLOW_NODE_STATUS_UNEXECUTED;
        if (status !== FLOW_NODE_STATUS_UNEXECUTED && branchEnd && isNotUnexecuted) {
            const finishNodes = getIdsBetweenNodes({ from: id, to: branchEnd, data });
            finishNodes.forEach((node) => {
                // eslint-disable-next-line no-param-reassign
                data[data.findIndex((dataItem) => dataItem.id === node)].branchFinished = true;
            });
        }
    }
    return flowElement;
});
