import React from 'react';
// @ts-ignore
import { isEmail } from 'commons-validator-es';
import { Field, ArrayField, ObjectField, observer, useField } from '../formily-react';
import { FormItem } from './FormItem';
import { Input, Switch, Select, Button, Form, Icon, useConfig, InputAdornment } from 'tea-component';
import { RelationSelector } from './RelationSelector';
import Location, { locationProperties } from './Location';
import { EnumSelect } from './Enum';
import { IDatePicker, ITimePicker } from './Date';
import { IFormSelect, IDepartSelect, IUserSelect } from './Select';
import Space from '../Space';
import { formatNumberFix } from '../Table/FieldRender';
import { getTypeDefaultValue } from './utils';
import { formatTimeByType, isValueNone } from '../../utils';
import { Uploader, RichText, UploaderFile } from '@cloudbase/weda-ui';
import ErrorBoundary from '../ErrorBoundary';
// 支持自动校验的 format 类型
const AUTO_FORMAT = {
    url: 'url',
    phone: 'phone',
    // email: 'email',
};
// 字段类型和名称转译
const FIELD_TYPE_NAME = {
    array: '数组',
    object: '对象',
    string: '字符串',
    number: '数字',
    boolean: '布尔值',
};
/**
 * 获取 form field 信息
 */
export const getFormField = (field, key, extra = {}) => {
    var _a, _b;
    const { name, type, title, description, format, items } = field;
    const { parentIsArray } = extra || {};
    let component = null;
    let props = { field, parentIsArray };
    const validator = getValidator(field);
    // 最新 schema 格式
    if (type === 'string') {
        if (format === 'x-image') {
            // 图片
            component = Uploader;
            props = {
                ...props,
                single: true,
                maxUploadCount: 1,
                maxSize: Math.round(parseInt(field === null || field === void 0 ? void 0 : field['x-image-size'], 10) / 1024 / 1024) || 10,
                acceptTypes: ((field === null || field === void 0 ? void 0 : field['x-image-support']) || ['*']).map((d) => {
                    const d1 = `${d}`.toLowerCase();
                    return d1.includes('image') ? d1 : `image/${d1}`;
                }),
                decorator: null,
                showType: 'pc',
            };
        }
        else if (format === 'x-rtf') {
            // 富文本
            component = RichText;
            props = {
                ...props,
                labelVisible: false,
                decorator: null,
                maxLength: field.maxLength || 32768,
            };
        }
        else if (format === 'x-enum') {
            // 枚举（新）
            component = EnumSelect;
        }
        else if (format === 'related' || format === 'father-son') {
            // 关联关系
            const xParent = field['x-parent'];
            // 父数据源为部门用户， 展示部门用户组件
            if ((xParent === null || xParent === void 0 ? void 0 : xParent.parentDataSourceName) === 'sys_department') {
                component = IDepartSelect;
            }
            else if ((xParent === null || xParent === void 0 ? void 0 : xParent.parentDataSourceName) === 'sys_user') {
                component = IUserSelect;
            }
            else {
                component = RelationSelector;
            }
        }
        else if (format === 'url') {
            // 网址
            component = BeforeAdornInput;
            props = { ...props, options: [{ value: 'http://' }, { value: 'https://' }] };
        }
        else if (format === 'x-file') {
            // 文件
            component = UploaderFile;
            props = {
                ...props,
                single: true,
                maxUploadCount: (field === null || field === void 0 ? void 0 : field.maxItems) || 9,
                maxSize: parseInt(field === null || field === void 0 ? void 0 : field['x-file-size']) || 1024,
                labelVisible: false,
                decorator: null,
            };
        }
        else if (format === 'x-area-code') {
            // 地区选择
            component = IFormSelect;
        }
        else if (format === 'x-long-text') {
            // 长文本
            component = InputTextAreaComponent;
            props = { ...props, size: 'full' };
        }
        else {
            // 普通字符串
            component = InputComponent;
            props = { ...props, size: 'full' };
        }
    }
    else if (type === 'number') {
        if (format === 'date' || format === 'datetime') {
            // 日期，日期时间
            component = IDatePicker;
            props = { ...props, format };
        }
        else if (format === 'time') {
            // 时间
            component = ITimePicker;
        }
        else {
            // 普通数字
            component = InputNumberComponent;
            props = { ...props, size: 'full' };
        }
    }
    else if (type === 'boolean') {
        props = { ...props, style: { marginTop: '10px' } };
        component = Switch;
    }
    else if (type === 'array') {
        if ((items === null || items === void 0 ? void 0 : items.format) === 'x-image') {
            // 图片，特殊处理
            component = Uploader;
            props = {
                ...props,
                single: false,
                maxUploadCount: field.maxItems || 9,
                maxSize: Math.round(parseInt(items === null || items === void 0 ? void 0 : items['x-image-size'], 10) / 1024 / 1024) || 10,
                acceptTypes: ((items === null || items === void 0 ? void 0 : items['x-image-support']) || ['*']).map((d) => {
                    const d1 = `${d}`.toLowerCase();
                    return d1.includes('image') ? d1 : `image/${d1}`;
                }),
                decorator: null,
                showType: 'pc',
            };
        }
        else if ((items === null || items === void 0 ? void 0 : items.format) === 'x-file') {
            // 文件，特殊处理
            component = UploaderFile;
            props = {
                ...props,
                single: false,
                maxUploadCount: field.maxItems || 9,
                maxSize: parseInt(items === null || items === void 0 ? void 0 : items['x-file-size']) || 1024,
                labelVisible: false,
                decorator: null,
            };
        }
        else if (format === 'x-enum') {
            // 枚举（新）
            component = EnumSelect;
        }
        else {
            if ((_a = props.field) === null || _a === void 0 ? void 0 : _a.items) {
                props.field.items['x-powerDisabled'] = field['x-powerDisabled'];
            }
            component = ArrayComponent;
        }
    }
    else if (type === 'object') {
        component = ObjectComponent;
        if (format === 'x-location') {
            component = Location;
        }
    }
    // 兼容历史 schema 格式
    if ((_b = field['x-enumObj']) === null || _b === void 0 ? void 0 : _b.length) {
        // 枚举-旧
        component = Select;
        props = {
            ...props,
            searchable: true,
            virtual: true,
            size: 'full',
            matchButtonWidth: true,
            appearance: 'button',
            options: field['x-enumObj'].map((enumItem) => ({
                value: enumItem.value,
                text: enumItem.label,
            })),
        };
    }
    const fieldProps = {
        key,
        title,
        name,
        validator,
        required: field['x-required'],
        disabled: field['x-powerDisabled'] ? true : false,
        decorator: extra.withoutDecorator
            ? []
            : [
                FormItem,
                {
                    style: {},
                    align: 'middle',
                    description,
                    required: field['x-required'],
                },
            ],
        component: [component, { ...props }],
    };
    // 最终表单项组件
    let FieldComponent = null;
    switch (type) {
        case 'array':
            if (['x-image', 'x-file'].includes(items === null || items === void 0 ? void 0 : items.format)) {
                FieldComponent = React.createElement(Field, { ...fieldProps });
                break;
            }
            FieldComponent = React.createElement(ArrayField, { ...fieldProps });
            break;
        case 'object':
            if (format === 'x-location') {
                FieldComponent = React.createElement(Field, { ...fieldProps });
                break;
            }
            FieldComponent = React.createElement(ObjectField, { ...fieldProps });
            break;
        default:
            FieldComponent = React.createElement(Field, { ...fieldProps });
            break;
    }
    return React.createElement(ErrorBoundary, { fallbackRender: () => React.createElement("span", null, "\u274C \u6570\u636E\u5F02\u5E38") }, FieldComponent);
};
/**
 * 数字输入组件，因为小数点的存在，小数点补齐0后，展示为string类型，传输为number类型
 */
const InputNumberComponent = observer((props) => {
    const [inner, setInner] = React.useState(formatNumberFix(props === null || props === void 0 ? void 0 : props.value, props.field, true));
    return (React.createElement(Input, { ...props, type: "number", value: inner, onChange: (val) => {
            var _a;
            const value = parseFloat(val);
            setInner(value);
            (_a = props === null || props === void 0 ? void 0 : props.onChange) === null || _a === void 0 ? void 0 : _a.call(props, value);
        }, onBlur: (e) => {
            var _a, _b, _c, _d, _e;
            const value = ((_a = e === null || e === void 0 ? void 0 : e.target) === null || _a === void 0 ? void 0 : _a.value) ? formatNumberFix((_b = e === null || e === void 0 ? void 0 : e.target) === null || _b === void 0 ? void 0 : _b.valueAsNumber, props === null || props === void 0 ? void 0 : props.field) : null;
            const innerValue = ((_c = e === null || e === void 0 ? void 0 : e.target) === null || _c === void 0 ? void 0 : _c.value) ? formatNumberFix((_d = e === null || e === void 0 ? void 0 : e.target) === null || _d === void 0 ? void 0 : _d.valueAsNumber, props === null || props === void 0 ? void 0 : props.field, true) : null;
            setInner(innerValue);
            (_e = props === null || props === void 0 ? void 0 : props.onChange) === null || _e === void 0 ? void 0 : _e.call(props, value);
        } }));
});
/**
 * 文本输入组件，添加 onBlur 失焦时转变值类型为字符串
 */
const InputComponent = observer((props) => (React.createElement(Input, { ...props, onBlur: (e) => { var _a, _b, _c; return (_a = props === null || props === void 0 ? void 0 : props.onChange) === null || _a === void 0 ? void 0 : _a.call(props, `${(_c = (_b = e === null || e === void 0 ? void 0 : e.target) === null || _b === void 0 ? void 0 : _b.value) !== null && _c !== void 0 ? _c : ''}`); } })));
/**
 * 长文本组件，添加 onBlur 失焦时转变值类型为字符串
 */
const InputTextAreaComponent = observer((props) => (React.createElement(Input.TextArea, { ...props, onBlur: (e) => { var _a, _b, _c; return (_a = props === null || props === void 0 ? void 0 : props.onChange) === null || _a === void 0 ? void 0 : _a.call(props, `${(_c = (_b = e === null || e === void 0 ? void 0 : e.target) === null || _b === void 0 ? void 0 : _b.value) !== null && _c !== void 0 ? _c : ''}`); } })));
/**
 * 带前缀输入组件，例如网址
 */
const BeforeAdornInput = observer((props) => {
    // @ts-ignore
    const { options = [], onChange, value = '', ...rest } = props;
    const beforeOpt = [].concat(options).find((item) => `${value}`.startsWith(`${item === null || item === void 0 ? void 0 : item.value}`));
    let [beforeValue, inputValue] = ['', value];
    if (beforeOpt) {
        beforeValue = beforeOpt.value;
        inputValue = `${value}`.replace(beforeValue, '');
    }
    const disabled = props === null || props === void 0 ? void 0 : props.field['x-powerDisabled'];
    return (React.createElement(InputAdornment, { className: "model-form__before-adorn-input", before: React.createElement(Select, { options: options, value: beforeValue, disabled: disabled, onChange: (val) => onChange === null || onChange === void 0 ? void 0 : onChange(`${val}${inputValue}`) }) },
        React.createElement(Input, { value: inputValue, disabled: disabled, onChange: (val) => onChange === null || onChange === void 0 ? void 0 : onChange(`${beforeValue}${val}`) })));
});
/**
 * 数组输入组件
 */
const ArrayComponent = observer((props) => {
    var _a, _b, _c, _d, _e, _f, _g;
    const field = useField();
    const defaultValue = getTypeDefaultValue((_a = props.field) === null || _a === void 0 ? void 0 : _a.items, ((_b = props.field) === null || _b === void 0 ? void 0 : _b['x-required']) === true);
    const isChildForm = ((_c = props.field) === null || _c === void 0 ? void 0 : _c.items.type) === 'object' && props.field.items.format !== 'x-location';
    const { classPrefix } = useConfig();
    return (React.createElement(React.Fragment, null, 
    // @ts-ignore
    (_d = field.value) === null || _d === void 0 ? void 0 :
        _d.map((item, index) => isChildForm ? (React.createElement("div", { className: "_wedamodal-form-nested", style: {} },
            React.createElement("div", { className: "_wedamodal-form-nested__op" },
                React.createElement(Icon, { type: "close", size: "s", onClick: () => {
                        // @ts-ignore
                        field.remove(index);
                    } })),
            getFormField(Object.assign({}, props.field.items, { name: index }), index, {
                withoutDecorator: true,
                parentIsArray: true,
            }))) : (React.createElement("div", { key: index, style: { display: 'flex-block', marginBottom: '10px' } },
            React.createElement(Space, { className: "_wedamodel-array" },
                getFormField(Object.assign({}, props.field.items, { name: index }), index, {
                    withoutDecorator: false,
                    parentIsArray: true,
                }),
                React.createElement(Button, { style: { borderStyle: 'dashed' }, onClick: () => {
                        // @ts-ignore
                        field.remove(index);
                    } },
                    React.createElement("i", { className: "wedatea2td-icon wedatea2td-icon-delete" })))))),
        isChildForm ? (React.createElement("div", null,
            React.createElement(Button, { type: "weak", className: `${
                // @ts-ignore
                ((_e = field.value) === null || _e === void 0 ? void 0 : _e.length) ? '_wedamodal-btn--full' : ''}  _wedamodal-btn--dashed`, onClick: () => {
                    // @ts-ignore
                    field.push(defaultValue);
                } },
                React.createElement("i", { className: `${classPrefix}-icon ${classPrefix}-icon-plus--primary` }),
                "\u589E\u52A0"))) : (React.createElement(Button, { type: "weak", 
            // @ts-ignore
            className: `${((_f = field.value) === null || _f === void 0 ? void 0 : _f.length) ? '_wedamodal-btn--full' : ''}  _wedamodal-btn--dashed`, onClick: () => {
                // @ts-ignore
                field.push(defaultValue);
            }, disabled: (_g = props.field) === null || _g === void 0 ? void 0 : _g.items['x-powerDisabled'] },
            React.createElement("i", { className: `${classPrefix}-icon ${classPrefix}-icon-plus--primary` }),
            "\u589E\u52A0"))));
});
/**
 * 对象输入组件
 */
const ObjectComponent = observer((props) => (React.createElement(Form
// @ts-ignore
// @ts-ignore
, { 
    // @ts-ignore
    // @ts-ignore
    onBlur: (e) => { var _a; return (_a = props === null || props === void 0 ? void 0 : props.onChange) === null || _a === void 0 ? void 0 : _a.call(props, props === null || props === void 0 ? void 0 : props.value); } }, Object.entries(props.field.properties || {}).map(([name, field]) => getFormField(Object.assign({}, field, { name }, { 'x-powerDisabled': props.field['x-powerDisabled'] }), name)) || '')));
/**
 * 最大最小值校验语句
 */
const minAndMaxMessage = (format, title, value, isgt) => {
    const gtStr = isgt ? '大于' : '小于';
    let formatValue = value;
    if (['datetime', 'date', 'time'].includes(format)) {
        formatValue = formatTimeByType(format, value);
    }
    return `${title}不能${gtStr}${formatValue}`;
};
/**
 * 判断当前值是否存在
 */
const isExist = (_, checkIsNum = true) => {
    if (!checkIsNum)
        return _ !== undefined && _ !== null;
    return _ !== undefined && _ !== null && typeof _ === 'number';
};
/**
 * 获取字段校验参数
 */
const getValidator = (field) => {
    const { minLength, maxLength, minimum, maximum, pattern, format, title, type } = field;
    const validators = [];
    // 历史数据与字段类型校验，https://iwiki.woa.com/pages/viewpage.action?pageId=1083488829
    validators.push({
        validator: (value) => {
            const vlaueType = typeof value;
            const errorTypeStr = FIELD_TYPE_NAME[type] ? `应为${FIELD_TYPE_NAME[type]}类型，` : '';
            let hasError = false;
            if (!(isValueNone(value) || type === vlaueType || (type === 'array' && Array.isArray(value)))) {
                hasError = true;
            }
            return hasError ? `"${title || '当前'}"字段格式填写有误，${errorTypeStr}请重新填写` : '';
        },
    });
    // 输入长度校验
    if (isExist(maxLength) || isExist(minLength)) {
        validators.push({
            validator: (value) => {
                if ((value === null || value === void 0 ? void 0 : value.length) === 0) {
                    return '';
                }
                if ((value === null || value === void 0 ? void 0 : value.length) < minLength) {
                    return `${title}长度不能小于${minLength}`;
                }
                if ((value === null || value === void 0 ? void 0 : value.length) > maxLength) {
                    return `${title}长度不能大于${maxLength}`;
                }
            },
        });
    }
    // 最大最小值范围校验
    if (isExist(maximum) || isExist(minimum)) {
        validators.push({
            validator: (value) => {
                if (isValueNone(value)) {
                    return '';
                }
                if (parseFloat(value) < minimum) {
                    return minAndMaxMessage(format, title, minimum, false);
                }
                if (parseFloat(value) > maximum) {
                    return minAndMaxMessage(format, title, maximum, true);
                }
            },
        });
    }
    // 正则校验
    if (isExist(pattern, false)) {
        validators.push({
            pattern,
        });
    }
    // 邮箱校验
    if (isExist(format, false) && format === 'email') {
        validators.push({
            validator: (value) => {
                if (isEmail(value, false, true) || isValueNone(value)) {
                    return '';
                }
                return '该邮箱不可用，请检查';
            },
        });
    }
    // 固定电话校验
    if (format === 'tel') {
        validators.push({
            validator: (value) => {
                if (/^(0\d{2,3}-)?(\d{7,8})$/.test(value) || isValueNone(value)) {
                    return '';
                }
                return '该字段不是有效的固定电话';
            },
        });
    }
    // 地理位置校验
    if (isExist(format, false) && format === 'x-location') {
        const isRequired = field === null || field === void 0 ? void 0 : field['x-required'];
        validators.push({
            validator: (value) => {
                var _a;
                const { address, geopoint } = value || {};
                const [longitude, latitude] = (geopoint === null || geopoint === void 0 ? void 0 : geopoint.coordinates) || [];
                const innerFieldValidator = (fields, value) => {
                    var _a;
                    const validators = getValidator(fields).slice(1);
                    const results = validators.map((d) => { var _a; return (_a = d === null || d === void 0 ? void 0 : d.validator) === null || _a === void 0 ? void 0 : _a.call(d, value); });
                    return (_a = results === null || results === void 0 ? void 0 : results.filter((info) => info)) === null || _a === void 0 ? void 0 : _a[0];
                };
                const availableCount = (_a = [address, longitude, latitude].filter((v) => !['', null, undefined].includes(v))) === null || _a === void 0 ? void 0 : _a.length;
                const lngCheck = innerFieldValidator(locationProperties.longitude, longitude);
                const latCheck = innerFieldValidator(locationProperties.latitude, latitude);
                if (longitude && lngCheck) {
                    return lngCheck;
                }
                if (latitude && latCheck) {
                    return latCheck;
                }
                if ((isRequired && availableCount === 3) || (!isRequired && [0, 3].includes(availableCount))) {
                    return '';
                }
                return '请填写完整地理位置信息';
            },
        });
    }
    // formily 自带校验
    if (isExist(format, false) && AUTO_FORMAT[format]) {
        validators.push({
            format: AUTO_FORMAT[format],
        });
    }
    return validators;
};
