import React from 'react';
import { Form, Button } from 'tea-component';
import { mapProps, connect, observer, useForm } from '../formily-react';
import { isEmpty } from '../../utils/config';
import { SelectableBlock } from '../SelectableBlock';
/**
 * 桥接 Tea 输入组件
 */
export const FormItem = connect(Form.Item, mapProps({
    title: 'label',
}, 
// @ts-ignore
(props, field) => {
    var _a;
    const takeMessage = () => {
        const split = (messages) => messages.reduce((buf, text, index) => {
            if (!text)
                return buf;
            return index < messages.length - 1 ? buf.concat([text, ', ']) : buf.concat([text]);
        }, []);
        // @ts-ignore
        if (field.validating)
            return;
        // @ts-ignore
        if (field.errors.length)
            return split(field.errors);
        // @ts-ignore
        if (field.warnings.length)
            return split(field.warnings);
        // @ts-ignore
        if (field.successes.length)
            return split(field.successes);
        // 描述的优先级最低
        // @ts-ignore
        if (props.description)
            return props.description;
    };
    return {
        ...props,
        align: 'middle',
        message: takeMessage(),
        // @ts-ignore
        required: field.required,
        // @ts-ignore
        status: ((_a = field.errors) === null || _a === void 0 ? void 0 : _a.length) ? 'error' : undefined,
    };
}));
/**
 * 提交按钮
 */
export const Submit = observer(({ onSubmit, onCancel, onClick, buttons, selectableBlockEvents, compId, exportValue, field, ...rest }) => {
    const form = useForm();
    const [isDisabled, setIsDisabled] = React.useState(false);
    React.useEffect(() => {
        if (!isEmpty(form.errors)) {
            setIsDisabled(true);
        }
        else {
            setIsDisabled(false);
        }
    }, [form.errors]);
    const actionMap = {
        submit: async (e) => {
            onClick && onClick(e);
            try {
                setIsDisabled(true);
                await form.submit(onSubmit);
            }
            catch (e) {
                console.error('模型：提交报错。', e);
            }
            finally {
                setIsDisabled(false);
            }
        },
        cancel: () => {
            onCancel === null || onCancel === void 0 ? void 0 : onCancel();
        },
    };
    return (React.createElement(React.Fragment, null, buttons && Array.isArray(buttons) ? (buttons.map((props, index) => {
        const { loading, action, disabled } = props;
        return (React.createElement(SelectableBlock, { ...props, loading: loading || (action === 'submit' && form.submitting), disabled: disabled || (action === 'submit' && isDisabled), htmlType: () => {
                if (action === 'submit') {
                    return onSubmit ? 'button' : 'submit';
                }
                return 'button';
            }, component: "Button", index: index, compId: compId, blockKey: "buttons", exportValue: exportValue, actionMap: actionMap, style: { marginBottom: '12px', ...props.style }, selectableBlockEvents: selectableBlockEvents }));
    })) : (React.createElement(React.Fragment, null,
        React.createElement(Button, { ...rest, disabled: isDisabled, type: "primary", htmlType: onSubmit ? 'button' : 'submit', loading: rest.loading !== undefined ? rest.loading : form.submitting, onClick: actionMap === null || actionMap === void 0 ? void 0 : actionMap.submit }, rest.children),
        React.createElement(Button, { onClick: () => onCancel === null || onCancel === void 0 ? void 0 : onCancel() }, "\u53D6\u6D88")))));
}, {
    forwardRef: true,
});
