import React, { useEffect } from 'react';
import classNames from 'classnames';
import { Form, Input, Text } from 'tea-component';
import './Location.css';
export const locationProperties = {
    address: {
        title: '地点',
        type: 'string',
        name: 'address',
    },
    longitude: {
        title: '经度',
        type: 'number',
        name: 'longitude',
        maximum: 180,
        minimum: -180,
    },
    latitude: {
        title: '纬度',
        type: 'number',
        name: 'latitude',
        maximum: 90,
        minimum: -90,
    },
};
const convertNumber = (number) => {
    const value = parseFloat(number);
    return value || value === 0 ? value : null;
};
const Location = (props) => {
    var _a, _b, _c, _d, _e;
    const { onChange, field, value, readonly, parentIsArray } = props || {};
    const initAddress = value === null || value === void 0 ? void 0 : value.address;
    const isRequired = field === null || field === void 0 ? void 0 : field['x-required'];
    const disabled = field === null || field === void 0 ? void 0 : field['x-powerDisabled'];
    // 新建或编辑时设置空值状态下的初始值
    useEffect(() => {
        if (!initAddress && !isRequired) {
            onChange === null || onChange === void 0 ? void 0 : onChange({
                address: '',
                geopoint: {
                    type: 'Point',
                    coordinates: [null, null],
                },
            });
        }
    }, [initAddress]);
    const onTrigger = (changeValue) => {
        var _a, _b, _c, _d;
        const currentValue = {
            address: value === null || value === void 0 ? void 0 : value.address,
            longitude: (_b = (_a = value === null || value === void 0 ? void 0 : value.geopoint) === null || _a === void 0 ? void 0 : _a.coordinates) === null || _b === void 0 ? void 0 : _b[0],
            latitude: (_d = (_c = value === null || value === void 0 ? void 0 : value.geopoint) === null || _c === void 0 ? void 0 : _c.coordinates) === null || _d === void 0 ? void 0 : _d[1],
        };
        const newValue = {
            ...currentValue,
            ...changeValue,
        };
        const finalValue = {
            address: (newValue === null || newValue === void 0 ? void 0 : newValue.address) || '',
            geopoint: {
                type: 'Point',
                coordinates: [convertNumber(newValue === null || newValue === void 0 ? void 0 : newValue.longitude), convertNumber(newValue === null || newValue === void 0 ? void 0 : newValue.latitude)],
            },
        };
        onChange === null || onChange === void 0 ? void 0 : onChange(finalValue);
    };
    const options = [
        {
            name: 'address',
            title: '地址',
            ...(_a = field === null || field === void 0 ? void 0 : field.properties) === null || _a === void 0 ? void 0 : _a.address,
            props: {
                value: value === null || value === void 0 ? void 0 : value.address,
                size: 'full',
                disabled,
                onChange: (value) => {
                    onTrigger({ address: value });
                },
            },
        },
        {
            name: 'longitude',
            title: '经度',
            props: {
                value: (_c = (_b = value === null || value === void 0 ? void 0 : value.geopoint) === null || _b === void 0 ? void 0 : _b.coordinates) === null || _c === void 0 ? void 0 : _c[0],
                size: 'full',
                type: 'number',
                disabled,
                onChange: (value) => {
                    onTrigger({ longitude: value });
                },
            },
        },
        {
            name: 'latitude',
            title: '纬度',
            props: {
                value: (_e = (_d = value === null || value === void 0 ? void 0 : value.geopoint) === null || _d === void 0 ? void 0 : _d.coordinates) === null || _e === void 0 ? void 0 : _e[1],
                size: 'full',
                type: 'number',
                disabled,
                onChange: (value) => {
                    onTrigger({ latitude: value });
                },
            },
        },
    ];
    const renderFormLabel = (option) => {
        if (parentIsArray) {
            return React.createElement("div", { className: "weda-business-location-form-array-label" }, option.title);
        }
        return option.title;
    };
    return (React.createElement(Form, { readonly: readonly }, options.map((option) => {
        var _a;
        return (React.createElement(Form.Item, { className: classNames({ ['weda-business-location-form-item']: parentIsArray }), label: renderFormLabel(option) }, readonly ? (React.createElement(Form.Text, null,
            React.createElement(Text, null, (_a = option === null || option === void 0 ? void 0 : option.props) === null || _a === void 0 ? void 0 : _a.value))) : (React.createElement(Input, { ...option.props, className: "weda-business-location-input" }))));
    })));
};
export default Location;
