import React, { useState, useEffect } from 'react';
import { observer, useField } from '../formily-react';
import { Input, Button, Modal, TagSelect } from 'tea-component';
import { BaseTable } from '../Table';
import { resolveFieldsFromSchema, getDataSourceByName } from '../../utils';
import { useAuthFields } from '../../hooks';
export const RelationSelector = ({ value, onChange, style, field, disabled }) => {
    var _a, _b;
    const [visible, setVisible] = useState(false);
    const [displayValue, setDisplayValue] = useState((value === null || value === void 0 ? void 0 : value.text) || (value === null || value === void 0 ? void 0 : value.value) || '');
    const [showFields, setShowFields] = useState([]);
    const dbName = ((_a = field === null || field === void 0 ? void 0 : field.parentDatasource) === null || _a === void 0 ? void 0 : _a.name) || ((_b = field === null || field === void 0 ? void 0 : field['x-parent']) === null || _b === void 0 ? void 0 : _b.parentDataSourceName);
    const onSelect = () => setVisible(true);
    useEffect(() => {
        async function getUseAuthFields() {
            const dataSource = await getDataSourceByName(dbName);
            const allFieldNames = resolveFieldsFromSchema(dataSource === null || dataSource === void 0 ? void 0 : dataSource.schema).map((item) => item === null || item === void 0 ? void 0 : item.name);
            const { showFields } = await useAuthFields(dbName, allFieldNames, true);
            setShowFields(showFields);
        }
        getUseAuthFields();
    }, [dbName]);
    useEffect(() => {
        var _a;
        // onChange 会主动触发formily表单校验
        const primaryColumn = (_a = field.parentDatasource) === null || _a === void 0 ? void 0 : _a.schema;
        if (value === null || value === void 0 ? void 0 : value.value) {
            onChange(value === null || value === void 0 ? void 0 : value.value);
        }
        else if (primaryColumn && primaryColumn['x-primary-column']) {
            value && onChange(value);
        }
        else {
            setDisplayValue(value);
            value && onChange(value);
        }
    }, [field]);
    // 获取默认选定
    const getDefaultKeys = () => {
        const keys = [];
        if (value === null || value === void 0 ? void 0 : value.value) {
            keys.push(value === null || value === void 0 ? void 0 : value.value);
        }
        else if (value) {
            keys.push(value);
        }
        return keys;
    };
    return (React.createElement(React.Fragment, null,
        React.createElement(SingleSelector, { ...{ style, onSelect, displayValue, disabled } }),
        React.createElement(Modal, { className: "model-relation-selector-modal", size: "xl", visible: visible, caption: `选择${field === null || field === void 0 ? void 0 : field.title}`, onClose: () => {
                setVisible(false);
            } },
            React.createElement(Modal.Body, null,
                React.createElement(BaseTable, { dbName: dbName, fields: showFields, onSelectCancel: () => setVisible(false), onSelectConfirm: (record) => {
                        var _a;
                        const primaryColumn = (_a = field === null || field === void 0 ? void 0 : field.parentDatasource) === null || _a === void 0 ? void 0 : _a.schema['x-primary-column'];
                        setVisible(false);
                        record[primaryColumn] ? setDisplayValue(record[primaryColumn]) : setDisplayValue(record._id);
                        onChange(record._id);
                    }, multipleSelect: false, defaultsSlectedKeys: getDefaultKeys(), tableAction: false, actionColumn: false, defaultSearchValue: undefined, compId: undefined, buttons: undefined, settings: undefined, selectableBlockEvents: undefined, filterFields: undefined, rowColor: undefined, columnSets: undefined, slots: undefined, events: undefined, isModel: true })))));
};
// 选择单个关联数据
const SingleSelector = ({ style, onSelect, displayValue, disabled }) => (React.createElement("div", { style: { ...style, position: 'relative', display: 'inline-block', width: '100%' } },
    React.createElement(Input, { value: displayValue, disabled: disabled, size: "full", onFocus: (e) => {
            var _a, _b;
            // HACK 禁止输入
            (_b = (_a = e.target).blur) === null || _b === void 0 ? void 0 : _b.call(_a);
            onSelect();
        } }),
    React.createElement("div", { style: { position: 'absolute', right: '4px', top: '2px' } },
        React.createElement(Button, { icon: "plus", title: "\u9009\u62E9", onClick: () => onSelect() }))));
// 选择多个关联数据
const ArraySelector = observer(({ style, onSelect, onChange }) => {
    const field = useField();
    return (React.createElement("div", { style: style, className: "array-selector" },
        React.createElement("div", { style: { position: 'relative' } },
            React.createElement(TagSelect, { open: false, 
                // @ts-ignore
                value: field.value || [], onChange: (v) => onChange(v), onFocus: (e) => {
                    var _a, _b;
                    // HACK 禁止输入
                    // @ts-ignore
                    (_b = (_a = e.target).blur) === null || _b === void 0 ? void 0 : _b.call(_a);
                    onSelect();
                } }),
            React.createElement("div", { style: { position: 'absolute', right: '4px', top: '2px' } },
                React.createElement(Button, { icon: "plus", title: "\u9009\u62E9", onClick: () => onSelect() })))));
});
