import React from 'react';
import { ConfigProvider, Form, FormAction } from 'tea-component';
import { createForm, FormProvider } from '../formily-react';
import { filterInvalidFormValue } from '../../utils';
import { getFormField } from './Field';
import { getInitialValues } from './utils';
import { useMediaQuery } from '../../hooks';
import { Submit } from './FormItem';
import { EnumProvider } from '../../context';
import './index.less';
export const ModelForm = ({ fields, action = 'create', initialData, onSubmit, compId, buttons, selectableBlockEvents, }) => {
    /**
     * 去除空字段，隐藏计算字段
     */
    const trimFields = fields.filter((field) => {
        var _a;
        return field &&
            typeof field === 'object' &&
            ((_a = Object.keys(field)) === null || _a === void 0 ? void 0 : _a.length) &&
            field.format !== 'function';
    });
    /**
     * 根据 fields 获取初始值
     * 仅首次渲染时可以设定初始值
     */
    const form = React.useMemo(() => createForm({
        initialValues: getInitialValues(action, trimFields, initialData),
        validateFirst: true,
    }), [action, fields, initialData]);
    const isH5 = useMediaQuery('(max-width: 1024px)');
    const onSubmitFormValue = async (formValues) => {
        const filteredValue = filterInvalidFormValue(formValues, trimFields);
        await (onSubmit === null || onSubmit === void 0 ? void 0 : onSubmit(filteredValue));
    };
    return (React.createElement(ConfigProvider, { classPrefix: "wedatea2td" },
        React.createElement(FormProvider, { form: form },
            React.createElement(EnumProvider, { fields: trimFields },
                React.createElement(Form, { className: "model-form", layout: isH5 ? 'vertical' : 'default' }, (trimFields === null || trimFields === void 0 ? void 0 : trimFields.length)
                    ? trimFields.map((field, i) => getFormField(field, i, { relation: initialData === null || initialData === void 0 ? void 0 : initialData[`@${field.name}`] }))
                    : '')),
            React.createElement(FormAction, null,
                React.createElement(Submit, { compId: compId, exportValue: { data: form.values }, buttons: buttons, selectableBlockEvents: selectableBlockEvents, onCancel: () => {
                        window.app.navigateBack();
                    }, onSubmit: onSubmitFormValue })))));
};
