import { formatTimeToNumber } from '../../utils/date';
/**
 * 获取表单初始值或默认值
 */
export const getInitialValues = (action, fields, selectedContent = {}) => {
    let initialValues = selectedContent;
    if (action === 'create') {
        initialValues = fields === null || fields === void 0 ? void 0 : fields.reduce((prev, field) => {
            const defaultValue = getTypeDefaultValue(field);
            return {
                ...prev,
                [field.name]: defaultValue,
            };
        }, {});
    }
    else if (action === 'edit') {
        fields === null || fields === void 0 ? void 0 : fields.forEach((field) => {
            const { type, name, format } = field;
            const fieldValue = selectedContent[name];
            // 布尔值默认为 false
            if (type === 'boolean' && typeof fieldValue !== 'boolean') {
                initialValues[name] = false;
            }
            // 数字默认为 0
            if (type === 'number' && typeof fieldValue !== 'number') {
                initialValues[name] = 0;
            }
            if (format === 'date' || format === 'datetime') {
                initialValues[name] = Date.now();
            }
        });
    }
    else if (action === 'update') {
        fields === null || fields === void 0 ? void 0 : fields.forEach((field) => {
            var _a, _b, _c, _d;
            const { name, format } = field;
            if (selectedContent && selectedContent[name]) {
                const fieldValue = selectedContent[name];
                const primaryColumn = (_a = field.parentDatasource) === null || _a === void 0 ? void 0 : _a.schema;
                // 排除部门用用户组件类型字段
                if (primaryColumn &&
                    primaryColumn['x-primary-column'] &&
                    !(format === 'father-son' && ['sys_department', 'sys_user'].includes((_b = field['x-parent']) === null || _b === void 0 ? void 0 : _b.parentDataSourceName))) {
                    if (selectedContent && selectedContent[`@${name}`]) {
                        const selectedName = selectedContent[`@${name}`];
                        const record = (_c = selectedName === null || selectedName === void 0 ? void 0 : selectedName.v1) === null || _c === void 0 ? void 0 : _c.record;
                        const primaryColumn = (_d = selectedName === null || selectedName === void 0 ? void 0 : selectedName.v1) === null || _d === void 0 ? void 0 : _d.primaryColumn;
                        if ((record === null || record === void 0 ? void 0 : record._id) === fieldValue) {
                            const text = record[primaryColumn];
                            initialValues[name] = { value: initialValues[name], text };
                        }
                    }
                }
                // }
            }
        });
    }
    return initialValues;
};
/**
 * 获取类型对应的默认值，若schema没有默认值则保留 undefined
 */
export const getTypeDefaultValue = (field, typeDefault) => {
    const { type, format, properties } = field || {};
    // 用户设置了默认值，按照用户设置进行显示。
    let defaultValue = field === null || field === void 0 ? void 0 : field.default;
    // 如果是boolean类型，用户未设置默认值，则在必填情况下，默认值为否，非必填情况下，默认值为空
    if (defaultValue === undefined && type === 'boolean' && field['x-required']) {
        defaultValue = false;
    }
    // 除对象外，无默认值时保留 undifiend
    if (!typeDefault && defaultValue === undefined && type !== 'object') {
        return defaultValue;
    }
    // 字符串，默认为 ''
    if (type === 'string' && typeof defaultValue !== 'string') {
        defaultValue = '';
    }
    // 布尔，默认为 false
    if (type === 'boolean' && typeof defaultValue !== 'boolean') {
        defaultValue = false;
    }
    // 数字，默认为 0
    if (type === 'number') {
        if (format === 'date' || format === 'datetime') {
            defaultValue = typeof defaultValue === 'number' ? defaultValue : Date.now();
        }
        else if (format === 'time') {
            defaultValue = typeof defaultValue === 'number' ? defaultValue : formatTimeToNumber();
        }
        else {
            if (typeof defaultValue !== 'number') {
                defaultValue = 0;
            }
        }
    }
    // 对象，不存在默认值，需要遍历
    if (type === 'object' && properties) {
        defaultValue = Object.values(properties).reduce((prev, subField) => {
            const subDefaultValue = getTypeDefaultValue(subField);
            return {
                ...prev,
                [subField.name]: subDefaultValue,
            };
        }, {});
    }
    // 数组，不存在默认值，只有新增时才会出现默认值
    if (type === 'array' && !Array.isArray(defaultValue)) {
        defaultValue = [];
    }
    return defaultValue;
};
