import React, { useState, useEffect, useRef } from 'react';
import { Status, List } from 'tea-component';
import { navigateToModelPage } from '../../utils';
import './index.css';
const StatusTitle = {
    blank: '内容为空',
    chart: '图表为空或异常',
    loading: '加载中',
    network: '网络异常',
    pay: '待续费',
    search: '搜索无结果',
    upload: '图片上传',
};
/**
 * 页面内容容器组件
 * 继承样式展示
 */
export const PageContent = ({ loading, children, style, ...props }) => {
    let status = '';
    if (loading) {
        status = 'loading';
    }
    return (React.createElement("div", { style: style, ...props }, status ? (React.createElement(Status, { size: "l", 
        // @ts-ignore
        icon: status, title: StatusTitle[status] || '', description: "\u6B63\u5728\u52A0\u8F7D\u9875\u9762" })) : (children)));
};
/**
 * 无字段权限时倒计时返回主页面
 */
export const PageCountDown = ({ dbName, description, ...props }) => {
    const [count, setCount] = useState(5);
    const timeRef = useRef(null);
    useEffect(() => {
        count <= 0 && navigateToModelPage('list', { dbName });
    }, [count]);
    useEffect(() => {
        clearInterval(timeRef.current);
        timeRef.current = setInterval(() => {
            setCount((count) => (count > 0 ? count - 1 : 0));
        }, 1000);
        return () => clearInterval(timeRef.current);
    }, []);
    return (React.createElement("div", { ...props, style: { margin: '20px 0' }, key: dbName },
        description || '当前页面无具备编辑的任何字段',
        "\uFF0C",
        count,
        " \u79D2\u540E\u81EA\u52A8\u8FD4\u56DE\u5217\u8868\u9875"));
};
/**
 * 当前页面未绑定数据源
 */
export const PageNullDb = (props) => {
    return (React.createElement("div", { key: "page_null_db", ...props }, "\u672A\u7ED1\u5B9A\u6570\u636E\u6E90"));
};
/**
 * 错误信息展示
 *
 */
export const PageErrorContent = ({ errorMessageInfo, ...props }) => {
    const { title, message, code } = errorMessageInfo;
    const errorInfoList = {
        错误码: code || '暂无错误码',
        错误信息: message || '暂无错误信息',
    };
    return (React.createElement(Status, { size: 'l', title: title, description: React.createElement("div", { className: "weda-errormessage-status__detail" },
            React.createElement("div", { className: "weda-errormessage-status__detail_wrap" },
                React.createElement("div", { className: "weda-errormessage-status__detail_wrap_header" },
                    React.createElement("p", null, "\u6545\u969C\u6392\u9664\u8BE6\u7EC6\u4FE1\u606F\uFF1A"),
                    React.createElement("a", { href: "https://docs.cloudbase.net/lowcode/faq", target: "_blank" }, "\u5E38\u89C1\u95EE\u9898\u6307\u5F15")),
                React.createElement("div", { className: "weda-errormessage-status__detail_wrap_content" },
                    React.createElement(List, null, Object.entries(errorInfoList).map((item) => (React.createElement(List.Item, null,
                        item[0],
                        ":",
                        item[1]))))))) }));
};
