import React from 'react';
import { ConfigProvider, Button, Popover } from 'tea-component';
import { isInIde, isWXIDE } from '../../utils';
import './index.less';
/**
 * 选区组件
 */
export function SelectableBlock({ index, show, component, action, actionMap, actionParams, compId, selectableBlockEvents, selectableBlock, exportValue, blockKey, isSelectable = true, // 当前元素是否为选区，默认为true
...rest }) {
    /**
     * selectableBlockEvents中
     * events: 事件面板的events
     * onCustomEvent: 预览和发布中的onClick事件
     * onClick: 编辑器的onClick事件
     * onMouseOver: 编辑器的onMouseOver事件
     * onMouseOut: 编辑器的onMouseOut事件
     */
    const { events, onCustomEvent, onMouseOver, onMouseOut, ...restEvents } = selectableBlockEvents || {};
    const isSelectableMix = isInIde() && isSelectable;
    const SelectableBlockButton = () => (React.createElement(Button, { key: index, ...rest, ...restEvents, onMouseOver: isSelectableMix ? onMouseOver : null, onMouseOut: isSelectableMix ? onMouseOut : null, onClick: (e) => {
            const params = {
                event: e,
                compId,
                order: index,
                blockKey,
            };
            if (isInIde()) {
                const { onClick } = selectableBlockEvents || {};
                isSelectable && onClick && onClick(params);
            }
            else {
                if (action === 'custom') {
                    // 1、onCustomEvent是运行态的render传过来的方法
                    // 2、点击按钮时，执行onCustomEvent，回调render
                    // 3、render中onCustomEvent根据params里的index等参数去拿到第几个选区的listenInstance
                    // 4、listenInstance转换为events，放到selectableBlockEvents里面，回传到选区
                    onCustomEvent && onCustomEvent(params);
                    // 1、上一步selectableBlockEvents里面的events，即为当前点击按钮所需的事件
                    // 2、执行该按钮的事件
                    // 3、setTimeout作用是保证先执行上一步，拿到events，才执行下一步的该events.tap
                    setTimeout(() => {
                        events.tap && events.tap({ value: { ...exportValue } }, { originEvent: e });
                    }, 0);
                }
                else if (action) {
                    const excuteAction = actionMap[action];
                    excuteAction && excuteAction(e, actionParams);
                }
            }
        } }, rest.text || `按钮${index + 1}`));
    const componentMap = {
        Button: SelectableBlockButton,
    };
    return (show && (React.createElement(ConfigProvider, { classPrefix: "wedatea2td" }, isSelectableMix ? (React.createElement(Popover, { placement: "top-start", placementOffset: "-1", escapeWithReference: true, overlay: React.createElement("div", null,
            React.createElement("div", { style: {
                    display: 'inline-block',
                    background: isWXIDE() ? '#00b26a' : '#0052d9',
                    color: 'white',
                    padding: 3,
                    borderRadius: '0 0 0 2px',
                    cursor: 'default',
                    fontSize: '12px',
                } }, "\u9009\u533A"),
            React.createElement("div", { style: {
                    display: 'inline-block',
                    padding: 2,
                    borderRadius: '0 2px 2px 0',
                    border: isWXIDE() ? '1px solid #00b26a' : '1px solid #0052d9',
                    cursor: 'default',
                    background: '#f3f4f7',
                    boxSizing: 'border-box',
                    fontSize: '12px',
                } }, `${blockKey}${index + 1}`)) }, componentMap[component] && componentMap[component]())) : (componentMap[component] && componentMap[component]()))));
}
