import React, { useEffect, useState } from 'react';
import { Modal, Button, Stepper, Dropdown, List, Status, Icon } from 'tea-component';
import { UploadCSVFile } from './uploadCSVFile';
import { alertErrorMessage } from '../../../utils/feedback';
import { useImportingStatus } from '../hooks/useImportingStatus';
import './index.less';
import { callWedaApi } from '../../../utils';
// 字段映射
const FileTitleDropdown = ({ fileHeader, fileItem, upddateFileJsonMap }) => {
    const [currentItem, setCurrentItem] = useState(fileItem === null || fileItem === void 0 ? void 0 : fileItem.fileTitle);
    return (React.createElement(Dropdown, { clearable: true, trigger: "click", appearance: "button", style: { marginRight: 10 }, button: currentItem, onClear: () => {
            setCurrentItem(null);
            upddateFileJsonMap(null);
        } },
        React.createElement(List, { type: "option" }, fileHeader.map((i, index) => (React.createElement(List.Item, { key: index, onClick: () => {
                setCurrentItem(i);
                upddateFileJsonMap(i);
            } }, i))))));
};
// 提交前校验是否存在重复字段
const repeatabilityCheck = (fileJsonMap) => {
    let repeatList = [];
    const fileTitleList = [];
    fileJsonMap.forEach((i) => {
        if (!fileTitleList.includes(i.fileTitle)) {
            fileTitleList.push(i.fileTitle);
        }
        else {
            repeatList.push(i.fileTitle);
            repeatList = [...new Set(repeatList)];
        }
    });
    return repeatList;
};
/**
 * 菜单导航配置弹窗
 */
export function ImportFileModalByApi({ onCancel, handleInOrOutRecordModealVisible, visible, fields, datasourceTitle, dbName, }) {
    const [file, setFile] = useState(null);
    const [fileId, setFileIds] = useState(null);
    const [fileHeader, setFileHeader] = useState([]);
    const [fileJsonMap, setFileJsonMap] = useState([]);
    const [disableNext, setDisableNext] = useState(true);
    const [status, setStatus] = useState(null);
    const [errorMessage, setErrorMessage] = useState([]);
    const { fileImportStatus, fileImportErrorMessage } = useImportingStatus(fileId, dbName, 'import');
    const [createDataImportQuery, setCreateDataImportQuery] = useState({
        writeType: 'create',
        isTriggerWorkflow: false,
        fileId: '',
        fileName: '',
        fileSize: 0,
    });
    const importFileHandle = async () => {
        var _a;
        try {
            // 设置上传参数
            // 去除空字段，隐藏计算字段
            const trimFields = fileJsonMap.filter((field) => { var _a; return field && typeof field === 'object' && ((_a = Object.keys(field)) === null || _a === void 0 ? void 0 : _a.length) && field.format !== 'function'; });
            const fieldMapAction = (_a = trimFields === null || trimFields === void 0 ? void 0 : trimFields.map(({ name, fileTitle, format, updateRelateByFieldsKey }) => {
                const result = {
                    FieldId: name,
                    ColumnName: fileTitle,
                };
                if (format === 'related' || format === 'x-enum' || format === 'father-son') {
                    result['Strategy'] = updateRelateByFieldsKey;
                }
                return result;
            })) === null || _a === void 0 ? void 0 : _a.filter((item) => item === null || item === void 0 ? void 0 : item.ColumnName); // 没有选择表列字段的过滤掉
            const { fileSize, fileName, fileId, isTriggerWorkflow, writeType } = createDataImportQuery;
            // 启动导入接口
            const queryParams = {
                action: 'CreateDataImport',
                data: {
                    FileId: fileId,
                    FileName: fileName,
                    FileSize: fileSize,
                    WriteType: writeType,
                    TriggerWorkflow: isTriggerWorkflow,
                    DataSourceName: dbName,
                    FieldMap: fieldMapAction,
                    UpdateBy: 'identity',
                },
            };
            const { FileId, Status, ErrorMessage } = await callWedaApi(queryParams);
            if (Status === 'success') {
                setStatus('success');
            }
            else if (Status === 'failure') {
                setErrorMessage(['导入文件失败', [ErrorMessage || '']]);
            }
            else if (Status === 'importing' || Status === 'init') {
                // 轮询查询状态
                setFileIds(FileId);
            }
        }
        catch (error) {
            setStatus('error');
            setErrorMessage(['导入文件请求失败', [(error === null || error === void 0 ? void 0 : error.message) || '']]);
            console.error('importFileHandle error', error);
        }
    };
    // 枚举 关联关系 主子明细根据可选「选项标识」「选项值」「主列字段」或「数据标识」作为导入匹配标识
    const getUpdateRelateByFields = ({ format, updateRelateByFieldsValue, name }) => {
        const relatedMenuList = [
            {
                key: 'identity',
                value: '数据标识',
            },
            {
                key: 'primary',
                value: '主列字段',
            },
        ];
        const menuList = {
            related: relatedMenuList,
            'x-enum': [
                {
                    key: 'key',
                    value: '选项标识',
                },
                {
                    key: 'value',
                    value: '选项值',
                },
            ],
            'father-son': relatedMenuList,
        };
        const menu = (format) => {
            var _a;
            return (React.createElement(List, { type: "option" }, (_a = menuList === null || menuList === void 0 ? void 0 : menuList[format]) === null || _a === void 0 ? void 0 : _a.map((item) => (React.createElement(List.Item, { key: item.key, onClick: () => {
                    const list = fileJsonMap.map((fileItem) => {
                        if (fileItem.name === name) {
                            fileItem.updateRelateByFieldsValue = item.value;
                            fileItem.updateRelateByFieldsKey = item.key;
                        }
                        return fileItem;
                    });
                    setFileJsonMap(list);
                } }, item.value)))));
        };
        if (format === 'related' || format === 'x-enum' || format === 'father-son') {
            return (React.createElement(Dropdown, { trigger: "click", clickClose: true, button: updateRelateByFieldsValue, appearance: "button", className: "file-json-map-content_update_fileds" }, menu(format)));
        }
        return null;
    };
    const addUpdateFieldsBy = (fileJsonMapList, fileHeader, type) => {
        const isExidsId = fileHeader === null || fileHeader === void 0 ? void 0 : fileHeader.find((item) => item === '数据标识');
        const isSelectUpdateBy = (fileJsonMapList === null || fileJsonMapList === void 0 ? void 0 : fileJsonMapList.findIndex((item) => item.name === '_id')) !== -1;
        if (!isSelectUpdateBy && type === 'add') {
            fileJsonMapList === null || fileJsonMapList === void 0 ? void 0 : fileJsonMapList.unshift({
                fileTitle: isExidsId || '请指定更新依据字段',
                title: '数据标识（更新依据字段）',
                name: '_id',
            });
        }
        if (isSelectUpdateBy && type === 'remove') {
            fileJsonMapList === null || fileJsonMapList === void 0 ? void 0 : fileJsonMapList.shift();
        }
        setFileJsonMap(fileJsonMapList);
    };
    useEffect(() => {
        if (fileImportStatus === 'success') {
            setStatus('success');
        }
        else if (fileImportStatus === 'failure') {
            setStatus('error');
            setErrorMessage([fileImportErrorMessage === null || fileImportErrorMessage === void 0 ? void 0 : fileImportErrorMessage.ErrorMessage]);
        }
    }, [fileImportStatus, fileImportErrorMessage]);
    const steps = [
        {
            id: 'uploadFile',
            label: '选择文件',
            component: (React.createElement(UploadCSVFile, { file: file, setFile: setFile, datasourceTitle: datasourceTitle, dbName: dbName, fields: fields, parseFile: ({ fileHeader }) => {
                    setFileHeader(fileHeader);
                    const fileJsonMapList = fields.map((i) => {
                        const item = { ...i };
                        item.fileTitle = fileHeader.find((j) => j === item.title);
                        if (i.format === 'x-enum') {
                            item.updateRelateByFieldsValue = '选项标识';
                            item.updateRelateByFieldsKey = 'key';
                        }
                        if (i.format === 'related' || i.format === 'father-son') {
                            item.updateRelateByFieldsValue = '主列字段';
                            item.updateRelateByFieldsKey = 'primary';
                        }
                        return item;
                    });
                    setFileJsonMap(fileJsonMapList);
                    setDisableNext(false);
                }, setDisableNext: setDisableNext, createDataImportQuery: createDataImportQuery, setCreateDataImportQuery: setCreateDataImportQuery })),
        },
        {
            id: 'fieldMap',
            label: '映射字段',
            component: (React.createElement("div", { className: "file-json-map-wrap" },
                React.createElement("div", { className: "file-json-map-header" },
                    React.createElement("p", null, "\u6587\u6863\u5B57\u6BB5"),
                    React.createElement("span", { className: "arrow" }),
                    React.createElement("p", null, "\u8868\u683C\u5B57\u6BB5")),
                React.createElement("div", { className: "file-json-map-container" }, fileJsonMap.map((i, index) => (React.createElement("div", { className: "file-json-map-content", key: index },
                    React.createElement(FileTitleDropdown, { fileHeader: fileHeader, fileItem: i, upddateFileJsonMap: (fileTitle) => {
                            const list = fileJsonMap.map((j) => {
                                const item = { ...j };
                                if (i.title === j.title) {
                                    item.fileTitle = fileTitle;
                                }
                                return item;
                            });
                            setFileJsonMap(list);
                        } }),
                    React.createElement("span", { className: "arrow" }),
                    React.createElement("span", { className: "table-field" }, i.title),
                    getUpdateRelateByFields(i))))))),
            nextAction: () => {
                const { writeType } = createDataImportQuery;
                const updateFieldsBystatus = writeType === 'update' || writeType === 'upsert';
                addUpdateFieldsBy(fileJsonMap, fileHeader, updateFieldsBystatus ? 'add' : 'remove');
                setCurrent(next ? next.id : null);
            },
        },
        {
            id: 'importFile',
            label: '导入文件',
            component: (React.createElement("div", { className: "import-file-result-wrap" },
                status === 'error' && (React.createElement("div", { className: "error-container" },
                    React.createElement("div", { className: "error-tip" },
                        React.createElement(Icon, { type: "error", size: "s" }),
                        React.createElement("p", null, "\u6570\u636E\u5BFC\u5165\u5931\u8D25")),
                    React.createElement("div", { className: "error-content" },
                        React.createElement(List, null, Array.isArray(errorMessage)
                            ? errorMessage === null || errorMessage === void 0 ? void 0 : errorMessage.map((i, index) => React.createElement(List.Item, { key: index }, i))
                            : errorMessage)))),
                status !== 'validateError' && status !== 'error' && (React.createElement("div", { className: `status-wrap ${status}` },
                    React.createElement(Status, { icon: 'blank', size: 'm', title: status === 'success' ? '数据导入完成' : '数据导入中，请稍后...', description: status !== 'success' ? (React.createElement("div", null,
                            "\u5173\u95ED\u5F39\u7A97\u540E\uFF0C\u5BFC\u5165\u4EFB\u52A1\u4F1A\u7EE7\u7EED\u6267\u884C\uFF0C\u53EF\u5728",
                            React.createElement("a", { onClick: () => {
                                    handleInOrOutRecordModealVisible('import');
                                    onCancel();
                                } }, "\u6211\u7684\u5BFC\u5165\u8BB0\u5F55"),
                            "\u67E5\u770B\u4EFB\u52A1\u8FDB\u5EA6")) : (React.createElement("div", null,
                            React.createElement("span", null,
                                "\u6210\u529F", fileImportErrorMessage === null || fileImportErrorMessage === void 0 ? void 0 :
                                fileImportErrorMessage.Success,
                                "\u6761\uFF0C\u5931\u8D25", fileImportErrorMessage === null || fileImportErrorMessage === void 0 ? void 0 :
                                fileImportErrorMessage.Failure,
                                "\u6761"),
                            !!(fileImportErrorMessage === null || fileImportErrorMessage === void 0 ? void 0 : fileImportErrorMessage.Failure) && (React.createElement("a", { target: '_blank', href: fileImportErrorMessage === null || fileImportErrorMessage === void 0 ? void 0 : fileImportErrorMessage.ErrorDownloadUrl, rel: "noreferrer", style: { marginLeft: '6px' } }, "\u4E0B\u8F7D\u9519\u8BEF\u62A5\u544A")))) }))))),
            nextAction: () => {
                var _a;
                // 更新依据字段
                const isSelectUpdateBy = fileJsonMap.find((item) => item.fileTitle === '请指定更新依据字段');
                if (isSelectUpdateBy) {
                    alertErrorMessage({ message: `请指定更新依据字段才能更新数据` });
                    return;
                }
                const repeatList = (_a = repeatabilityCheck(fileJsonMap)) === null || _a === void 0 ? void 0 : _a.filter((item) => !!item);
                if (repeatList.length) {
                    let text = repeatList.reduce((res, i) => `${res}、${i}`);
                    text = text ? text === null || text === void 0 ? void 0 : text.replace('null', '空') : '空';
                    const modal = Modal.show({
                        size: 's',
                        children: (React.createElement(React.Fragment, null,
                            React.createElement(Modal.Body, null,
                                React.createElement(Modal.Message, { icon: "warning", message: "\u6587\u6863\u5B57\u6BB5\u91CD\u590D\u6620\u5C04", description: `文档的「${text}」字段被映射到多个表格字段，是否确认导入` })),
                            React.createElement(Modal.Footer, null,
                                React.createElement(Button, { onClick: () => {
                                        modal.destroy();
                                    } }, "\u53D6\u6D88"),
                                React.createElement(Button, { type: "primary", onClick: async () => {
                                        modal.destroy();
                                        importFileHandle();
                                        setCurrent(next ? next.id : null);
                                    } }, "\u786E\u8BA4\u5BFC\u5165")))),
                        onClose: () => modal.destroy(),
                    });
                }
                else {
                    importFileHandle();
                    setCurrent(next ? next.id : null);
                }
            },
        },
    ];
    const [current, setCurrent] = useState('uploadFile');
    const currentIndex = current ? steps.findIndex((x) => x.id === current) : -1;
    const next = current && steps[currentIndex + 1];
    const prev = current ? steps[currentIndex - 1] : steps[steps.length - 1];
    return (React.createElement(Modal, { caption: "\u5BFC\u5165\u6570\u636E", visible: visible, onClose: onCancel, destroyOnClose: true, size: 'l', className: `weda-import-csv-file-modal weda-import-by-api-csv-file-modal ${current === 'importFile' && 'none-border'}` },
        React.createElement(Modal.Body, null,
            React.createElement("div", null,
                React.createElement(Stepper, { steps: steps, current: current }),
                React.createElement("section", { className: "modal-section" }, steps[currentIndex] && steps[currentIndex].component))),
        React.createElement(Modal.Footer, null,
            current === 'uploadFile' && (React.createElement(Button, { type: "link", style: { float: 'left' }, onClick: () => {
                    handleInOrOutRecordModealVisible('import');
                    onCancel();
                } }, "\u6211\u7684\u5BFC\u5165\u8BB0\u5F55")),
            current !== 'importFile' && (React.createElement(React.Fragment, null,
                React.createElement(Button, { disabled: !prev, onClick: () => setCurrent(prev.id) }, "\u4E0A\u4E00\u6B65"),
                React.createElement(Button, { type: "primary", disabled: (!next && !current) || disableNext, onClick: () => {
                        if (next.nextAction) {
                            next.nextAction();
                        }
                    }, style: { marginLeft: 10 } }, next ? '下一步' : '开始导入'))))));
}
