import React, { useState } from 'react';
import { Upload, Form, Text, Status, Select } from 'tea-component';
import { exportData, getXlsxFileHeader } from '../../../utils/file';
import { callWedaApi } from '../../../utils';
import csvTemplate from './csvTemplate.json';
// 下载模版
const getCSVTemplate = (fields, datasourceTitle) => {
    const header = [];
    const row = [];
    fields.forEach((i) => {
        var _a;
        const value = (_a = csvTemplate.find((j) => j.type === i.type && j.format === i.format)) === null || _a === void 0 ? void 0 : _a.value;
        header.push(i.title);
        row.push(value || '');
    });
    exportData([header, row], 'xlsx', `_${datasourceTitle}_导入模版`);
};
// 上传csv文件
export function UploadCSVFile({ parseFile, fields, datasourceTitle, dbName, setDisableNext, setFile, file, createDataImportQuery, setCreateDataImportQuery, }) {
    const [status, setStatus] = useState(null);
    const [errorMsg, setErrorMsg] = useState('');
    function handleStart(file) {
        setFile(file);
        getXlsxFileHeader(file, ({ headers }) => {
            parseFile({ fileHeader: headers });
        });
    }
    function handleSuccess(value) {
        const { code } = value;
        if (code === 'BAD_REQUEST') {
            setStatus('error');
            setDisableNext(true);
        }
        else {
            setStatus('success');
        }
    }
    function handleError(error) {
        console.error('handleError', error);
        setStatus('error');
        setErrorMsg(`上传文件失败:${error}`);
        setDisableNext(true);
    }
    function beforeUpload(file, fileList, isAccepted) {
        if (!isAccepted) {
            setStatus('error');
        }
        return isAccepted;
    }
    async function getUploadUrl(file) {
        try {
            const { name, size } = file;
            const queryParams = {
                action: 'CreateDataPreImport',
                data: { FileName: name, DataSourceName: dbName },
            };
            const { UploadUrl, FileId } = await callWedaApi(queryParams);
            setCreateDataImportQuery({
                ...createDataImportQuery,
                fileId: FileId,
                fileName: name,
                fileSize: size,
            });
            return UploadUrl;
        }
        catch (error) {
            setErrorMsg(`上传文件 CreateDataPreImport 接口调用失败，${(error === null || error === void 0 ? void 0 : error.message) || ''}`);
            console.error('getUploadUrl error', error);
        }
    }
    return (React.createElement("div", { className: "modal-upload-wrap" },
        React.createElement(Form, { className: "weda_modal-upload-wrap_import_type" },
            React.createElement(Form.Item, { label: "\u5BFC\u5165\u6A21\u5F0F" },
                React.createElement(Select, { size: "m", appearance: "button", options: [
                        { value: 'create', text: '仅新增' },
                        { value: 'update', text: '仅更新' },
                        { value: 'upsert', text: '新增或更新' },
                    ], value: createDataImportQuery.writeType, onChange: (value) => setCreateDataImportQuery({
                        ...createDataImportQuery,
                        writeType: value,
                    }) }))),
        React.createElement(Form.Control, { status: status, message: status === 'error' ? errorMsg : '' },
            React.createElement(Upload, { method: "put", action: getUploadUrl, send: (file) => file, accept: ".xlsx", maxSize: 1024 * 1024 * 2, onStart: handleStart, onSuccess: handleSuccess, onError: handleError, beforeUpload: beforeUpload }, ({ open, isDragging }) => (React.createElement(Upload.Dragger, { filename: file && file.name, button: React.createElement(Status, { className: "csv-file", icon: 'blank', size: 's', description: file && file.name, operation: React.createElement(React.Fragment, null,
                        React.createElement("a", { onClick: open }, "\u91CD\u65B0\u4E0A\u4F20"),
                        "\u00A0/\u00A0",
                        React.createElement("a", { onClick: () => {
                                setFile(null);
                                setDisableNext(true);
                            } }, "\u5220\u9664")) }) }, isDragging ? ('释放鼠标') : (React.createElement(Status, { className: "upload-file", icon: 'blank', size: 's', operation: React.createElement(React.Fragment, null,
                    React.createElement("a", { onClick: open }, "\u70B9\u51FB\u4E0A\u4F20"),
                    " / ",
                    React.createElement(Text, { theme: "weak" }, "\u62D6\u62FD\u5230\u6B64\u533A\u57DF")) })))))),
        React.createElement("div", { className: "modal-upload-text-wrap" },
            React.createElement("ul", null,
                React.createElement("li", null,
                    "1\u3001\u8BF7\u4E0B\u8F7D\u4F7F\u7528",
                    React.createElement("span", { onClick: () => {
                            getCSVTemplate(fields, datasourceTitle);
                        } }, "\u00A0 \u5BFC\u5165\u6A21\u7248"),
                    "\uFF0C\u5E76\u6309\u7167\u793A\u4F8B\u586B\u5199\u6570\u636E\uFF0C\u4EE5\u4FDD\u8BC1\u6570\u636E\u5BFC\u5165\u987A\u5229"),
                React.createElement("li", null, "2\u3001\u652F\u6301xlsx\u7C7B\u578B\u6587\u4EF6\uFF0C\u8981\u6C42\u6587\u4EF6\u5927\u5C0F2M\u4EE5\u5185\uFF0C\u4E0D\u80FD\u8D85\u8FC710000\u884C"),
                React.createElement("li", null,
                    React.createElement("div", null, "3\u3001\u5BFC\u5165\u66F4\u65B0\u8BF4\u660E"),
                    React.createElement("div", null, "\u00A01\uFF09\u652F\u6301\u4EE5\u300C\u6570\u636E\u6807\u8BC6\u300D\u4F5C\u4E3A\u4F9D\u636E\u5B57\u6BB5\uFF0C\u5BF9\u5DF2\u6709\u6570\u636E\u8FDB\u884C\u5339\u914D\u66F4\u65B0"),
                    React.createElement("div", null, "\u00A02\uFF09\u5BFC\u5165\u6A21\u5F0F\u4E3A\u300C\u65B0\u589E\u6216\u66F4\u65B0\u300D\u65F6\uFF0C\u300C\u6570\u636E\u6807\u8BC6\u300D\u5217\u4E3A\u7A7A\u65F6\uFF0C\u8BE5\u884C\u6570\u636E\u4F1A\u6267\u884C\u65B0\u589E\u64CD\u4F5C")),
                React.createElement("li", null,
                    React.createElement("div", null, "4\u3001\u6570\u636E\u683C\u5F0F\u8BF4\u660E"),
                    React.createElement("div", null, "\u00A0 1\uFF09\u679A\u4E3E\u7C7B\u578B\u6570\u636E\uFF1A\u652F\u6301\u9009\u62E9\u9009\u9879\u6807\u8BC6\u6216\u9009\u9879\u503C\u8FDB\u884C\u5BFC\u5165\uFF1A\u9009\u9879\u6807\u8BC6\u4E3A\u8868\u683C\u4E2D\u5C55\u793A\u7684\u4E1A\u52A1\u540D\u79F0\uFF1B\u9009\u9879\u503C\u4E3A\u9009\u9879\u7684\u7CFB\u7EDF\u552F\u4E00\u6807\u8BC6"),
                    React.createElement("div", null, "\u00A02\uFF09\u5173\u8054\u5173\u7CFB\u3001\u4E3B\u5B50\u660E\u7EC6\u7C7B\u578B\u6570\u636E\uFF1A\u652F\u6301\u9009\u62E9\u4E3B\u5217\u5B57\u6BB5\u6216\u6570\u636E\u6807\u8BC6\u8FDB\u884C\u5BFC\u5165\uFF1A\u4E3B\u5217\u5B57\u6BB5\u4E3A\u8868\u683C\u4E2D\u5C55\u793A\u7684\u5173\u8054\u6570\u636E\u4E1A\u52A1\u540D\u79F0\uFF1B\u6570\u636E\u6807\u8BC6\u4E3A\u5173\u8054\u6570\u636E\u7684\u7CFB\u7EDF\u552F\u4E00\u6807\u8BC6"),
                    React.createElement("div", null, "\u00A03\uFF09\u5E03\u5C14\u7C7B\u578B\u6570\u636E\uFF1A\u652F\u6301\u5BFC\u5165\u300C\u662F/\u5426\u300D\u6216\u300Ctrue/false\u300D"),
                    React.createElement("div", null, "\u00A04\uFF09\u56FE\u7247\u7C7B\u578B\u6570\u636E\uFF1A\u652F\u6301\u5BFC\u5165https\u6216cloud\u534F\u8BAE\u7684\u56FE\u7247\u5730\u5740"),
                    React.createElement("div", null, "\u00A05\uFF09\u6587\u4EF6\u7C7B\u578B\u6570\u636E\uFF1A\u652F\u6301cloud\u534F\u8BAE\u7684\u6587\u4EF6\u5730\u5740"),
                    React.createElement("div", null, "\u00A06\uFF09\u6682\u4E0D\u652F\u6301\u6570\u7EC4\u3001\u5BF9\u8C61\u3001\u5730\u7406\u4F4D\u7F6E\u7C7B\u578B\u6570\u636E\u7684\u5BFC\u5165"))))));
}
