import React, { useState } from 'react';
import { useInOutFieldsRecords } from './hooks/useInOutFieldsRecords';
import { mutate } from 'swr';
import { Modal, Table, Button, Text, StatusTip, Icon, List } from 'tea-component';
import moment from 'moment';
const fieldsStatusTextColor = {
    init: 'warning',
    importing: 'warning',
    success: 'text',
    failure: 'danger',
    partialFailure: 'danger',
};
/**
 * 菜单导航配置弹窗
 */
export function InOrOutRecordModeal({ visible, onCancel, dbName }) {
    var _a, _b, _c, _d;
    const isImportRecord = visible && visible === 'import';
    const titlePre = isImportRecord ? '导入' : '导出';
    const fieldsStatus = {
        init: `${titlePre}中`,
        importing: `${titlePre}中`,
        success: '成功',
        failure: '失败',
        partialFailure: '部分失败',
    };
    const [isLodaing, setIsLodaing] = useState(null);
    const [errorFiles, setErrorFiles] = useState(null);
    const { data, isError } = useInOutFieldsRecords({
        type: visible || 'import',
        dbName,
        setIsLodaing,
    });
    const tableRecords = ((_a = data === null || data === void 0 ? void 0 : data.Files) === null || _a === void 0 ? void 0 : _a.map((item) => {
        const result = { ...item };
        // 整理出部分失败状态，成功条数不为0，失败条数不为0，partialFailure
        const { ErrorMessage, ErrorDownloadUrl } = item;
        const isPartialFailure = (ErrorMessage === null || ErrorMessage === void 0 ? void 0 : ErrorMessage.Failure) && (ErrorMessage === null || ErrorMessage === void 0 ? void 0 : ErrorMessage.Success);
        if (ErrorMessage && ErrorDownloadUrl) {
            ErrorMessage['ErrorDownloadUrl'] = ErrorDownloadUrl;
        }
        if (isPartialFailure) {
            result['Status'] = 'partialFailure';
        }
        result['ErrorMessage'] = ErrorMessage;
        return result;
    })) || [];
    const renderStatus = (item) => {
        const status = item === null || item === void 0 ? void 0 : item.Status;
        return (React.createElement("div", null,
            status && React.createElement(Text, { theme: fieldsStatusTextColor[status] }, fieldsStatus[status]),
            status !== 'success' && React.createElement("span", { style: { color: '#E7E7E7', margin: '0 7px' } }, "|"),
            (status === 'importing' || status === 'init') && (React.createElement(Button, { type: "link", onClick: () => {
                    mutate(`${dbName}DescribeDataImportList`);
                } }, "\u5237\u65B0")),
            (status === 'failure' || status === 'partialFailure') && (React.createElement(Button, { type: "link", onClick: () => {
                    setErrorFiles(item === null || item === void 0 ? void 0 : item.ErrorMessage);
                } }, (item === null || item === void 0 ? void 0 : item.ErrorMessage) ? '查看原因' : '未查询到失败原因'))));
    };
    const { scrollable } = Table.addons;
    const columns = [
        { key: 'FileName', header: '文件名称' },
        {
            key: 'CreateDate',
            header: `${titlePre}时间`,
            render: (item) => moment(item === null || item === void 0 ? void 0 : item.CreateDate).format('YYYY-MM-DD HH:mm:ss'),
        },
        { key: 'Status', header: '状态', render: (item) => renderStatus(item) },
    ];
    const getTopTip = (isError, isLodaing, tableRecords) => {
        let type = null;
        if (isError) {
            type = 'error';
        }
        else if (isLodaing) {
            type = 'loading';
        }
        else if (!(tableRecords === null || tableRecords === void 0 ? void 0 : tableRecords.length)) {
            type = 'empty';
        }
        return type && React.createElement(StatusTip, { status: type });
    };
    const getTitle = (isErrorMessage) => {
        if (isErrorMessage) {
            return (React.createElement(React.Fragment, null,
                React.createElement("span", { className: "weda-import-or-modal__arrowarp", onClick: () => {
                        setErrorFiles([]);
                    } },
                    React.createElement("i", { className: "weda-import-or-modal__arrow" })),
                React.createElement("span", null),
                "\u5931\u8D25\u539F\u56E0\uFF1A", errorFiles === null || errorFiles === void 0 ? void 0 :
                errorFiles.FileName));
        }
        return `${titlePre}数据`;
    };
    return (React.createElement(Modal, { caption: getTitle((_b = errorFiles === null || errorFiles === void 0 ? void 0 : errorFiles.ErrorMessage) === null || _b === void 0 ? void 0 : _b.length), visible: visible, onClose: onCancel, destroyOnClose: true, size: 'l', className: `weda-import-or-modal` },
        React.createElement(Modal.Body, null, ((_c = errorFiles === null || errorFiles === void 0 ? void 0 : errorFiles.ErrorMessage) === null || _c === void 0 ? void 0 : _c.length) ? (React.createElement("div", { className: "error-container" },
            React.createElement("div", { className: "error-tip" },
                React.createElement(Icon, { type: "error", size: "s" }),
                React.createElement("p", null, "\u6570\u636E\u5BFC\u5165\u5931\u8D25")),
            React.createElement("div", { className: "error-content" },
                React.createElement(List, null,
                    (errorFiles === null || errorFiles === void 0 ? void 0 : errorFiles.ErrorDownloadUrl) && (React.createElement(List.Item, null,
                        React.createElement("a", { target: '_blank', href: errorFiles === null || errorFiles === void 0 ? void 0 : errorFiles.ErrorDownloadUrl, rel: "noreferrer" }, "\u4E0B\u8F7D\u9519\u8BEF\u62A5\u544A"))),
                    Array.isArray(errorFiles === null || errorFiles === void 0 ? void 0 : errorFiles.ErrorMessage)
                        ? (_d = errorFiles === null || errorFiles === void 0 ? void 0 : errorFiles.ErrorMessage) === null || _d === void 0 ? void 0 : _d.map((i, index) => React.createElement(List.Item, { key: index }, i))
                        : errorFiles === null || errorFiles === void 0 ? void 0 : errorFiles.ErrorMessage)))) : (React.createElement(Table, { records: tableRecords, recordKey: "CreateDate", columns: columns, topTip: getTopTip(isError, isLodaing, tableRecords), addons: [
                scrollable({
                    maxHeight: 420,
                }),
            ] }))),
        React.createElement(Modal.Footer, null,
            React.createElement(Button, { type: "text", style: { float: 'left', pointerEvents: 'none' } }, "\u4EC5\u5C55\u793A24\u5C0F\u65F6\u5185\u5BFC\u5165/\u5BFC\u51FA\u8BB0\u5F55\uFF0C\u8BF7\u5C3D\u5FEB\u4E0B\u8F7D"))));
}
