import { useEffect, useState } from 'react';
import { PollingInterface } from '../../../utils/pollingInterface';
import { callWedaApi } from '../../../utils';
export const useImportingStatus = (fileId, DataSourceName, Type) => {
    const [fileImportStatus, setFileImportStatus] = useState(null);
    const [fileImportErrorMessage, setFileImportErrorMessage] = useState(null);
    useEffect(() => {
        if (!fileId)
            return;
        const Polling = new PollingInterface(getImportingStatus, 5000, 40);
        async function getImportingStatus() {
            try {
                const queryParams = {
                    action: 'DescribeDataImportDetail',
                    data: { DataSourceName, fileId, Type },
                };
                const { Status, ErrorMessage, ErrorDownloadUrl } = await callWedaApi(queryParams);
                // 文件状态: 初始化 init、导入中 importing、导出中 exporting、成功 success、失败 failure
                if (Status === 'success') {
                    setFileImportStatus('success');
                    setFileImportErrorMessage({ ...ErrorMessage, ErrorDownloadUrl });
                    Polling.stop();
                }
                else if (Status === 'failure') {
                    setFileImportStatus('failure');
                    setFileImportErrorMessage({ ...ErrorMessage, ErrorDownloadUrl });
                }
                else if (Status === 'importing' || Status === 'init') {
                    setFileImportStatus('importing');
                    Polling.start();
                }
            }
            catch (error) {
                setFileImportStatus('failure');
                console.error('getApi error', error);
                setFileImportErrorMessage({
                    ErrorMessage: [`查询导出请求失败${(error === null || error === void 0 ? void 0 : error.message) || ''}`],
                });
            }
        }
        Polling.setCallback(() => {
            setFileImportStatus('failure');
            setFileImportErrorMessage('查询导出请求超时');
            setFileImportErrorMessage({
                ErrorMessage: ['查询导出请求超时'],
            });
        });
        Polling.start();
        return () => {
            Polling.stop();
            setFileImportStatus(null);
            setFileImportErrorMessage({});
        };
    }, [fileId]);
    return {
        fileImportStatus,
        fileImportErrorMessage,
    };
};
