import React, { useState, useEffect } from 'react';
import { Dropdown, Status, Tree, SearchBox, Button, StatusTip } from 'tea-component';
import { useRequest } from 'ahooks';
import { getDeparts, searchByCondition, initRootOrgs } from '../../services';
import { SelectViewContainer, getNode, dealUserData, dealOrgData, getRelationData } from './index';
import { useGlobalContext, globalContextConnect } from '../../context';
import cloneDeep from 'lodash.clonedeep';
import './dimTreeSelect.less';
const { LoadingTip } = StatusTip;
const DimTreeSelectComponent = (props) => {
    const { onChange, value, departData, multiple = false, relation, // 存储关联关系的信息
    ...rest } = props;
    const [treeData, setTreeData] = useState([]);
    const [selectIds, setSelectIds] = useState([]);
    // 用户单选
    const [curSelectItem, setCurSelectItem] = useState({});
    const [open, setOpen] = useState(false);
    const [originalTreeData, setOriginalTreeData] = useState([]);
    // 初始化部门数据
    const { run: runInitRootOrgs } = useRequest(initRootOrgs, {
        manual: true,
    });
    const { callWedaApi, showToast, showError } = useGlobalContext();
    // 节点高亮
    const [activeIds, setActiveIds] = useState([]);
    const commonParams = [callWedaApi, showToast, showError];
    // 初始化数据
    const init = async () => {
        const resData = await runInitRootOrgs({}, ...commonParams);
        const data = dealOrgData(resData);
        setTreeData(data);
        setOriginalTreeData(cloneDeep(data));
    };
    useEffect(() => {
        init();
        const defaultValue = getRelationData(relation);
        if (defaultValue === null || defaultValue === void 0 ? void 0 : defaultValue.id) {
            setCurSelectItem({ orgId: defaultValue === null || defaultValue === void 0 ? void 0 : defaultValue.id, ...defaultValue });
        }
    }, []);
    const reset = (close) => {
        setSelectIds([]);
        setCurSelectItem({});
        setActiveIds([]);
        onChange === null || onChange === void 0 ? void 0 : onChange(null);
        close && setOpen(false);
    };
    useEffect(() => {
        if (open) {
            // @ts-ignore
            const orgId = curSelectItem === null || curSelectItem === void 0 ? void 0 : curSelectItem.orgId;
            const node = getNode(treeData, orgId, 'orgId');
            setActiveIds(node ? [node.id] : []);
            setSelectIds(node ? [node.id] : []);
        }
    }, [open, treeData]);
    return (React.createElement(Dropdown, { className: "department-tree-select", destroyOnClose: false, open: open, onOpenChange: (open) => setOpen(open), button: React.createElement(SelectViewContainer, { curSelectItem: curSelectItem, multiple: multiple, placeholder: "\u8BF7\u9009\u62E9\u90E8\u95E8", onChange: () => reset() }), size: "full", style: { width: '100%' }, appearance: multiple ? 'pure' : 'button', clickClose: false, ...rest }, () => (React.createElement("div", { className: "dim-tree-select" },
        React.createElement(DimTreeContent, { data: treeData, setTreeData: setTreeData, selectedIds: selectIds, activeIds: activeIds, needUser: false, searchType: "SearchByOrg", onActive: (ids) => {
                setActiveIds(ids);
                setSelectIds(ids);
            }, onReset: () => setTreeData(originalTreeData), onCancel: () => setOpen(false), onClear: () => reset(false), onConfirm: () => {
                const selectId = selectIds === null || selectIds === void 0 ? void 0 : selectIds[0];
                if (selectId) {
                    const node = getNode(treeData, selectId, 'id');
                    if (node) {
                        setCurSelectItem(node);
                        onChange === null || onChange === void 0 ? void 0 : onChange(node.orgId);
                        setOpen(false);
                    }
                }
            } })))));
};
export const DimTreeContent = (props) => {
    const { selectedIds, onSelect, onActive, activeIds, treeClassName, data: treeData, setTreeData, onCancel, onConfirm, needUser, onReset, } = props;
    const { callWedaApi, showToast, showError } = useGlobalContext();
    // 获取部门信息
    const { run: runGetDeparts, loading: getDepartsLoading } = useRequest(initRootOrgs, {
        manual: true,
    });
    // 异步加载
    const { run: lazyLoaded } = useRequest(getDeparts, {
        manual: true,
    });
    const commonParams = [callWedaApi, showToast, showError];
    const loadDepartData = (id) => new Promise(async (resolve, reject) => {
        const node = getNode(treeData, id, 'id');
        if (node.children) {
            resolve(true);
            return;
        }
        try {
            // 部门
            const { Orgs = [], Users = [] } = await lazyLoaded({
                OrgId: node.orgId,
                DescribeCondition: {
                    NeedUser: !!needUser,
                },
            }, ...commonParams);
            // 如果不存在数据
            if (!(Orgs === null || Orgs === void 0 ? void 0 : Orgs.length) && !(Users === null || Users === void 0 ? void 0 : Users.length)) {
                node.expandable = false;
            }
            else {
                // 用来收集返回的部门和用户
                // @ts-ignore
                const children = dealOrgData(Orgs).concat(needUser ? dealUserData(Users) : []);
                Object.assign(node, { children });
            }
            setTreeData([...treeData]);
            resolve(true);
        }
        catch (error) {
            reject(error);
        }
    });
    // 根据关键词搜索
    const { run: runSearch, loading: searchLoading } = useRequest(searchByCondition, {
        manual: true,
    });
    // 搜索
    const search = async (key) => {
        let resData = [];
        if (key) {
            const { Orgs = [], Users = [] } = await runSearch({
                KeyWords: key,
                SearchCondition: {
                    SearchType: 'SearchByOrg',
                    NeedUser: needUser,
                },
            }, ...commonParams);
            // @ts-ignore
            resData = dealOrgData(Orgs).concat(needUser ? dealUserData(Users) : []);
        }
        else {
            resData = dealOrgData((await runGetDeparts({}, ...commonParams)) || []);
        }
        setTreeData(resData);
    };
    return (React.createElement(React.Fragment, null,
        React.createElement(SearchBox, { onSearch: search, onClear: () => onReset(), onChange: (val) => !val && onReset() }),
        React.createElement("div", { className: "tree-search-loading" }, (searchLoading || getDepartsLoading) && React.createElement(LoadingTip, null)),
        !(treeData === null || treeData === void 0 ? void 0 : treeData.length) && !searchLoading && !getDepartsLoading && (React.createElement(Status, { className: "dim-tree-select--blank", icon: 'blank', size: 's', title: `暂无${needUser ? '用户' : '部门'}数据` })),
        !!(treeData === null || treeData === void 0 ? void 0 : treeData.length) && !searchLoading && !getDepartsLoading && (React.createElement(React.Fragment, null,
            React.createElement("div", { className: "tree-select-warp" },
                React.createElement(Tree, { selectable: false, selectStrictly: true, activable: true, activeIds: activeIds, selectedIds: selectedIds, data: treeData, onLoad: loadDepartData, onActive: onActive, onSelect: onSelect, className: `tree-select ${treeClassName}` })),
            React.createElement("div", { className: "dim-tree-footer" },
                React.createElement(Button, { type: "weak", className: "din-btn din-btn--cancle", onClick: onCancel }, "\u53D6\u6D88"),
                React.createElement(Button, { type: "primary", className: "din-btn din-btn--primary", onClick: onConfirm }, "\u786E\u5B9A"))))));
};
export const DimTreeSelect = globalContextConnect(DimTreeSelectComponent);
