import React, { useState, useEffect } from 'react';
import { useRequest } from 'ahooks';
import { Dropdown, Status, Tree, SearchBox, Tabs, TabPanel, StatusTip, Button, } from 'tea-component';
import { getRoleList, getUsersByRoleId, searchByCondition, initRootOrgs } from '../../services';
import { SelectViewContainer, DimTreeContent, dealRoleData, getNode, dealUserData, getRelationData, dealOrgData, } from './index';
import { useGlobalContext, globalContextConnect } from '../../context';
import classNames from 'classnames';
import cloneDeep from 'lodash.clonedeep';
import './dimTreeSelect.less';
const { LoadingTip } = StatusTip;
const tabs = [
    { id: 'depart', label: '按部门' },
    { id: 'role', label: '按角色' },
];
export const UserTreeSelectComponent = (props) => {
    const { onChange, value, departData, multiple = false, treeClassName = '', relation, ...rest } = props;
    const [currentTab, setCurrentTab] = useState('depart');
    const [selectIds, setSelectIds] = useState([]);
    const [selectUserIds, setSelectUserIds] = useState([]);
    // 用户单选
    const [curSelectItem, setCurSelectItem] = useState({});
    // 用户多选
    const [userTreeData, setUserTreeData] = useState([]);
    const { callWedaApi, showToast, showError } = useGlobalContext();
    const [orgsTreeData, setOrgsTreeData] = useState([]);
    // 节点高亮
    const [activeIds, setActiveIds] = useState([]);
    const [userActiveIds, setUserActiveIds] = useState([]);
    const [open, setOpen] = useState(false);
    const [originalOrgData, setOriginalOrgData] = useState([]);
    const [originalUsersData, setOriginalUsersData] = useState([]);
    const commonParams = [callWedaApi, showToast, showError];
    // 角色-查
    const { run: getRoleListRun, loading: roleListLoading } = useRequest(getRoleList, {
        manual: true,
    });
    // 用户-查
    const { run: getUsersByRoleIdRun } = useRequest(getUsersByRoleId, {
        manual: true,
    });
    // 获取节点信息
    const { run: runGetDeparts } = useRequest(initRootOrgs, {
        manual: true,
    });
    // 根据关键词搜索
    const { run: runSearch, loading: searchLoading } = useRequest(searchByCondition, {
        manual: true,
    });
    useEffect(() => {
        initOrgsData();
        getRoleListData();
        const defaultValue = getRelationData(relation);
        if (defaultValue === null || defaultValue === void 0 ? void 0 : defaultValue.id) {
            setCurSelectItem({ userId: defaultValue === null || defaultValue === void 0 ? void 0 : defaultValue.id, ...defaultValue });
        }
    }, []);
    useEffect(() => {
        if (open) {
            // @ts-ignore
            const userId = curSelectItem === null || curSelectItem === void 0 ? void 0 : curSelectItem.userId;
            const isOrgs = getNode(orgsTreeData, userId, 'userId');
            const isRole = getNode(userTreeData, userId, 'userId');
            if (!userId) {
                setUserActiveIds([]);
                setSelectUserIds([]);
                setUserActiveIds([]);
                setSelectUserIds([]);
            }
            if (isOrgs) {
                setActiveIds([isOrgs.id]);
                setSelectIds([isOrgs.id]);
                setUserActiveIds([]);
                setSelectUserIds([]);
            }
            if (isRole) {
                setUserActiveIds([isRole.id]);
                setSelectUserIds([isRole.id]);
                setActiveIds([]);
                setSelectIds([]);
            }
        }
    }, [open, orgsTreeData, userTreeData]);
    const getRoleListData = async () => {
        const res = await getRoleListRun({
            PageNo: 1,
            PageSize: 1000,
            RoleIds: [],
        }, ...commonParams);
        const data = dealRoleData(res || []);
        setUserTreeData(data);
        setOriginalUsersData(cloneDeep(data));
    };
    // 初始化数据
    const initOrgsData = async () => {
        const resData = (await runGetDeparts({}, ...commonParams)) || [];
        const data = dealOrgData(resData);
        setOrgsTreeData(data);
        setOriginalOrgData(cloneDeep(data));
    };
    const reset = (close) => {
        setSelectIds([]);
        setCurSelectItem({});
        setActiveIds([]);
        onChange === null || onChange === void 0 ? void 0 : onChange(null);
        close && setOpen(false);
    };
    const loadUserTreeData = (RoleId) => new Promise(async (resolve, reject) => {
        const params = {
            PageNo: 1,
            PageSize: 1000,
            RoleIds: [RoleId],
            NeedPrimaryColumn: true,
        };
        const node = getNode(userTreeData, RoleId, 'id');
        if (node.children) {
            resolve(true);
            return;
        }
        try {
            const { UserList = [] } = await getUsersByRoleIdRun(params, ...commonParams);
            Object.assign(node, {
                children: dealUserData(UserList, true),
            });
            setUserTreeData([...userTreeData]);
            resolve(true);
        }
        catch (error) {
            reject(error);
        }
    });
    return (React.createElement(Dropdown, { className: classNames('role-tree-select', `role-tree-select--${currentTab}`), destroyOnClose: false, open: open, onOpenChange: (open) => setOpen(open), button: React.createElement(SelectViewContainer, { curSelectItem: curSelectItem, multiple: multiple, placeholder: "\u8BF7\u9009\u62E9\u7528\u6237", onChange: () => reset() }), size: "full", style: { width: '100%' }, appearance: multiple ? 'pure' : 'button', clickClose: false, ...rest }, (close) => (React.createElement("div", { className: "dim-tree-select dim-tree-select--user" },
        React.createElement(React.Fragment, null,
            React.createElement(Tabs, { className: "dim-tree-select-tabs", tabs: tabs, destroyInactiveTabPanel: false, onActive: (tab) => {
                    if (tab && tab.id) {
                        setCurrentTab(tab.id);
                    }
                } },
                React.createElement(TabPanel, { id: "depart" },
                    React.createElement(DimTreeContent, { data: orgsTreeData, setTreeData: setOrgsTreeData, selectedIds: selectIds, activeIds: activeIds, needUser: true, onActive: (ids) => {
                            const selectedId = ids[0];
                            const node = getNode(orgsTreeData, selectedId, 'id');
                            if (node === null || node === void 0 ? void 0 : node.isUser) {
                                setActiveIds(ids);
                                setSelectIds(ids);
                                // 置空角色选择的用户
                                setUserActiveIds([]);
                                setSelectUserIds([]);
                            }
                        }, onReset: () => setOrgsTreeData(originalOrgData), onCancel: () => setOpen(false), onClear: () => reset(false), onConfirm: () => {
                            const selectId = selectIds === null || selectIds === void 0 ? void 0 : selectIds[0];
                            if (selectId) {
                                const node = getNode(orgsTreeData, selectId, 'id');
                                if (node) {
                                    setCurSelectItem(node);
                                    onChange === null || onChange === void 0 ? void 0 : onChange(node.userId);
                                    setOpen(false);
                                }
                            }
                        } })),
                React.createElement(TabPanel, { id: "role" },
                    React.createElement(SearchBox, { onSearch: async (v) => {
                            if (v.trim()) {
                                try {
                                    const { Users = [], Roles = [] } = await runSearch({
                                        KeyWords: v.trim(),
                                        SearchCondition: {
                                            SearchType: 'SearchByRole',
                                            NeedUser: true,
                                        },
                                    }, ...commonParams);
                                    // @ts-ignore
                                    setUserTreeData(dealRoleData(Roles).concat(dealUserData(Users)));
                                }
                                catch (error) { }
                            }
                            else {
                                setUserTreeData(originalUsersData);
                            }
                        }, onChange: (val) => !val && setUserTreeData(originalUsersData), onClear: () => setUserTreeData(originalUsersData), onHelp: () => console.log('help') }),
                    (userTreeData === null || userTreeData === void 0 ? void 0 : userTreeData.length) === 0 && !searchLoading && !roleListLoading && (React.createElement(Status, { className: "dim-tree-select--blank", icon: 'blank', size: 's', title: '暂无用户数据' })),
                    React.createElement("div", { className: "tree-search-loading" }, (searchLoading || roleListLoading) && React.createElement(LoadingTip, null)),
                    !!(userTreeData === null || userTreeData === void 0 ? void 0 : userTreeData.length) && !searchLoading && !roleListLoading && (React.createElement(React.Fragment, null,
                        React.createElement("div", { className: "tree-select-warp" },
                            React.createElement(Tree, { data: userTreeData, className: `tree-select ${treeClassName}`, selectedIds: selectUserIds, selectStrictly: true, onLoad: loadUserTreeData, activable: true, activeIds: userActiveIds, onActive: (ids) => {
                                    const selectedId = ids === null || ids === void 0 ? void 0 : ids[0];
                                    const node = getNode(userTreeData, selectedId, 'id');
                                    if (node === null || node === void 0 ? void 0 : node.isUser) {
                                        setUserActiveIds(ids);
                                        setSelectUserIds(ids);
                                    }
                                } })),
                        React.createElement("div", { className: "dim-tree-footer" },
                            React.createElement(Button, { type: "weak", className: "din-btn din-btn--cancle", onClick: () => setOpen(false) }, "\u53D6\u6D88"),
                            React.createElement(Button, { type: "primary", className: "din-btn din-btn--primary", onClick: () => {
                                    const selectId = selectUserIds === null || selectUserIds === void 0 ? void 0 : selectUserIds[0];
                                    if (selectId) {
                                        const node = getNode(userTreeData, selectId, 'id');
                                        if (node) {
                                            setCurSelectItem(node);
                                            onChange === null || onChange === void 0 ? void 0 : onChange(node.userId);
                                            setOpen(false);
                                        }
                                    }
                                } }, "\u786E\u5B9A")))))))))));
};
export const UserTreeSelect = globalContextConnect(UserTreeSelectComponent);
