import React, { createContext, useContext, useState, useEffect } from 'react';
import { appCloudCallWedaApi, appCloudShowWedaToast, appCloudIsInWedaIde } from '../../utils';
import { getEnumOptions } from './utils';
const Context = createContext({ enumOptions: {}, loading: true });
export const EnumProvider = ({ children, fields = [] }) => {
    const [enumOptions, setEnumOptions] = useState({});
    const [loading, setLoading] = useState(true);
    useEffect(() => {
        setLoading(true);
        getEnumOptions({
            fields,
            callWedaApi: appCloudCallWedaApi,
            showToast: appCloudShowWedaToast,
            isWedaIde: appCloudIsInWedaIde(),
            successCallback: ({ enumOptions, loading }) => {
                setEnumOptions(enumOptions);
                setLoading(loading);
            },
        });
    }, [fields.length]);
    return (React.createElement(Context.Provider, { value: {
            enumOptions,
            loading,
        } }, children));
};
export const useEnumContext = () => {
    const value = useContext(Context);
    return value;
};
