import { resolveFieldsFromDatasource } from '../utils';
/**
 * 获取过滤权限后的字段
 * 返回格式: {readFields:[], writeFields: [], requireNoRwFields: []}，分别为：只读字段，读写字段，必填但无写权限字段
 * 权限接口规则：有返回字段时以字段权限为准，无字段返回时，数据源所有字段都有读写权限，默认内置字段均有读写权限
 * 2021-11-25 暂时下掉列权限，优化性能
 * 2022-08-06
 * 新增返回 showFields:[] 能够展示的字段，包含读写字段和只读字段，其中只读字段的表单项为disable状态
 */
export const useAuthFields = async (dbName, enabledFieldNames, displaySystem, isUseAll = false, isModel = true, connectorMethod) => {
    const allFields = await resolveFieldsFromDatasource({
        dbName,
        enabledFieldNames,
        displaySystem,
        isUseAll,
        isModel,
        connectorMethod,
    });
    const [readFields, writeFields, requireNoRwFields, authLoading, showFields] = [
        [].concat(allFields),
        [].concat(allFields),
        [],
        false,
        [].concat(allFields === null || allFields === void 0 ? void 0 : allFields.filter((item) => (item === null || item === void 0 ? void 0 : item.readWriteAuth) === 'rw' || (item === null || item === void 0 ? void 0 : item.readWriteAuth) === 'r')),
    ];
    return { readFields, writeFields, requireNoRwFields, authLoading, showFields };
};
