import { useReducer, useState, useEffect } from 'react';
export function useForm(initState, validator) {
    const [isFirstValidate, setIsFirstValidate] = useState(true);
    const [formData, setFormData] = useReducer(useFormReducer, initState);
    const [formMsgMap, setFormMsgMap] = useState({});
    const [formValidated, setFormValidated] = useState(true);
    useEffect(() => {
        if (typeof validator !== 'function') {
            return;
        }
        const res = validator(formData);
        if (!(res === null || res === void 0 ? void 0 : res.length)) {
            setFormValidated(true);
            setFormMsgMap({});
        }
        else {
            setFormValidated(false);
            // 很细的逻辑：在第一次验证的时候，不更新验证信息
            // 原因：表单在初始化时进行验证，但是由于用户没操作，直接显示验证信息效果太差（满屏的红色提示）
            if (isFirstValidate) {
                setIsFirstValidate(false);
            }
            else {
                setFormMsgMap({ [res[0]]: res[1] });
            }
        }
    }, [formData]);
    return {
        formData,
        setFormData,
        formMsgMap,
        formValidated,
    };
}
function useFormReducer(state, action) {
    const [key, value] = action;
    if (!(key in state)) {
        console.error(`The key(${key}) doesn't exist on formdata.`);
        return { ...state };
    }
    return {
        ...state,
        [key]: value,
    };
}
