import { NO_CALLWEDAAPI_TIPS } from '../constants';
/*
 * 初始化加载部门数据
 * @author servernnyu
 */
export async function initRootOrgs(data, callWedaApi, showError, showToast) {
    if (callWedaApi) {
        try {
            const res = await callWedaApi({
                action: 'InitRootOrgs',
                data,
            });
            return Promise.resolve(res);
        }
        catch (error) {
            showError(error);
        }
    }
    else {
        showToast(NO_CALLWEDAAPI_TIPS);
        Promise.reject(NO_CALLWEDAAPI_TIPS.title);
    }
}
/*
 * 懒加载部门数据 LazyDescribeOrgs
 * OrgId：组织架构Id，为根节点就不传了，我就会去查根节点
 * DescribeCondition：查询条件，现在可以传{"Key":"NeedUser","Value": "true"}
 */
export async function getDeparts(data, callWedaApi, showError, showToast) {
    try {
        const res = await callWedaApi({
            action: 'LazyDescribeOrgs',
            data,
        });
        return Promise.resolve(res);
    }
    catch (error) {
        showError(error);
    }
}
/*
 SearchByCondition接口
入参：
KeyWords：关键词
SearchCondition：搜索条件，当前要传俩，一个是是否关联用户同上，一个是SeachType，SearchByRole和SearchByOrg, NeedUser
 */
export async function searchByCondition(data, callWedaApi, showError, showToast) {
    try {
        const res = await callWedaApi({
            action: 'SearchByCondition',
            data,
        });
        return Promise.resolve(res);
    }
    catch (error) {
        showError(error);
    }
}
/**
 * 获取weda角色列表-DescribeWedaRoles
 */
export async function getRoleList(data, callWedaApi, showError, showToast) {
    if (callWedaApi) {
        try {
            const res = await callWedaApi({
                action: 'DescribeRolePage',
                data,
            });
            return Promise.resolve(res);
        }
        catch (error) {
            showError(error);
        }
    }
    else {
        showToast(NO_CALLWEDAAPI_TIPS);
        Promise.reject(NO_CALLWEDAAPI_TIPS.title);
    }
}
/*
 * 根据角色id获取用户
 * @author servernnyu
 */
export async function getUsersByRoleId(data, callWedaApi, showError, showToast) {
    if (callWedaApi) {
        try {
            const res = await callWedaApi({
                action: 'DescribeUsersByRoleId',
                data,
            });
            return Promise.resolve(res);
        }
        catch (error) {
            showError(error);
        }
    }
    else {
        showToast(NO_CALLWEDAAPI_TIPS);
        Promise.reject(NO_CALLWEDAAPI_TIPS.title);
    }
}
/*
 * 根据角色id获取用户
 * @author servernnyu
 */
export async function searchUserByName({ Name }, callWedaApi, showError, showToast) {
    if (callWedaApi) {
        try {
            const res = await callWedaApi({
                action: 'DescribeUserServerByName',
                data: { Name },
            });
            return Promise.resolve(res);
        }
        catch (error) {
            showError(error);
        }
    }
    else {
        showToast(NO_CALLWEDAAPI_TIPS);
        Promise.reject(NO_CALLWEDAAPI_TIPS.title);
    }
}
/**
 * 根据用户或者部门id获取组织机构详情
 *
 */
// 先注释 这期先不做
// export async function getOrgsDetail(data, callWedaApi, showError, showToast) {
//   if (callWedaApi) {
//     try {
//       return new Promise((resolve) => {
//         setTimeout(() => {
//           resolve({
//             Data: [
//               {
//                 OrgName: '市政府2',
//                 OrgId: 1,
//                 OrgIdentity: '2323',
//                 RelatedUsers: [],
//                 level: 2,
//               },
//               {
//                 OrgName: '市政府1',
//                 OrgId: 1,
//                 OrgIdentity: '2323',
//                 RelatedUsers: [],
//                 level: 1,
//               },
//             ],
//           });
//         }, 1000);
//       });
//     } catch (error) {
//       showError(error);
//     }
//   } else {
//     showToast(NO_CALLWEDAAPI_TIPS);
//     Promise.reject(NO_CALLWEDAAPI_TIPS.title);
//   }
// }
