import { parse } from 'querystring';
import { callWedaApi } from './datasource';
import { alertErrorMessage } from './feedback';
/* **** 应用相关方法 **** */
/**
 * 跳转到相关的模型页面
 */
export const navigateToModelPage = (action, params = {}) => {
    // eslint-disable-next-line prefer-const
    let { id, dbName } = params;
    // 没有指定数据库标识
    if (!dbName) {
        alertErrorMessage({ message: '页面信息错误，缺少“数据源名称”' });
        return;
    }
    window.app.navigateTo({
        pageId: `${dbName}_${action}`,
        params: { id },
    });
};
/**
 * 获取 query 参数
 */
export const getPageQuery = () => {
    const { hash } = window.location;
    const qsIndex = hash.indexOf('?');
    const sharpIndex = hash.indexOf('#');
    if (qsIndex !== -1) {
        if (qsIndex > sharpIndex) {
            return parse(hash.split('?')[1]);
        }
        return parse(hash.slice(qsIndex + 1, sharpIndex));
    }
    return {};
};
/**
 * 获取数据id
 */
export const getRecordId = (propsId) => {
    const { id: queryId } = getPageQuery();
    return { id: queryId || propsId || null };
};
/**
 * 当前是否在编辑器环境中
 */
export const isInIde = () => window.app.cloud.IS_WEDA_IDE;
/**
 * 判断是否微信IDE环境
 */
export const isWXIDE = () => /wechatideplugin/i.test(navigator.userAgent);
/**
 * 判断对应的后端接口是否，为了兼容私有化环境后端接口未兼容问题
 */
export const isPrivateInterfaceIsExit = async (params) => {
    try {
        await callWedaApi(params);
        return true;
    }
    catch (e) {
        const reg = RegExp(/接口不存在/);
        const isExit = reg.test(e);
        return !isExit;
    }
};
