import { ApprovalHistoryConstants } from '../constants';
import moment from 'moment';
const { APPROVAL_TYPE, APPROVAL_TYPE_TEXT_MAP, APPROVAL_STATUS_MAP, COMMENT_TYPE_MAP, ELEMENT_TYPE_MAP, START_EVENT_TEXT, END_EVENT_TEXT, APPROVER_NODE_STATUS_TEXT_MAP, COMMENT_TYPE, TRIGGER_TYPE, TRIGGER_FUNC_NAME_MAP, TRIGGER_TYPE_EVENT_MAP, TRIGGER_FIELD_METHOD, TRIGGER_FIELD_METHOD_NAME_MAP, TRIGGER_ELEMENT_TYPE_MAP, TRIGGER_ELEMENT_TYPE_NAME_MAP, } = ApprovalHistoryConstants;
const formatUTCTime = (utcTime) => (utcTime ? moment(utcTime).format('YYYY-MM-DD HH:mm') : '');
const getFieldUpdateHistoryList = (comment) => {
    const { Status, DatasourceFieldInfoList: dataSourceFieldInfoList = [] } = (comment === null || comment === void 0 ? void 0 : comment.Message) || {};
    return dataSourceFieldInfoList === null || dataSourceFieldInfoList === void 0 ? void 0 : dataSourceFieldInfoList.map((info) => {
        const isUpdate = (info === null || info === void 0 ? void 0 : info.Method) === TRIGGER_FIELD_METHOD.UPDATE;
        const isError = Status === 'FAILED';
        if (isUpdate && !isError) {
            return {
                fieldName: info === null || info === void 0 ? void 0 : info.FieldName,
                sourceValue: info === null || info === void 0 ? void 0 : info.SourceValue,
                targetValue: info === null || info === void 0 ? void 0 : info.TargetValue,
                isUpdate,
            };
        }
        return {
            fieldName: TRIGGER_FIELD_METHOD_NAME_MAP === null || TRIGGER_FIELD_METHOD_NAME_MAP === void 0 ? void 0 : TRIGGER_FIELD_METHOD_NAME_MAP[info === null || info === void 0 ? void 0 : info.Method],
            itemId: (info === null || info === void 0 ? void 0 : info.ItemId) || '',
            isError,
        };
    });
};
const isTriggerNode = (processNode) => Object.values(TRIGGER_ELEMENT_TYPE_MAP).includes(processNode === null || processNode === void 0 ? void 0 : processNode.ElementType);
const isTriggerSubNode = (comment) => Object.keys(TRIGGER_TYPE_EVENT_MAP).includes(comment === null || comment === void 0 ? void 0 : comment.Type);
class UserNode {
    constructor({ comment, userName, nodeApprovalStatus }) {
        var _a, _b, _c;
        const status = this.getApproveUserStatus(comment, nodeApprovalStatus);
        this.userName = userName;
        this.status = status;
        this.statusName = APPROVER_NODE_STATUS_TEXT_MAP[status];
        this.commentContent = (_a = comment === null || comment === void 0 ? void 0 : comment.Message) === null || _a === void 0 ? void 0 : _a.Text;
        this.commentTime = formatUTCTime(comment === null || comment === void 0 ? void 0 : comment.Time);
        this.fieldUpdateList = getFieldUpdateHistoryList(comment);
        this.relatedUser = ((_c = (_b = comment === null || comment === void 0 ? void 0 : comment.Message) === null || _b === void 0 ? void 0 : _b.RelatedUser) === null || _c === void 0 ? void 0 : _c.UserName) || '';
        this.isApproving = status === APPROVAL_STATUS_MAP.APPROVING;
    }
    getApproveUserStatus(comment, nodeApprovalStatus) {
        const isRejectNode = nodeApprovalStatus === APPROVAL_STATUS_MAP.REJECTED;
        const { APPROVING, TERMINATION } = APPROVAL_STATUS_MAP;
        if (!(comment === null || comment === void 0 ? void 0 : comment.Type)) {
            return isRejectNode ? TERMINATION : APPROVING;
        }
        return APPROVAL_STATUS_MAP[COMMENT_TYPE_MAP[(comment === null || comment === void 0 ? void 0 : comment.Type) || '']];
    }
}
class TriggerSubNode {
    constructor({ comment, triggerNodeInfo, fatherIsTriggerNode = false }) {
        this.userName = '系统自动';
        this.status = APPROVAL_STATUS_MAP.APPROVED;
        this.commentTime = formatUTCTime(comment === null || comment === void 0 ? void 0 : comment.Time);
        this.fieldUpdateList = getFieldUpdateHistoryList(comment);
        this.setTriggerSubNodeInfo(comment, triggerNodeInfo, fatherIsTriggerNode);
    }
    // 处理系统自动子节点信息
    setTriggerSubNodeInfo(comment, triggerNodeInfo, fatherIsTriggerNode) {
        var _a, _b, _c, _d, _e;
        const triggerType = comment === null || comment === void 0 ? void 0 : comment.Type;
        const eventType = TRIGGER_TYPE_EVENT_MAP === null || TRIGGER_TYPE_EVENT_MAP === void 0 ? void 0 : TRIGGER_TYPE_EVENT_MAP[triggerType];
        const dataSourceExt = fatherIsTriggerNode
            ? triggerNodeInfo === null || triggerNodeInfo === void 0 ? void 0 : triggerNodeInfo.DatasourceExtension
            : (_b = (_a = triggerNodeInfo === null || triggerNodeInfo === void 0 ? void 0 : triggerNodeInfo.filter((event) => (event === null || event === void 0 ? void 0 : event.TriggerAfterEvent) === eventType)) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.DatasourceExtension;
        // 针对数据操作类节点的处理
        if (triggerType === TRIGGER_TYPE.OPT_DATASOURCE) {
            this.statusName = (_d = (_c = dataSourceExt === null || dataSourceExt === void 0 ? void 0 : dataSourceExt.TriggerFuncList) === null || _c === void 0 ? void 0 : _c.map((name) => TRIGGER_FUNC_NAME_MAP[name])) === null || _d === void 0 ? void 0 : _d.join('/');
            this.relatedUser = dataSourceExt === null || dataSourceExt === void 0 ? void 0 : dataSourceExt.Datasource;
        }
        // 针对通知类节点的处理
        if (triggerType === TRIGGER_TYPE.SEND_MESSAGE) {
            this.statusName = '发送通知';
            this.relatedUser = (_e = comment === null || comment === void 0 ? void 0 : comment.Message) === null || _e === void 0 ? void 0 : _e.Text;
        }
    }
}
class HistoryNode {
    constructor({ processNode, commentInfoList, order, nextNode, processBeginTime }) {
        var _a, _b;
        const { startUser, userNodeApprovalType } = this.getInitialValue(processNode);
        this.id = processNode === null || processNode === void 0 ? void 0 : processNode.NodeKey;
        // 节点名称
        this.nodeName = order === 0 ? startUser : (_b = (_a = processNode === null || processNode === void 0 ? void 0 : processNode.ManualUserNode) === null || _a === void 0 ? void 0 : _a.NodeName) !== null && _b !== void 0 ? _b : '';
        // 节点类型 - 开始节点、审批节点、结束节点
        this.elementType = this.getElementType(order);
        // 审批类型 - 单人或签 多人会签
        this.approvalType = this.getApprovalType(order, userNodeApprovalType);
        // 整体节点的状态
        this.status = this.getNodeApproveStatus(commentInfoList, userNodeApprovalType);
        // 节点下的审批人列表
        this.approvalUserList = this.splitUserListByStatus([
            ...this.getNodeApproveList(processNode, commentInfoList || [], this.status),
            ...this.getNodeApprovalRoleList(processNode, this.status),
            ...this.getNodeApprovalDataSourceUserList(processNode, this.status),
        ]);
        // 拒绝至XXX
        this.rejectTo = this.getRejectTo(this.status, nextNode);
        // 触发时间
        this.triggerTime = formatUTCTime(processBeginTime);
    }
    splitUserListByStatus(list) {
        const { APPROVED, TURN_SIGN, BACKWARD_SIGN, FORWARD_SIGN } = APPROVAL_STATUS_MAP;
        const isFront = [APPROVED, TURN_SIGN, BACKWARD_SIGN, FORWARD_SIGN];
        return [
            ...list.filter((i) => isFront.includes(i.status)),
            ...list.filter((i) => !isFront.includes(i.status)),
        ];
    }
    getApprovalType(order, userNodeApprovalType) {
        var _a;
        if (order === 0) {
            return START_EVENT_TEXT;
        }
        return (_a = APPROVAL_TYPE_TEXT_MAP[userNodeApprovalType]) !== null && _a !== void 0 ? _a : '';
    }
    getInitialValue(processNode) {
        var _a, _b;
        return {
            startUser: (_a = processNode === null || processNode === void 0 ? void 0 : processNode.TriggerNode) === null || _a === void 0 ? void 0 : _a.StartUser,
            userNodeApprovalType: (_b = processNode === null || processNode === void 0 ? void 0 : processNode.ManualUserNode) === null || _b === void 0 ? void 0 : _b.ApprovalType,
        };
    }
    getElementType(order) {
        if (order === 0) {
            return ELEMENT_TYPE_MAP.START_EVENT;
        }
        if (order === ELEMENT_TYPE_MAP.END_EVENT) {
            return ELEMENT_TYPE_MAP.END_EVENT;
        }
        return ELEMENT_TYPE_MAP.APPROVAL_EVENT;
    }
    // 获取节点审批人列表
    getNodeApproveList(processNode, commentInfoList, nodeApprovalStatus) {
        const { ApprovalType: currentNodeApprovalType, ApprovalUserNameList: currentNodeApprovalUserList = [], ApprovalUserList: currentNodeApprovalUserIdList = [], ApproveMethodExtension: approveMethodExtension = {}, RejectMethodExtension: rejectMethodExtension = {}, } = (processNode === null || processNode === void 0 ? void 0 : processNode.ManualUserNode) || {};
        const nodeApproveUserList = [];
        const userIdListInCommentInfoList = [];
        // 获取节点的操作数据表节点
        const { TriggerAfterEventExtensionList: triggerNodeInfo } = nodeApprovalStatus === APPROVAL_STATUS_MAP.APPROVED
            ? approveMethodExtension
            : rejectMethodExtension;
        // 收集评论列表中的审批人
        commentInfoList.forEach((comment) => {
            var _a, _b;
            const userName = (_a = comment === null || comment === void 0 ? void 0 : comment.AuthorInfo) === null || _a === void 0 ? void 0 : _a.UserName;
            const userId = (_b = comment === null || comment === void 0 ? void 0 : comment.AuthorInfo) === null || _b === void 0 ? void 0 : _b.UserId;
            const userNode = isTriggerSubNode(comment)
                ? new TriggerSubNode({ comment, triggerNodeInfo })
                : new UserNode({
                    comment,
                    userName,
                    nodeApprovalStatus,
                });
            userIdListInCommentInfoList.push(userId);
            nodeApproveUserList.push(userNode);
        });
        // 收集固定审批人列表中的审批人
        currentNodeApprovalUserIdList.forEach((userId, index) => {
            if (!userIdListInCommentInfoList.includes(userId)) {
                const userNode = new UserNode({
                    comment: null,
                    userName: currentNodeApprovalUserList[index],
                    nodeApprovalStatus,
                });
                nodeApproveUserList.push(userNode);
            }
        });
        // 根据审批类型返回不同形式的审批人列表
        if (currentNodeApprovalType === APPROVAL_TYPE.APPROVE_MULTI) {
            return nodeApproveUserList;
        }
        if (currentNodeApprovalType === APPROVAL_TYPE.APPROVE_SINGLE) {
            if (nodeApprovalStatus === APPROVAL_STATUS_MAP.APPROVING) {
                return nodeApproveUserList;
            }
            return nodeApproveUserList.filter((userNode) => userNode === null || userNode === void 0 ? void 0 : userNode.commentTime);
        }
        return [];
    }
    // 获取节点角色列表
    getNodeApprovalRoleList(processNode, nodeApprovalStatus) {
        var _a, _b, _c;
        const currentNodeApprovalType = (_a = processNode === null || processNode === void 0 ? void 0 : processNode.ManualUserNode) === null || _a === void 0 ? void 0 : _a.ApprovalType;
        const approvalRoleIdList = ((_b = processNode === null || processNode === void 0 ? void 0 : processNode.ManualUserNode) === null || _b === void 0 ? void 0 : _b.ApprovalRoleIdList) || [];
        const approvalRoleNameList = ((_c = processNode === null || processNode === void 0 ? void 0 : processNode.ManualUserNode) === null || _c === void 0 ? void 0 : _c.ApprovalRoleNameList) || [];
        const handledRoleIdList = (processNode === null || processNode === void 0 ? void 0 : processNode.HandledRoleIdList) || [];
        const approvalRoleNodeList = approvalRoleIdList.map((roleId, index) => new UserNode({
            comment: { Type: handledRoleIdList.includes(roleId) ? COMMENT_TYPE.APPROVE : null },
            userName: `【角色】${approvalRoleNameList[index]}`,
            nodeApprovalStatus,
        }));
        // return approvalRoleNodeList;
        if (currentNodeApprovalType === APPROVAL_TYPE.APPROVE_MULTI) {
            return approvalRoleNodeList;
        }
        if (currentNodeApprovalType === APPROVAL_TYPE.APPROVE_SINGLE) {
            if (nodeApprovalStatus === APPROVAL_STATUS_MAP.APPROVING) {
                return approvalRoleNodeList;
            }
            return approvalRoleNodeList.filter((userNode) => { var _a; return (_a = userNode === null || userNode === void 0 ? void 0 : userNode.comment) === null || _a === void 0 ? void 0 : _a.Type; });
        }
        return [];
    }
    getNodeApprovalDataSourceUserList(processNode, nodeApprovalStatus) {
        var _a, _b;
        const getUserList = (userList, onlyName = false) => userList.reduce((list, data) => {
            data === null || data === void 0 ? void 0 : data.UserFieldNameList.forEach((name, index) => {
                if (onlyName) {
                    list.push(name);
                }
                else {
                    list.push({ name, title: data === null || data === void 0 ? void 0 : data.UserFieldTitleList[index] });
                }
            });
            return list;
        }, []);
        const currentNodeApprovalType = (_a = processNode === null || processNode === void 0 ? void 0 : processNode.ManualUserNode) === null || _a === void 0 ? void 0 : _a.ApprovalType;
        const allDataSourceUserList = getUserList(((_b = processNode === null || processNode === void 0 ? void 0 : processNode.ManualUserNode) === null || _b === void 0 ? void 0 : _b.DatasourceUserFieldList) || []);
        const handleDataSourceUseNameList = getUserList((processNode === null || processNode === void 0 ? void 0 : processNode.HandledDatasourceUserFieldList) || [], true);
        const dataSourceUserNodeList = allDataSourceUserList.map((user) => new UserNode({
            comment: {
                Type: handleDataSourceUseNameList.includes(user === null || user === void 0 ? void 0 : user.name) ? COMMENT_TYPE.APPROVE : null,
            },
            userName: user === null || user === void 0 ? void 0 : user.title,
            nodeApprovalStatus,
        }));
        if (currentNodeApprovalType === APPROVAL_TYPE.APPROVE_MULTI) {
            return dataSourceUserNodeList;
        }
        if (currentNodeApprovalType === APPROVAL_TYPE.APPROVE_SINGLE) {
            if (nodeApprovalStatus === APPROVAL_STATUS_MAP.APPROVING) {
                return dataSourceUserNodeList;
            }
            return dataSourceUserNodeList.filter((userNode) => { var _a; return (_a = userNode === null || userNode === void 0 ? void 0 : userNode.comment) === null || _a === void 0 ? void 0 : _a.Type; });
        }
        return [];
    }
    /**
     * 获取某个审批节点的审批状态
     * @param {*} commentInfoList
     * @param {*} approvalType
     * @returns
     */
    getNodeApproveStatus(commentInfoList, approvalType) {
        const { REJECT, APPROVE, BACKWARD_SIGN, FORWARD_SIGN } = COMMENT_TYPE;
        const { APPROVED, REJECTED, APPROVING } = APPROVAL_STATUS_MAP;
        const realCommentInfoList = commentInfoList.filter((c) => !Object.keys(TRIGGER_TYPE_EVENT_MAP).includes(c === null || c === void 0 ? void 0 : c.Type));
        if (approvalType === APPROVAL_TYPE.APPROVE_MULTI) {
            const hasRejected = realCommentInfoList.some((c) => (c === null || c === void 0 ? void 0 : c.Type) === REJECT);
            const isAllApproved = realCommentInfoList.every((c) => (c === null || c === void 0 ? void 0 : c.Type) === APPROVE);
            if (!hasRejected && !isAllApproved) {
                return APPROVING;
            }
            return hasRejected ? REJECTED : APPROVED;
        }
        if (approvalType === APPROVAL_TYPE.APPROVE_SINGLE) {
            const executedComment = realCommentInfoList.find((c) => [APPROVE, REJECT, BACKWARD_SIGN, FORWARD_SIGN].includes(c === null || c === void 0 ? void 0 : c.Type));
            if (executedComment) {
                const commentType = COMMENT_TYPE_MAP[executedComment === null || executedComment === void 0 ? void 0 : executedComment.Type];
                return APPROVAL_STATUS_MAP[commentType];
            }
            return APPROVING;
        }
        return APPROVED;
    }
    getRejectTo(status, nextNode) {
        var _a, _b;
        if (status === APPROVAL_STATUS_MAP.REJECTED) {
            if (!nextNode) {
                return ['开始节点'];
            }
            return (((_b = (_a = nextNode === null || nextNode === void 0 ? void 0 : nextNode.ProcessNode) === null || _a === void 0 ? void 0 : _a.ManualUserNode) === null || _b === void 0 ? void 0 : _b.ApprovalUserNameList) || []).join(',');
        }
        return null;
    }
}
class TriggerNode {
    constructor({ processNode, commentInfoList }) {
        this.id = processNode === null || processNode === void 0 ? void 0 : processNode.NodeKey;
        // 节点名称
        this.nodeName = '自动化节点'; // 固定名称
        // 节点类型 - 发送通知、操作数据表
        this.approvalType = TRIGGER_ELEMENT_TYPE_NAME_MAP === null || TRIGGER_ELEMENT_TYPE_NAME_MAP === void 0 ? void 0 : TRIGGER_ELEMENT_TYPE_NAME_MAP[processNode === null || processNode === void 0 ? void 0 : processNode.ElementType];
        // 触发节点列表
        this.approvalUserList = [];
        this.setApprovalUserList(processNode, commentInfoList);
    }
    setApprovalUserList(processNode, commentInfoList) {
        const { AutoServiceNode: autoServiceNode = {} } = processNode || {};
        this.approvalUserList = commentInfoList.map((comment) => new TriggerSubNode({
            comment,
            triggerNodeInfo: autoServiceNode,
            fatherIsTriggerNode: true,
        }));
    }
}
export default class ApprovalHistoryEntity {
    constructor(resData) {
        Object.defineProperty(this, "getLastNode", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (historyNodeList, status) => {
                var _a;
                const hasEndNode = status.toLowerCase() !== APPROVAL_STATUS_MAP.APPROVING;
                const hasRevokeNode = status.toLowerCase() === APPROVAL_STATUS_MAP.REVOKED;
                if (historyNodeList.length && hasEndNode && !hasRevokeNode) {
                    historyNodeList.push({
                        id: 'endNode',
                        elementType: ELEMENT_TYPE_MAP.END_EVENT,
                        nodeName: END_EVENT_TEXT,
                    });
                }
                if (historyNodeList.length && hasRevokeNode) {
                    historyNodeList.push({
                        id: 'revokeNode',
                        elementType: ELEMENT_TYPE_MAP.REVOKE_EVENT,
                        nodeName: (_a = historyNodeList[0]) === null || _a === void 0 ? void 0 : _a.nodeName,
                        approvalType: '撤回',
                        extra: '流程',
                    });
                }
            }
        });
        this.historyNodeList = [];
        this.init(resData);
    }
    init(resData) {
        var _a;
        if (resData) {
            const processBeginTime = resData === null || resData === void 0 ? void 0 : resData.ProcessInstanceBeginTime;
            const processNodeDetailList = ((resData === null || resData === void 0 ? void 0 : resData.ProcessNodeDetailList) || []).concat();
            processNodeDetailList.shift();
            this.historyNodeList = this.transformNodeList(processNodeDetailList, processBeginTime);
            this.getLastNode(this.historyNodeList, (_a = resData === null || resData === void 0 ? void 0 : resData.Status) !== null && _a !== void 0 ? _a : '');
        }
    }
    /**
     * 将节点数组扁平化处理,对审批历史组件进行适配
     * @param {*} nodeList 节点列表
     * @returns
     */
    transformNodeList(nodeList, processBeginTime) {
        if (nodeList.length) {
            return nodeList.map((node, index) => {
                const { ProcessNode: processNode, CommentInfoList: commentInfoList } = node;
                return isTriggerNode(processNode)
                    ? new TriggerNode({ processNode, commentInfoList })
                    : new HistoryNode({
                        processNode,
                        commentInfoList: commentInfoList || [],
                        order: index,
                        nextNode: nodeList[index + 1],
                        processBeginTime,
                    });
            });
        }
        return [];
    }
}
