import { isInIde } from './app';
import { alertErrorMessage } from './feedback';
/* **** 数据源相关方法 **** */
export const deleteRecord = () => { };
export const callWedaApi = (...args) => { var _a, _b; return (_b = (_a = window === null || window === void 0 ? void 0 : window.app) === null || _a === void 0 ? void 0 : _a.cloud) === null || _b === void 0 ? void 0 : _b.callWedaApi(...args); };
export const callFunction = (...args) => { var _a, _b; return (_b = (_a = window === null || window === void 0 ? void 0 : window.app) === null || _a === void 0 ? void 0 : _a.cloud) === null || _b === void 0 ? void 0 : _b.callFunction(...args); };
export class DataSource {
    constructor(name) {
        Object.defineProperty(this, "dbName", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.dbName = name;
    }
    async getRecord(id) {
        try {
            const res = await window.app.cloud.callDataSource({
                dataSourceName: this.dbName,
                methodName: 'wedaGetItem',
                params: {
                    _id: id,
                },
            });
            return res;
        }
        catch (error) {
            if (!isInIde()) {
                console.error('错误', error);
                alertErrorMessage({ message: `获取数据源字段失败,错误信息:${error.message}` });
            }
            return {};
        }
    }
    async getRecordList(params = {}, db = this.dbName) {
        try {
            const res = await window.app.cloud.callDataSource({
                params,
                dataSourceName: db,
                methodName: 'wedaGetRecords',
            });
            return res;
        }
        catch (error) {
            if (!isInIde()) {
                console.error('错误', error);
                alertErrorMessage({ message: `获取数据源字段失败,错误信息:${error.message}` });
            }
            return {};
        }
    }
    async createRecord(data) {
        return window.app.cloud.callDataSource({
            dataSourceName: this.dbName,
            methodName: 'wedaCreate',
            params: data,
        });
    }
    async batchDeleteRecord(ids) {
        let params = {};
        let methodName = '';
        if (ids.length === 0)
            return;
        if (ids.length === 1) {
            params = { _id: ids[0] };
            methodName = 'wedaDelete';
        }
        else {
            params = { where: [{ Key: '_id', Rel: 'in', Val: ids }] };
            methodName = 'wedaBatchDelete';
        }
        return window.app.cloud.callDataSource({
            dataSourceName: this.dbName,
            methodName,
            params,
        });
    }
    async batchCreateRecord(data) {
        return window.app.cloud.callDataSource({
            dataSourceName: this.dbName,
            methodName: 'wedaBatchCreate',
            params: { records: data },
        });
    }
    async updateRecord(id, data) {
        return window.app.cloud.callDataSource({
            dataSourceName: this.dbName,
            methodName: 'wedaUpdate',
            params: {
                _id: id,
                ...data,
            },
        });
    }
    /**
     * API方法调用
     */
    async getConnect(method, params) {
        const { where, ...newParams } = params;
        // console.log('getConnect', where?.length <= 0 ? newParams : params)
        return window.app.cloud.callConnector({
            dataSourceName: this.dbName,
            methodName: method,
            params: Object.prototype.toString.call(params) === '[object Object]' ? ((where === null || where === void 0 ? void 0 : where.length) <= 0 ? newParams : params) : {},
        });
    }
}
export const getDataSourceByName = async (dbName, GetColumnAuth = false) => {
    var _a, _b;
    try {
        // dbName 为空时
        if (!dbName)
            return;
        // 兼容私有环境，新接口不存在的情况
        if ((window === null || window === void 0 ? void 0 : window._isPrivate) && !((_b = (_a = window === null || window === void 0 ? void 0 : window.app) === null || _a === void 0 ? void 0 : _a.cloud) === null || _b === void 0 ? void 0 : _b.getDataSourceProfileAsync)) {
            return getDataSourceByNameCompatibleProvite(dbName);
        }
        const datasource = await window.app.cloud.getDataSourceProfileAsync(GetColumnAuth ? { Name: dbName, GetColumnAuth: true } : dbName);
        return datasource;
    }
    catch (error) {
        alertErrorMessage({ message: `获取数据源字段失败,错误信息:${error}` });
        console.error(`获取数据源字段失败,错误信息:${error}`);
        return {};
    }
};
// 兼容私有环境 getDataSourceProfileAsync 方法不存在的情况
export const getDataSourceByNameCompatibleProvite = async (dbName) => {
    try {
        // dbName 为空时
        if (!dbName)
            return;
        const datasource = await window.app.cloud.getDataSourceProfile(dbName);
        return datasource;
    }
    catch (error) {
        alertErrorMessage({ message: `获取数据源字段失败,错误信息:${error}` });
        console.error(`获取数据源字段失败,错误信息:${error}`);
        return {};
    }
};
/**
 * 获取数据源权限字段
 * 返回对象格式
 */
export const getAuthFieldsByName = async (dbName) => {
    var _a;
    const dataSource = await getDataSourceByName(dbName, true);
    const id = dataSource === null || dataSource === void 0 ? void 0 : dataSource.id;
    const fieldsAuthObj = {};
    try {
        if (callWedaApi && id) {
            const resp = await callWedaApi({
                action: 'DescribeAuthStrategy',
                data: {
                    ResourceId: id,
                },
            });
            const fieldsAuth = (_a = resp === null || resp === void 0 ? void 0 : resp.PermissionMap) === null || _a === void 0 ? void 0 : _a[id];
            if (fieldsAuth) {
                Object.keys(fieldsAuth).forEach((d) => {
                    const pre = `${id}#`;
                    if (d.startsWith(pre)) {
                        const fieldKey = d.replace(pre, '');
                        fieldKey && (fieldsAuthObj[fieldKey] = fieldsAuth[d]);
                    }
                });
            }
        }
    }
    catch (e) {
        console.error('模型：请求权限出错.', e);
    }
    return fieldsAuthObj;
};
/**
 * 添加数据源字段权限
 * 接口文档：https://iwiki.woa.com/pages/viewpage.action?pageId=1920884250
 * 新增读写权限字段 readWriteAuth
 * n : 列操作无权限 ; r : 列操作只读权限 ; rw : 列操作读写权限
 */
export const getDataSourceFieldsWithAuth = (authList, resourceCheckInfos) => {
    try {
        if (isInIde() || (window === null || window === void 0 ? void 0 : window._isPrivate)) {
            // 编辑器编辑区 和 兼容私有环境，新接口不存在的情况
            return authList.map((item) => ({ ...item, readWriteAuth: 'rw' }));
        }
        if ((resourceCheckInfos === null || resourceCheckInfos === void 0 ? void 0 : resourceCheckInfos.length) > 0 && (authList === null || authList === void 0 ? void 0 : authList.length) > 0) {
            const resourceCheckInfosList = resourceCheckInfos.map((item) => item.ReadWrite);
            return authList.map((authItem) => {
                const authInfoList = resourceCheckInfosList.filter((item) => !!item[authItem.name]);
                // 匹配到了权限字段
                if (authInfoList[0]) {
                    authItem.readWriteAuth = authInfoList[0][authItem.name];
                    if (authInfoList[0][authItem.name] === 'r') {
                        // 只读字段标单项disable状态
                        authItem['x-powerDisabled'] = true;
                    }
                }
                return authItem;
            });
        }
        // 没有返回字段权限列表
        return authList;
    }
    catch (err) {
        console.log('getDataSourceFieldsWithAuth err', err);
    }
};
/*
  判断接口是否存在
*/
export const getIsExitApi = async (params) => {
    var _a, _b;
    try {
        await callWedaApi(params);
        return true;
    }
    catch (error) {
        const message = (_b = (_a = error === null || error === void 0 ? void 0 : error.original) === null || _a === void 0 ? void 0 : _a.result) === null || _b === void 0 ? void 0 : _b.message;
        const status = (message === null || message === void 0 ? void 0 : message.indexOf('Action不存在')) != -1 || (message === null || message === void 0 ? void 0 : message.indexOf('接口不存在')) != -1;
        return !status;
    }
};
