import moment from 'moment';
import 'moment/locale/zh-cn';
import { isSystemField } from '../utils';
moment.locale('zh-cn');
// 'time' | 'date' | 'datetime'
export const formatTimeByType = (dateType, v = Date.now(), field = undefined) => {
    // 时间
    if (dateType === 'time') {
        const timeStr = formatNumberToStr(v);
        return timeStr;
    }
    // 日期，但不包括系统字段
    if (dateType === 'date' && !isSystemField(field)) {
        return moment(v).format('YYYY-MM-DD');
    }
    // 兼容逻辑，有一部分数据源的系统字段里的创建和更新时间被设置为了date，这里当作datetime来处理
    return moment(v).format('YYYY-MM-DD HH:mm:ss');
};
export const getFullDate = (v = Date.now()) => moment(v).format('YYYY-MM-DD');
export const addZero = (v) => (v < 10 ? `0${v}` : v);
/**
 * 24小时毫秒数转字符串
 */
export const formatNumberToStr = (v) => {
    let timeStr = '-';
    try {
        const time = moment.duration(parseInt(v), 'milliseconds');
        const hours = addZero(time.hours());
        const minutes = addZero(time.minutes());
        const seconds = addZero(time.seconds());
        timeStr = `${hours}:${minutes}:${seconds}`;
    }
    catch (e) { }
    return timeStr;
};
/**
 * 时间字符串转24小时毫秒数
 */
export const formatTimeToNumber = (t = '') => {
    let time = moment();
    try {
        t && (time = moment(t));
    }
    catch (e) { }
    const hours = time.hours();
    const minutes = time.minutes();
    const seconds = time.seconds();
    const value = (hours * 60 * 60 + minutes * 60 + seconds) * 1000;
    return value;
};
/**
 * 日期字符串转24小时毫秒数
 */
export const formatDateToNumber = (dateType = 'date', t = '') => {
    const formatType = {
        time: 'HH:mm:ss',
        date: 'YYYY-MM-DD',
        datetime: 'YYYY-MM-DD HH:mm:ss',
    };
    let time = moment().format(formatType[dateType]).valueOf();
    try {
        // @ts-ignore
        t && (time = moment(t, formatType[dateType]).valueOf());
    }
    catch (e) { }
    if (dateType === 'time') {
        return formatTimeToNumber(time);
    }
    return time;
};
/**
 * 日期，日期时间按照毫秒区间转换为范围查询
 */
export const dateCalculation = ({ key, val, rel }, filterType) => {
    const dateSearchvalue = [];
    // 1.date日期时间查询精确到秒，换算成毫秒时间戳存储，区间相隔一秒
    // 2.datetime日期查询精确到日，日期换算成毫秒时间戳存储，区间相隔一日
    // 3.time时间查询精确到毫秒,换算成毫秒存储，区间相隔一秒
    const endTime = filterType === 'date' ? 86400000 : 999;
    switch (rel) {
        case 'eq':
            dateSearchvalue.push({
                key,
                val,
                rel: 'gte',
            });
            dateSearchvalue.push({
                key,
                val: val + endTime,
                rel: 'lte',
            });
            break;
        case 'neq':
            dateSearchvalue.push({
                key,
                val,
                rel: 'lt',
            });
            dateSearchvalue.push({
                key,
                val: val + endTime,
                rel: 'gt',
            });
            break;
        case 'gt':
            dateSearchvalue.push({
                key,
                val: val + endTime,
                rel: 'gt',
            });
            break;
        case 'lt':
            dateSearchvalue.push({
                key,
                val,
                rel: 'lt',
            });
            break;
        case 'gte':
            dateSearchvalue.push({
                key,
                val,
                rel: 'gte',
            });
            break;
        case 'lte':
            dateSearchvalue.push({
                key,
                val: val + endTime,
                rel: 'lte',
            });
            break;
        default:
            break;
    }
    return dateSearchvalue;
};
