import { START_EVENT, APPROVAL_NODE_TYPE, RESPONSE, DATA, X_REQUIRED, APPROVING, } from '../constants';
/**
 *  组装节点数据
 */
export const getFlowStepNodes = (ProcessNodeList, SequenceFlowList) => {
    const nodes = [];
    const startNode = ProcessNodeList.find((node) => node.ElementType === START_EVENT);
    const startSequenceFlow = SequenceFlowList.find((node) => node.SourceNodeKey === startNode.NodeId);
    nodes.push(startNode);
    const getLineNodes = (sequenceFlow) => {
        ProcessNodeList.forEach((node) => {
            if (
            // eslint-disable-next-line
            sequenceFlow.TargetNodeKey === node.NodeId &&
                nodes.findIndex((n) => n.NodeId === node.NodeId) === -1) {
                nodes.push(node);
                const order = SequenceFlowList.find((o) => o.SourceNodeKey === node.NodeId);
                if (!order)
                    return;
                getLineNodes(order);
            }
        });
    };
    getLineNodes(startSequenceFlow);
    return nodes;
};
/**
 *  获取审批节点
 */
export const getApprovalNode = (nodes) => nodes.find((node) => node.ElementType === APPROVAL_NODE_TYPE);
export const formatResData = (resData) => resData[RESPONSE] && resData[RESPONSE][DATA];
// 合并字段信息
export const mergeFieldsInfo = (properties = {}, configurableFields = []) => {
    const fields = [];
    configurableFields.forEach((field) => {
        const { Key, Required } = field;
        const f = properties && properties[Key];
        if (f) {
            fields.push({
                ...f,
                [X_REQUIRED]: Required,
                name: Key,
            });
        }
    });
    return fields;
};
// setState obj
export const setObjectData = (oldValue, newValue) => {
    const data = { ...oldValue };
    // eslint-disable-next-line no-restricted-syntax
    for (const key in newValue) {
        // eslint-disable-next-line no-prototype-builtins
        if (newValue.hasOwnProperty(key)) {
            data[key] = newValue[key];
        }
    }
    return data;
};
export const isCurrentApprover = (wedaUserId, approvalUserList = [], IS_WEDA_IDE) => {
    const userId = wedaUserId || '';
    if (IS_WEDA_IDE)
        return true;
    return approvalUserList.findIndex((u) => u === userId) !== -1;
};
// 是否渲染审批驳回 通过按钮 流程操作按钮区域：只有当前节点审批人可见 且流程状态是审批中
export const isRenderOperateButton = (...params) => {
    const [approvalUserList = [], status, taskId, approvalRoleIdList = [], datasourceUserFieldList = [], wedaUserId, wedaRelatedRoles, IS_WEDA_IDE,] = params || [];
    const isCurApprover = isCurrentApprover(wedaUserId, approvalUserList, IS_WEDA_IDE);
    const isApproving = status === APPROVING;
    // 当前用户所属的角色
    const relatedRoles = wedaRelatedRoles || [];
    let isIn = false;
    approvalRoleIdList.forEach((roleId) => {
        if (relatedRoles.map((v) => v.Id).includes(roleId)) {
            isIn = true;
        }
    });
    // 如果当前用户不在审批人列表以及角色列表里，那么显示条件为 1.DatasourceUserFieldList不为空 2.审批中 3.tasId不为空
    if (!(isCurApprover || isIn)) {
        return (datasourceUserFieldList === null || datasourceUserFieldList === void 0 ? void 0 : datasourceUserFieldList.length) && isApproving && taskId;
    }
    return (isCurApprover || isIn) && isApproving && taskId;
};
// 是否渲染撤回按钮， 条件：1.流程是审批中 2.RevokeTypeList 非空数组 3.当前登陆用户为发起人 4.触发方式为即时触发
export const isRenderRevokeButton = ({ nodes, status }, wedaUserId) => {
    var _a;
    // const currentFlowStatus =
    const serviceTriggerTaskNode = nodes.find((node) => { var _a; return ((_a = node === null || node === void 0 ? void 0 : node.ProcessNode) === null || _a === void 0 ? void 0 : _a.ElementType) === 'ServiceTriggerTask'; });
    const { RevokeTypeList: revokeTypeList, StartUserId: startUserId, TriggerType: triggerType, } = ((_a = serviceTriggerTaskNode === null || serviceTriggerTaskNode === void 0 ? void 0 : serviceTriggerTaskNode.ProcessNode) === null || _a === void 0 ? void 0 : _a.TriggerNode) || {};
    const isApproving = status === 'APPROVING'; // 1.流程是审批中
    const isRevokeType = revokeTypeList === null || revokeTypeList === void 0 ? void 0 : revokeTypeList.length; // 2.RevokeTypeList 非空数组
    const currentUserIsStartUser = startUserId === wedaUserId; // 3.当前登陆用户为发起人
    const isDataTrigger = triggerType === 'DataTrigger'; // 4.触发方式为即时触发
    return isApproving && isRevokeType && currentUserIsStartUser && isDataTrigger;
};
export const getVariables = (values, fields) => {
    const data = [];
    // eslint-disable-next-line no-restricted-syntax
    for (const key in values) {
        // eslint-disable-next-line no-prototype-builtins
        if (values.hasOwnProperty(key)) {
            const temp = fields.find((v) => v.name === key);
            if (temp) {
                data.push(JSON.stringify({
                    name: key,
                    value: values[key],
                }));
            }
        }
    }
    return data;
};
