export class PollingInterface {
    constructor(fn, interval = 5000, count = 10) {
        Object.defineProperty(this, "timeObj", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        }); // 轮询对象
        Object.defineProperty(this, "timeId", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        }); // 轮询id
        Object.defineProperty(this, "isFirst", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: true
        }); // 是否首次轮询
        this.fn = fn; // 轮询方法体
        this.interval = interval; // 轮询时间
        this.defaultCount = count; // 默认轮询次数
        this.count = count; // 轮询次数
    }
    setCallback(cb) {
        this.cb = cb;
    }
    // 开启轮询
    start() {
        // eslint-disable-next-line no-plusplus
        const id = this.timeId++;
        this.timeObj[id] = true;
        const timerFn = () => {
            if (!this.timeObj[id])
                return;
            this.fn();
        };
        // 若是首次，直接执行方法体，并修改是否首次状态为false，并减少次数
        if (this.isFirst) {
            timerFn();
            // eslint-disable-next-line no-plusplus
            this.count--;
            this.isFirst = false;
            return;
        }
        if (this.count > 0) {
            setTimeout(timerFn, this.interval);
            // eslint-disable-next-line no-plusplus
            this.count--;
        }
        else {
            this.stop();
            typeof this.cb === 'function' && this.cb(this.count);
        }
    }
    // 停止轮询
    stop() {
        this.count = this.defaultCount;
        this.timeObj = {};
        this.isFirst = true;
    }
}
