/* **** 通用工具相关 **** */
/**
 * 生成随机字符串
 */
export const createRandomString = (len = 32) => {
    const count = Math.ceil(Number(len) / 10) + 1;
    let ret = '';
    for (let i = 0; i < count; i++) {
        ret += Math.random().toString(36).substr(2);
    }
    return ret.substr(0, len);
};
/**
 * 表单数据过滤
 * @param {*} formValues 表单提交的数据
 * @param {*} formFieldsEntityList 表单的字段信息
 */
export const filterInvalidFormValue = (formValues, formFieldsEntityList) => {
    const has = Object.prototype.hasOwnProperty;
    // 过滤规则，针对一些特定的空值过滤规则在此进行扩展
    const filterRules = (value, fieldEntity) => [
        // 过滤掉 值为undefined 的字段
        [undefined].includes(value),
        // 过滤掉 枚举类型 且 值为空字符串 的字段
        has.call(fieldEntity, 'x-enumObj') && value === '',
    ];
    // 遍历字段信息,执行过滤规则
    return formFieldsEntityList.reduce((newFormValue, fieldEntity) => {
        const fieldName = fieldEntity === null || fieldEntity === void 0 ? void 0 : fieldEntity.name;
        const fieldValue = formValues === null || formValues === void 0 ? void 0 : formValues[fieldName];
        const isEffectiveFieldValue = !filterRules(fieldValue, fieldEntity).includes(true);
        if (isEffectiveFieldValue) {
            // eslint-disable-next-line no-param-reassign
            newFormValue[fieldName] = fieldValue;
        }
        return newFormValue;
    }, {});
};
/**
 * 字符串安全转JSON
 */
export const safeJsonParse = (val) => {
    let result = val;
    if (typeof val === 'string') {
        try {
            result = JSON.parse(val);
        }
        catch (e) { }
    }
    return result;
};
/**
 * 判断值是否为空
 */
export const isValueNone = (val) => [undefined, '', null].includes(val);
/**
 * 判断值是否为空，在 isValueNone 基础上加上空数组
 */
export const isValueEmpty = (val) => isValueNone(val) || (Array.isArray(val) && (val === null || val === void 0 ? void 0 : val.length) === 0);
/**
 * 比较两个数组是否一样
 */
export const isEqualArray = (a1, a2) => {
    if (!Array.isArray(a1) || !Array.isArray(a2) || (a1 === null || a1 === void 0 ? void 0 : a1.length) !== (a2 === null || a2 === void 0 ? void 0 : a2.length))
        return false;
    for (let i = 0; i < a1.length; i++) {
        if (a1[i] !== a2[i])
            return false;
    }
    return true;
};
